	SUBROUTINE ALPHA
C  THIS SUBROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE THE FOLLOWING:
C  ALPHAX(I,J,NL,K) - CHARGE EXCHANGE ABSORPTION COEFFICIENT IN ZONE I
C                  FOR HYDROGENIC NEUTRAL SPECIE K BORN IN ZONE J
C		   CREATING HYDROGENIC NEUTRAL SPECIES NL IN
C		   ZONE I.
C  ALPHAT(I,J,K) - TOTAL ABSORPTION COEFFICIENT IN ZONE I FOR
C                  HYDROGENIC NEUTRAL SPECIE K BORN IN ZONE J
C		   INTERACTING WITH ALL CHARGED SPECIES IN ZONE I.
C  BETAX(I,NL,K)    - CHARGE EXCHANGE ABSORPTION COEFFICIENT IN ZONE I
C                  FOR HYDROGENIC NEUTRAL SPECIES K BORN AT THE WALL
C                  CREATING HYDROGENIC NEUTRAL SPECIES NL IN
C                  ZONE I.
C  BETAT(I,K)    - TOTAL ABSORPTION COEFFICIENT IN ZONE I FOR
C                  HYDROGENIC NEUTRAL SPECIE K BORN AT THE WALL
C		   INTERACTING WITH ALL CHARGED SPECIES IN ZONE I.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	DIMENSION DSVEL(CNT$3)
C
	EQUIVALENCE (DSVEL(1),TSTOR1(1,1))
C
	DATA CON1/1.929691E15/
C  CON1= 2*C*C/(KEV/AMU)
C
C  I NOW FILL TEMPORARY STORAGE ARRAY TSTOR1 WITH DENE*<SIGMA*V>EL
C  AT ALL ZONE LOCATIONS.
	DO 10 I=1,NZONES
	  DSVEL(I)=DEN(I,1)*SVEL(TEMP(I,1))
10	CONTINUE
C  I NOW LOOP OVER THE DIFFERENT NEUTRAL SPECIES.
	DO 80 K=1,NNEUT
C  I NOW DEFINE THE MASS, Z, AND WALL VELOCITY FOR THIS NEUTRAL SPECIES.
	  NK=NSPEC(K)
	  TSKA=APLAS(NK)
	  TSKZ=ZPLAS(NK)
C  CHECK TO SEE IF NEUTRAL IS HYDROGENIC.
	  IF(TSKZ.GT.1.01) GO TO 90
	  EWALL=1.5E00*TWALL(K)
	  VWALL=SQRT(CON1*EWALL/TSKA)
C  I NOW LOOP OVER ALL SPECIES WHICH THIS NEUTRAL CAN INTERACT WITH.
	  DO 70 L=2,NPLAS
	    LM1=L-1
	    TSLA=APLAS(L)
	    TSLZ=ZPLAS(L)
C  I NOW LOOP OVER THE BIRTH ZONES OF THE MULTI-GENERATION NEUTRALS.
C  NOTE THAT THIS SAME LOOP WILL BE USED AS DEATH ZONES FOR WALL
C  NEUTRALS.
	    DO 60 J=1,NZONES
C  I NOW FIND THE ENERGY AND VELOCITY OF THE BIRTH NEUTRAL.
	      EBORN=1.5E00*TEMP(J,NK)
	      ECL=1.5E00*TEMP(J,L)
	      VBORN=SQRT(CON1*EBORN/TSKA)
C  I NOW FIND THE CREATED NEUTRAL'S INDEX.
	      NL=ISPEC(LM1)
	      IF(.NOT.ESFLAG) GO TO 30
C  I NOW FIND THE RELATIVE ENERGY AND VELOCITY BETWEEN A WALL NEUTRAL
C  OF SPECIES K AND AN ION OF SPECIE L AT LOCATION J.
	      ERELW=EWALL+TSKA*ECL/TSLA
	      VRELW=SQRT(CON1*ERELW/TSKA)
C  I NOW FIND THE ION DENSITY AT THIS ZONE.
	      DENJ=DEN(J,L)
C  I NOW FIND (SIGMA)LOSS AND (SIGMA)CX FOR THE WALL NEUTRAL
C  INTERACTING AT ZONE J
	      SIGL=SIGLOS(ERELW,TSKA,TSLZ)
	      IF(TSLZ.GT.1.01) GO TO 20
C  WE CAN CHARGE EXCHANGE OFF A HYDROGENIC SPECIE.
C  I NOW CHECK IF THIS IS AN ALLOWED NEUTRAL.
	      IF(NL.EQ.0) GO TO 20
	      SIGX=SIGCX(ERELW,TSKA)
	      BETAX(J,NL,K)=DENJ*SIGX*VRELW/VWALL
20	      IF(L.EQ.2) THEN
	      TSWBII=BII(J,K)/VWALL
	      TSWBCX=BCX(J,K)/VWALL
	      BETAT(J,K)=DSVEL(J)/VWALL+TSWBII+TSWBCX
	      END IF
	      BETAT(J,K)=DENJ*SIGL*VRELW/VWALL+BETAT(J,K)
C  I NOW LOOP OVER ALL INTERACTION ZONES.
30	      IF(.NOT.MCFLAG) GO TO 60
	      DO 50 I=1,NZONES
C  I NOW FIND THE ENERGY OF THE ION IN THIS ZONE AND CALCULATE
C  THE RELATIVE ENERGY AND VELOCITY BETWEEN THIS ION AND THE
C  NEUTRAL FROM ZONE J.
	        EC=1.5E00*TEMP(I,L)
	        EREL=EBORN+TSKA*EC/TSLA
	        VREL=SQRT(CON1*EREL/TSKA)
	        DENI=DEN(I,L)
C  I NOW FIND (SIGMA)LOSS AND (SIGMA)CX FOR THE NEUTRAL
C  BORN IN ZONE J INTERACTING WITH THE IONS IN ZONE I.
	        SIGL=SIGLOS(EREL,TSKA,TSLZ)
	        IF(TSLZ.GT.1.01) GO TO 40
C  WE CAN CHARGE EXCHANGE OF A HYDROGENIC SPECIE.
	        IF(NL.EQ.0) GO TO 40
	        SIGX=SIGCX(EREL,TSKA)
	        ALPHAX(I,J,NL,K)=DENI*SIGX*VREL/VBORN
40	        IF(L.EQ.2) THEN
	        TSMBII=BII(I,K)/VBORN
	        TSMBCX=BCX(I,K)/VBORN
	        ALPHAT(I,J,K)=DSVEL(I)/VBORN+TSMBII+TSMBCX
	        END IF
	        ALPHAT(I,J,K)=DENI*SIGL*VREL/VBORN+ALPHAT(I,J,K)
50	      CONTINUE
60	    CONTINUE
70	  CONTINUE
80	CONTINUE
	RETURN
90	WRITE(LUN6,100) K
100	FORMAT(T2,'NEUTRAL SPECIES ',I2,' IS NOT HYDROGENIC')
	STOP
	END
C
	PARAMETER CNT$1=2,CNT$2=5,CNT$3=20,CNT$4=CNT$2-1,
     >CNT$5=CNT$3+1,CNT$6=2*CNT$3-1
C
C  DEFINITION OF PARAMETERS.
C  CNT$1     - MAXIMUM NUMBER OF NEUTRALS.
C  CNT$2     - MAXIMUM NUMBER OF PLASMA SPECIES (ELECTRONS AND IONS).
C  CNT$3     - MAXIMUM NUMBER OF ZONES.
C  CNT$4     - MAXIMUM NUMBER OF IONS IN THE PLASMA.
C  CNT$5     - MAXIMUM NUMBER OF ZONE BAOUDARIES.
C  CNT$6     - ARRAY SIZE FOR TRAJECTORY TRACKING
C
	LOGICAL DEBUG,ESFLAG,VSFLAG,MCFLAG,MSFLAG,INFLAG,RCFLAG,
     >IBFLAG
C
	COMMON/CNEUT/APLAS(CNT$2),ZPLAS(CNT$2),NSPEC(CNT$1),
     >DEN(CNT$3,CNT$2),
     >TEMP(CNT$3,CNT$2),DNEUT(CNT$3,CNT$1),TNEUT(CNT$3,CNT$1),
     >ZONEC(CNT$3),NZONES,NBOUND,
     >NPLAS,DEBUG,ESFLAG,VSFLAG,MCFLAG,MSFLAG,INFLAG,RCFLAG,IBFLAG,
     >NNEUT,ALPHAX(CNT$3,CNT$3,CNT$1,CNT$1),ALPHAT(CNT$3,CNT$3,CNT$1),
     >BETAX(CNT$3,CNT$1,CNT$1),
     >BETAT(CNT$3,CNT$1),TSTOR1(CNT$3,CNT$1),TWALL(CNT$1),LUN6,
     >RBOUND(CNT$5),Y(CNT$3),
     >DIST(CNT$6),NZCROS(CNT$6),NBCROS,MID,DCOLD(CNT$3,CNT$1),
     >WALL(CNT$3,CNT$1),
     >GFUN(CNT$3,CNT$3,CNT$1,CNT$1),VOL(CNT$3),
     >FLUXIN(CNT$1),ISPEC(CNT$4),S0COLD(CNT$3,CNT$1,CNT$1),
     >VOLSOR(CNT$3,CNT$1),SVRC(CNT$3),BII(CNT$3,CNT$1),BCX(CNT$3,CNT$1),
     >SORI(CNT$3,CNT$1),SORN(CNT$3,CNT$1),SORE(CNT$3),PRC(CNT$3),
     >PCX(CNT$3),PII(CNT$3),AMINOR,WWALL(CNT$1),
     >PLOSS,ELOSS,PLOSS1,ELOSS1
C
C  DEFINITION OF COMMON BLOCK VARIABLES.
CI APLAS  - ARRAY GIVING AMU FOR EACH PLASMA SPECIES.  (AMU)
CI ZPLAS  - ARRAY GIVING Z FOR EACH PLASMA SPECIES.
CI NSPEC  - ARRAY GIVING CORRESPONDENCE BETWEEN NEUTRAL SPECIES
C           AND CHARGE SPECIES. I.E. NSPEC(1)=3 MEANS THAT
C           NEUTRAL SPECIES 1 IS THE SAME AS PLASMA SPECIES 3.
CI DEN    - 2-D ARRAY GIVING THE DENSITY FOR EACH PLASMA SPECIES
C           AT THE VARIOUS MINOR RADII LOCATIONS.  (1/CM**3)
CI TEMP   - 2-D ARRAY GIVING THE TEMPERATURE FOR EACH PLASMA SPECIES
C           AT THE VARIOUS MINOR RADII LOCATIONS.  (KEV)
C  DNEUT  - 2-D ARRAY GIVING THE DENSITY FOR EACH NEUTRAL SPECIES
C           AT THE VARIOUS MINOR RADII LOCATIONS.  (1/CM**3)
C  TNEUT  - 2-D ARRAY GIVING THE TEMPERATURE FOR EACH NEUTRAL SPECIES
C           AT THE VARIOUS MINOR RADII LOCATIONS.  (KEV)
CI ZONEC  - ARRAY GIVING THE VARIOUS MINOR RADII LOCATIONS AT
C           ZONE CENTERS.  (CM)
CI NZONES - THE ACTUAL NUMBER OF ZONE CENTERS.
C  NBOUND - THE ACTUAL NUMBER OF ZONE BOUNDARIES.
CI NPLAS  - THE ACTUAL NUMBER OF PLASMA SPECIES.
CI DEBUG  - DEBUG FLAG.
C	    TRUE = DEBUG PRINTOUT IS DONE.
C	    FALSE = NO DEBUG PRINTOUT DONE.
CI ESFLAG - EDGE SOURCE FLAG.
C	    TRUE - EDGE NEUTRAL DENSITY AND SOURCE IS CALCULATED.
C	    FALSE - EDGE CONTRIBUTION IS NOT INCLUDED.
CI VSFLAG - VOLUME SOURCE FLAG.
C	    TRUE - DO VOLUME SOURCE CALCULATION.
C	    FALSE - DO NOT INCLUED VOLUME SOURCE.
CI MCFLAG - MATRIX CALCULATION FLAG.
C	    TRUE - MATRIX IS CALCULATED.
C	    FALSE - OLD MATRIX IS USED.
C  MSFLAG - MATRIX STATUS FLAG.
C	    TRUE - A MATRIX HAD BEEN CALCULATED.
C	    FALSE - MATRIX HAS NEVER BEEN CALCULATED.
CI INFLAG - INITIALIZATION FLAG.
C	    TRUE - COMPLETE INITIALIZATION IS DONE.
C	    FALSE - NO INITIALIZATION IS DONE.
CI RCFLAG - RECOMBINATION FLAG.
C	    TRUE - RECOMBINATION IS INCLUDED.
C	    FALSE - RECOMBINATION IS NOT INCLUDED.
CI IBFLAG - BEAM FLAG
C           TRUE - USE BII AND BCX TO DELETE NEUTRALS.
C	    FALSE - DON'T USE BII AND BCX.
CI NNEUT  - THE ACTUAL NUMBER OF NEUTRAL SPECIES.
C  ALPHAX - 4-D ARRAY GIVING THE CHARGE EXCHANGE ABSORPTION
C           COEFICIENT FOR THERMAL NEUTRALS.  (1/CM)
C  ALPHAT - 3-D ARRAY GIVING THE TOTAL ABSORPTION COEFICIENT
C           FOR THERMAL NEUTRALS.  (1/CM)
C  BETAX  - 3-D ARRAY GIVING THE CHARGE EXCHANGE ABSORPTION
C           COEFICIENT FOR WALL NEUTRALS.  (1/CM)
C  BETAT  - 2-D ARRAY GIVING THE TOTAL ABSORPTION COEFICIENT
C           FOR WALL NEUTRALS.  (1/CM)
C  TSTOR1 - 2-D ARRAY GIVING TEMPORARY STORAGE.
CI TWALL  - ARRAY GIVING THE WALL TEMPERATURE FOR THE DIFFERENT
C           NEUTRAL SPECIES.  (KEV)
CI LUN6   - LOGICAL UNIT FOR PRINTING ERROR MESSAGES.
C  RBOUND - ARRAY GIVING THE ZONE BOUNDARIES.  (CM)
C  Y      - ARRAY GIVING THE DISTANCE OF CLOSEST APPROACH.  (CM)
C  DIST   - ARRAY GIVING THE PATH LENGTH OF THE TRAJECTORY IN
C           EACH ZONE.
C  NZCROS - ARRAY GIVING THE ZONE NUMBER AS CROSSED FROM LEFT
C           TO RIGHT.
C  NBCROS - THE NUMBER OF ZONE BOUNDARIES CROSSED FOR THE GIVEN
C           TRAJECTORY.
C  MID    - THE ZONE NUMBER FOR THE MIDDLE ELEMENT OF THE TRAJECTORY.
C  DCOLD  - 2-D ARRAY GIVING THE DENSITY FOR EACH WALL NEUTRAL SPECIES
C           AT THE VARIOUS MINOR RADII LOCATIONS.  (1/CM**3)
C  WALL   - 2-D GIVING THE PROBABILITY THAT NEUTRAL SPECIE
C	    K BORN IN ZONE I ESCAPES TO THE WALL.
C  VOL    - ARRAY GIVING THE VOLUME/LENGTH OF THE GIVEN ZONE.  (CM**2)
CI FLUXIN - ARRAY GIVING THE FLUX OF NEUTRALS OF SPECIES K
C           FROM THE WALL.  (#/(SEC*CM**2))
C  ISPEC  - ARRAY GIVING CORRESPONDENCE BETWEEN CHARGED SPECIES
C           AND NEUTRAL SPECIES. I.E. ISPEC(3)=1 MEANS THAT
C           CHARGED SPECIES 3 IS THE SAME AS NEUTRAL SPECIES 1.
C           NOTE THAT A VALUE OF 0 MEANS THAT FOR CHARGED PARTICLE
C           I THERE IS NO NEUTRAL SPECIES.
C  S0COLD - 3-D ARRAY GIVING THE SOURCE RATE OF NEW NEUTRAL RESULTING
C           FROM THE INTERACTION OF COLD NEUTRALS OF SPECIES K
C           INTERACTING WITH CHARGE SPECIES L IN ZONE I.
CI VOLSOR - 2-D ARRAY GIVING THE VOLUME SOURCE RATE OF NEUTRALS
C	    OF SPECIES NL IN ZONE I.  (#/(SEC*CM**3))
C  SVRC   - ARRAY GIVING DENE*<SIGMA*V> FOR ELECTRON RECOMBINATION.
C           (#/SEC)
CI BII    - 2-D ARRAY GIVING NBEAM*<SIGMA*V>II FOR BEAM ION 
C	    INTERACTING WITH NEUTRAL SPECIES K IN ZONE I.  (1/SEC)
CI BCX    - 2-D ARRAY GIVING NBEAM*<SIGMA*V>CX FOR BEAM ION
C	    INTERACTING WITH NEUTRAL SPECIES K IN ZONE I.  (1/SEC)
C  SORI   - 2-D ARRAY GIVING THE NET SOURCE OF IONS AT RADIAL POINTS.
C	    (#/(SEC*CM**3))
C  SORN   - 2-D ARRAY GIVING THE NET SOURCE OF NEUTRALS AT RADIAL POINTS.
C	    (#/(SEC*CM**3))
C  SORE   - ARRAY GIVING THE NET SOURCE OF ELECTRONS AT RADIAL POINTS.
C	    (#/(SEC*CM**3))
C  PRC    - ARRAY GIVING THE RECOMBINATION POWER AT THE GIVEN RADIAL
C           LOCATION.  (WATTS/CM**3)
C  PCX    - ARRAY GIVING CHARGE EXCHANGE POWER FROM THERMAL IONS
C	    TO NEUTRALS AT RADIAL POINTS.  (WATTS/CM**3)
C  PII    - ARRAY GIVING IONIZATION POWER FROM NEUTRALS TO
C	    THERMAL IONS AT RADIAL POINTS.  (WATTS/CM**3)
CI AMINOR - THE MINOR RADIUS OF THE PLASMA.  (CM)
C  WWALL  - THE FLUX OF WALL NEUTRALS LEAVING PLASMA FOR SPECIES K.
C	    (#/(SEC*CM**2))
C  PLOSS  - NET NEUTRAL FLOW OUT OF PLASMA.  (#/SEC)  (+=OUT)
C  ELOSS  - NET POWER FROM NEUTRALS TO IONS.  (WATTS)
C  PLOSS1 - NET NEUTRAL SOURCE RATE.  (#/SEC)  (-=SINK)
C  ELOSS1 - NET POWER FROM IONS TO NEUTRALS.  (WATTS)
C
C  NOTE THAT QUANTITES MARKED CI ARE INPUT VARIABLES TO NEUTS AND
C  ASSOCIATED ROUTINES.
	COMMON /CNEUT1/RA(21),DENEA(21),TEA(21),TIA(21),NDENE,NTEMPE,
     >NTEMPI
C  RA     - ARRAY OF MINOR RADIA AT WHICH PROFILES ARE GIVEN.  (CM)
C  DENEA  - ARRAY OF ELECTRON DENSITY AT RA POINTS.  (1/CM**3)
C  TEA    - ARRAY OF ELECTRON TEMPERATURES AT RA POINTS.  (KEV)
C  TIA    - ARRAY OF ION TEMPERATURES AT RA POINTS.  (KEV)
C  NDENE  - NUMBER OF DENSITY POINTS.
C  NTEMPE - NUMBER OF ELECTRON TEMPERATURE POINTS.
C  NTEMPI - NUMBER OF ION TEMPERATURE POINTS.
	SUBROUTINE DEINIT(RA,DENEA,NDENE,XA,DENE,NSHELL,IFIT,IERR)
C  5/29/78
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS ROUTINE WILL COMPUTE THE VALUES
C  OF THE ELECTRON DENSITY AT EACH POINT IN ARRAY XA AND STORE
C  THE RESULTS IN ARRAY DENE.
C  PARAMETERS:
C  RA	  - ARRAY CONTAINING THE PLASMA HALF-WIDTHS AT WHICH THE
C	    ELECTRON DENSITY IS KNOWN.	(CM)
C  DENEA  - THE VALUE OF THE ELECTRON DENSITY AT THE POINTS ABOVE.
C	    (1/CM**3)
C  NDENE  - THE NUMBER OF POINTS IN THE ABOVE ARRAYS.
C  XA	  - THE ARRAY OF HALF-WIDTHS AT WHICH THE DENSITY IS TO BE
C	    EVALUATED.	(CM)
C  DENE   - THE ARRAY CONTAINING THE ELECTRON DENSITY AT EACH OF THE XA
C	    POINTS.  (1/CM**3)
C  NSHELL - THE # OF POINTS IN THE ABOVE ARRAYS.
C  IFIT   - IF IFIT .EQ. 0 AN ANALYTIC SOLUTION WILL BE CREATED USING
C	    THE INFORMATION IN ARRAY DENEA.  OTHERWISE THE ORGINAL DATA
C	    WILL BE FIT WITH CUBIC SPLINES AND INTERPOLATED.
C  IERR   - THE LOGICAL UNIT USED FOR PRINTING ERROR MESSAGES.
	DIMENSION RA(NDENE),DENEA(NDENE),XA(NSHELL),DENE(NSHELL)
	IF(IFIT.EQ.0) GO TO 10
	CALL DFITS(RA,DENEA,NDENE,XA,DENE,NSHELL,IERR)
	RETURN
10	DO 20 I=1,NSHELL
20	DENE(I)=(DENEA(1)-DENEA(4))*(1.E00-(XA(I)/DENEA(5))**DENEA(2))**
     >DENEA(3)+DENEA(4)
	RETURN
	END
	SUBROUTINE DFITS(X,Y,NX,U,S,M,IERR)
C  5/29/78
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON PLASMA
C  PHYSICS LAB.  THIS ROUTINE WILL FIT THE Y POINTS WITH CUBIC SPLINES
C  AND EVALUATE THE FITS AT POINTS U STORING THE RESULTS IN S.
C  PARAMETERS:
C  X	  - VECTOR NX CONTAINING THE ABSCISSAE OF THE NX DATA POINTS.
C  Y	  - VECTOR NX CONTAINING THE DATA VALUES.
C  NX	  - # OF DATA VALUES.
C  U	  - VECTOR M CONTAINING ABSCISSAE OF POINTS AT WHICH THE SPLINE
C	    FITS ARE TO BE EVALUATED.
C  S	  - VECTOR M CONTAINING THE INTERPOLATED VALUES AT THE ABOVE
C	    POINTS.
C  M	  - THE # OF POINTS IN THE ABOVE ARRAY.
C  IERR   - THE LOGICAL UNIT USED TO WRITE INFORMATION.
	DIMENSION X(NX),Y(NX),U(M),S(M),SPCOEF(21,3),BPAR(4)
	DATA NMAX/21/
C  I NOW FIND BPAR SUCH THAT FIRST DEIVATIVES ARE 0 AT ORIGIN.
	BPAR(1)=1.E00
	BPAR(2)=6.E00*(Y(2)-Y(1))/(X(2)-X(1))**2
	BPAR(3)=0.E00
	BPAR(4)=0.E00
C  I NOW FIND THE SPLINE COEFFICIENTS.
	IER=0
	CALL ICSICU(X,Y,NX,BPAR,SPCOEF,NMAX,IER)
	IF(IER.NE.0) GO TO 10
C  I NOW INTERPOLATE.
	CALL ICSEVU(X,Y,NX,SPCOEF,NMAX,U,S,M,IER)
	RETURN
10	WRITE(IERR,20) IER
20	FORMAT(/,T2,'DFITS:  ICSICU ERROR',I3)
	STOP
	END
	SUBROUTINE EDITR
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CONSTRUCT THE OUTPUT
C  RESULTS OF DNEUT, TNEUT, EDEP, RDOT AND SPECTRUM.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	DATA CON2/2.8945365E15/
C  CON2 = 3*C*C/(KEV/AMU)
C  I NOW LOOP OVER ALL ORGINAL NEUTRAL SPECIES.
	DO 30 K=1,NNEUT
	  NNK=NSPEC(K)
	  TSKA=APLAS(NNK)
C  I NOW FIND THE WALL VELOCITY FOR THIS SPECIES.
	  VWALL=SQRT(CON2*TWALL(K)/TSKA)
C  I NOW LOOP OVER ALL ZONES
	  DO 20 I=1,NZONES
C  I NOW LOOP OVER ALL CHARGED PARTICLES WHICH COULD
C  RESULT IN AN INTERACTION.
	    IF(ESFLAG) DCOLD(I,K)=S0COLD(I,1,K)/
     >(BETAX(I,1,K)*VWALL)
	    NL=1
	    LP1=NSPEC(NL)
	    SUM2=0.E00
	    SUM3=0.E00
C  I NOW LOOP OVER ALL BIRTH ZONES
	    DO 10 J=1,NZONES
	      VNEUT=SQRT(CON2*TEMP(J,NNK)/TSKA)
C I NOW FIND THE SOURCE RATE
	      GF=-GFUN(I,J,NL,K)
	      IF(J.EQ.I.AND.NL.EQ.K) GF=GF+1.E00
C  TS2 CONTAINS THE SOURCE RATE OF NEUTRALS NL IN ZONE I
C  DUE TO NEUTRALS OF TYPE K BORN IN ZONE J CHARGE EXCHANGING
C  ON CHARGED PARTICLE L.
	      TS2=GF*TSTOR1(J,K)
C  TS4 IS THE NUMBER OF MULTI-GENERATION NEUTRALS IN ZONE I DUE
C  TO NEUTRAL K BORN IN ZONE J INTERACTING WITH CHARGED PARTICLE
C  L.
	      TS4=TS2/(ALPHAX(I,J,NL,K)*VNEUT)
	      SUM2=SUM2+TS4
	      SUM3=SUM3+TS4*TEMP(J,LP1)
10	    CONTINUE
	    DNEUT(I,K)=SUM2
	    TNEUT(I,K)=SUM3
20	  CONTINUE
30	CONTINUE
	IF(.NOT.ESFLAG) GO TO 60
	DO 50 K=1,NNEUT
	  DO 40 I=1,NZONES
	  TOTALN=DNEUT(I,K)+DCOLD(I,K)
	  IF(TOTALN.LT.1.E-10) GO TO 40
	    TNEUT(I,K)=(TNEUT(I,K)+DCOLD(I,K)*TWALL(K))/TOTALN
40	  CONTINUE
50	CONTINUE
	GO TO 90
60	DO 80 K=1,NNEUT
	  DO 70 I=1,NZONES
	    TNEUT(I,K)=TNEUT(I,K)/DNEUT(I,K)
70	  CONTINUE
80	CONTINUE
90	RETURN
	END
	SUBROUTINE EDITS
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE THE NET SOURCES
C  FOR:
C  ELECTRONS
C  HYDROGENIC ION
C  HYDROGENIC NEUTRALS
C  POWER FLOW DUE TO CHARGE EXCHANGE  (IONS TO NEUTRALS)
C  POWER FLOW DUE TO IONIZATION  (NEUTRALS TO IONS)
C
	INCLUDE 'CNEUT.CMN/LIST'
C
	DATA CON1,CON2,TWOPI/1.6021892E-16,2.8945365E15,6.283185308E00/
C  CON1 = JOULES/KEV
C  CON2 = 3*C*C/(KEV/AMU)
C
	ELOSMG=0.E00
	VIPCX=0.E00
	VIPII=0.E00
	VIPRC=0.E00
	VIVS=0.E00
	VIPVS=0.E00
	FLUXMG=0.E00
	PLOSS1=0.E00
	ELOSW=0.E00
	FLUXT=0.E00
	EFLUX=0.E00
	FLUXW=0.E00
	WAREA=TWOPI*AMINOR
	DO 70 K=1,NNEUT

	  IF(.NOT.ESFLAG) GO TO 10
	  TWALLK=TWALL(K)
	  FLUXT=FLUXT+FLUXIN(K)
	  EFLUX=EFLUX+FLUXIN(K)*TWALLK
	  WWALL(K)=4.E00*WWALL(K)*FLUXIN(K)/WAREA
	  FLUXW=FLUXW+WWALL(K)
	  ELOSW=ELOSW+WWALL(K)*TWALLK
10	  KC=NSPEC(K)
C  KC GIVES THE ION NUMBER OF THE KTH NEUTRAL.
	  TKA=APLAS(KC)
C  I NOW FIND THE WALL VELOCITY FOR THIS NEUTRAL SPECIES.
	  VWALL=SQRT(CON2*TWALLK/TKA)
C  I NOW LOOP OVER ALL DEATH ZONES.
	  DO 60 I=1,NZONES
	  TS4=TSTOR1(I,K)/WAREA
	    FLUXMG=FLUXMG+WALL(I,K)*TS4
	    ELOSMG=ELOSMG+WALL(I,K)*TS4*TEMP(I,KC)
	    USUM1=0.E00
C  USUM1 WILL CONTAIN THE SOURCE OF ELECTRONS RESULTING FROM
C  THE INTERACTION OF THE K NEUTRAL SPECIES.
	    PCXMG=0.E00
C  PCXMG WILL CONTAIN THE CHARGE EXCHANGE POWER FLOW IN ZONE I
C  DUE TO MG NEUTRAL PARTICAL K INTERACTING WITH ALL CHARGED
C  PARTICLES.
	    PIIMG=0.E00
C  PIIMG WILL CONTAIN THE IONIZATION POWER FLOW IN ZONE I DUE
C  TO MG NEUTRAL PARTICLES BEING IONIZED.
	    PCXW=0.E00
C  PCXW WILL CONTAIN THE CHARGE EXCHANGE POWER FLOW IN ZONE I
C  DUE TO WALL NEUTRAL PARTICAL K INTERACTING WITH ALL CHARGED
C  PARTICLES.
	    PIIW=0.E00
C  PIIW WILL CONTAIN THE IONIZATION POWER FLOW IN ZONE I DUE TO
C  WALL NEUTRALS.
	    TSUM3=0.E00
C  TSUM3 WILL CONTAIN THE TOTAL MG REACTION RATE/CM**3 FOR THE
C  K NEUTRAL INTERACTING IN ZONE I.
	    TS1=DCOLD(I,K)*VWALL
	    SUM=0.E00
C  SUM CONTAINS THE WALL SOURCE OF NEUTRALS PRODUCED BY THE K NEUTRAL
C  INTERACTING WITH ALL CHARGED PARTICLES.
	    SUM2=0.E00
C  SUM2 CONTAINS THE POWER OF THE K MG NEUTRAL SPECIES IN ZONE I
C  SUMMED OVER ALL BIRTH ZONES AND OVER ALL CHARGED PARTICLES.
	    SUM3=0.E00
C  SUM3 CONTAINS THE POWER OF THE K WALL NEUTRAL SPECIES IN ZONE I
C  SUMMED OVER ALL CHARGED PARTICLES.
	    DO 40 NL=1,NNEUT
C  NL REPRESENTS THE CHARGED HYDROGENIC SPECIES
	      NLC=NSPEC(NL)
	      TSUM1=0.E00
C  TSUM1 WILL CONTAIN THE SOURCE OF ALL CX MG NEUTRALS IN ZONE I
C  SUMMED OVER ALL BIRTH ZONES FOR THE K NEUTRAL AND THE NLC PLASMA
C  SPECIE.
C  SAME AS ABOVE BUT FOR CROSS SPECIES.
	      TEMPI=TEMP(I,NLC)
C  TEMPI IS THE TEMPERATURE OF THE NLC PLASMA SPECIES IN ZONE I.
C  I NOW LOOP OVER THE BIRTH ZONES OF THE MULTI-GENERATION NEUTRALS.
	      DO 30 J=1,NZONES
	        TEMPJ=TEMP(J,KC)
C  TEMPJ IS THE TEMPERATURE OF THE K NEUTRAL SPECIE BORN IN J.
C  I NOW FIND THE VELOCITY AT WHICH THE NEUTRAL WAS BORN.
	        VNEUT=SQRT(CON2*TEMPJ/TKA)
C  I NOW FIND THE SOURCE OF NEUTRALS
	        SMG=-GFUN(I,J,NL,K)
	        IF(J.EQ.I.AND.NL.EQ.K) SMG=SMG+1.E00
	        SMG=SMG*TSTOR1(J,K)
C  SMG IS NOW THE SOURCE RATE OF MULTI-GENERATION NEUTRALS OF SPECIES
C  NL IN ZONE I DUE TO NEUTRALS OF TYPE K BORN IN ZONE J CHARGE
C  EXCHANGING ON CHARGED PARTICLE L.
	        TSUM1=TSUM1+SMG
	        PCXMG=PCXMG+(TEMPI-TEMPJ)*SMG
	        SUM2=SUM2+TEMPJ*SMG
	        IF(NL.GT.1) GO TO 20
C  INCLUDE TOTAL MG REACTIONS.
	        TS8=ALPHAT(I,J,K)*SMG/ALPHAX(I,J,1,K)
	        TSUM3=TSUM3+TS8
	        USUM1=USUM1+TS8		!SOURCE DUE TO MG IONIZATION
	        PIIMG=PIIMG+TS8*TEMPJ
C  NOTE THAT THE NEUTRAL POWER (SUM2) WILL BE SUBTRACTED LATER.
20	        CONTINUE
30	      CONTINUE
C  NEXT 3 LINES GIVE CONTRIBUTION DUE TO MG NEUTRALS.
	      SORI(I,NL)=SORI(I,NL)-TSUM1	!SINK DUE TO MG CX
	      SORN(I,NL)=SORN(I,NL)+TSUM1	!SOURCE DUE TO MG CX
	      USUM1=USUM1-TSUM1		!CORRECTION FOR MG CX
	      IF(.NOT.ESFLAG) GO TO 40
	      TS2=S0COLD(I,NL,K)
	      SORI(I,NL)=SORI(I,NL)-TS2		!SINK DUE TO WALL CX
	      SORN(I,NL)=SORN(I,NL)+TS2		!SOURCE DUE TO WALL CX
	      PCXW=PCXW+(TEMPI-TWALLK)*TS2
	      SUM3=SUM3+TS2*TWALLK
	      SUM=SUM+TS2
40	    CONTINUE
C  NEXT 3 LINES GIVE CONTRIBUTION DUE TO MG NEUTRALS.
	    PIIMG=PIIMG-SUM2
	    SORI(I,K)=SORI(I,K)+TSUM3		!SOURCE DUE TO MG ION.
	    SORN(I,K)=SORN(I,K)-TSUM3		!SINK DUE TO MG ION.
	    IF(.NOT.ESFLAG) GO TO 50
	    TS5=BETAT(I,K)*TS1
	    SORI(I,K)=SORI(I,K)+TS5		!SOURCE DUE TO WALL ION.
	    SORN(I,K)=SORN(I,K)-TS5		!SINK DUE TO WALL ION.
	    USUM1=USUM1+TS5-SUM		!SOURCE DUE TO WALL IONIZATION
	    PIIW=PIIW+TS5*TWALLK-SUM3
50	    CONTINUE
C  THE ELECTRON SOURCE IS NOW CORRECTED FOR BEAM CX.
	    IF(IBFLAG) USUM1=USUM1-BCX(I,K)*(DCOLD(I,K)+DNEUT(I,K))
	    PCX(I)=PCX(I)+PCXMG+PCXW
	    PII(I)=PII(I)+PIIMG+PIIW
	    SORE(I)=SORE(I)+USUM1
	    IF(.NOT.RCFLAG) GO TO 60
	    TS6=SVRC(I)*DEN(I,KC)
	    SORI(I,K)=SORI(I,K)-TS6
	    SORE(I)=SORE(I)-TS6
	    SORN(I,K)=SORN(I,K)+TS6
	    PRC(I)=PRC(I)+TS6*TEMP(I,KC)
60	  CONTINUE
70	CONTINUE
C  I NOW CONVERT POWER TERMS TO WATTS
	DO 100 I=1,NZONES
	  PCX(I)=PCX(I)*CON1*1.5E00
	  PII(I)=PII(I)*CON1*1.5E00
	  VOLI=VOL(I)
	  VIPII=VIPII+PII(I)*VOLI
	  VIPCX=VIPCX+PCX(I)*VOLI
	  IF(.NOT.RCFLAG) GO TO 80
	  PRC(I)=PRC(I)*CON1*1.5E00
	  VIPRC=VIPRC+PRC(I)*VOLI
80	  DO 90 K=1,NNEUT
	    KC=NSPEC(K)
	    PLOSS1=PLOSS1+SORN(I,K)*VOLI
	    IF(.NOT.VSFLAG) GO TO 90
	    TS3=VOLSOR(I,K)*VOLI
	    VIVS=VIVS+TS3
	    VIPVS=VIPVS+TS3*TEMP(I,KC)
90	  CONTINUE
100	CONTINUE
	PLOSS=WAREA*(FLUXMG+FLUXW-FLUXT)-VIVS
	ELOSS=(WAREA*(ELOSMG+ELOSW-EFLUX)-VIPVS)*1.5E00*CON1
	ELOSS1=VIPCX+VIPRC-VIPII
	RETURN
	END
	SUBROUTINE EXTREM(ARRAY,NUM,XMIN,XMAX)
C	07/23/76
C	THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C	PLASMA PHYSICS LAB.  THIS ROUTINE WILL FIND THE EXTREM VALUES
C	IN ARRAY.
C	PARAMETER LIST:
C	ARRAY - AN ARRAY WHICH CONTAINS THE VALUES TO BE SEARCHED.
C	NUM - THE NUMBER OF VALUES IN THE ABOVE ARRAY.
C	XMIN - THE SMALLEST VALUE IN THE ARRAY.
C	XMAX - THE LARGEST VALUE IN THE ARRAY.
	DIMENSION ARRAY(NUM)
C	I NOW INITIALIZE XMIN AND XMAX.
	XMIN=1.E+38
	XMAX=-1.E+38
C	NOW NOW COMPARE ALL VALUES.
	DO 1000 I=1,NUM
	X=ARRAY(I)
	IF(X.GT.XMAX) XMAX=X
1000	IF(X.LT.XMIN) XMIN=X
	RETURN
	END
	SUBROUTINE EXTRM2(ARRAY,IROW,N1,N2,XMIN,XMAX)
C  12/14/78
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS ROUTINE WILL FIND THE MIMIMUM AND
C  MAXIMUM VALUES WITHIN A 2-D ARRAY.
C
C  PARAMETERS:
C  ARRAY  - THE NAME OF THE 2-D ARRAY.
C  IROW   - THE ROW DIMENSION OF THIS ARRAY AS GIVEN IN THE CALLING
C	    PROGRAM.  (ROW DIMENSION IS THE LEFT ARGUEMENT)
C  N1	  - THE NUMBER OF ROW ELEMENTS TO BE SEARCHED.	N1 .LE. IROW
C  N2	  - THE NUMBER OF COLUMN ELEMENTS TO BE SEARCHED.
C  XMIN   - THE VALUE OF THE MINIMUM ELEMENT.
C  XMAX   - THE VALUE OF THE MAXIMUM ELEMENT.
C
	DIMENSION ARRAY(IROW,N2)
C
C  I NOW INITIALIZE XMIN AND XMAX.
	XMIN=1.E30
	XMAX=-1.E30
C  I NOW LOOP OVER ALL ELEMENTS.
	DO 10 I=1,N1
C  I IS THE LOOP OVER THE ROWS.
	DO 10 J=1,N2
C  J IS THE LOOP OVER COLUMNS.
	TEST=ARRAY(I,J)
	IF(TEST.GT.XMAX) XMAX=TEST
	IF(TEST.LT.XMIN) XMIN=TEST
10	CONTINUE
	RETURN
	END
	SUBROUTINE GAUSSE(A,IROW,N,M,X,EPS,IW)
C	11/7/76
C	THIS PROGRAM WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C	PLASMA PHYSICS LAB.  THIS ROUTINE WILL SOLVE A SYSTEM OF LINEAR
C	EQUATIONS BY USING GAUSS ELIMATION AND BACKWORD SUBSITUTION.
C	PARAMETER LIST:
C	A - THE N BY M AUGMENTED MATRIX WITH THE RIGHT HAND VECTOR
C		IN COLUMN M.
C	IROW - THE ROW DIMENSION OF A AS DEFINED BY THE CALLING PROGRAM.
C	N - THE # EQUATIONS.
C	M - N+1.
C	X - SOLUTION VECTOR WITH N ELEMENTS.
C	EPS - EVERY PIVOT ELEMENT MUST BE GREATER THAN EPS.  IF
C		A PIVOT ELEMENT IS SMALLER THE EQUATIONS ARE
C		DECLARED SINGULAR.
C	IW - THE LOGICAL UNIT USED FOR PRINTING.
C	NOTE THAT THE MATRIX A IS DESTROYED.
	DIMENSION A(IROW,M),X(N)
	NN=N-1
C	I FIRST REDUCE MATRIX A.
	DO 320 J=1,NN
C	MAXROW WILL GIVE THE ROW IN WHICH THE MAXIMUM ELEMENT WAS FOUND.
	MAXROW=J
	AMAX=0.E00
C	I NOW SEARCH FOR THE MAXIMUM ELEMENT IN ROW J AND GREATER.
	DO 270 I=J,N
	ABSA=ABS(A(I,J))
	IF(ABSA-AMAX) 270,270,260
260	AMAX=ABSA
	MAXROW=I
270	CONTINUE
C	NOW THE MAXIMUM ELEMENT IN COL. J HAS THE VALUE OF AMAX AND
C	IS IN ROW=MAXROW.
C	I NOW CHECK TO SEE IF EQUATIONS ARE SINGULAR.
	IF(AMAX.GT.EPS) GO TO 275
	WRITE(IW,50) MAXROW,J
50	FORMAT(1X'A PIVOT ELEMENT WAS FOUND TO BE LESS THAN EPS',
     >' THE ELEMENT IS (',I3,','I3,')')
	STOP
C	IF MAXROW DOES NOT EQUAL J SWITCH ROWS.
275	IF(MAXROW.EQ.J) GO TO 290
	DO 280 I=J,M
	STORE=A(J,I)
	A(J,I)=A(MAXROW,I)
280	A(MAXROW,I)=STORE
C	ROWS ARE NOW SWITCHED.
C	I NOW NORMALIZE ROW J SO THAT A(J,J)=1.
290	ANORM=A(J,J)
	DO 295 I=J,M
295	A(J,I)=A(J,I)/ANORM
C	I NOW SUBTRACT ROW J FROM ROWS GREATER THEN J SUCH THAT
C	THERE ARE ZEROS IN COLUMN J.
	JJ=J
300	JJ=JJ+1
	ASUB=A(JJ,J)
	DO 310 I=J,M
310	A(JJ,I)=A(JJ,I)-A(J,I)*ASUB
	IF(JJ.LT.N) GO TO 300
320	CONTINUE
C	I NOW PREFORM BACKWARD SUBSITION.
	X(N)=A(N,M)/A(N,N)
	DO 340 K=1,NN
	SUM=0.E00
	L1=N-K
	DO 330 I=1,K
330	SUM=SUM+A(L1,L1+I)*X(L1+I)
340	X(L1)=A(L1,M)-SUM
	RETURN
	END
	SUBROUTINE GMAT
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE THE CHARGE
C  EXCHANGE RATE IN EACH ZONE DUE TO NEUTRALS BORN IN ALL ZONES
C  AND FROM THE WALL BY INTEGRATING ALONG EACH TRAJECTORY AND
C  SUMMING OVER ALL TRAJECTORIES.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
C
	DATA PID2/1.570796327E00/
C
C I NOW DEFINE THE H3 FUNCTION.
	H3(X)=H3X(X)
C
	IZMIN = NZCROS(MID)
	DELTR=RBOUND(IZMIN+1)-RBOUND(IZMIN)
C  I NOW TRACK FROM LEFT TO RIGHT AND CALCULATE THE WALL FLUX
C  WHICH CHARGE EXCHANGES IN ZONE I AND THE FRACTION WHICH GETS
C  TRANSMITTED TO THE RIGHT.
	DO 50 K=1,NNEUT
	  X1=0.E00
	  DO 40 I=1,NBCROS
	    IZA=NZCROS(I)
	    ZDISTI=DIST(I)
	    IF(.NOT.ESFLAG) GO TO 10
	    TAUB=BETAT(IZA,K)*ZDISTI
	    X2=X1+TAUB
C	    IF(X2.GT.87.5E00) GO TO 10  !CHECK
	    S0COLD(IZA,1,K)=S0COLD(IZA,1,K)+(H3(X1)-H3(X2))*DELTR
	    X1=X2
10	    CONTINUE
	IF(.NOT.MCFLAG) GO TO 40
C  THE ATTENUATION IN ZONE IZA DUE TO ATOMS BORN IN THE
C  SAME ZONE IS NOW FOUND.
	    TAUA=ZDISTI*ALPHAT(IZA,IZA,K)
	    TS1=PID2-H3(TAUA)
	    GFUN(IZA,IZA,1,K)=GFUN(IZA,IZA,1,K)+(2.E00*TAUA-TS1)*DELTR
	    IF(I.EQ.NBCROS) GO TO 30
C  THE ATTENUATION IN EACH ZONE IZB, TO THE RIGHT OF ZONE
C  IZA, DUE TO ATOMS BORN IN ZONE IZA AND THE FRACTION TRANSMITTED
C  TO THE NEXT ZONE IS NOW CALCULATED.
	    TAUC=0.E00
	    DO 20 J=I+1,NBCROS
	      IZB=NZCROS(J)
	      TAUC=TAUC+DIST(J)*ALPHAT(IZB,IZA,K)
	      TS2=H3(TAUC)-H3(TAUC+TAUA)
	      GFUN(IZB,IZA,1,K)=GFUN(IZB,IZA,1,K)+(TS1-TS2)*DELTR
	      TS1=TS2
20	    CONTINUE
C  ATOMS BORN IN IZA WHICH GET ALL THE WAY TO THE WALL.
30	    WALL(IZA,K)=WALL(IZA,K)+TS1*DELTR
40	  CONTINUE
	IF(ESFLAG) WWALL(K)=WWALL(K)+H3(X2)*DELTR
50	CONTINUE
	RETURN
	END
	FUNCTION H3X(XX)
	DATA PIO2/1.570796327E00/
	EEE1(Y)=PIO2-Y*(.4292037-Y*(.3561945-Y*(.462238-Y*(.404604
     >-Y*(.178267-Y*.030169)))))
	EEE2(Y)=SQRT(Y)*(2.50663-Y*(4.18947-Y*(13.9212-Y*(46.3264
     >-Y*(92.7633-Y*73.3359)))))
	IF(XX.LE.2.E00) THEN
	  H3X=EXP(-XX)*EEE1(XX)
	ELSE IF (XX .LT. 86.) THEN
	  H3X=EXP(-XX)*EEE2(1.E00/XX)
	ELSE
	  H3X = 0.
	END IF
	RETURN
	END
C	SUBROUTINE ICSEVU (X,Y,NX,C,IC,U,S,M,IER)
C
C-ICSEVU--------S-------LIBRARY 2---------------------------------------
C
C   FUNCTION		- EVALUATION OF A CUBIC SPLINE.
C   USAGE		- CALL ICSEVU(X,Y,NX,C,IC,U,S,M,IER)
C   PARAMETERS	 X	- VECTOR OF LENGTH NX CONTAINING THE ABSCISSAE
C				  OF THE NX DATA POINTS (X(I),Y(I)) I=1,...,
C				  NX (INPUT). X MUST BE ORDERED SO THAT
C				  X(I) .LT. X(I+1).
C		   Y	  - VECTOR OF LENGTH NX CONTAINING THE ORDINATES
C				  (OR FUNCTION VALUES) OF THE NX DATA POINTS
C				  (INPUT).
C		   NX	  - NUMBER OF ELEMENTS IN X AND Y (INPUT).
C				  NX MUST BE .GE. 2.
C		   C	 - SPLINE COEFFICIENTS (INPUT). C IS AN NX-1 BY
C				  3 MATRIX.
C		   IC	- ROW DIMENSION OF MATRIX C IN THE CALLING
C				  PROGRAM (INPUT). IC MUST BE .GE. NX-1.
C		   U	  - VECTOR OF LENGTH M CONTAINING THE ABCISSAE
C				  OF THE M POINTS AT WHICH THE CUBIC SPLINE
C				  IS TO BE EVALUATED (INPUT).
C		   S	  - VECTOR OF LENGTH M (OUTPUT).
C				  THE VALUE OF THE SPLINE APPROXIMATION AT
C				  U(I) IS
C				  S(I) = ((C(J,3)*D+C(J,2))*D+C(J,1))*D+Y(J)
C				  WHERE X(J) .LE. U(I) .LT. X(J+1) AND
C				  D = U(I)-X(J).
C		   M	  - NUMBER OF ELEMENTS IN U AND S (INPUT).
C		   IER	  - ERROR PARAMETER.
C				WARNING ERROR
C				  IER = 33, U(I) IS LESS THAN X(1).
C				  IER = 34, U(I) IS GREATER THAN X(NX).
C   PRECISION		- SINGLE
C   LANGUAGE		- FORTRAN
C-----------------------------------------------------------------------
C   LATEST REVISION	- AUGUST 9, 1974
C					  DEC		
	SUBROUTINE ICSEVU  (X,Y,NX,C,IC,U,S,M,IER)
C
	DIMENSION	   X(NX),Y(NX),C(IC,3),U(M),S(M)
	DATA			 I/1/,ZERO/0.0/
C					  INITIALIZE ERROR PARAMETERS
	JER = 0
	KER = 0
	IF (M .LE. 0) GO TO 100
	NXM1 = NX-1
	IF (I .GT. NXM1) I = 1
C					  EVALUATE SPLINE AT M POINTS
	DO 80 K=1,M
C					  FIND THE PROPER INTERVAL
	D = U(K)-X(I)
	IF (D) 10,50,30
10	IF (I .EQ. 1) GO TO 60
	I = I-1
	D = U(K)-X(I)
	IF (D) 10,50,40
20	I = I+1
	D = DD
30	IF (I .GE. NX) GO TO 70
	DD = U(K)-X(I+1)
	IF (DD .GE. ZERO) GO TO 20
	IF (D .EQ. ZERO) GO TO 50
C					  PERFORM EVALUATION
40	S(K) = ((C(I,3)*D+C(I,2))*D+C(I,1))*D+Y(I)
	GO TO 80
50	S(K) = Y(I)
	GO TO 80
C					  WARNING - U(I) .LT. X(1)
60	JER = 33
	GO TO 40
C					  IF U(I) .GT. X(NX) - WARNING
70	IF (DD .GT. ZERO) KER = 34
	D = U(K)-X(NXM1)
	I = NXM1
	GO TO 40
80	CONTINUE
	IER = MAX0(JER,KER)
90	CONTINUE
100	RETURN
	END
C	SUBROUTINE ICSICU (X,Y,NX,BPAR,C,IC,IER)
C
C-ICSICU--------S-------LIBRARY 2---------------------------------------
C
C   FUNCTION		- INTERPOLATORY APPROXIMATION BY CUBIC SPLINES
C				  WITH ARBITRARY SECOND DERIVATIVE END
C				  CONDITIONS.
C   USAGE		- CALL ICSICU(X,Y,NX,BPAR,C,IC,IER)
C   PARAMETERS	 X	- VECTOR OF LENGTH NX CONTAINING THE ABSCISSAE
C				  OF THE NX DATA POINTS (X(I),Y(I)) I=1,...,
C				  NX (INPUT). X MUST BE ORDERED SO THAT
C				  X(I) .LT. X(I+1).
C		   Y	  - VECTOR OF LENGTH NX CONTAINING THE ORDINATES
C				  (OR FUNCTION VALUES) OF THE NX DATA POINTS
C				  (INPUT).
C		   NX	  - NUMBER OF ELEMENTS IN X AND Y (INPUT). NX
C				  MUST BE .GE. 2.
C		   BPAR   - VECTOR OF LENGTH 4 CONTAINING THE END
C				  CONDITION PARAMETERS (INPUT).
C				  2.0*SPP(1)+BPAR(1)*SPP(2) = BPAR(2),
C				  BPAR(3)*SPP(NX-1)+2.0*SPP(NX) = BPAR(4),
C				  WHERE SPP(I) = SECOND DERIVATIVE OF THE
C				  CUBIC SPLINE FUNCTION S EVALUATED AT X(I).
C		   C	 - SPLINE COEFFICIENTS (OUTPUT). C IS AN NX-1 BY
C				  3 MATRIX. THE VALUE OF THE SPLINE
C				  APPROXIMATION AT T IS
C				  S(T) = ((C(I,3)*D+C(I,2))*D+C(I,1))*D+Y(I)
C				  WHERE X(I) .LE. T .LT. X(I+1) AND
C				  D = T-X(I).
C		   IC	- ROW DIMENSION OF MATRIX C IN THE CALLING
C				  PROGRAM (INPUT). IC MUST BE .GE. NX-1.
C		   IER	  - ERROR PARAMETER.
C				TERMINAL ERROR
C				  IER = 129, IC IS LESS THAN NX-1.
C				  IER = 130, NX IS LESS THAN 2.
C				  IER = 131, INPUT ABSCISSA ARE NOT ORDERED
C				SO THAT X(1) .LT. X(2) ... .LT. X(NX).
C   PRECISION		- SINGLE
C   LANGUAGE		- FORTRAN
C-----------------------------------------------------------------------
C   LATEST REVISION	- JULY 29, 1974
C					  DEC		
	SUBROUTINE ICSICU  (X,Y,NX,BPAR,C,IC,IER)
C
	DIMENSION	   X(NX),Y(NX),BPAR(4),C(IC,3)
	EQUIVALENCE	   (DXJ,YPPB),(PJ,SIXI),(DXJP1,YPPA)
	DATA			 ZERO/0.0/,HALF/0.5/,ONE/1.0/,
     >			 TWO/2.0/,SIX/6.0/
C					  CHECK ERROR CONDITIONS
	IER = 0
	NXM1 = NX-1
	IF (IC .LT. NXM1) GO TO 60
	IF (NX .LT. 2) GO TO 70
	IF (NX .EQ. 2) GO TO 20
C					  COMPUTE COEFFICIENTS AND RIGHT
C					  HAND SIDE OF THE TRIDIAGONAL
C					  SYSTEM DEFINING THE SECOND
C					  DERIVATIVES OF THE SPLINE
C					  INTERPOLANT FOR (X,Y)
C					  C(J,1) = LAMBDA(J)
C					  C(J,2) = MU(J)
C					  C(J,3) = D(J)
	DXJ = X(2)-X(1)
	IF (DXJ .LE. ZERO) GO TO 80
	DYJ = Y(2)-Y(1)
	DO 10 J=2,NXM1
	DXJP1 = X(J+1)-X(J)
	IF (DXJP1 .LE. ZERO) GO TO 80
	DYJP1 = Y(J+1)-Y(J)
	DXP = DXJ+DXJP1
	C(J,1) = DXJP1/DXP
	C(J,2) = ONE-C(J,1)
	C(J,3) = SIX*(DYJP1/DXJP1-DYJ/DXJ)/DXP
	DXJ = DXJP1
	DYJ = DYJP1
10	CONTINUE
C					  FACTOR THE TRIDIAGONAL MATRIX
C					  AND SOLVE FOR U
C					  C(J,2) = U(J)
C					  C(J,1) = Q(J)
C					  BPAR(1) = LAMBDA(1)
C					  BPAR(2) = D(1)
C					  BPAR(3) = MU(NX)
C					  BPAR(4) = D(NX)
20	C(1,1) = -BPAR(1)*HALF
	C(1,2) = BPAR(2)*HALF
	IF (NX .EQ. 2) GO TO 40
	DO 30 J=2,NXM1
	PJ = C(J,2)*C(J-1,1)+TWO
	C(J,1) = -C(J,1)/PJ
	C(J,2) = (C(J,3)-C(J,2)*C(J-1,2))/PJ
30	CONTINUE
C					  SOLVE FOR CUBIC COEFFICIENTS
C					  OF SPLINE INTERPOLANT
C					  C(J,1), C(J,2), AND C(J,3)
40	YPPB = (BPAR(4)-BPAR(3)*C(NXM1,2))/(BPAR(3)*C(NXM1,1)+TWO)
	SIXI = ONE/SIX
	DO 50 I=1,NXM1
	J = NX-I
	YPPA = C(J,1)*YPPB+C(J,2)
	DX = X(J+1)-X(J)
	C(J,3) = SIXI*(YPPB-YPPA)/DX
	C(J,2) = HALF*YPPA
	C(J,1) = (Y(J+1)-Y(J))/DX-(C(J,2)+C(J,3)*DX)*DX
	YPPB = YPPA
50	CONTINUE
	GO TO 100
60	IER = 129
	GO TO 90
70	IER = 130
	GO TO 90
80	IER = 131
90	CONTINUE
100	RETURN
	END
	SUBROUTINE KERNEL
C  THIS SUBROUTINE WAS WRITTEN BY HARRY. H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL FINISH CONSTRUCTING
C  THE S0COLD AND GFUN ARRAYS.  IT WILL ALSO CONSTRUCT THE KERNEL
C  FROM GFUN.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
C  I NOW LOOP OVER ALL NEUTRAL SPECIES.
	DO 40 K=1,NNEUT
	  TS1=4.E00*FLUXIN(K)
C  I NOW LOOP OVER ALL BIRTH ZONES.
	  DO 30 I=1,NZONES
	    IF(.NOT.MCFLAG) GO TO 50
	    TS3=1.E00/ALPHAT(I,I,K)
	    WALL(I,K)=WALL(I,K)*TS3
50	    IF(.NOT.ESFLAG) GO TO 100
	    TS2=BETAT(I,K)*VOL(I)
	    TS4=S0COLD(I,1,K)
C  I NOW LOOP OVER ALL ALLOWED CHARGED PARTICLES.
100	    DO 20 NL=NNEUT,1,-1
	      IF(ESFLAG) S0COLD(I,NL,K)=TS4*TS1*BETAX(I,NL,K)/TS2
	      IF(.NOT.MCFLAG) GO TO 20
C  I NOW LOOP OVER ALL ZONES.
	      DO 10 J=1,NZONES
	        GA=GFUN(J,I,1,K)*TS3
	        TS0=-ALPHAX(J,I,NL,K)/(ALPHAT(J,I,K)*VOL(J))
	        GFUN(J,I,NL,K)=GA*TS0
	        IF(J.EQ.I.AND.NL.EQ.K) GFUN(J,I,NL,K)=GFUN(J,I,NL,K)+
     >          1.E00
10	      CONTINUE
20	    CONTINUE
30	  CONTINUE
40	CONTINUE
	RETURN
	END
	PROGRAM MNEUT
C  THIS PROGRAM WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL DRIVE SUBROUTINE NEUTS.
C
C  LUN6  - WHERE ERROR MESSAGES ARE PRINTED.
C  LUN21 - WHERE NAMELIST PARAMETERS ARE READ.
C  LUN22 - WHERE NAMELIST PARAMETERS AND RESULTS ARE WRITTEN.
C
	INCLUDE 'CNEUT.CMN/LIST'
	INCLUDE 'CNEUT1.CMN/LIST'
C
	DIMENSION DFAC(4),FRAC(2)
C
	NAMELIST/NDATA/APLAS,ZPLAS,NSPEC,DENEA,RA,TEA,TIA,
     >IDEFIT,ITEFIT,ITIFIT,NDENE,NTEMPE,NTEMPI,AMINOR,NNEUT,NZONES,
     >NPLAS,TWALL,LUN6,ZEFF,FLUXIN,DEBUG,ESFLAG,VSFLAG,MCFLAG,
     >INFLAG,RCFLAG,IBFLAG,FRAC
C
	DATA APLAS/5.4858026E-4,2.013553229E00,16.E00,0.E00,0.E00/
	DATA DENEA/3.E13,2.E00,1.E00,1.E12,40.E00,16*0.E00/
	DATA RA/40.E00,20*0.E00/
	DATA TEA/1.5E00,2.E00,2.E00,.03E00,40.E00,16*0.E00/
	DATA TIA/2.0E00,2.E00,2.E00,.03E00,40.E00,16*0.E00/
	DATA ZPLAS/-1.E00,1.E00,8.E00,1.E00,1.E00/
	DATA NSPEC/2,3/
	DATA DEBUG,VSFLAG,IBFLAG/3*.FALSE./
	DATA ESFLAG,MCFLAG,INFLAG,RCFLAG/4*.TRUE./
	DATA ZEFF,AMINOR,NZONES,NNEUT/2.E00,40.E00,20,1/
	DATA IDEFIT,ITEFIT,ITIFIT/3*0/
	DATA NDENE,NTEMPE,NTEMPI/3*5/
	DATA LUN6,LUN21,LUN22/6,21,22/
	DATA TWALL,NPLAS/.030,.030,3/
	DATA FLUXIN/CNT$1*5.E13/
	DATA FRAC/1.,0./
C
C  READ NAMELIST PARAMETERS.
10	READ(LUN21,NDATA,END=600)
	Z2=ZPLAS(2)
	IF(IBFLAG) READ(73) (BII(I,1),I=1,NZONES)
	IF(.NOT.VSFLAG) GO TO 40
	DO 30 NL=1,NNEUT
	  READ(77) (VOLSOR(I,NL),I=1,NZONES)
30	CONTINUE
40	Z3=ZPLAS(NPLAS)
	IF (NPLAS .EQ. 2) Z3 = ZPLAS(3)
C  WRITE NAMELIST PARAMETERS.
	WRITE(LUN22,50)
50	FORMAT('1')
	WRITE(LUN22,NDATA)
C
C  I NOW OBTAIN PROFILE INFORMATION.
C  SET NPLAS = 2 IF ZEFF = 1.
	IF(ZEFF.EQ.1.E00) NPLAS=2
C  FIND THE SPACING BETWEEN ZONE CENTERS.
	DPROF=AMINOR/NZONES
C  FILL UP THE ZONE CENTER ARRAY.
	DO 430 I=1,NZONES
	  ZONEC(I)=(I-.5E00)*DPROF
430	CONTINUE
C   ESTABLISH THE DENSITY FACTORS.  (DEN(N)=DFAC(N)*DENE)
	DFAC(1)=1.E00
	TFAC=(ZEFF-Z3)/(Z2*(Z2-Z3))
	DO 440 I=2,NPLAS-1
	  DFAC(I)=TFAC*FRAC(I-1)
440	CONTINUE
	DFAC(NPLAS)=(1.E00-(ZEFF-Z3)/(Z2-Z3))/Z3
	IF(NPLAS.EQ.2) DFAC(2)=1.E00/Z2
C  FIND THE DENSITY PROFILES.
	CALL DEINIT(RA,DENEA,NDENE,ZONEC,DEN(1,1),NZONES,IDEFIT,IERR)
	DO 460 I=2,NPLAS
	  FAC=DFAC(I)
	  DO 450 J=1,NZONES
	    DEN(J,I)=FAC*DEN(J,1)
450	  CONTINUE
460	CONTINUE
C  GET THE ELECTRON TEMPERATURE PROFILE.
	CALL TEINIT(RA,TEA,NTEMPE,ZONEC,TEMP(1,1),NZONES,ITEFIT,IERR)
C  GET THE ION TEMPERATURE PROFILE.
	CALL TIINIT(RA,TIA,NTEMPI,ZONEC,TEMP(1,2),NZONES,ITIFIT,IERR)
C  FILL THE IMPURITY TEMPERATURE ARRAY.
	DO 480 I=3,NPLAS
	  DO 470 J=1,NZONES
	    TEMP(J,I)=TEMP(J,2)
470	  CONTINUE
480	CONTINUE
C  I NOW CALL NEUTS TO CALCULATE THE NEUTRAL DENSITIES.
	CALL CPTIME(START)
	CALL NEUTS
C		DO 6000 K=1,NNEUT
C		DO 6000 NL=1,NNEUT
C		DO 6000 I=1,NZONES
C6000		VOLSOR(I,NL)=VOLSOR(I,NL)+S0COLD(I,NL,K)
C		DO 6010 NL=1,NNEUT
C		WRITE(77) (VOLSOR(I,NL),I=1,NZONES)
C6010		CONTINUE
	CALL CPTIME(STOP)
	TIME=(STOP-START)*1.E-2
	WRITE(6,490) TIME
490	FORMAT(T2,'EXECUTION TIME=',1PE9.2,' SEC')
	IF(.NOT.DEBUG) GO TO 580
	DO 530 K=1,NNEUT
	  DO 530 NL=1,NNEUT
	    WRITE(LUN22,500) NL,K
500	    FORMAT(/,T2,'S0COLD(I,'I1,',',I1,')')
	    WRITE(LUN22,510) (S0COLD(I,NL,K),I=1,NZONES)
510	    FORMAT(T2,1P5E14.5)
	    DO 530 J=1,NZONES
	      WRITE(LUN22,520) J,NL,K
520	      FORMAT(/,T2'GFUN(I,'I2,',',I1,',',I1,')')
	      WRITE(LUN22,510) (GFUN(I,J,NL,K),I=1,NZONES)
530	CONTINUE
	DO 550 K=1,NNEUT
	  DO 550 NL=1,NNEUT
	    WRITE(LUN22,540) NL,K
540	    FORMAT(T2,'BETAX(I,'I1,',',I1,')')
	    WRITE(LUN22,510) (BETAX(I,NL,K),I=1,NZONES)
550	CONTINUE
	DO 570 K=1,NNEUT
	  WRITE(LUN22,560) K
560	  FORMAT(/,T2'BETAT(I,'I1,')')
	  WRITE(LUN22,510) (BETAT(I,K),I=1,NZONES)
570	CONTINUE
580	CONTINUE
	CALL PRES(LUN22)
	CALL VRES
	GO TO 10
590	CONTINUE
600	CALL FINITT(0,780)
	STOP
	END
	SUBROUTINE NEUTS
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL DETERMINE THE NEUTRAL
C  DENSITY FOR MULTIPLE SPECIES. THE BASIC ALGORITHEM USED
C  IS THAT DEVELOPED BY STEVE TAMOR IN HIS ANTIC CODE.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	PARAMETER NN1=CNT$3*CNT$1,NN2=NN1+1
C
	DIMENSION DD(NN1,NN2),TSTOR2(NN1)
C
	DATA PI,MSFLAG/3.141592654E00,.FALSE./
C
	IF(.NOT.ESFLAG.AND..NOT.VSFLAG.AND..NOT.RCFLAG) GO TO 280
	IF(IBFLAG) GO TO 30
	DO 20 K=1,NNEUT
	  DO 10 I=1,NZONES
	    BII(I,K)=0.E00
	    BCX(I,K)=0.E00
10	  CONTINUE
20	CONTINUE
30	DO 50 I=1,NZONES
	  PCX(I)=0.E00
	  PII(I)=0.E00
	  PRC(I)=0.E00
	  SORE(I)=0.E00
50	CONTINUE
	DO 40 J=1,NNEUT
	  DO 35 I=1,NZONES
	    SORI(I,J)=0.E00
	    SORN(I,J)=0.E00
	    DCOLD(I,J)=0.E00
35	  CONTINUE
40	  CONTINUE
	IF(MSFLAG) GO TO 70
	IF(INFLAG.AND.MCFLAG) GO TO 70
	WRITE(LUN6,60)
60	FORMAT(T2,'NO MATRIX CALCULATED.  DOING INITIALIZATION',
     >' WITH MATRIX CALCULATION')
	INFLAG=.TRUE.
	MCFLAG=.TRUE.
70	IF(.NOT.INFLAG) GO TO 120
C  I NOW FILL ISPEC ARRAY
	DO 100 I=1,NPLAS-1
	  IP1=I+1
	  DO 80 J=1,NNEUT
	    INEUT=J
	    IF(NSPEC(J).EQ.IP1) GO TO 90
80	  CONTINUE
	  ISPEC(J)=0
	  GO TO 100
90	  ISPEC(I)=INEUT
100	CONTINUE
C  I NOW FIND THE BOUNDARY VALUES FOR EACH ZONE. NOTE THAT ZONE
C  CENTER I LIES BETWEEN ZONE BOUNDARY I AND I+1.
	NBOUND=NZONES+1
	RBOUND(1)=0.E00
	TS1=0.E00
	DO 110 I=1,NZONES
	  TS2=2.E00*ZONEC(I)-RBOUND(I)
	  RBOUND(I+1)=TS2
	  VOL(I)=PI*(TS2*TS2-TS1*TS1)
	  TS1=TS2
110	CONTINUE
C  I NOW INITALIZE ALPHA AND BETA ARRAYS
120	IF(MCFLAG.OR.ESFLAG) CALL ALPHA
	IF(.NOT.INFLAG) GO TO 150
C  I NOW FIND ARRAY Y WHICH CONTAINS THE DISTANCE OF CLOSEST
C  APPOACH. SEE PROGRAM WRITE UP FOR DETAILS.
	Y(1)=RBOUND(2)*.6189908923E00
	DO 140 I=2,NZONES
	  RPI=RBOUND(I+1)
	  RPI2=RPI*RPI
	  R=RBOUND(I)
	  VLT=.25E00*PI*RPI2
	  VLM=0.E00
	  DO 130 J=1,I-1
	    VLM=VLM+(RBOUND(J+1)-RBOUND(J))*SQRT(RPI2-Y(J)**2)
130	  CONTINUE
	  TS=(VLT-VLM)/(RPI-R)
	  Y(I)=SQRT(RPI2-TS*TS)
140	CONTINUE
C  I NOW INITIALIZE SOME ARRAYS.
150	IF(.NOT.ESFLAG.AND..NOT.MCFLAG) GO TO 180
	DO 160 K=1,NNEUT
	  WWALL(K)=0.E00
	  DO 160 I=1,NZONES
	    IF(MCFLAG) WALL(I,K)=0.E00
	    IF(ESFLAG) S0COLD(I,1,K)=0.E00
	    DO 160 J=1,NZONES
	      IF(MCFLAG) GFUN(J,I,1,K)=0.E00
160	CONTINUE
C  I NOW LOOP OVER ALL TRAJECTORIES
	DO 170 I=1,NZONES
	  CALL TRACKR(Y(I))
	  CALL GMAT
170	CONTINUE
	CALL KERNEL
	MSFLAG=.TRUE.
C  I NOW MAKE A COPY OF 4-D ARRAY GFUN + S0COLD SO THAT
C  THE SOLUTION VECTOR CAN BE FOUND.
C  I NOW LOOP OVER THE NEUTRAL WHICH WILL DIE IN ZONE I.
180	DO 190 K=1,NNEUT
	  JSTART=NZONES*(K-1)
C  I NOW LOOP OVER THE NEUTRAL CREATED IN ZONE I.
	  DO 190 NL=1,NNEUT
	    ISTART=NZONES*(NL-1)
C  I NOW LOOP OVER ALL BIRTH ZONES OF THE K NEUTRAL.
	    DO 190 J=1,NZONES
	      J1=JSTART+J
C  I NOW LOOP OVER ALL DEATH ZONES OF THE K NEUTRAL.
	      DO 190 I=1,NZONES
	        I1=ISTART+I
	        DD(I1,J1)=GFUN(I,J,NL,K)
190	CONTINUE
C  I NOW ATTACH THE SOLUTION VECTOR.
	IROW=NZONES*NNEUT
	JCOL=IROW+1
	DO 200 I=1,IROW
200	DD(I,JCOL)=0.E00
	IF(.NOT.ESFLAG) GO TO 220
	DO 210 K=1,NNEUT
	  DO 210 NL=1,NNEUT
	    ISTART=NZONES*(NL-1)
	    DO 210 I=1,NZONES
	      I1=ISTART+I
	      DD(I1,JCOL)=S0COLD(I,NL,K)+DD(I1,JCOL)
210	CONTINUE
220	IF(.NOT.VSFLAG) GO TO 240
	DO 230 NL=1,NNEUT
	  ISTART=NZONES*(NL-1)
	  DO 230 I=1,NZONES
	    I1=ISTART+I
	    DD(I1,JCOL)=VOLSOR(I,NL)+DD(I1,JCOL)
230	CONTINUE
240	IF(.NOT.RCFLAG) GO TO 270
C  I NOW FILL UP ARRAY SVRC
	DO 250 I=1,NZONES
	  SVRC(I)=DEN(I,1)*SIGVRC(TEMP(I,1))
250	CONTINUE
	DO 260 NL=1,NNEUT
	  L=NSPEC(NL)
	  ISTART=NZONES*(NL-1)
	  DO 260 I=1,NZONES
	    I1=ISTART+I
	    DD(I1,JCOL)=SVRC(I)*DEN(I,L)+DD(I1,JCOL)
260	CONTINUE
C  I NOW CALL GAUSSE TO SOLVE THE EQUATIONS
270	CALL GAUSSE(DD,NN1,IROW,JCOL,TSTOR2,1.E-4,6)
C
C  FILL TSTOR1 ARRAY.
C
	DO 272 K = 1,NNEUT
	  IOFF = (K - 1)*NZONES
	  DO 271 I = 1,NZONES
	    TSTOR1(I,K)=TSTOR2(IOFF+I)
271	CONTINUE
272	CONTINUE
	CALL EDITR
	CALL EDITS
	RETURN
280	WRITE(LUN6,290)
290	FORMAT(T2,'NO SOURCES MEANS NO NEUTRALS MEANS NO WORK.  NEUTS')
	STOP
	END
	SUBROUTINE PLOTI(N)
C  12/14/78
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS ROUTINE IS A COLLECTION OF
C  PLOT-10 CALLS WHICH SET VARIOUS PLOT PARAMETERS.
C
C  PARAMETERS:
C  N	  - THE NUMBER OF POINTS TO BE PLOTED.
	CALL BINITT
	CALL NPTS(N)
	CALL XFRM(3)
	CALL YFRM(3)
	CALL XMFRM(3)
	CALL YMFRM(3)
	CALL SLIMX(222,802)
	CALL SLIMY(100,680)
	RETURN
	END
	SUBROUTINE PRES(LUN)
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB. THIS ROUTINE WILL PRINT THE RESULTS FROM
C  SUBROUTINE NEUTS.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
C  PARAMETER:
C  LUN    - THE LOGICAL UNIT USED TO PRINT THE RESULTS.
C
	WRITE(LUN,10)
10	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T21'TEMPE',T32'TEMPI',
     >T44'DENE')
	DO 30 I=1,NZONES
	  WRITE(LUN,20) I,ZONEC(I),TEMP(I,1),TEMP(I,2),DEN(I,1)
20	  FORMAT(2X,I2,1X,4(2X,1PE9.2))
30	CONTINUE
	IC1=1
40	IC2=IC1+1
	IF(IC2.GT.NNEUT) GO TO 80
	WRITE(LUN,50) IC1,IC1,IC1,IC2,IC2,IC2
50	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T20'DCOLD(',I1,')',
     >T31'DNEUT(',I1,')',T42'TNEUT(',I1,')',:,T53'DCOLD(',I1,')',
     >T64'DNEUT(',I1,')',T75'TNEUT(',I1,')')
	DO 70 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),((DCOLD(I,K),DNEUT(I,K),TNEUT(I,K)),
     >K=1,2)
60	  FORMAT(2X,I2,1X,7(2X,1PE9.2))
70	CONTINUE
	IC1=IC2+1
	IF(IC1.GT.NNEUT) GO TO 100
	GO TO 40
80	WRITE(LUN,50) IC1,IC1,IC1
	DO 90 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),DCOLD(I,IC1),DNEUT(I,IC1),TNEUT(I,IC1)
90	CONTINUE
100	K=1
	KC=NSPEC(K)
	K2=K+1
	IF(K2.GT.NNEUT) GO TO 120
	KC2=NSPEC(K2)
	WRITE(LUN,110) KC,K,KC2,K2
110	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T22'SORE',
     >T32'SORI(',I1,')',T43'SORN(',I1,')',:,T53'SORI(',I1,')',
     >T65'SORN(',I1,')')
	GO TO 130
120	WRITE(LUN,110) KC,K
	K2=K
130	DO 140 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),SORE(I),((SORI(I,J),SORN(I,J)),J=K,K2)
140	CONTINUE
150	K=K2+1
	IF(K.GT.NNEUT) GO TO 200
	KC=NSPEC(K)
	K2=K+1
	IF(K2.GT.NNEUT) GO TO 170
	KC2=NSPEC(K2)
	WRITE(LUN,160) KC,K,KC2,K2
160	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T21'SORI(',I1,')',
     >T31'SORN(',I1,')',:,T43'SORI(',I1,')',T53'SORN(',I1,')')
	GO TO 180
170	WRITE(LUN,160) KC,K
	K2=K
180	DO 190 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),((SORI(I,J),SORN(I,J)),J=K,K2)
190	CONTINUE
	GO TO 150
200	WRITE(LUN,210)
210	FORMAT('1',5(/),T2'ZONE',T10'ZONEC',T23'PCX',T34'PII')
	DO 220 I=1,NZONES
	  WRITE(LUN,60) I,ZONEC(I),PCX(I),PII(I)
220	CONTINUE
	WRITE(LUN,230)
230	FORMAT(3(/),T2'CONSERVATION CHECKS')
	WRITE(LUN,240)
240	FORMAT(2(/),T2'NEUTRALS ABSORBED IN PLASMA (#/SEC)')
	WRITE(LUN,250) -PLOSS
250	FORMAT(T5'EDGE+VOLUME SOURCE BALANCE: ',1PE11.4)
	WRITE(LUN,260) -PLOSS1
260	FORMAT(T13'VOLUME INTEGRATION: ',1PE11.4)
	WRITE(LUN,270)
270	FORMAT(2(/),T2'POWER OUT FLOW FROM PLASMA (WATTS)')
	WRITE(LUN,250) ELOSS
	WRITE(LUN,260) ELOSS1
	RETURN
	END
	FUNCTION SIGCX(EREL,ANEUT)
C
C  3/28/79
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS FUNCTION WILL CALCULATE THE CHARGE
C  EXCHANGE CROSS SECTION AS GIVEN BY EQ. 4 OF RIVIERE'S PAPER.
C
C  PARAMETER LIST:
C  EREL	  - THE RELATIVE ENERGY BETWEEN THE NEUTRAL AND THE CHARGED
C           PARTICLE.  (KEV)
C	    EG. WNEUT+ANEUT*1.5*TI/APLASMA.
C  ANEUT  - THE ATOMIC WEIGHT OF THE NEUTRAL PARTICLE.
C
C  EE IS THE RELATIVE ENERGY (IN EV) PER AMU.
	EE=EREL*1.E3/ANEUT
C  THE CROSS-SECTION IS NOW EVALUATED.
	AL10=ALOG10(EE)
	SIGCX=(1.6666E-16*AL10-2.1505E-15)*AL10+.6937E-14
	SIGCX=SIGCX/(1.E00+.1112E-14*EE**3.3)
	RETURN
	END
	FUNCTION SIGION(EREL,ANEUT)
C
C  1/28/80
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS FUNCTION WILL CALCULATE THE PROTON
C  IONIZATION CROSS SECTION AS GIVEN BY FREEMAN AND JONES IN
C  TABLE 2.
C
C  PARAMETER LIST:
C  EREL   - THE RELATIVE ENERGY BETWEEN THE NEUTRAL PARTICLE
C           AND THE CHARGED PARTICLE.  (KEV)
C	    EG. WNEUT+ANEUT*1.5*TI/APLASMA.
C  ANEUT  - THE ATOMIC WEIGHT OF THE NEUTRAL PARTICLE.
C
C  SIGION HAS UNITS OF CM**2.
C
	DATA A0,A1,A2,A3,A4,A5,A6/-.4203309E2,.3557321E1,-.1045134E1,
     >.3139238E00,-.7454475E-1,.8459113E-2,-.3495444E-3/
	DATA EXP1/2.718281828E00/
C
C  EE IS THE RELATIVE ENERGY (IN KEV) PER AMU.
	EE=EREL/ANEUT
	ALE=ALOG(EE)
C  THE CROSS-SECTION IS NOW EVALUATED.
	SIGION=EXP1**((((((A6*ALE+A5)*ALE+A4)*ALE+A3)*ALE+A2)*ALE+A1)
     >*ALE+A0)
	RETURN
	END
	FUNCTION SIGLOS(EREL,ANEUT,ZIMP)
C  3/29/79
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS FUNCTION WILL CALCULATE THE CROSS
C  SECTION FOR ELECTRON LOSS (SUM OF CHARGE EXCHANGE AND
C  ION IMPACT IONIZATION) FOR CHARGED PARTICLES COLLIDING
C  WITH ATOMIC HYDROGEN.  FOR ZIMP .GT. 1 THE FORMULA
C  IS BASED ON OLSON ET. AL. PHYSICAL REVIEW LETTERS
C  VOL 41 NUM 3 P. 163.  FOR ZIMP .EQ. 1 THE CHARGE EXCHANGE
C  AND ION IMPACT IONIZATION CROSS-SECTIONS FROM RIVIERE ARE
C  USED.  THE VALUE OF THE CROSS-SECTION IS RETURNED
C  BY SIGLOS AND HAS UNITS OF CM**2.
C
C  PARAMETERS:
C  EREL   - THE RELATIVE ENERGY BETWEEN THE NEUTRAL PARTICLE AND
C           THE IMPURITY.  (KEV)
C	    EG. WNEUT+ANEUT*1.5*TI/APLASMA.
C  ANEUT  - THE ATOMIC MASS OF THE NEUTRAL PARTICLE.  (AMU)
C  ZIMP   - THE CHARGE STATE OF THE IMPURITY.
C
	IF(ZIMP.LT.1.01E00) GO TO 30
C  I NOW USE THE FORMULA FROM OLSON.
	CON1=EREL/(32.E00*ZIMP*ANEUT)
	IF(CON1.GT.87.E00) GO TO 10
	CON2=1.E00-EXP(-CON1)
	GO TO 20
10	CON2=1.E00
20	SIGLOS=4.6E-16*ZIMP*CON2/CON1
	RETURN
30	SIGLOS=SIGCX(EREL,ANEUT)+SIGION(EREL,ANEUT)
	RETURN
	END
	FUNCTION SIGVRC(TE)
C  5/15/80
C  THIS FUNCTION WAS WRITTEN BY HARRY H. TOWNER AND WILL CALCULATE
C  <SIGMA*V> FOR RECOMBINATION OF HYDROGEN BY ELECTRONS.  SEE
C  PAPER BY Yu. S. Gordeev, et al. JETP Lett., Vol. 25 No. 3
C  Page 204.
C  PARAMETERS:
C  TE     - THE ELECTRON TEMPERATURE.  (KEV)
C
	IF(TE.GT..4E00) GO TO 100
	SIGVRC=1.48E-14/(SQRT(TE)*(1.E00+4.34E1*TE))
	RETURN
100	SIGVRC=3.58E-16/TE**1.388
	RETURN
	END
	FUNCTION SVEL(TEMPE)
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL CALCULATE <SIGMA*V>
C  FOR ELECTRON IMPACT IONIZATION BY USING THE FIT GIVEN BY
C  FREEMAN (CLM-R137 TABLE 3).
C
	DIMENSION COEF(7)
C
	DATA COEF/-3.173850E1,1.143818E1,-3.833998E00,7.046692E-1,
     >-7.431486E-2,4.153749E-3,-9.486967E-5/
	DATA EXP1/2.718281828E00/
C  COEF ARE THE COEFFICIENTS USED TO MAKE THE FIT TO <SIGMA*V>
	ALN=ALOG(TEMPE*1.E3)
	SVEL=COEF(1)+ALN*(COEF(2)+ALN*(COEF(3)+ALN*(COEF(4)+
     >ALN*(COEF(5)+ALN*(COEF(6)+ALN*COEF(7))))))
	SVEL=EXP1**SVEL
	RETURN
	END
	SUBROUTINE TEINIT(RA,TEA,NTEMPE,XA,TEMPE,NSHELL,IFIT,IERR)
C  5/29/78
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS ROUTINE WILL COMPUTE THE VALUES OF
C  THE ELECTRON TEMPERATURE AT EACH POINT IN ARRAY XA AND STORES
C  THE RESULTS IN TEMPE.
C  PARAMETERS:
C  RA	  - AN ARRAY CONTAINING THE PLASMA HALF WIDTHS AT WHICH THE
C	    ELECTRON TEMPERATURE IS KNOWN.  (CM)
C  TEA	  - THE VALUE OF THE ELECTRON TEMPERATURE AT THE POINTS ABOVE.
C	    (KEV)
C  NTEMPE - THE # OF POINTS IN THE ABOVE ARRAYS.
C  XA	  - THE ARRAY CONTAINING THE PLASMA HALF-WIDTH.  (CM)
C  TEMPE  - THE VALUE OF THE ELECTRON TEMPERATURE AT THE ABOVE POINTS.
C	    (KEV)
C  NSHELL - # OF POINTS IN THE ABOVE ARRAY.
C  IFIT   - IF IFIT .EQ. 0 AN ANALYTIC SOLUTION WILL BE CREATED USING
C	    THE INFORMATION IN ARRAY DENEA.  OTHERWISE THE ORGINAL DATA
C	    WILL BE FIT WITH CUBIC SPLINES AND INTERPOLATED.
C  IERR   - THE LOGICAL UNIT USED FOR PRINTING ERROR MESSAGES.
	DIMENSION RA(NTEMPE),TEA(NTEMPE),XA(NSHELL),TEMPE(NSHELL)
	IF(IFIT.EQ.0) GO TO 10
	CALL DFITS(RA,TEA,NTEMPE,XA,TEMPE,NSHELL,IERR)
	RETURN
10	DO 20 I=1,NSHELL
	  TEMPE(I)=(TEA(1)-TEA(4))*(1.E00-(XA(I)/TEA(5))**TEA(2))**
     >TEA(3)+TEA(4)
20	CONTINUE
	RETURN
	END
	SUBROUTINE TIINIT(RA,TIA,NTEMPI,XA,TEMPI,NSHELL,IFIT,IERR)
C  5/29/78
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.	THIS ROUTINE WILL COMPUTE THE VALUES OF
C  THE ION TEMPERATURE AT EACH POINT IN ARRAY XA AND STORES
C  THE RESULTS IN TEMPI.
C  PARAMETERS:
C  RA	  - AN ARRAY CONTAINING THE PLASMA HALF WIDTHS AT WHICH THE
C	    ION TEMPERATURE IS KNOWN.  (CM)
C  TIA	  - THE VALUE OF THE ION TEMPERATURE AT THE POINTS ABOVE.
C	    (KEV)
C  NTEMPI - THE # OF POINTS IN THE ABOVE ARRAYS.
C  XA	  - THE ARRAY CONTAINING THE PLASMA HALF-WIDTH.  (CM)
C  TEMPI  - THE VALUE OF THE ION TEMPERATURE AT THE ABOVE POINTS.
C	    (KEV)
C  NSHELL - # OF POINTS IN THE ABOVE ARRAY.
C  IFIT   - IF IFIT .EQ. 0 AN ANALYTIC SOLUTION WILL BE CREATED USING
C	    THE INFORMATION IN ARRAY DENEA.  OTHERWISE THE ORGINAL DATA
C	    WILL BE FIT WITH CUBIC SPLINES AND INTERPOLATED.
C  IERR   - THE LOGICAL UNIT USED FOR PRINTING ERROR MESSAGES.
	DIMENSION RA(NTEMPI),TIA(NTEMPI),XA(NSHELL),TEMPI(NSHELL)
	IF(IFIT.EQ.0) GO TO 10
	CALL DFITS(RA,TIA,NTEMPI,XA,TEMPI,NSHELL,IERR)
	RETURN
10	DO 20 I=1,NSHELL
	  TEMPI(I)=(TIA(1)-TIA(4))*(1.E00-(XA(I)/TIA(5))**TIA(2))**
     >TIA(3)+TIA(4)
20	CONTINUE
	RETURN
	END
	SUBROUTINE TRACKR(ELEV)
C  THIS ROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL TRACK A GIVEN
C  TRAJECTORY THROUGH THE MESH FOR A GIVEN Y.  IT WILL CALCULATE
C  THE NUMBER OF BOUNDARIES CROSSED (NBCROS), THE NUMBER OF
C  THE ZONE CROSSED FROM LEFT TO RIGHT (NZCROS(I)) AND THE
C  PATH LENGTH IN EACH ZONE (DIST(I)).
C
C  PARAMETER:
C  ELEV  - THE DISTANCE OF CLOSEST APPROACHED.  (CM)
C
	INCLUDE 'CNEUT.CMN/NOLIST'
C
	ELEV2=ELEV*ELEV
C  I NOW LOOP OVER ALL BOUNDARIES TO FIND THE BOUNDARY WHICH
C  IS GREATER THAN Y.
	DO 10 I=2,NBOUND
	  IC=I
	  RB=RBOUND(I)
	  IF(RB.GE.ELEV) GO TO 20
10	CONTINUE
C  MZONE CONTAINS THE ZONE NUMBER FOR THE CENTER REGION.
20	MZONE=IC-1
C  NBCROS CONTAINS THE NUMBER OF BOUNDARIES CROSSED = NUMBER OF ZONES
C  TRAJECTORY PASSED THROUGH.
	NBCROS=2*(NZONES-MZONE)+1
C  I NOW FIND THE ARRAY ELEMENT FOR THE MID ZONE.
	MID=NBCROS/2+1
C  I NOW FILL NZCROS AND DIST FOR THE CENTER REGION.
	NZCROS(MID)=MZONE
	DISTM=SQRT(RB*RB-ELEV2)
	DIST(MID)=2.E00*DISTM
	IF(MID.EQ.1) GO TO 40
C  DUE TO SYMMETRY I NOW ONLY HAVE TO LOOP OVER MID-1 POINTS
C  TO FILL UP DIST AND NZCROS ARRAYS
	DO 30 I=1,MID-1
	  IP=MID+I
	  IM=MID-I
	  MZP1=MZONE+I
	  DISTP=SQRT(RBOUND(IC+I)**2-ELEV2)
	  XDIST=DISTP-DISTM
	  DISTM=DISTP
	  NZCROS(IP)=MZP1
	  NZCROS(IM)=MZP1
	  DIST(IP)=XDIST
	  DIST(IM)=XDIST
30	CONTINUE
40	RETURN
	END
	SUBROUTINE VRES
C  THIS SUBROUTINE WAS WRITTEN BY HARRY H. TOWNER OF THE PRINCETON
C  PLASMA PHYSICS LAB.  THIS ROUTINE WILL DISPLAY THE RESULTS FROM
C  SUBROUTINE NEUTS.
C
	INCLUDE 'CNEUT.CMN/NOLIST'
	INCLUDE 'CNEUT1.CMN/NOLIST'
C
	DIMENSION XTITLE(20),YTITLE(20),TITLE(20),TWORK(CNT$3)
C
	EQUIVALENCE (TWORK(1),TSTOR1(1,1))
C
	CALL INITT(120)
C
C  DENSITY PLOT
C  THE DENSITY RANGE IS NOW FOUND.
	CALL EXTRM2(DEN,CNT$3,NZONES,NPLAS,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL DLIMY(YMIN,YMAX)
	CALL STEPS(4)
	DO 20 I=1,NPLAS
	  CALL SYMBL(48+I)
	  IF(I.GT.1) GO TO 10
	  CALL CHECK(ZONEC,DEN(1,1))
	  CALL DSPLAY(ZONEC,DEN(1,1))
	  GO TO 20
10	  CALL CPLOT(ZONEC,DEN(1,I))
20	CONTINUE
	ENCODE(80,30,XTITLE)
30	FORMAT('RMINOR  (CM)$')
	CALL AGXCHS(XTITLE)
	ENCODE(80,40,YTITLE)
40	FORMAT('#/CM**3$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,50,TITLE)
50	FORMAT('PLASMA DENSITY$')
	CALL AGXCGT(0,TITLE)
	IF(IDEFIT.EQ.0) GO TO 60
	CALL SYMBL(1)
	CALL NPTS(NDENE)
	CALL STEPS(1)
	CALL LINE(-1)
	CALL CPLOT(RA,DENEA)
	CALL LINE(0)
60	CALL UPAUSE(1)
C
C  TEMPERATURE PLOTS
C  THE TEMPERATURE RANGE IS NOW FOUND.
	CALL EXTRM2(TEMP,CNT$3,NZONES,NPLAS,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL DLIMY(YMIN,YMAX)
	CALL STEPS(4)
	DO 80 I=1,NPLAS
	  CALL SYMBL(48+I)
	  IF(I.GT.1) GO TO 70
	  CALL CHECK(ZONEC,TEMP(1,1))
	  CALL DSPLAY(ZONEC,TEMP(1,1))
	  GO TO 80
70	  CALL CPLOT(ZONEC,TEMP(1,I))
80	CONTINUE
	CALL AGXCHS(XTITLE)
	ENCODE(80,90,YTITLE)
90	FORMAT('KEV$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,100,TITLE)
100	FORMAT('PLASMA TEMPERATURE$')
	CALL AGXCGT(0,TITLE)
	IF(ITEFIT.EQ.0) GO TO 110
	CALL SYMBL(1)
	CALL NPTS(NTEMPE)
	CALL STEPS(1)
	CALL LINE(-1)
	CALL CPLOT(RA,TEA)
	CALL SYMBL(0)
	CALL LINE(0)
110	IF(ITIFIT.EQ.0) GO TO 120
	CALL SYMBL(1)
	CALL NPTS(NTEMPI)
	CALL STEPS(1)
	CALL LINE(-1)
	CALL CPLOT(RA,TIA)
	CALL SYMBL(0)
	CALL LINE(0)
120	CALL UPAUSE(1)
C
C  NEUTRAL DENSITY PLOTS.
C  THE NEUTRAL DENSITY RANGE IS NOW FOUND.
	DO 170 I=1,NNEUT
	  DO 130 J=1,NZONES
	    TWORK(J)=DNEUT(J,I)
	    IF(ESFLAG) TWORK(J)=TWORK(J)+DCOLD(J,I)
130	  CONTINUE
	  CALL EXTREM(TWORK,NZONES,YMIN,YMAX)
	  CALL PLOTI(NZONES)
	  CALL YTYPE(2)
	  CALL DLIMY(YMIN,YMAX)
	  CALL CHECK(ZONEC,TWORK)
	  CALL DSPLAY(ZONEC,TWORK)
	  CALL LINE(34)
	  CALL CPLOT(ZONEC,DNEUT(1,I))
	  IF(.NOT.ESFLAG) GO TO 140
	  CALL LINE(5434)
	  CALL CPLOT(ZONEC,DCOLD(1,I))
140	  CALL LINE(0)
	  CALL AGXCHS(XTITLE)
	  ENCODE(80,150,YTITLE)
150	  FORMAT('#/CM**3$')
	  CALL AGXCVS(YTITLE)
	  ENCODE(80,160,TITLE) I
160	  FORMAT('NEUTRAL DENSITY FOR SPECIE ',I1,'$')
	  CALL AGXCGT(0,TITLE)
170	CALL UPAUSE(1)
C
C  WALL NEUTRAL TEMPERATURE
C  THE NEUTRAL TEMPERATURE RANGE IS NOW FOUND.
	CALL EXTRM2(TNEUT,CNT$3,NZONES,NNEUT,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL DLIMY(YMIN,YMAX)
	CALL STEPS(4)
	DO 190 I=1,NNEUT
	  CALL SYMBL(48+I)
	  IF(I.GT.1) GO TO 180
	  CALL CHECK(ZONEC,TNEUT(1,1))
	  CALL DSPLAY(ZONEC,TNEUT(1,1))
	  GO TO 190
180	  CALL CPLOT(ZONEC,TNEUT(1,I))
190	CONTINUE
	CALL AGXCHS(XTITLE)
	ENCODE(80,200,YTITLE)
200	FORMAT('KEV$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,210,TITLE)
210	FORMAT('NEUTRAL TEMPERATURE$')
	CALL AGXCGT(0,TITLE)
	CALL UPAUSE(1)
C
C  NET SOURCE FOR IONS
C  THE RANGE IS NOW FOUND
	CALL EXTRM2(SORI,CNT$3,NZONES,NNEUT,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL YTYPE(2)
	IF((SIGN(1.E00,YMIN)/SIGN(1.E00,YMAX)).LT.0.) CALL YTYPE(1)
	CALL DLIMY(YMIN,YMAX)
	CALL STEPS(4)
	DO 230 I=1,NNEUT
	  IC=NSPEC(I)
	  CALL SYMBL(48+IC)
	  IF(I.GT.1) GO TO 220
	  CALL CHECK(ZONEC,SORI(1,1))
	  CALL DSPLAY(ZONEC,SORI(1,1))
	  CALL LINE(0)
	  GO TO 230
220	  CALL CPLOT(ZONEC,SORI(1,I))
230	CONTINUE
	CALL AGXCHS(XTITLE)
	ENCODE(80,240,YTITLE)
240	FORMAT('#/(SEC*CM**3)$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,250,TITLE)
250	FORMAT('NET SOURCE FOR IONS$')
	CALL AGXCGT(0,TITLE)
	CALL UPAUSE(1)
C
C  NET SOURCE FOR NEUTRALS
C  THE RANGE IS NOW FOUND.
	CALL EXTRM2(SORN,CNT$3,NZONES,NNEUT,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL YTYPE(2)
	IF((YMIN/YMAX).LT.0.) CALL YTYPE(1)
	CALL DLIMY(-YMAX,-YMIN)
	CALL STEPS(4)
	DO 270 I=1,NNEUT
	DO 255 J=1,NZONES
	TWORK(J)=-SORN(J,I)
255	CONTINUE
	  CALL SYMBL(48+I)
	  IF(I.GT.1) GO TO 260
	  CALL CHECK(ZONEC,TWORK)
	  CALL DSPLAY(ZONEC,TWORK)
	  GO TO 270
260	  CALL CPLOT(ZONEC,TWORK)
270	CONTINUE
	CALL AGXCHS(XTITLE)
	ENCODE(80,280,YTITLE)
280	FORMAT('#/(SEC*CM**3)$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,290,TITLE)
290	FORMAT('NET SINK FOR NEUTRALS$')
	CALL AGXCGT(0,TITLE)
	CALL UPAUSE(1)
C
C  NET SOURCE FOR ELECTRONS.
C  RANGE IS NOW FOUND
	CALL EXTREM(SORE,CNT$3,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL YTYPE(2)
	IF((YMIN/YMAX).LT.0.) CALL YTYPE(1)
	CALL DLIMY(YMIN,YMAX)
	CALL CHECK(ZONEC,SORE)
	CALL DSPLAY(ZONEC,SORE)
	CALL AGXCHS(XTITLE)
	ENCODE(80,300,YTITLE)
300	FORMAT('#/(SEC*CM**3)$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,310,TITLE)
310	FORMAT('NET SOURCE FOR ELECTRONS$')
	CALL AGXCGT(0,TITLE)
	CALL UPAUSE(1)
C
C  CHARGE EXCHANGE POWER.
C  RANGE IS NOW FOUND
	CALL EXTREM(PCX,CNT$3,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL DLIMY(YMIN,YMAX)
	CALL CHECK(ZONEC,PCX)
	CALL DSPLAY(ZONEC,PCX)
	CALL AGXCHS(XTITLE)
	ENCODE(80,320,YTITLE)
320	FORMAT('WATTS/CM**3$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,330,TITLE)
330	FORMAT('CHARGE EXCHANGE POWER FROM ION TO NEUTRAL$')
	CALL AGXCGT(0,TITLE)
	CALL UPAUSE(1)
C
C  IONIZATION POWER.
C  RANGE IS NOW FOUND
	CALL EXTREM(PII,CNT$3,YMIN,YMAX)
	CALL PLOTI(NZONES)
	CALL DLIMY(YMIN,YMAX)
	CALL CHECK(ZONEC,PII)
	CALL DSPLAY(ZONEC,PII)
	CALL AGXCHS(XTITLE)
	ENCODE(80,340,YTITLE)
340	FORMAT('WATTS/CM**3$')
	CALL AGXCVS(YTITLE)
	ENCODE(80,350,TITLE)
350	FORMAT('IONIZATION POWER FROM NEUTRAL TO ION$')
	CALL AGXCGT(0,TITLE)
	CALL UPAUSE(1)
	RETURN
	END
