      SUBROUTINE DEPO
C
C     THIS PROGRAM CALCULATES DEPOSITION PROFILES AS
C     A FUNCTION OF MINOR RADIUS AND PITCH ANGLE FOR
C     THE FPP PROGRAM
C
C	IT USES A FINITE BANANA WIDTH MODEL TO TRANSFORM
C	THE PHYSICAL DEPOSITION, TO DEPOSITION AT <R> AND AT
C	A ZETA0 DETERMINED FROM MU CONSERVATION AT RMAJ+<R>
C
C 17-May-1989 GWH: upgraded to general geometry.  Improvements which
C could still be made include upgrading the BMFP calculation to general
C plasma species (not just 3 species), and upgrading to a variable step
C size so that it goes faster.  Could also perhaps use variable spacing
C between the pencil beamlets, to concentrate beamlets near the center
C where more resolution is needed.  Could also interpolate in pitch
C angle, as it already interpolates in minor radius.
C
c  improved to calculate the beam attenuation with Towner's
c	bmfp subroutine.  The old method used the central Te to
c	calculate electron impact ionization thus overestimating
c	attenuation, and ignored CX with impurities thus
c	underestimating attenuation.  For one test case, this only
c	made a 10 % difference.   Greg Hammett April 17, 1985.
C
C       PLOTS CONDENSED TO 1 PAGE FOR EACH SOURCE
C       RON RADEZTSKY   1/15/86
C
C     SET UP COMMON BLOCKS
C
	include 'FPPCOM'
 
c--see ADI for definitions of these parameters:
	parameter (jm=maxxsi/2)	!replaces the old 26
	parameter (jp=maxxsi/2+1)	!replaces the old 27
	parameter (imxBmltR=30)
	parameter (imxBmltZ=30)
 
c--The beam is divided into an array of up to imxBmltR*imxBmltZ
c beamlets.   BEAM is the relative weight of each beamlet as it
c passes through the plasma, with the sum of all the beamlet weights
c equal to 1 coming out of the beam duct.
	real BEAM(imxBmltR,imxBmltZ)
	real  PLOTH(maxshe)
	DIMENSION BHR(3,maxshe), PLOTH1(maxxsi),PLOTH2(maxxsi),
	1	 PLOTH3(maxxsi), XARRAY(maxxsi)
 
	DIMENSION PI(maxshe), PE(maxshe), TSTOP(maxshe),BEDEN(maxshe)
 
c
        CHARACTER*9 iTextRR(3)
C
	real temp(3),den(3)	!for electrons, ions, and impurities
	real almfp(maxshe)	!mean free path on each shell
C
	DATA	iTextRR/'Full     ','Half     ','Third    '/
 
C zero out dep:
	do 7 i=1,nshell
	do 7 j=0,maxxsi
	do 7 k=1,3
7	dep(i,j,k)=0.0
 
c Set up some parameters if we are to use the more advanced model
c implemented in January 1988 which includes beam divergence and
c limiting apertures with the error function model.
c (This model still ignores any focussing of the beam.)
 
	if(nfpdepsw .eq. 0) goto 50
	call momry(0.0,0.0,0,zRaxis,zth,zJac)
	z2=zRaxis**2-Rtan**2
	if(z2 .lt. 0.0) z2=0.0
	ztopl=fplbtn(nbemnum)-z2**0.5
	zsigZ=ztopl*fpbmdz !Beam error function height sigma_Z
	zsigR=fplbtn(nbemnum)*fpbmdR  !Beam error function sigma_R
	bmedghw=fpbahw+2*zsigR*(1.0-fplbap(nbemnum)/fplbtn(nbemnum))
	if(fpbshw+2*zsigR .lt. bmedghw)bmedghw=fpbshw+2*zsigR
	bmedghh=fpbaHH+2*zsigZ*(1.0-fplbap(nbemnum)/ztopl)
	if(fpbshh+2*zsigZ .lt. bmedghh)bmedghh=fpbshh+2*zsigZ
50	continue
 
C step size in major radius for the sightline integrals (cm):
C some good guesses for how large S can be:
	s=Max(abs(Rtan),Rinwal)**2*(2./maxxsi)
	1   /Max(abs(Rtan),Rinwal/maxxsi/2.)
	s=Min(s,rmin/nshell)
c to be more accurate, reduce s by a factor of zgrids:
c zgrids=1.0 for the minimum needed step size:
	data zgrids /3.0/
	s=s/zgrids
C
C     BEAM PROFILE INFORMATION
C
C The beam is divided up into nBmltR*nBmltZ parallel beamlets, which
C have have a gaussian weighting out to a sharp cutoff.  Because up-down
C symmetry is assumed, the # of vertical beamlets is effectively
C 2*nBmltZ-1.
 
C Select a reasonable number of beamlets as a good compromise between
C accuracy and speed:
	zdr=Max(abs(Rtan),Rinwal)**2*(2./maxxsi)
	1   /Max(abs(Rtan),Rinwal/maxxsi/2.)
	zdr=min(zdr,rmin/nshell)
 
c zgridR=agridZ=1.0 for the minimum needed step size:
	data zgridR /2.0/ !increase # of R grid points by *zgridR
	data zgridZ /3.0/ !increase # of Z grid points by *zgridZ
	nBmltR=2*BmEdgHW/(zdr/zgridR)
	nBmltZ=BmEdgHH/(zdr/zgridZ)
	if(nBmltR .lt. 1) nBmltR=1
	if(nBmltZ .lt. 1) nBmltZ=1
c force NBmltR to be odd so that one beamlet goes through the
c center of the beam:
	if(nBmltR .eq. (NBmltR/2)*2) NBmltR=nBmltR+1
 
	if(nBmltR .gt. imxBmltR) then
	    write(lsysout,*)'DEPO warning:  optimum nBmltR=',nBmltr	
	    nBmltR=imxBmltR
	    write(lsysout,*)'      being reduced to nBmltR=',nBmltR
	endif
	if(nBmltZ .gt. imxBmltZ) then
	    write(lsysout,*)'DEPO warning:  optimum nBmltZ=',nBmltz
	    nBmltZ=imxBmltZ
	    write(lsysout,*)'      being reduced to nBmltZ=',nBmltZ
	endif
 
c  BmEdgHW, BmEdgHH = beam edge half width and half height
c  delr, delz = spacing between beamlets.
	DELR=2*BmEdgHW/nBmltR
	DELZ=2*BmEdgHH/(2*nBmltZ-1)
C
	DIR=SIGN(1.0,RTAN)
	RTAN=ABS(RTAN)
C
C     BEAM DEPOSITION
C
	DO 100 IEN=1,3
	blost(ien)=0.0
	bunion(ien)=0.0
	IF(PtFrBeam(IEN).le. 0.0) GOTO 100
 
c  calculate beam mean free path on each minor radius shell:
	E=VBeamInj/FLOAT(IEN)
	ekev=e/1000.
	VEL=1.38E6*SQRT(E/AB)
	DHY=(rZimp-rZeff)/(rZimp-1.)
	DIM=(rZeff-1.)/((rZimp-1.)*rZimp)
 
 
	do 640 ish=1,nshell
	den(1)=1.0e13*denpro(ish)	!electron density  (per cc)
	den(2)=den(1)*dhy		!hydrogenic density
	den(3)=den(1)*dim		!impurity density
	temp(1)=1.0e-3*tepro(ish)
	temp(2)=1.0e-3*tipro(ish)
	temp(3)=temp(2)
640	almfp(ish)=bmfp(ekev,temp,APlasAMU,ab,den,aplasz,rZbeam,NumPlSp)
 
C
C     WEIGHT THE BEAMLETS
C
	TOT=0.0
	DO 600 M=1,nBmltZ
	ZB=(M-1)*delZ
	DO 600 N=1,nBmltR
	RB=-BmEdgHW + (N-0.5)*delR
 
	if(nfpdepsw .eq. 0)then
c original simple gaussian width model:
	    WGHT=EXP(-RB**2/(2.E00*FALLR**2)-ZB**2/(2.E00*FALLZ**2))
	else
c new error function model (evaluating all these error function
c takes negligible time compared with the sightline tracking!):
 
	zbw=(fpbahw-rb*fplbap(nbemnum)/fplbtn(nbemnum))/
	1   (1.-fplbap(nbemnum)/fplbtn(nbemnum))
	if(fpbaHW .lt. zbw) zbw=fpbaHW
	zaw=-(fpbahw+rb*fplbap(nbemnum)/fplbtn(nbemnum))/
	1   (1.-fplbap(nbemnum)/fplbtn(nbemnum))
	if(-fpbaHW .gt. zaw) zaw=-fpbaHW
 
	zbh=(fpbahh-zb*fplbap(nbemnum)/ztopl)/
	1   (1.-fplbap(nbemnum)/ztopl)
	if(fpbaHH .lt. zbh) zbh=fpbaHH
	zah=-(fpbahh+zb*fplbap(nbemnum)/ztopl)/
	1   (1.-fplbap(nbemnum)/ztopl)
	if(-fpbaHH .gt. zah) zah=-fpbaHH
 
	WGHT=(erf((zbw-rb)/2**.5/zsigR)-
	1   erf((zaw-rb)/2**.5/zsigR))/2.
	WGHT=WGHT*(erf((zbh-zb)/2**.5/zsigZ)-
	1   erf((zah-zb)/2**.5/zsigZ))/2.
	endif
 
c  Because up-down symmetry is assumed, the m=1 (ZB=0) beamlets would get
c  counted twice unless we cut their weight in half:
	if(m .eq. 1)wght=wght/2.0
	BEAM(N,M)=WGHT
	TOT=TOT+BEAM(N,M)
600	CONTINUE
 
c normalize so that sum_n sum_m beam_(n,m) =1.0 (half of power since
c  up-down symmetry is assumed)	
 
	DO 650 N=1,nBmltR
	DO 650 M=1,nBmltZ
	BEAM(N,M)=BEAM(N,M)/TOT
650	CONTINUE
C
C     TRACK THE BEAM INTO THE PLASMA
C
c  I am not happy with the way the sightline integrals are done.  They
c  look right (although perhaps there are 1/INS errors), but they are not
c  clear.
 
	call momry(1.0,0.0,0,zRmax,zy0,zjac)
	zRmax=zRmax+BmEdgHw
	ins=(zRmax-Rtan)/abs(s)+1
	s=(zRmax-Rtan)/ins
 
c sightline integral broken in two halves:
c  kop = 1 = inward in R, 2 = outward in R
	DO 711 KOP=1,2
	IF(KOP.EQ.1) S=ABS(S)
	IF(KOP.EQ.2) S=-ABS(S)
 
c integral over a half sightline in INS steps:
	DO 700 KIP=1,INS
	IF(KOP.EQ.1)then
	RC=zRmax-S*(KIP-1)
	else
	RC=RTAN-S*(KIP-1)
	endif
 
c loop over all beamlets, first in the vertical direction:
	DO 123 M=1,nBmltZ
	Z=(M-1)*DELZ
 
c  then in the horizontal direction:
	DO 123 N=1,nBmltR
	zdiffR=-BmEdgHW+(N-0.5)*delR
	RT=RTAN+zdiffR
	IF(RT.LT.RINWAL.AND.KOP.EQ.2) GO TO 124
	R=ABS(RC+zdiffR)
	PT= (R-S)*(R-S) - RT*RT
	IF(PT.LT.0.0) PT=0.0
	PT2=R*R-RT*RT
	IF(PT2.LT.0.0) PT2=0.0
	DX = ABS(SQRT(   PT2   ) - SQRT(PT))
	RI=R-0.5*S
c rl = minor radius of this interval:
	call nbimap(RI,Z,zxi,zth,ier)
	rl=zxi*rmin
c skip if this is outside the plasma:
	IF(RL.GT.RMIN) GOTO 124
 
c interpolate in minor radius to find the beam attenuation:
c k is the shell closer in than radius rl, l=k+1 is the shell
c  further out.  del is the relative weighting:
	K=INT(RL/RMIN*NSHELL+0.5)
	L=K+1
	if(l .gt. nshell)l=nshell
	DEL=RL/RMIN*NSHELL+0.5-K
	IF(K.EQ.0) K=1
	akmfp=del/almfp(L) + (1.-del)/almfp(K)
 
c  replacing exp(-x) with 1./(1.+x) here and in the beamlet weighting
c  above only speeds up DEPO by 4%, so leave the more accurate exp()
c  function in :
	transm=exp(-dx*akmfp)
	db=(1.-transm)*beam(n,m)
	beam(n,m)=beam(n,m)*transm
C
C	.....DETERMINE <R>, AND ZETA AT RMAJ+<R>, THETA = 0
C	.....USING FINITE BANANA WIDTH MODEL
C
	zE=EkeV
	zVparV=dir*RT/RI
	zR=Ri
	zZ=Z
	call orbshift(zE,zVparV,zR,zZ,zrav,Zeta0,iunconf)
 
c handle unconfined orbits:
	if(iunconf .eq. 1)then
	    BLost(ien)=Blost(ien)+db
	    goto 124
	endif
 
	rdep=zrav
 
c assign it to a radial zone (uniformly spaced radial zones assumed):
2003	K=INT(RDEP/RMIN*NSHELL+1.0)
 
c  find the index L which corresponds to zeta.
c  this fancy binary search routine only speeded up DEPO from
c  138 cpu secs to 117 cpu secs.  Most of the time is being spent
c  somewhere else.
	L=LKUPR(zeta0,xsi,maxxsi)
	if(L .eq. maxxsi)go to 3001
	if(zeta0 .gt. .5*(xsi(L)+xsi(L+1)) ) L=L+1
 
3001	DEP(K,L,IEN)=DEP(K,L,IEN)+DB
  124 CONTINUE
  123 CONTINUE
  700 CONTINUE
  711 CONTINUE
	DO 699 N=1,nBmltR
	DO 699 M=1,nBmltZ
	BUNION(IEN)=BUNION(IEN)+BEAM(N,M)
  699 CONTINUE
  100 CONTINUE
 
C
C     PLOT PROFILES OF H(RHO) AND NORMALIZE DEP TO PARTICLES PER SECOND
c
c  according to some notes from Harry Towner, H(r) is defined by:
c
c 	d n(r) / dt  = (I_beam/ ze)  H(r)
c                       -----------------
c                       2 pi R pi a**2
c  where d n(r) /dt is the beam density input rate, LBeam is the neutral beam
c  equivalent current, and 2 pi R pi a**2 is the plasma volume.  You can see
c  that H(r) is completely dimensionless, and that H(r) = 1 corresponds to
c  completely uniform beam deposition.
c
c  first zero BHR:
	do 1234 ien=1,3
	do 1234 ish=1,nshell
1234	bhr(ien,ish)=0.0
 
	PAREQ=PBeamInj*6.2418E18/VBeamInj*1.0E6
	FRTOT=PtFrBeam(1)+PtFrBeam(2)/2.+PtFrBeam(3)/3.
	VOLTOT=Volfp(nshell)
	DO 200 K=1,NSHELL
	VOL=dVolfp(k)
	DO 200 L=1,maxxsi
	DO 200 IEN=1,3
	IF(PtFrBeam(IEN) .le. 0.0) GOTO 200
	BHR(IEN,K)=BHR(IEN,K)+DEP(K,L,IEN)/VOL*VOLTOT
	DEP(K,L,IEN)=DEP(K,L,IEN)*PtFrBeam(IEN)*PAREQ/FRTOT
  200 CONTINUE
 
C       CALCULATE TOTAL SHINE AND PROMPT ORBIT LOSS IN KW
        BKWR = PtFrBeam(1) + PtFrBeam(2)/2 + PtFrBeam(3)/3
        BUKWE = PBeamInj * 1000. * (BUNION(1)*PtFrBeam(1)
	1   + BUNION(2)*PtFrBeam(2)/2 + BUNION(3)*PtFrBeam(3)/3)
        BLKWE = PBeamInj * 1000. * (BLOST(1)*PtFrBeam(1)
	1   + BLOST(2)*PtFrBeam(2)/2 + BLOST(3)*PtFrBeam(3)/3)
        BUKW = BUKWE / BKWR
        BLKW = BLKWE / BKWR
 
	IF(LPlotH.NE.1) GOTO 555
	DO 222 J=1, NSHELL
	XARRAY(J)=(J-0.5)/FLOAT(NSHELL)
  222 CONTINUE
C       FIND MAX OF H(RHO) TO SET GRAPH LIMIT
        zValMax = 1.E-15
        DO 223 J = 1,3
        DO 223 K = 1,NSHELL
        zValMax = AMAX1(zValMax,BHR(J,K))
  223 CONTINUE
        zUpLim = zValMax * 1.05
        zBotLim = 0.0
C     PLOT H(RHO)
        CALL INITT(240)
        CALL BINITT
	call fpgrlab
 
        CALL NPTS(NSHELL)
        CALL CHRSIZ(2)
        CALL XFRM(3)
        CALL YFRM(3)
        CALL XMFRM(3)
        CALL YMFRM(3)
	call place('ul4  ')
        CALL DLIMX(0.0,1.0)
        CALL DLIMY(zBotLim,zUpLim)
        IEN = 1
        DO 333 K = 1,NSHELL
        PLOTH(K) = BHR(IEN,K)
  333 CONTINUE
        CALL CHECK(XARRAY,PLOTH)
        CALL DSPLAY(XARRAY,PLOTH)
C     PLOT 2ND AND 3RD CURVES
        DO 335 IEN = 2,3
        IF (PtFrBeam(IEN) .le. 0.0) GOTO 335
        IF (IEN .EQ. 2) LINTYP = 7474
        IF (IEN .EQ. 3) LINTYP = 5454
        CALL LINE(LINTYP)
        DO 334 K = 1,NSHELL
        PLOTH(K) = BHR(IEN,K)
  334 CONTINUE
        CALL CHECK(XARRAY,PLOTH)
        CALL CPLOT(XARRAY,PLOTH)
  335 CONTINUE
        CALL FRAME    !@
c@	call dframe
 
	WRITE(TEXT1,15)
   15   FORMAT('h(rho) for full, 1/2, 1/3 energy$')
        CALL AGTCGT(0,TEXT1)
        CALL CHRSIZ(1)
C     PLOT TABLE OF LOSSES
        JRHRX = 575
        JRHRY = 725
        JRHRST = 30
	WRITE(TEXT1,26) NBEMNUM
26	FORMAT('Source #',I2,'$')
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
        JRHRY = JRHRY - JRHRST
	WRITE(TEXT1,27) DIR*RTAN ! dir*Rtan<0 means counter injection
27	FORMAT('Rtan:  ',F8.1,' cm$')
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
        JRHRY = JRHRY - JRHRST
	WRITE(TEXT1,28) PBEAMINJ
   28 FORMAT('Pinj:  ',F8.2,' MW$')
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
        JRHRY = JRHRY - JRHRST
        EINJR = VBeamInj/1000.0
	WRITE(TEXT1,29) EINJR
   29 FORMAT('Einj:  ',F8.2,' keV$')
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
        JRHRY = JRHRY - JRHRST
        JRHRY = JRHRY - JRHRST
	WRITE(TEXT1,20)
   20   FORMAT('       Shinethru   Prompt Loss$')
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
        JRHRY = JRHRY - JRHRST
        DO 336 IEN = 1,3
        BUNI = BUNION(IEN)*100.
        BLOS = BLOST(IEN)*100.
	WRITE(TEXT1,21) ITEXTRR(IEN),BUNI,BLOS
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
        JRHRY = JRHRY - JRHRST
   21 FORMAT(A9,F6.1,'%       ',F6.1,'%$')
  336 CONTINUE
 
	WRITE(TEXT1,24) BUKW,BLKW
   24 FORMAT('Total  ',F6.1,' kW     ',F6.1,' kW$')
        CALL AGTMHS(JRHRX,JRHRY,TEXT1,0)
 
c
c  Plot deposition vs. pitch angle for a single shell if desired.
c  This feature is used only for debugging, and will be turned off for
c  normal runs.
c
c for the debug version, uncomment the following two lines:
c	CALL ISBATCH(IRET)
c	IF (IRET .EQ. 1) GO TO 666
c and comment out the next line:
	goto 666
 
320	call finitt(0,0) !switch to alpha mode temporarily
321	WRITE(6,11)
11	FORMAT(' WHAT SHELL WOULD YOU LIKE TO EXAMINE ? (0 FOR NONE)')
	READ(5,*) ishell
	IF(ishell.EQ.0) then
	    call initt(0) !switch back to graphics without erasing screen
	    GOTO 666
	endif
 
	DO 111 L=1,maxxsi
	PLOTH1(L)=DEP(ishell,L,1)
	PLOTH2(L)=DEP(ishell,L,2)
	PLOTH3(L)=DEP(ishell,L,3)
	xarray(L)=xsi(L)
111	CONTINUE
 
C Rob's original code would now symmetrix the ploth1,2, and 3 arrays
C for trapped particles.  But don't bother.
 
	XARRAY(1)=maxxsi
	PLOTH1(1)=maxxsi
	PLOTH2(1)=maxxsi
	PLOTH3(1)=maxxsi
 
	call initt(240)
	CALL BINITT
	call fpgrlab
	CALL CHECK(XARRAY,PLOTH1)
	CALL DSPLAY(XARRAY,PLOTH1)
	CALL LINE(7474)
	CALL CPLOT(XARRAY,PLOTH2)
	CALL LINE(5454)
	CALL CPLOT(XARRAY,PLOTH3)
	WRITE(TEXT1,45) ISHELL
45	FORMAT('SHELL #',I2,'$')
	CALL AGTCGT(750,TEXT1)
	WRITE(TEXT1,55)
55	FORMAT('DEPOSITION VS. PITCH ANGLE$')
	CALL AGTCGT(0,TEXT1)
	WRITE(TEXT1,65)
65	FORMAT('PARTICLES/(CM**3-SEC)$')
	CALL AGTCVS(TEXT1)
	WRITE(TEXT1,75)
75	FORMAT('VPAR/V$')
	CALL AGTCHS(TEXT1)
	CALL UPAUSE(0)
	GOTO 320
666    	CONTINUE
C
C	CALCULATE HEATING PROFILES USING SIMPLIFYING
C	ASSUMPTIONS -- E FIELD = 0, NO LOSS ORBITS, STEADY STATE
C
	Q3=SQRT(3.)
	PBI=0.
	PBE=0.
	DIM=(rZeff-1.)/((rZimp-1.)*rZimp)
	DHY=(rZimp-rZeff)/(rZimp-1.)
	DO 1000 I=1,NSHELL
	PI(I)=0.
	PE(I)=0.
	BEDEN(I)=0.
	TIS=TIPRO(I)
	TES=TEPRO(I)
	DES=DENPRO(I)*1.0E13
C
C	...CALCULATE SLOWING-DOWN PARAMETERS
C
	ALLE=ALOG(TES*3.95E10/SQRT(DES))
	ALLHY=ALOG(SQRT(TES)*VBeamInj/2.*1.032E10/SQRT(DES)*AP/(AP+AB))
	ALLIM=ALOG(SQRT(TES)*VBeamInj/2.*1.032E10/SQRT(DES)*AIM/rZimp/
     >  (AIM+AB))
	Z2=DHY*ALLHY/ALLE+DIM*rZimp**2/AIM*ALLIM/ALLE*AP
	ZSC=DHY*ALLHY/ALLE+DIM*rZimp**2*ALLIM/ALLE
	TAUS=6.32E8*AB*TES**1.5/(DES*ALLE)
	EC=AB*(Z2/AP)**.66666*14.8*TES
	EC32=EC**1.5
	TSTOP(I)=TAUS/3.*ALOG((VBeamInj**1.5+EC32)/
     >  ((1.5*TIS)**1.5+EC32))*1000.
	XARRAY(I)=(I-.5)/FLOAT(NSHELL)
	VOL=dVolfp(i)
	DO 1000 J=1,3
	E0=VBeamInj/FLOAT(J)
	P=BHR(J,I)*PtFrBeam(J)*PBeamInj*1.0E6/(FLOAT(J)*FRTOT*VOLTOT)
	X=E0/EC
	Y=SQRT(X)
	GI=ALOG((1.+Y**3)/(1.+Y)**3)/(3.*X)
     >	+2.*ATAN2(2.*Y-1.,Q3)/(Q3*X)
     >  -2.*ATAN2(-1.,Q3)/(Q3*X)
	PI(I)=PI(I)+P*GI-P*(1.5*TIPRO(I))/(VBeamInj/J)
	PE(I)=PE(I)+P*(1.-GI)
	BEDEN(I)=P/E0*6.2418E18*TSTOP(I)/1.0E16+BEDEN(I)
	PBI=PBI+(P*GI-P*1.5*TIPRO(I)/(VBeamInj/J))*VOL/1000.
	PBE=PBE+P*(1.-GI)*VOL/1000.
1000	CONTINUE
C       GET READY TO PLOT POWER TO IONS / ELECTRONS:
C       FIND MAX OF PI AND PE
        zValMax = 1.0E-15
        zValMaxI = 1.0E-15
        zValMaxE = 1.0E-15
        DO 1003 I = 1,NSHELL
        zValMaxI = AMAX1(zValMaxI,PI(I))
        zValMaxE = AMAX1(zValMaxE,PE(I))
 1003   CONTINUE
        zValMax = AMAX1(zValMaxI,zValMaxE)
C       CHECK TO SEE IF OK TO PLOT ON SAME SCALE
        IOK = 0
        zValRat = zValMaxI / zValMaxE
        IF (zValRat .GT. 3.0 .OR. zValRat .LT. 0.33) IOK=1
C       PLOT IONS ON ION SCALE
 
	CALL BINITT
        CALL NPTS(NSHELL)
        CALL CHRSIZ(2)
        CALL XFRM(3)
        CALL YFRM(3)
        CALL XMFRM(3)
        CALL YMFRM(3)
	call place('lr4  ')
        zBotLim = 0.0
        IF (IOK .EQ. 1) THEN
          zUpLim = zValMaxI*1.05
        ELSE
          zUpLim = zValMax*1.05
        ENDIF
        CALL DLIMX(0.0,1.0)
        CALL DLIMY(zBotLim,zUpLim)
        CALL CHECK(XARRAY,PI)
        CALL DSPLAY(XARRAY,PI)
C       PLOT ELECTRONS ON ELECTRON SCALE
        CALL YLOCRT(0)
        CALL LINE(7474)
        IF (IOK .EQ. 1) THEN
          zUpLim = zValMaxE*1.05
        ELSE
          zUpLim = zValMax*1.05
        ENDIF
        CALL DLIMX(0.0,1.0)
        CALL DLIMY(zBotLim,zUpLim)
        CALL CHECK(XARRAY,PE)
        CALL DSPLAY(XARRAY,PE)
        CALL FRAME    !@
c@	call dframe
	WRITE(TEXT1,1015)
 1015 FORMAT('Pbi / Pbe  (no losses)$')
        CALL AGTCGT(375,TEXT1)
	WRITE(TEXT1,1065)
 1065 FORMAT('(WATTS/CM**3)$')
        CALL YLOC(0)
        CALL AGTCGT(355,TEXT1)
        CALL AGTCHS('r/a$')
	WRITE(TEXT1,1035) PBI,PBE
1035	FORMAT('Pbi =',F6.0,' kW   Pbe =',f6.0,' kW$')
	call agtcgt(335,text1)
 
C       PLOT STOPPING TIME / BEAM DENSITY
	CALL BINITT
	CALL NPTS(NSHELL)
        CALL CHRSIZ(2)
        CALL XFRM(3)
        CALL YFRM(3)
        CALL XMFRM(3)
        CALL YMFRM(3)
	call place('ll4  ')
	CALL CHECK(XARRAY,TSTOP)
	CALL DSPLAY(XARRAY,TSTOP)
	CALL AGTCHS('r/a$')
        CALL DINITY
        CALL YLOCRT(0)
	CALL NPTS(NSHELL)
        CALL LINE(7474)
	CALL CHECK(XARRAY,BEDEN)
	CALL DSPLAY(XARRAY,BEDEN)
        CALL FRAME    !@
c@	call dframe
	WRITE(TEXT1,1045)
1045	FORMAT('Full Energy Stopping Time/ n_beam$')
	CALL AGTCGT(375,TEXT1)
	WRITE(TEXT1,1046)
1046	FORMAT('(msec)       /     (10**13/cm**3)$')
        CALL AGTCGT(350,TEXT1)
        CALL CHRSIZ(1)
	CALL UPAUSE(0)
	call finitt(0,0) !return to text screen
C       PLOTS FINISHED FOR THIS SOURCE
555	RTAN=RTAN*DIR
	RETURN
	END
