C
      SUBROUTINE DETECV
C
C     THIS PROGRAM CALCULATES THE DETECTOR SENSITIVITIES
C     FOR THE GIVEN SIGHTLINES AS A FUNCTION OF MINOR
C     RADIUS AND PITCH-ANGLE, AND STORES THEM IN 'EVENT MODE'
C     FOR THE FPP PROGRAM
C
C     DETECTOR IS ASSUMED TO BE ON THE BOTTOM AND THE SLIGHT LINES
C     ARE FOLLOWED UPWARDS.
C
c 17-May-1989 GWH: upgraded to general geometry.
c Improvements which could still be made include upgrading the
c BMFP calculation to general plasma species (not just 3 species),
C and upgrading to a variable step size so that it goes faster.
c Could also interpolate in pitch angle, as it already interpolates in
C minor radius.
C
C     11/19/86 - MODIFIED BY R. KAITA IN ACCORDANCE WITH CHANGES MADE
C		 TO DETECH BY G. HAMMETT (see detech for more details...)
C
C	CHANGES MADE ON 11/24/86:
C
C	1) ACCOMMODATION OF ARBITRARY NEUTRAL DENSITY PROFILES.
C	2) ABILITY TO TURN ATTENUATION ON AND OFF.
C	3) MAXIMUM NUMBER OF ENERGY GRID POINTS CHANGED TO 'MAXEN.'
C	4) MEAN FREE PATH AND SIGMA V CALCULATIONS TAKEN OUT OF INNER
C	   SIGHTLINE INTEGRAL LOOP TO SPEED UP CODE.
C	5) ORBIT PHYSICS UNCHANGED, BUT 'CORRECTLY' NORMALIZED SIGMA V
C	   (AS USED IN DETECH) INCLUDED.
c
C     SET UP COMMON BLOCKS
	include 'FPPCOM'
C
	REAL XARRAY(MAXXSI/2),YARRAY(MAXXSI/2),TEMP(3),DEN(3)
	REAL ALMFP(MAXSHE)
	REAL SMALLC(MAXSHE)
C
	DATA CON1,PI/1.92691111E15,3.141592654E00/
	DATA TWOPI/6.283185308/
 
C function which indexes into the SENS array, giving
c the sensitivity of the CX analyzer looking at energy i
C and sightline j to f on shell k at pitch angle L:
 
	IND(I,J,K,L)=(I-1+NENCX*(J-1+NLINE*(K-1+NSHELL*(L-1))))+1
 
	call finitt(0,0) !return to alpha mode before typing...
	IF(NDNB.GT.2*MAXSHE)THEN
		IDUM=2*MAXSHE
		write(lsysout,*)
     1		'FATAL ERROR: NDNB EXCEEDS THE LIMIT OF 2*MAXSHE=',IDUM
		STOP
	ENDIF
C
	IF(NENCX.EQ.0)NENCX=MAXEN
	IF(NENCX.GT.MAXEN)THEN
		NENCX=MAXEN
		write(lsysout,*) 'WARNING: NENCX REDUCED TO',NENCX
	ENDIF
	NSENS=MXSENS/(NLINE*NSHELL*MAXXSI/2)
	IF(NENCX.GT.NSENS)THEN
		NENCX=NSENS
		write(lsysout,*) 'WARNING: NENCX REDUCED TO',NENCX
	ENDIF
C
	IF(NENCX.LT.1)THEN						
	  write(lsysout,*)
     1	' WARNING: NOT ENOUGH ROOM IN ARRAYS FOR CALCULATING'
	  write(lsysout,*)
     1	' ANY CHARGE-EXCHANGE SPECTRA.	TRY REDUCING # OF '
	  write(lsysout,*) ' CHARGE-EXCHANGE DETECTORS TO 1.'
	  Ldir=-1
	  RETURN
	ENDIF
 
c Find magnetic axis zRaxis:
	call momry(0.0,0.0,0,zRaxis,zY,zjac)
 
c find the maximum height of the plasma above the midplane
c so we know where to start the vertical sightline integral:
 
	zrmax=0.0
	do 15 i=1,maxxsi/2
	zth=(pi*i)/(maxxsi/2.+1.)
	call momry(1.0,zth,0,zR,zY,zJac)
15	zrmax=max(zrmax,zY)
	zrmax=1.2*zrmax	!make it larger for safety
 
C
C     MAIN ION/ELECTRON DENSITY
C
      DHY=(rZimp-rZeff)/(rZimp-aplasz(2))/aplasz(2)
C
C     N(IMP)/N(E)
C
      DIM=(rZeff-aplasz(2))/((rZimp-aplasz(2))*rZimp)
C
C! determine energies at which the CX flux is to be calculated:
C! Take the first nencx grid points below elim2:
	ITOP=0
	DO 50 I=MAXEN,1,-1
		IF(ENEREV(I).GT.ELIM2)GO TO 50
		ITOP=I
		GO TO 51
50	CONTINUE
51	IF(ITOP.LT.NENCX)ITOP=NENCX
	NFENCX=ITOP+1-NENCX !starting energy index
C
C     ENERGY LOOP
C
      DO 100 IEN=1,NENCX
C
      E=ENEREV(NFENCX+IEN-1)/1000. !E in keV
	V=SQRT(E*CON1/AB)
 
C
C	DEFINE TERMS FOR CALCULATING LAMBDA MFP
C
	DO 70 ISH=1,NSHELL
	DEN(1)=1.0E13*DENPRO(ISH)
	DEN(2)=DEN(1)*DHY
	DEN(3)=DEN(1)*DIM
	TEMP(1)=1.0E-3*TEPRO(ISH)
	TEMP(2)=1.0E-3*TIPRO(ISH)
	TEMP(3)=TEMP(3)
	ALMFP(ISH)=BMFP(E,TEMP,APlasAMU,AB,DEN,aplasz,rZbeam,NumPlSp)
 
C!also calculate a small correction due to approximate evaluation
C! of <sigma v>cx when the data is analyzed:
 
	smallc(ish)=sigvbt(E/Ab*1000.,tnupro(ish)/APlasAMU(2),'CX')
	1   /Psgvcx(E/Ab*1000.)
70	SMALLC(ISH)=SMALLC(ISH)*25*SQRT(3.14159)
 
C!smallc contains the normalizations necessary to be consistent
C! with Davis, Mueller, & Keane's conventions.	See the comments
C! below, just before the SENS calculations.
C
C
C     TRACK THE BEAMS INTO THE PLASMA
C
      DO 100 ILIN=1,NLINE
C
C step size in major radius for the sightline integrals (cm):
c (make good guesses for how large S can be):
C
	STAN = ABS(TANS(ILIN) - zRaxis)
	s=rmin/nshell
c this would be the very small criterion:
c	s=abs(stan)*(1./maxxsi)**2
c	s=Min(s,rmin/nshell)
c so we need a variable step size.
 
c for accuracy, reduce s by a factor of zgrids.
c zgrids=1.0 for the minimum necessary step size:
	data zgrids /30.0/
	s=s/zgrids
 
	BEAM=1.0
	ins=(zrmax-stan)/abs(s)+1.0
	s=(zrmax-stan)/ins
 
      DO 711 KOP=1,2
      IF(KOP.EQ.1) S=ABS(S)
      IF(KOP.EQ.2) S=-ABS(S)
	PHIPA=1.5E00*PI
 
	DO 700 KIP=1,INS
	IF(KOP.EQ.1)then
	R=zrmax-S*KIP+S
	else
	R=STAN-S*KIP+S
	endif
 
      RT=STAN
      PT= (R-S)*(R-S) - RT*RT
      IF(PT.LT.0.0) PT=0.0
      PT2=R*R-RT*RT
      IF(PT2.LT.0.0) PT2=0.0
      DX = ABS(SQRT(   PT2   ) - SQRT(PT))
      RL = ABS(R-S/2.)
      B = RL**2 - (zRaxis - TANS(ILIN))**2
	YP=SQRT(B)*SIGN(1.E00,KOP-1.5)
	call nbimap(Tans(ilin),Yp,zxi,zth,ier)
	rl=rmin*zxi
      IF(RL.GT.RMIN) GOTO 124
C!interpolate in minor radius to find neutral density, beam attenuation:
C!k is the shell closer in than radius rl, L=K+1 is the shell
C! further out.	 del is the relative weighting:
      K=INT(RL/RMIN*NSHELL+0.5)
	L=K+1
	IF(L.GT.NSHELL) L=NSHELL
	DEL=RL/RMIN*NSHELL+0.5-K
	IF(K.EQ.0) K=1
	D=1.0E13*(DEL*DENPRO(L)+(1.-DEL)*DENPRO(K))
C
c take care to interpolate dnu (the neutral density) properly beyond
c the last zone center.
C
	DNUUP=NEUPRO(L)
	IF(K.EQ.NSHELL)DNUUP=NEUPRO(K)**2/NEUPRO(K-1)
	DNU=EXP(DEL*ALOG(DNUUP)+(1.-DEL)*ALOG(NEUPRO(K)))
C
C	ASYMMETRIC NEUTRAL DENSITY PROFILE
C
	IF(LNASYM .AND. Tans(ilin) .GT. zRaxis)THEN
		DNU=NEUPRO(1)
	ENDIF
C
C	ARBITRARY NEUTRAL DENSITY PROFILE
c Note that dnbneu(dnbrmj) is actually n0(r), a function of minor
c radius for vertically viewing CX analyzers.  This is because we
c don't expect the neutral density profile to be updown asymmetric,
c while it may be in-out asymmetric and would affect the horizontally
c viewing (DETECH) CX detectors.
C
	IF(LDNB)THEN
	    DNU=FPINTRP(rl,DNBNEU,DNBRMJ,NDNB)
	ENDIF
C
	TNU=DEL*TNUPRO(L)+(1.-DEL)*TNUPRO(K)
C
C	CALCULATE LAMBDA MFP.
C
	AKMFP=DEL/ALMFP(L)+(1.-DEL)/ALMFP(K)
C
C	THE FOLLOWING IS USED TO CALCULATE 'SENS' USING
C	NORMALIZED SIGMA V (IMPLEMENTED 11/24/86).
C
	SVCORR=DEL*SMALLC(L)+(1.-DEL)*SMALLC(K)
C
	BEAM=BEAM*EXP(-DX*AKMFP)
C
C	CONTROL ATTENUATION
C
	IF(.NOT.LATTEN)BEAM=1.0
C
      IF(RL.GT.RMIN)   GOTO 124
C***********************************************************************
C
C	.....DETERMINE <R>, AND ZETA AT RMAJ+<R>, THETA = 0
C	.....USING FINITE BANANA WIDTH MODEL
C
 
	zE=E
	zVparV=0.0 !particles detected at their banana tips
	zR=Tans(ILIN)
	zZ=yp
	call orbshift(zE,zVparV,zR,zZ,zrav,Zeta0,iunconf)
	if(iunconf .eq. 1) goto 124 !skip unconfined orbits
	rdep=zrav
 
C!At this point we have assigned a minor radius rdep that this
C! orbit comes from.  To find f at this rdep we interpolate beween
C! shells.  Kin is the shell inside of rdep, Kout the shell outside
C! of rdep.
	kin=int(rdep/rmin*nshell + 0.5)
	kout=kin+1
	if(kout .gt. nshell)kout=nshell
	weight=rdep/rmin*nshell -kin +0.5
	if(kin .lt. 1)kin=1
 
C!determine pitch angle index.	Since a single sightline only
C! views co or counter particles, we can cut our pitch angle
C! space in half:
	L=LKUPR(zeta0,xsi,maxxsi)
	if(L .lt. maxxsi)then
	    if(zeta0 .gt. .5*(xsi(L)+xsi(L+1)) ) L=L+1
	endif
	if(L .gt. maxxsi/2)L=L-maxxsi/2
 
	indexi=ind(ien,ilin,kin,L)
	indexo=ind(ien,ilin,kout,L)
 
C!The svcorr factor in the formula for SENS includes some
C! normalization factors. One factor of (2 pi)**1.5 is to
C! get the standard normalization defined by Davis, Mueller, & Keane.
C! Another factor of 25/(2 pi sqrt(2)) converts Rob's f used
C! in the code to the physical f (see SCCF).
C!
C! CX detector sensitivity at energy ien and sightline ilin,
C! to f on shell l with pitch angle k:
	SENS(indexi)=SENS(indexi)+DX*BEAM*DNU*svcorr
	1	*(1.-weight)
	SENS(indexo)=SENS(indexo)+DX*BEAM*DNU*svcorr
	1	*weight
C***********************************************************************
 
C  I NOW INCLUDE THE CHARGE EXCANGE BETWEEN THE BEAM ION AND THE
C  VOLUNE SOURCE NEUTRAL.
	IF(LBeam.EQ.0) GO TO 124
	PHIL=PHITNS(ILIN)
	IF(PHIL.LT.0.) PHIL=PHIL+TWOPI
	IF(PHIL.GT.TWOPI) PHIL=PHIL-TWOPI
	IF(PHIL.LT.PHIMIN.OR.PHIL.GT.PHIMAX) GO TO 124
	IPHI=(PHIL-FppPhi1)/DPHI+1.5
	IRN=(TANS(ILIN)-RMAJ1)/DRP+1.5
	IYN=(YP-ELEV1)/DYP+1.5
	IF(Litrav.EQ.0) GO TO 122
	DNUVS=DN0VS(IRN,IYN,IPHI)
	IF(DNUVS.EQ.0.) GO TO 122
	TNUVS=T0VS(IRN,IYN,IPHI)/DNUVS
	s2cxv=psgvcx(E/Ab*1000.)
	s3cxv=sigvbt(e/ab*1000.,tnuvs/Aplasamu(2)*1000.,'CX')
	SENS(indexi)=SENS(indexi)+DX*BEAM*DNUvs*S3CXV/S2CXV
	1	*(1.-weight)*24.*sqrt(pi)
	SENS(indexo)=SENS(indexo)+DX*BEAM*DNUvs*S3CXV/S2CXV
	1	*weight*24.*sqrt(pi)
122	CONTINUE
C	I NOW INCLUDE THE CHARGE EXCHANGE BETWEEN THE BEAM ION AND
C	THE PRIMARY BEAM NEUTRALS
	COSPA=0.E00
	SINPA=1.E00
	COSPB=FppRTCEN/TANS(ILIN)
	IF(ABS(COSPB).GT.1.E00) COSPB=SIGN(1.E00,COSPB)
	SINPB=SQRT(1.-COSPB*COSPB)
	PHIPB=PI
	IF(NiCur*FppRTCEN.LT.0) GO TO 320
	IF(PHIL.GT.PHIBT) PHIPB=0.E00
	GO TO 330
320	IF(PHIL.LT.PHIBT) PHIPB=0.E00
330	COSX=COSPA*COSPB+SINPA*SINPB*COS(PHIPA-PHIPB)
	DO 310 IENG=1,NKENG1
	W0BEAM=EFULL/IENG
	V0B=SQRT(W0BEAM*CON1/BeamAMU)
	DNPB=DN0B(IRN,IYN,IPHI,IENG)
	VREL2=V*V+V0B*V0B-2.*V*V0B*COSX
	EREL=VREL2*BeamAMU/CON1
	s4cxv=psgvcx(erel/beamamu*1000.)
	SENS(indexi)=SENS(indexi)+DX*BEAM*DNpb*S4CXV/S2CXV
	1	*(1.-weight)*24.*sqrt(pi)
	SENS(indexo)=SENS(indexo)+DX*BEAM*DNpb*S4CXV/S2CXV
	1	*weight*24.*sqrt(pi)
310	CONTINUE
  124 CONTINUE
  700 CONTINUE
  711 CONTINUE
  100 CONTINUE  !end of the NENCX energies and NLINE sightlines loop
      IF(LPloD.NE.1) GOTO 555
c	CALL ISBATCH(IRET)
c	IF (IRET .EQ. 1) GO TO 555
 
C
C     EXAMINE SENS GRAPHICALLY
C
899	WRITE(6,820)
820	FORMAT(' INPUT SIGHTLINE # AND ENERGY # TO EXAMINE SENS,
     >	(0,0) TO EXIT')
	READ(5,*) NANG,NENG
	IF(NANG.EQ.0.AND.NENG.EQ.0) GOTO 555
890	WRITE(6,860)
  860 FORMAT(' WHAT SHELL #?  0 TO EXIT')
	READ(5,*) ishell
	CALL NEWPAG
	IF(ishell.EQ.0) GOTO 850
	DO 870 IJK=1,maxxsi/2
	YARRAY(IJK)=SENS(IND(NENG,NANG,ishell,IJK))
	j=IJK
	if(sign(1.0,Tans(nang)) .gt. 0.0) j=j+maxxsi/2
	xarray(ijk)=xsi(j)
870	continue
	call initt(240)
	CALL BINITT
	call fpgrlab
	CALL NPTS(maxxsi/2)
	CALL DLIMX(-1.0,1.0)
	CALL CHECK(XARRAY,YARRAY)
	CALL DSPLAY(XARRAY,YARRAY)
	WRITE(TEXT1,910)
910	FORMAT('DETECTOR SENSITIVITY$')
	CALL AGTCGT(750,TEXT1)
	WRITE(TEXT1,920) ISHELL, NENG, NANG
920	FORMAT('SHELL #',I2,3X,'ENERGY #',I2,3X,'SIGHTLINE #',I2,'$')
	CALL AGTCGT(0,TEXT1)
	WRITE(TEXT1,930)
930	FORMAT('VPAR/V$')
	CALL AGTCHS(TEXT1)
	CALL UPAUSE(0)
	GOTO 890
850	continue
	GOTO 899
555	RETURN
	END
