	subroutine difmod
!
! calculate a confinement time to model radial diffusion.
! a cold particle source to repace the particle lost to radial diffusion
! is calculated in MOMS and added to f1 in IMP
!
C     COMMON BLOCKS
C
	include 'FPPCOM'
 
	if(lradls) write(lsysout,*)
	1 'WARNING!: DIFMOD has not been upgraded to general geometry'
c equivalent circles are being used.
 
	zrOverR = rminpro(nshe)/Rmajpro(nshe)
 
	do 100 i=0,maxen
	do 100 j=1,maxxsi
	ratedl(i,j)=0.0 !initialize to no losses
 
	if(i .eq. 0)go to 100
	if(.not. lradls)goto 100
 
!evaluate ripple plateau diffusion D(E,xsi)
!(Goldston and Towner, J. Plasma Physics, v. 26, p. 283 (1981).)
	costh=((1.-xsi(j)**2)*(1.+zrOverR)-1.)/zrOverR
	if(costh .lt. -1.0) goto 100 !passing particles don't see ripple
	sinth=sqrt(1-costh**2)
	rloc=rminpro(nshe)
	R=Rmajpro(nshe)+rloc*costh
	y=rloc*sinth
 
!exponential model for ripple:
        zRipDlta=Ripdmin*exp(sqrt(y**2+(R-RipRmin)**2)/RipexpL)
 
! q(r) model assumes a current profile which is parabolic to the x power,
! where x = (qlim/q0)-1:
	denom=1.0-(1.0-(rloc/rmin)**2)**qrat
	qloc=qlim*(rloc/rmin)**2/denom
 
        zRipAlfa= zrOverR*sinth/(NumTF * qloc * zRipDlta)
!Steve Scott's approximation for the correction to the 1/alpha**2 dependence
!of the (radial step size)**2 taken from fig. 13 of Goldston and Towner:
        if(zRipAlfa .gt. 2.5) zRipCorr= 1.0
        if(zRipAlfa .gt. .92 .and. zRipAlfa .le. 2.5)
	1                      zRipCorr=1.4961/zRipAlfa**(.44)
        if(zRipAlfa .le. .92) zRipCorr= 1.902*zRipAlfa**2.68
 
        Dripple = XRipMult*3.613e9*enerev(i)**1.5 *Ab**.5 *sinth
	1   /(rZbeam**2 *Btor**2 *NumTF *rloc)
	1   *zRipCorr/zRipAlfa**2/btime(j)
 
! converting a D into a confinement time model is a non-trivial matter!
! LTauMod = 1 gives Tau = (a-r)(a+r)/(4 D):
	data LTauMod /1/
	if(lradls .and. LTauMod .eq. 1)
	1	    ratedl(i,j)=4*Dripple/(rmin-rloc)/(rmin+rloc)
 
! LTauMod = 2 uses a more complicated formula from Rob:
	if(lradls .and. LTauMod .eq. 2) then
	    Taue = tause(nshe)/3
	1	*alog( (Enerev(i)**1.5 + Ecrit(nshe)**1.5)
	1	/((Enerev(i)/exp(1.0))**1.5 + Ecrit(nshe)**1.5))
	    expfun=exp(-(rmin-rloc)/sqrt(Dripple * Taue))
	    ratedl(i,j)=expfun/Taue/(1-expfun)
	endif
 
100	continue
 
	return
	end
