	subroutine fpminf(f,zguess,zstep,zmin,zmax,zacc,imaxfn,zatmin)
c
c finds the minimum of f(z)
c
c Written by Greg Hammett, 26-Oct-1989.  This routine uses a simple
c bisection search algorithm, and could be speeded up by using a
c quadratic interpolation near the minimum.  This was written to replace
c IMSL routine UVMIF which forces zmin and zmax to be symmetric about
c zguess.
c
c f	function to be minimized (must be declared external in the
c	calling program).
c zguess  initial guess at z
c zstep	  initial step size in z for the search
c zmin	  minimum allowable value for z
c zmax	  maximum allowable value for z
c zacc	  desired accuracy in zatmin
c imaxfn  maximum number of evaluations of f
c zatmin  value of z at the minimum of f(z).
c
c Assumes that zguess+-zstep is still within the zmin and zmax bounds
c
	external f
c
	real f,zguess,zstep,zmin,zmax,zacc,zatmin
	integer imaxfn
 
	ncalls=2
	dz=zstep
 
	z0=zguess
	f0=f(z0)
	z1=z0+dz
c	z1=max(min(z1,zmax),zmin)
	f1=f(z1)
 
	if(f1 .ge. f0) then
	    dz=-dz
	else
	    f0=f1
	    z0=z1
	endif
 
20	z1=z0+dz
	z1=max(min(z1,zmax),zmin)
	f1=f(z1)
 
	if(f1 .ge. f0) then
	    if(abs(dz) .lt. zacc/4) goto 900
	    dz=-dz/4
	else
	    f0=f1
	    z0=z1
c if we run into the boundary, then turn around with a smaller step size:
	    if (z0 .eq. zmin) then
	       dz=abs(dz)/2
	    else if (z0 .eq. zmax) then
	       dz=-abs(dz)/2
	    else
c if we haven't run into the boundary, and we are still going in the
c same direction without finding the minimum yet, then increase the
c step size to find the minimum faster:
	    dz=dz*4
	    endif
	endif
	ncalls=ncalls+1
	if(ncalls .gt. imaxfn) then
	    type *,'  FPMINF exceeded imaxfn function calls'
	    goto 900
	endif
	goto 20
 
900	zatmin=z0
	if(zatmin .lt. zmin) then
	    type *,' FPMINF warning: zatmin < zmin!'
	endif
	if(zatmin .gt. zmax) then
	    type *,' FPMINF warning: zatmin > zmax!'
	endif
	return
	end
