      subroutine FPSPRUCE
c
C Interface routine between Greg Hammett's FPP and Dave Smithe's
C general geometry, finite difference, ICRF package SPRUCE.  FPSPRUCE
C calls Dave's routine ZDILE.
c
c Written by Greg Hammett, 15-Mar-1989.
c This routine evolved from FPSHOOT2.FOR, the interface to Dave's
c previous shooting code.
c
c There are a few places that I have had
c to change things or have questions.  They are marked with a "?".
c
c
c       include 'TRCOM/NOLIST'
	include 'FPPCOM'
	include 'SPRINC'
	include 'SPRUCOM'
 
	external ctrans2    ! maps (zichi,zjpsi) --> (R,Y)
			    ! and finds the error relative to the
			    ! desired (R,Y)
	real zxlb(2)
	real zxub(2)
 
c other local variables:
	character label*80
	logical vvinit
	data vvinit/.true./   ! =.t. if VV grid needs initialization
	save vvinit
 
c these moments are in meters (SPRUCE's units):
	real zVVRmoms(0:mxVVmoms) ! VV R(th) = Sum VVRmoms(i) cos(i th)
	real zVVZmoms(0:mxVVmoms) ! VV Z(th) = Sum VVZmoms(i) sin(i th)
 
 
!	real zthicrf(nticrf+1) ! poloidal angle grid
	real zPrfpro(maxshe) ! total RF power profile (W/m**3)
!	real zr2(nr+1,nticrf+1)	    ! (R,Z) grid for 2D plots (m)
!	real zz2(nr+1,nticrf+1)	
	character*6 iname(mxions+2) !names of ions+elec+MdCv
 
c quantities which vary more smoothly than Er and Eth, and
c can therefore be interpolated more accurately:
	real zEmag(nchi+1,npsi)    ! cabs(Eplus) on SPRUCE's grid
	complex zEpol(nchi+1,npsi) ! Eminus/Eplus on SPRUCE's grid
	real zEmagint		!interpolated zEmag
	complex zEpolint	!interpolated zEpol
 
	real zijguess(2)    ! guess of FPP-->SPRUCE map
	real zij(2)	    ! actual FPP-->SPRUCE map
 
	parameter (imaxcont=8) ! number of contours should be even
	real zconval(imaxcont)	    !contour values for 2D plots
	real zwallR(nchi)  ! (R,Z) of wall for 2D plots
	real zwallZ(nchi)
        common /plotbr/ Zemag,zepol   ! to pass these to sprout.for  jphe
 
	complex zEplus,zEminus,znperp
 
	zpi=abs(acos(-1.0))
 
c
c set up input arguments to ZDILE:
c
	nionsrf=nions
	power=1.0 ! 1 watt default
	if (Prftot .gt. 0.) power=Prftot
 
c rminorr and rmajorr control the numerical grid, set the origin
c of the grid to coincide with the plasma center.
c rminorr will be set by spruvv.
 
	call momry(0.0,0.0,0,rmajorr,zy,zjac)
	rmajorr=rmajorr/100.
 
c convert VV moments from centimeters to meters:
	if(VVRmoms(0) .le. 0.0) call aborter(lsysout,
	1   'You need to set the VV parameters in the input file?')
	do 40 i= 0,mxVVmoms
	zVVRmoms(i)=VVRmoms(i)/100.
40	zVVZmoms(i)=VVZmoms(i)/100.
 
c SPRUCE's VV grid only needs initialization once, unless it is
c being called from TRANSP when it needs initialization everytime
c because the plasma may be changing position:
	if(vvinit .or. Ltransp) then
c New Fourier representation of VV shape:
	call spruvv(zVVRmoms,zVVZmoms,mxVVmoms,vvRmax,vvRmin,vvZmax)
	draytol=rminorr*fppeps
	call setup
	vvinit=.false.
	endif
 
c Antenna description:
 
	freq=freqrf
	dkz=rkpar*100.
	nnstep=1	! number of kz's to do (only 1 allowed at the moment)
	awidth=antwidth/100.
	antRmj=antRcen/100.
	rmajkz=antRmj
	antzed=antHH/100.
	zphas1=zpi
	zphas2=zpi
	antspc=zpi/dkz      !jphe
	boxspc=2*antspc     !jphe
	pfrac1=1.0
 
c set up radial bins:
	nbin=nshell
	do 50 i=1,nbin
	rbins(i)=xradzb(i)*rmin
50	continue
 
	rmjbins(1)=dVolfp(1)/(zpi*rbins(1)**2)/2/zpi
	do 55 i=2,nbin
55	rmjbins(i)=dVolfp(i)/(zpi*(rbins(i)**2-rbins(i-1)**2))/2/zpi
 
c an average drbin for the OUTPUT routine:
	drbin=rmin/nbin
 
c Label the different channels of ICRF damping:
	do 65 i=1,nionsrf
	name(i)=namions(i)	! for Dave's plotting routine
65	iname(i)=namions(i)	! for my own plotting
	name(nionsrf+1)='Elec'
	iname(nionsrf+1)='Elec'
	iname(nionsrf+2)='MdCv'
	nionso=nionsrf
 
c try to adjust the fast ion density to give the same
c J.E power calculated by SPRUCE as was calculated
c quasilinearly by FPP:
c prevent the correction factor from exceeding *1.0 or *0.2.
c If rkpcpro>1, this is probably due either to numerical errors
c in calculating the damping in SPRUCE, or due to an increase in
c the fast ion density at resonance as the fast ions gain perpendicular
c energy and become trapped.  To include this properly in SPRUCE,
c we shouldn't just enhance the fast ion density everywhere, but
c we should calculate the minority density as a function of
c poloidal angle (as well as minor radius).
 
	data zqlmin /0.2/
	data zqlmax /1.0/
	do 70 i=1,nshell
	qldamp(i)=qldamp(i)*rkpcpro(i)
	qldamp(i)=min(zqlmax,max(zqlmin,qldamp(i)))
70	continue
 
c
c++++++++++++++++++++ main solution sequence +++++++++++++++++++++
	call zdile(power,nionsrf,rbins,rmjbins,nbin,aprsp,apion,
	1   apelc,apmc,
	1          resist,react, hzplt,Prf2d, akreal,akimag,akrmaj,
	1          akprav,transm,drkmax)
 
	if(nrloop .ne. 0 .and. nrloop .ne. mrloop-1 .and.
	1   nrloop .ne. mrloop) then
	    continue
	else
c  jphe  23/8/94 added sprout(-8) to get all possible outputs
	call sprout(-8)
	call sprout(-7)
	endif
 
c Total power to the fast ion species calculated from J.E by
c SPRUCE:
	Prfje0=Prftot*aPion(nfasti)/resist
 
c ???:
c define theta grids:
!!:	dth=2.*zpi/float(nt)
!!:	do 13 j=1,nt+1
!!:13	zthicrf(j)=(j-1)*dth
 
 
!!:c define (R,Z) grid for 2d plots
!!:	do 15 i=1,nr+1
!!:	do 15 j=1,nt+1
!!:	zR2(i,j)=Rmajvv+zricrf(i)*cos(zthicrf(j))
!!:15	zZ2(i,j)=zricrf(i)*sin(zthicrf(j))
!!:
c
C convert Er and Eth to cabs(Eplus) and Eplus/Eminus, which have a
C smoother radial variation and can be more accurately interpolated.
c
	do 612 j=1,npsi
	do 610 i=1,nchi
	zEplus= (Epsi(i,j)+(0.,1.)*Echi(i,j))/2.
	zEminus=(Epsi(i,j)-(0.,1.)*Echi(i,j))/2.
	zEmag(i,j)=cabs(zEplus)
	if(zEplus .ne. (0.,0.)) then
	    zEpol(i,j)=zEminus/zEplus
	else
	    zEpol(i,j)=1.0
	endif
610	continue
	zEmag(nchi+1,j)=zEmag(1,j)
	zEpol(nchi+1,j)=zEpol(1,j)
612	continue
 
c Interpolate from SPRUCE's grid to FPP's grid, and calculate all of
c the quantities which FPP needs.
c ( E(r=0) not defined )
 
	zxlb(1)=-200*zpi
	zxub(1)=200*zpi
	zxlb(2)=1
	zxub(2)=npsi
 
	do 650 i=1,nshell
	do 640 j=1,nticrf
 
c convert from FPP's flux surface coordinates:
	zxi=rminpro(i)/rmin
	ztheta=(j-1.0)*2*zpi/nticrf
	call momry(zxi,ztheta,0,findR,findY,zJac)
 
c to SPRUCE's Vaccuum Vessel coordinates:
c note that SPRUCE's arrays (such as Rmajich(nchi,npsi))
c have their indices reversed from the FPP convention.
 
	zijguess(2)=1+zxi*(npsi-1)
	if(zijguess(2) .gt. npsi) zijguess(2)=npsi
	zijguess(1)=1+ztheta/2/zpi*nchi
 
	zxtol=.01
	zerr=.01
	call fpminf0(ctrans2,zijguess,2,zxlb,zxub,zxtol,zerr)
	zij(1)=zijguess(1)
	zij(2)=zijguess(2)
 
	znchi=nchi
	zi=mod(zij(1),znchi)
	if(zi .lt. 1.0) zi=zi+znchi
	if(zi .lt. 1.0) zi=zi+znchi
	if(zi .lt. 1.0) zi=1.0
	if(zi .gt. nchi+1) zi=nchi+1
 
	zj=zij(2)
	if(zj .lt. 1.0) zj=1.0
	if(zj .gt. npsi)   zj=npsi
 
c defaults:
	zEmagint=0.0
	zEpolint=cmplx(1.0,0.0)
	znperp=0.0
 
c calculate interpolation weights:
c first the angle interpolation:
	i1=zi
	if(i1 .ge. nchi+1) i1=nchi
	i2=i1+1
	zw2=zi-i1
	zw1=1.0-zw2
 
c then the radial interpolation:
	j1=zj
	if(j1 .ge. npsi) j1=npsi-1
	j2=j1+1
	zwth2=zj-j1
	zwth1=1.0-zwth2
 
	zEmagint=zw1*(zwth1*zEmag(i1,j1)+zwth2*zEmag(i1,j2))
	1  +zw2*(zwth1*zEmag(i2,j1)+zwth2*zEmag(i2,j2))
	zEpolint=zw1*(zwth1*zEpol(i1,j1)+zwth2*zEpol(i1,j2))
	1   +zw2*(zwth1*zEpol(i2,j1)+zwth2*zEpol(i2,j2))
 
c Nperp**2 = 1./acoef:
	if(i2 .eq. nchi+1)i2=1
	znperp=zw1*(zwth1/acoef(i1,j1)**.5+zwth2/acoef(i1,j2)**.5)
	1   +zw2*(zwth1/acoef(i2,j1)**.5+zwth2/acoef(i2,j2)**.5)
 
c normalize Efields to current in antenna, and convert from SPRUCE's
c kz integral to FPP's n_toroidal summation.  Both calculations are
c done in a finite torus, i.e., SPRUCE knows that kz=n/R:
 
c to derive the formula below, we need to understand how to
c express continuous spectra in terms of discrete spectra.
c
c E(z) =   sum_over_n E_n exp(i (n/r) z)
c      =   sum_over_n dk R E_n exp(i k z)
c      =   integral dk E_k exp(i k z)
c
c so E_n = dk E_k.  If dk=1/R, then E_n=E_k/R.  However, SPRUCE
c allows dk=alpha/R, where alpha is any arbitrary number, not
c neccessarily alpha=1.  Then we need to go through the power
c formulas to make sure that the power in FPP's discrete spectrum
c is the same as the power in SPRUCE's continuous spectrum.
c
C This is the reason that zrenorm**2 scales like
c dkz/R, rather than 1/R^2.  (zrenorm is the field renormalization
C factor.  It is easier to think in terms of the power renormalization
C factor zrenorm**2.)  Also, dkz is SPRUCE's kz step size at the
C antenna, so it has to be scaled to other locations by 1/R.
c
c the fields are for unit current in each fourier mode of the antenna,
C aIofkz(i) gives the actual antenna current in each fourier mode.  The
C factor of 2 is because the spectrum was assumed to be symmetric and
C resist is the power for +- kz combined:
C
	zrenorm=sqrt(abs(2.*Prftot/resist*dkz/Rmajkz))
     1    *(Rmajkz*100./FindR)*cabs(aIofkz(1))
 
c Until 4/3/90, we used to try to renormalize the fields given to
c FPP so that FPP would calculate approximately the same power as
c SPRUCE did.  This renormalization was done by adding the line:
c
c	zrenorm=zrenorm/sqrt(rkpcorr)
c
c The rkpcorr factor was calculated to make the total volume
c integrated powers from the last SPRUCE/FPP iteration agree,
c so it wouldn't give exactly the same powers on this iteration,
c and the profiles given by SPRUCE and FPP would be different.
c
c We now do the renormalization in SYMRF, so that FPP and
c SPRUCE give exactly the same power profiles (at least for
c FPP's initial time step before f can change) without iteration.
c The correction factor rkpcpro is plotted vs. minor radius
c in subroutine PLOTPB.
 
c convert to statvolt/cm's:
	Eplus(i,j) = zEmagint*zrenorm/3.e4 ! jphe 7-nov-1994
 
	crfpol(i,j)=zEpolint
 
c enable debugging of kperp effects with an arbitrary multiplier:
	data zkmult /1.0/
c	skperp(i,j)=zkmult*abs(real(znperp))*sqrt(ako2)/100.
	skperp(i,j)=zkmult*cabs(znperp)*sqrt(ako2)/100.
640	continue
 
c f(theta=2pi) = f(theta=0)
        Eplus(i,nticrf+1)=Eplus(i,1)
        crfpol(i,nticrf+1)=crfpol(i,1)
        skperp(i,nticrf+1)=skperp(i,1)
 650     continue
c    jphe ! 16-nov-1994
c   set all "kick" determining quantities as for a brand new particle
c   fields have changed; old info in principle no more valid
c
c GWH: comment out monte carlo/transp stuff:
cc 	  IF  (NLMCR2) THEN
cc            DO JJJ=1,NSBEAM
cc            DO III=1,NUSE(JJJ)
cc            IND=NBNDEX(III,JJJ)
cc	    DELRSY(IND,JJJ)=EPSINV
cc	    RSCONY(IND,JJJ)=0.
cc	    DRSCONY(IND,JJJ)=0.
cc	    LRSOKY(IND,JJJ)=.FALSE.
cc     	    LGOSRFY(IND,JJJ)=.FALSE.
cc            RGOSRFY(IND,JJJ,1)=1.
cc	    RGOSRFY(IND,JJJ,2)=EPSINV
cc            ENDDO
cc            ENDDO
cc	ENDIF
c
c  jphe 4-nov-1994  renorm zemag to get it in volts/cm and call sprout
c                   to plot both zemag and crfpol, but on the spruce grid
c                   (in order not to have to write yet another plotroutine..)
c                   according to michel, output from spruce is in V/m;
C                   to convert to v/cm : divide by 100:
c
 	do 660 j=1,npsi
        do 660 i=1,nchi+1
 660    zemag(i,j)=zemag(i,j)*zrenorm/1.e2
c
c  end of additions  jphe
c
c Interpolate direct ICH Power profiles from SPRUCE grid to FPP grid:
 
	do 670 i=1,nshell
c prffast seems redundant with pichspec.  It is only used in
c fpprf.  So if we replace that usage with pichspec, then we
c can eliminate prffast...
	prffast(i)=aPrsp(i,nfasti)
	do 670 isp=1,nions+2
670	Pichspec(i,isp)=aPrsp(i,isp)
 
c Find total power profile:
	do 690 i=1,nshell
	zPrfpro(i)=0.0
	do 690 isp=1,nionsrf+2
690	zPrfpro(i)=zPrfpro(i)+aPrsp(i,isp)
 
c********************************************************************
c
c End of Calculations.  Make plots if desired.
c
c********************************************************************
 
c make plots only for the first and the last two iterations:
	if(nrloop .ne. 0 .and. nrloop .ne. mrloop-1 .and.
	1   nrloop .ne. mrloop) return
 
c       call sprout(-5)  ! jphe: plot eplus and cabs(crfpol)
c********************************************************************
c
c Plot power profiles P(r)
c
c********************************************************************
 
c switch to graphics screen & erase it.
	call initt(240)
	call binitt
	call fpgrlab
	call npts(nshell)
    	zYMIN=0.0
	zYMAX=0.0
	call mnmx(zPrfpro,zYMIN,zYMAX)
	do 1030 iion=1,nionsrf+2
1030	call mnmx(aPrsp(1,iion),zYmin,zYmax)
 
c set x & y axis limits
	call slimx(150,500)
	call slimy(125,700)
	call xneat(0) !use exact x-axis limits
	call dlimx(0.0,rmin)
	call dlimy(zYMIN,zYMAX)
c plot total Power profile
	call check(rminpro,zPrfpro)
	call dsplay(rminpro,zPrfpro)
c plot RF power to minority:
	ilintyp=7474
	call line(ilintyp)
	call check(rminpro,aPrsp(1,nfasti))
	call cplot(rminpro,aPrsp(1,nfasti))
 
c calculate power to other ions:
	do 1040 ir=1,nshell
	zPrfpro(ir)=0.0
	do 1040 iion=1,nionsrf
	if(iion .ne. nfasti) zPrfpro(ir)=zPrfpro(ir)+aPrsp(ir,iion)
1040	continue
 
c calculate sum of powers to other ions:
	ilintyp=5454
	call line(ilintyp)
	call check(rminpro,zPrfpro)
	call cplot(rminpro,zPrfpro)
 
c plot power to electrons
	ilintyp=3434
	call line(ilintyp)
	call check(rminpro,aPrsp(1,nionsrf+1))
	call cplot(rminpro,aPrsp(1,nionsrf+1))
 
c plot mode converted power
	ilintyp=1414
	call line(ilintyp)
	call check(rminpro,aPrsp(1,nionsrf+2))
	call cplot(rminpro,aPrsp(1,nionsrf+2))
 
	call agtchs('minor radius r (cm)')	!label the x axis
	call agtcvs('Prf (W/cm**3)')	!label the y axis
	call agtcgt(0,'Prf: total, Mnti, Othi, e, MdCv')
	write(label,1055)time
1055	format('Smithe Genl Geometry Fast Wave Calculation, t=',
	1 1pe10.3,' s$')
	call agtmhs(150,750,label,0)
 
c print out power split next to plot:
	write(label,1501)resist
1501     format('Rload = ',1pe10.3,' Ohm$')
	call agtmhs(550,675,label,0)
 
	write(label,1503) Prftot
1503     format('total Prf=',1pe10.3,' Watts$')
 	call agtmhs(550,625,label,0)
 
	iyloc=625
	do 1600 isp=1,nionsrf+2
	iyloc=iyloc-25
	if(isp .le. nions)then
	ztotal=aPion(isp)
	else if (isp .eq. nionsrf+1) then
	ztotal=Apelc
	else
	ztotal=aPmc
	endif
	ztotal=ztotal/resist
	write(label,1504) iname(isp),100.*ztotal
1504	format(a6,1x,1pe10.3, ' %$')
	call agtmhs(550,iyloc,label,0)
1600	continue
 
	call upause(0)  !ring the bell, pause for TTY input, no erase.
	call finitt(0,0)
 
!!:c********************************************************************
!!:c
!!:c Plot Nperp**2 vs. R.
!!:c
!!:c********************************************************************
!!:
!!:
!!:	call initt(240)
!!:	call binitt
!!:	call fpgrlab
!!:	call npts(2*npsi-1)
!!:	call xneat(0)
!!:	call dlimx(akRmaj(1),akRmaj(2*npsi-1))
!!:	zymin=0.0
!!:	zymax=0.0
!!:	call mnmx(akreal,zymin,zymax)
!!:	call mnmx(akimag,zymin,zymax)
!!:	call dlimy(zymin,zymax)
!!:	call slimx(150,725)
!!:	call slimy(125,700)
!!:	call check(akRmaj,akreal)
!!:	call dsplay(akRmaj,akreal)
!!:	call line(1414)
!!:	call check(akRmaj,akimag)
!!:	call cplot(akRmaj,akimag)
!!:	call agtchs('Major Radius (m)')
!!:	call agtcvs('N_perp**2')
!!:	call agtcgt(0,'Real and Imaginary N_perp**2 vs R @Z=0')
!!:	call upause(0)
!!:
!!:c********************************************************************
!!:c
!!:c Plot Real Hz(R,Y)
!!:c
!!:c********************************************************************
!!:
!!:c     wallr/z:  vessel wall curve for plots
!!:	do 753 j=1,ntp1
!!:	zwallr(j)=Rmajvv+awl*cos(zthicrf(j))
!!:753	zwallz(j)=awl*sin(zthicrf(j))
!!:
!!:	call initt(240)
!!:	call binitt
!!:	call fpgrlab
!!:c
!!:c CONTUR routine doesn't work unless you have already set up the
!!:c plotting area and the virtual to screen map.  Do this by first
!!:c plotting the vacuum vessel wall (we could have instead called
!!:c the AGRAPH routine to prepare for CONTUR, but AGRAPH doesn't allow
!!:c to set the x and y limits exacatly, not with xneat/yneat, not with
!!:c agixnt/agiynt).
!!:c
!!:	call npts(ntp1)
!!:	call xneat(0)
!!:	call yneat(0)
!!:	call slimx(150,725)
!!:	call slimy(125,700)
!!:	call dlimx(Rmajvv-awl,Rmajvv+awl)
!!:	call dlimy(-awl,awl)
!!:	call xfrm(2)
!!:	call yfrm(2)
!!:	call check(zwallr,zwallz)
!!:	call dsplay(zwallr,zwallz)
!!:	call frame
!!:	call agtchs('R (m)')
!!:	call agtcvs('Y (m)')
!!:	call agtcgt(750,'Hz Contours')
!!:
!!:	call npts(nrp1*ntp1)
!!:
!!:	k1=imaxcont
!!:	k2=imaxcont/2
!!:c calculate the contour spacing based on the average abs(hz):
!!:	zhmin=0.0
!!:	zhmax=0.0
!!:	call mnmx(hzplt,zhmin,zhmax)
!!:	zhmax=abs(zhmax)
!!:	if(abs(zhmin) .gt. zhmax)zhmax=abs(zhmin)
!!:
!!:	zdcont=2*zhmax/(imaxcont+1)
!!:	do 767 i=1,imaxcont
!!:767	zconval(i)=-zhmax+i*zdcont
!!:
!!:	call contur(k1,zconval,k2,hzplt,nrp1,zR2,1,nrp1,1,zZ2,1,ntp1,1)
!!:
!!:	call upause(0)
!!:
!!:c********************************************************************
!!:c
!!:c Plot Real Prf2d(R,Y)
!!:c
!!:c********************************************************************
!!:
!!:	call initt(240)
!!:	call binitt
!!:	call fpgrlab
!!:c
!!:c CONTUR routine doesn't work unless you have already set up the
!!:c plotting area and the virtual to screen map.  Do this by first
!!:c plotting the vacuum vessel wall (we could have instead called
!!:c the AGRAPH routine to prepare for CONTUR, but AGRAPH doesn't allow
!!:c to set the x and y limits exacatly, not with xneat/yneat, not with
!!:c agixnt/agiynt).
!!:c
!!:	call npts(ntp1)
!!:	call xneat(0)
!!:	call yneat(0)
!!:	call slimx(150,725)
!!:	call slimy(125,700)
!!:	call dlimx(Rmajvv-awl,Rmajvv+awl)
!!:	call dlimy(-awl,awl)
!!:	call xfrm(2)
!!:	call yfrm(2)
!!:	call check(zwallr,zwallz)
!!:	call dsplay(zwallr,zwallz)
!!:	call frame
!!:	call agtchs('R (m)')
!!:	call agtcvs('Y (m)')
!!:	write(label,768) imaxcont+1
!!:768	format('Prf(R,Y) 2D Contours (1/',i2,') spacing$')
!!:	call agtcgt(750,label)
!!:
!!:	call npts(nrp1*ntp1)
!!:
!!:	k1=imaxcont
!!:	k2=1
!!:c calculate the contour spacing:
!!:	zhmin=0.0
!!:	zhmax=0.0
!!:	call mnmx(Prf2d,zhmin,zhmax)
!!:
!!:	do 769 i=1,imaxcont
!!:769	zconval(i)=zhmax*i/(imaxcont+1.0)
!!:
!!:	call contur(k1,zconval,k2,Prf2d,nrp1,zR2,1,nrp1,1,zZ2,1,ntp1,1)
!!:	call upause(0)
!!:
!!:c********************************************************************
!!:c
!!:c Plot Vacuum Vessel and Plasma Boundary
!!:c
!!:c********************************************************************
!!:
!!:c     wallr/z:  vessel wall curve for plots
!!:   	do 780 j=1,ntp1
!!:	zwallr(j)=Rmajvv+awl*cos(zthicrf(j))
!!:780	zwallz(j)=awl*sin(zthicrf(j))
!!:
!!:	call initt(240)
!!:	call binitt
!!:	call fpgrlab
!!:	call npts(ntp1)
!!:	call xneat(0)
!!:	call yneat(0)
!!:	call slimx(150,725)
!!:	call slimy(125,700)
!!:	call dlimx(Rmajvv-awl,Rmajvv+awl)
!!:	call dlimy(-awl,awl)
!!:	call xfrm(2)
!!:	call yfrm(2)
!!:	call check(zwallr,zwallz)
!!:	call dsplay(zwallr,zwallz)
!!:
!!:	do 790 ixi=0,4
!!:	zxi=ixi/4.0
!!:   	do 785 j=1,ntp1
!!:	zth=(j-1.0)*2*zpi/nt
!!:	call momry(zxi,zth,0,zzR,zzY,zjac)
!!:	zwallr(j)=zzR/100.
!!:785	zwallz(j)=zzY/100.
!!:
!!:	call line(1414)
!!:	call check(zwallr,zwallz)
!!:790	call cplot(zwallr,zwallz)
!!:
!!:	call frame
!!:	call agtchs('R (m)')
!!:	call agtcvs('Y (m)')
!!:	call agtcgt(750,'Plasma surfaces')
!!:
!!:	call upause(0)
 
	if(nrloop .ne. 0 .and. nrloop .ne. mrloop-1 .and.
	1   nrloop .ne. mrloop) then
	    continue
	else
	call sprout(1)
	call sprout(-1)	    !plot sprout's grid
	endif
 
c check grid resolution of the waves:
c drkmax should be less than 1.0 for good resolution of
c the wave, and the wave becomes numerically evanescent if
c drkmax is greater than 2.0.  If it is greater than 1.7, cause
c a fatal error and suggest that the code should be recompiled.
 
c drkmax was measured along the midplane, so include an estimate
c of dr*kmax above the midplane including the VV elongation:
	drkmax=drkmax*vvZmax/(0.5*(vvRmax-vvRmin))
	write(lsysout,*) ' ICRF wave resolution, max k*dr = ',drkmax
	if(drkmax .gt. 1.7) call aborter(lsysout,
	1	'drkmax too big, need to recompile with npsi larger')
 
	return
	end
