 
        SUBROUTINE FSTART
 
 
C	Updated:
C  09/15/93 TBT  Added test for DragI.Eq.0
 
 
c--initialize f to be a maxwellian.
c  initialize fmxwel to be a source to keep n constant.
c  initialize fcx to be a maxwellian at temperature Tnus, which
c	is used as the CX source.
c  initialize fcold to be a source of cold particles to replace
c 	particles lost due to finite confinement time.
c  initialize feqTiTe to an f in equilibrium with ions and electrons
c  	simultaneously, and so has slope Ti at low energies and Te at high
c       energies.  This is used as the background contribution to the CX
c       flux in subroutine SPECT
c  finds rjac0 and aven0 which are related to the assumed form of
c       f near the origin.
c  calculates qjace, qjaca, and enint which are expressions which
c       frequently appear in velocity space integrals and are
c       calculated in advance to try to speed up subroutine moms.
 
	include 'FPPCOM'
 
c--des = electron density in this shell (cm**-3)
c Tis = ion temperature on this radial shell
c--the usual constant in front of a Maxwellian is:
        fconst=1./(2*3.14159*Tis)**1.5
c--to convert to Rob's units, need to multiply by:
        fconst=fconst * 2**.5 *2*3.14159/25
 
!initialize f and the source fmxwel to be maxwellians at the
! temperature Tis.
! For the time being, f is normalized to a density of 1 particle
! per cc.  The proper normalization is determined later.
 
!As noted in PREADI, the discrete collision operator has an
! equilibrium solution which is not quite the usual Maxwellian
! but is of order dE/E times hotter.  If f is initialized to the
! colder temperature then we get the annoying result that Pi < 0
! as f heats up to become a solution of the discrete collision
! operator.
! To avoid this, we can initialize f and the source to be an
! equilibrium solution of the discrete operator by setting the
! switch DISCRE to 0.0:
	data discre/0.0/
        do 50 i=0,maxen
	if(discre .ne. 0.0)then
        fmxwel(i)=fconst*exp(-enerev(i)/Tis)
	else
	if(i .eq. 0)fmxwel(i)=fconst
 
!when the energy dependence of these coefficients through log(lambda)
! can be ignored, you get tidum=tis.  However, using the more exact
! expression for tidum gives an initial f which is exactly in equilibrium
! with the ions, giving the pleasing result Pi=0.
 
	  if(i .ne. 0)then
	    If (DragI(I) .Ne. 0.0) Then
		tidum=diffei(i-1)/dragi(i)
	    Else
	        tidum=Tis
	    End If   ! DragI     tbt
 
 
	    fmxwel(i)=fmxwel(i-1)/(1+(enerev(i)-enerev(i-1))/tidum)
	  endif
 	endif
 
	if(i .eq. 0)feqTiTe(i)=fconst
	if(i .ne. 0)then
	    tidum=diffe(i-1)/drag(i)
	    feqTiTe(i)=feqTiTe(i-1)/(1+(enerev(i)-enerev(i-1))/tidum)
	endif
 
50	continue
 
! July 15, 1984, gwh:  FFPRF has been changed so that it now
! assumes that enerev(0) is not neccessarily zero, but that
! an analytic expression for f(E) for 0<E<enerev(0) is known.
! (This is usually possible because pitch angle scattering is
! rapid enough at low energies to make f independent of pitch
! angle, thus simplifying the equations to allow analytic solution.)
!  The density of particles near the origin for a general f
! is proportional to:
! Rjac0 = integral from 0 to E12 of dE/E12 E**.5 f(E)/f(E0)
!       = integral from 0 to E0  of dE/E12 E**.5 f(E)/f(E0)
!       + integral from E0 to E12 of dE/E12 of sqrt(E)
!
! where E12 = (E0 + E1)/2
!  We will assume that f is maxwellian for E < enerev(0), and Rjac0
!  works out to be:
        x0=enerev(0)/tis
        sqrtx0=sqrt(x0)
        dum=exp(x0)*sqrt(3.14159)/2*erf(sqrtx0) -sqrtx0
        rjac0=tis**1.5 *dum +2./3.*(delten(0)**1.5 -enerev(0)**1.5)
        rjac0=rjac0/delten(0)
!observe how rjac0 is used in the density calculation below.
 
!in addition to the density of particles in the analytic solution,
! we need to know their average energy:
!         integral from 0 to E12 of dE E**1.5 f(E)
! aven0 = ------------------------------------------
!         integral from 0 to E12 of dE E**.5 f(E)
!
!which for a maxwellian works out to be:
	aven0=-tis*enerev(0)**1.5 +1.5*tis**2.5*dum
	1	+.4*(delten(0)**2.5-enerev(0)**2.5)
	aven0=aven0/(tis**1.5*dum
	1	+2./3.*(delten(0)**1.5-enerev(0)**1.5))
 
c--now, we think fmxwel is normalized to 1 particle per cc.
c       Let's find out how much is really there and adjust it
c       accordingly:
c--calculate flux surface averaged particle density.
 
        sum=0.0
        sumeq=0.0
c--loop over all pitch angles:
        do 200 j=1,maxxsi
        sume=0.0
        sumeeq=0.0
c--sum over all energies:
        do 170 i=1,maxen
	sume=sume+delten(i)*enerev(i)**.5*fmxwel(i)*loss(i,j)
170	sumeeq=sumeeq+delten(i)*enerev(i)**.5*feqTiTe(i)*loss(i,j)
        sume=sume+delten(0)*rjac0*fmxwel(0)*loss(0,j)
        sumeeq=sumeeq+delten(0)*rjac0*feqTiTe(0)*loss(0,j)
        sum=sum+25.*dVolnorm*delxsi(j)*abs(xsi(j))*btime(j)*sume
        sumeq=sumeq+25.*dVolnorm*delxsi(j)*abs(xsi(j))*btime(j)*sumeeq
200     continue
c--sum is now the approximate particle density.
c--sorfra is the correction factor to give us exactly 1 per cc:
        sorfra=1.0/sum
 
c--adjust feqTiTe to be the thermal hydrogenic density, TIMES the fudge
c  factor bkgmax:
	zdenth=drions(nshe,nthermi)
	if(bkgmax .gt. 0.0 .and. nthermi .eq. nfasti) then
c  dmc -- addl output line on failure (8 May 1993)
	  write(lsysout,
     >      '(/''  BKGMAX='',1pe12.5,'' nthermi,nfasti='',2i8)')
     >      bkgmax,nthermi,nfasti
	  call aborter(lsysout,' BKGMAX inconsistency in FSTART!')
	endif
 
	do 275 i=0,maxen
275	feqTiTe(i)=feqTiTe(i)*bkgmax*zdenth/sumeq
 
c ZDENBT = density of the beam plus thermal component:
        if(nfasti .ne. nthermi) then
            zdenbt=drions(nshe,nfasti)+drions(nshe,nthermi)
        else
            zdenbt=drions(nshe,nfasti)
        endif
	if(llwbnd .eq. 0 .or. lfptherm .eq. 1) zdenbt=0.0
 
c--adjust fmxwel to exactly 1 per cc:
c       (for you dummies, 1 cc = 1 cubic centimeter)
        do i=0,maxen
           fmxwel(i)=fmxwel(i)*sorfra
        enddo
 
! calculate integrands which appear in velocity space moments
!  in moms:
        do 500 i=1,maxen
        qjace(i)=delten(i)*enerev(i)**.5
        qpjace(i)=qjace(i)*2.3151e-18*sqrt(AB)*enerev(i)**.5
	qwjace(i)=qjace(i)*1.3841e6*sqrt(enerev(i)/AB)
	bjinte(i)=delten(i)*enerev(i)*2.2176e-13*rZbeam/sqrt(AB)
500     enint(i)=delten(i)*enerev(i)**1.5
 
        qjace(0)=delten(0)*rjac0
	qpjace(0)=0.0
	qwjace(0)=0.0
	bjinte(0)=0.0
        enint(0)=delten(0)*rjac0*aven0
 
        do 550 j=1,maxxsi
	qjaca(j)=25.0*dVolnorm*delxsi(j)*abs(xsi(j))*btime(j)
	qpjaca(j)=qjaca(j)*rvparav(j)
	qwjaca(j)=qjaca(j)*vparvr(j)
	qijaca(j)=qjaca(j)*rmaj2av(j)*1.6726e-24*AB
c contribution of trapped particles to current is zero:
	if(j .lt. maxxsi/2+1-njtrap .or. j .gt. maxxsi/2+njtrap)then
	    bjinta(j)=25.0*delxsi(j)*xsi(j)
	else
	    bjinta(j)=0.0
	endif
550	continue
 
! The above formulas allow us to quickly calculate flux surface
! averaged particle density and energy density:
!       <n> = (sum over i) (sum over j) qjaca(j)*qjace(i)*f1(i,j)
!	j0  = (sum over i) (sum over j) bjinte(i)*bjinta(j)*f1(i,j)
!       <W> = (sum over i) (sum over j) qjaca(j)*enint(i)*f1(i,j)
!	<Ptor> = .....................  qpjaca(j)*qpjace(i)*f1(i,j)
!	<I> = ........................  qijaca(j)*qjace(i)*f1(i,j)
!	<Omega> = ......................qwjaca(j)*qwjace(i)*f1(i,j)
!
! where <n> is particle density in 1/cm^3
!       j0  is the unshielded beam parallel current density at Bmin
!	    in amps/cm^2
!	<W> is energy density in eV/cm^3
!	<Ptor> is the toroidal momentum density <m R vpar> (in the frame
!	    of reference rotating with the plasma).
!	<I> is the toriodal moment of inertia < m R**2>
!	<Omega> = <Vphi/R> is the average beam rotation frequency
!
! The Pebint and Pibint arrays found below allow us to quickly
! calculate the flux surface averaged power transfer to electrons and
! ions:
! d<We>/dt = sum over j sum over i qjaca(j) pebint(i) f1(i,j)
 
	do 600 i=2,maxen
	pebint(i)=(enerev(i)-enerev(i-1))*drage(i)
	1	-diffee(i) + diffee(i-1)
	pabint=(enerev(i)-enerev(i-1))*drag(i)
	1	-diffe(i) + diffe(i-1)
600	pibint(i)=pabint-pebint(i)
 
	diffe2=diffee(0)*(enerev(1)-aven0)/(enerev(1)-enerev(0))
	pebint(1)=(enerev(1)-aven0)*drage(1)
	1	-diffee(1)+diffe2
	pebint(0)=-diffe2
	diffa2=diffe(0)*(enerev(1)-aven0)/(enerev(1)-enerev(0))
	pabint=(enerev(1)-aven0)*drag(1)
	1	-diffe(1)+diffa2
	pibint(1)=pabint-pebint(1)
	pibint(0)=-diffa2-pebint(0)
 
c--calculate a normalization factor needed when f is isotropized
c       at the origin:
        coniso=0.0
        do j=1,maxxsi
           coniso=coniso+qjaca(j)
        enddo
 
! Calculate the CX source Fcx:
	fconst=1./(2.*3.14159*tnus)**1.5
	fconst=fconst*2**.5*2*3.14159/25
!initial guess from discrete Maxwellian at temperature Tnus
	fcx(0)=fconst
	do 640 i=1,maxen
640	fcx(i)=fcx(i-1)/(1+(enerev(i)-enerev(i-1))/Tnus)
! normalize to give proper discrete density of 1 per cc exactly:
	qjacas=0.0
	do 650 j=1,maxxsi
650	qjacas=qjacas+qjaca(j)
	sume=0.0
	sumn=0.0
	do 670 i=0,maxen
	sumn=sumn+qjace(i)*fcx(i)
670	sume=sume+enint(i)*fcx(i)
	tnudis=(2./3.)*sume/sumn
	do 680 i=0,maxen
680	fcx(i)=fcx(i)/sumn/qjacas
 
!calculate the CX rate on the maxwellian (for use in MOMS):
	rmaxcx=0.0
	do 690 i=0,maxen
690	rmaxcx=rmaxcx+qjace(i)*fmxwel(i)*rate(i)
	rmaxcx=rmaxcx*qjacas
 
! fcold is a source of cold particles to replace particles lost to
! diffusion or bad orbits.  fcold is normalized to 1 particle per
! cm**3:
	fcold=1./qjace(0)/qjacas
 
c--calculate velocity space Jacobian, which will be used in the
c explicit time advancement of the cross-derivative terms:
	do 750 i=0,maxen
	do 750 j=1,maxxsi
c note there is a factor of 2 here which has been factored out
c of the flux which will be calculated by FPCROSS:
750	fpdtjac(i,j)=dtFPP/(qjace(i)*qjaca(j)/(25.*dVolnorm))/2
 
c find the maximum energy particle which is confined for all
c pitch angle:
        ienloss=maxen
        do i=maxen,1,-1
           do j=1,maxxsi
              if(loss(i,j) .ne. 1) ienloss=i-1
           enddo
        enddo
 
c In moms, we will be thermalizing particles from E=0 to Ethmax,
c where Ethmax=4*Ti, unless the loss energy is too small.
c This is to avoid complications in the high energy part of the
c distribution, which is not necessary "Maxwellian" even in the
c absence of beams or RF.  (I.e., it may get distorted by bad orbit
c losses, charge-exchange losses, the ohmic electric field, etc.)
c
c Note that for a Maxwellian, 85% of the energy is contained below 4*Ti,
c and about 95% of the particles are contained below 4*Ti.
c
        mxthe=lkupr(4*trions(nshe,nthermi),enerev,maxen+1)-1
        mxthe=max(mxthe,0)
        if(mxthe .gt. ienloss) then
           ze=enerev(ienloss) - trions(nshe,nthermi)
           if(ze .lt. trions(nshe,nthermi)) ze=enerev(ienloss)*.9
           mxthe=lkupr(ze,enerev,maxen+1)-1
        endif
        if(mxthe .gt. ienloss) mxthe=ienloss/2
        mxthe=max(mxthe,0)
 
c calculate the particles and energy contained in fmxwel from E=0 to
c E=enerev(mxthe):
        zd=0.0
        ze=0.0
        do i=0,mxthe
           zd=zd+qjace(i)*fmxwel(i)
           ze=ze+enint(i)*fmxwel(i)
        enddo
        if(zd .ne. 0.0) then
           Tidis4=ze/zd/1.5
        else
           Tidis4=trions(nshe,nthermi)
        endif
        dthdis4=zd*coniso
 
! need to know the discrete ion temperature:
	sume=0.0
	do i=0,maxen
           sume=sume+enint(i)*fmxwel(i)
        enddo
        tidis=(2./3.)*sume*coniso
 
c--initialize f1 to almost ZDENBT per cc (except for instant losses):
 
        do j=1,maxxsi
           do i=0,maxen
              f1(i,j)=zdenbt*fmxwel(i)*loss(i,j)*sorfra
           enddo
        enddo
 
c--force f to be zero outside the boundary.  This simplifies
c       some of the formulas for boundary conditions.
 
        do 410 j=0,maxxsi+1
        f1(-1,j)=0.0
410     f1(maxen+1,j)=0.0
 
        do 430 i=0,maxen
        f1(i,0)=0.0
430     f1(i,maxxsi+1)=0.0
 
	return
	end
