	subroutine furate
c--calculate the beam-target fusion rate n_target*(sigma*v) for the
c proper fusion reaction.  This used to call Harry Towner's subroutines
c SIGMA and SIGE in the TRANSP CMS library (note that these return
c cross sections in m**2, while the ones in Harry's SLIB return cross
c sections in cm**2).  When FPP is called from TRANSP, it now uses
c TRANSP's SIGVBT routines, which include finite Ti effects.
c
c The list of fusion reactions which are calculated by SIGMA and SIGE:
c
c  nfutyp	incident + target --> products	  Traditional Notation
c
c      1.	D + T 	--> He4 + n + 17.6 MeV	  T(D,n)He4
c      2.	D + He3 --> He4 + p +18.3 MeV	  He3(D,p)He4
c      3.	D + D 	--> T   + p + 4.03 MeV	  D(D,p)T
c      4.	D + D 	--> He3 + n + 3.27 MeV	  D(D,n)He3
c      5.	T + T   --> He4 + 2n +		  T(T,2n)He4
c      6.	T + He3 --> X   + Y +		  He3(T,Y)X
c
c note: The reaction convention is
c               TARGET (INCIDENT, SMALL_PRODUCT) LARGE_PRODUCT
c
c******************************************************************
c
c For d-d reactions, we may wish to get fancier at some future date
c to better calculate self reactions for a Maxwellian, and not just
c rely on beam target reactions.  But don't forget to also divide
c by the factor of 2 so that the same reactions aren't counted twice.
c
c******************************************************************
 
	include 'FPPCOM'
	character icbeam*1,ictarget(mxions)*1,iccode*2
 
	do 5 i=0,maxen
	do 5 ireact=1,mxfureac
5	sigvfu(ireact,i) = 0.0
 
	iabeam=naions(nfasti)
	icbeam=' '
	if(iabeam .eq. 2 .and. nzions(nfasti) .eq. 1) icbeam='D'
	if(iabeam .eq. 3 .and. nzions(nfasti) .eq. 1) icbeam='T'
	if(iabeam .eq. 3 .and. nzions(nfasti) .eq. 2) icbeam='3'
	if(icbeam .eq. ' ')return
 
c identify the target ions:
	do 7 i=1,nfptherm
	ictarget(i)=' '
	if(naions(i) .eq. 2 .and. nzions(i) .eq. 1) ictarget(i)='D'
	if(naions(i) .eq. 3 .and. nzions(i) .eq. 1) ictarget(i)='T'
7	if(naions(i) .eq. 3 .and. nzions(i) .eq. 2) ictarget(i)='3'
 
c check for reactions with a Tritium beam:
	if(icbeam .eq. 'T') then
	    do 10 i=1,nfptherm
	    if(i .eq. nfasti) goto 10 !skip beam-beam collisions
            if(nbbfus(i) .eq. 1) goto 10
 
	    if(ictarget(i) .eq. 'D') then
	    do 8 ien=0,maxen
8	    sigvfu(1,ien)=sigvfu(1,ien)+
	1   drions(nshe,i)*sigvbt(enerev(ien),trions(nshe,i),'TD')
	    endif
 
c TRANSP's SIGVBT doesn't yet have TT reactions:
c	    if(ictarget(i) .eq. 'T') then
c	    do 9 ien=0,maxen
c9	    sigvfu(5,ien)=sigvfu(5,ien)+
c	1   drions(nshe,i)*sigvbt(enerev(ien),trions(nshe,i),'TT')
c	    endif
 
10	    continue
	endif
 
c check for reactions with a D beam.
	if(icbeam .eq. 'D') then
	    do 30 i=1,nfptherm
	    if(i .eq. nfasti) goto 30 !skip beam-beam collisions
            if(nbbfus(i) .eq. 1) goto 30
 
	    if(ictarget(i) .eq. 'D') then
	    do 18 ien=0,maxen
18	    sigvfu(4,ien)=sigvfu(4,ien)+
	1   drions(nshe,i)*sigvbt(enerev(ien),trions(nshe,i),'DD')
	    endif
 
	    if(ictarget(i) .eq. 'T') then
	    do 19 ien=0,maxen
19	    sigvfu(1,ien)=sigvfu(1,ien)+
	1   drions(nshe,i)*sigvbt(enerev(ien),trions(nshe,i),'DT')
	    endif
 
	    if(ictarget(i) .eq. '3') then
	    do 20 ien=0,maxen
20	    sigvfu(2,ien)=sigvfu(2,ien)+
	1   drions(nshe,i)*sigvbt(enerev(ien),trions(nshe,i),'D3')
	    endif
 
30	    continue
	endif
 
c check for reactions with a Helium-3 beam:
	if(icbeam .eq. '3') then
	    do 50 i=1,nfptherm
	    if(i .eq. nfasti) goto 50 !skip beam-beam collisions
            if(nbbfus(i) .eq. 1) goto 50
 
	    if(ictarget(i) .eq. 'D') then
	    do 38 ien=0,maxen
38	    sigvfu(2,ien)=sigvfu(2,ien)+
	1   drions(nshe,i)*sigvbt(enerev(ien),trions(nshe,i),'3D')
	    endif
 
50	    continue
	endif
 
c--fusene= energy released per fusion reaction
c200	if(nfutyp .eq. 1)fusene=17.6e6
c	if(nfutyp .eq. 2)fusene=18.3e6
c	if(nfutyp .eq. 3)fusene=24.9e6	!catalyzed d-d (T burns also)
c	if(nfutyp .eq. 4)fusene=24.9e6	!catalyzed d-d (T burns also)
c!for d-d, the cross section we use is for the neutron producing
c! branch only -- giving the neutron production rate.  To calculate
c! the total fusion energy released, we assume there is 1 reaction
c! via the proton branch for every neutron branch reaction.
c! This yields 4.03+3.27+17.6 = 24.9 MeV per neutron branch reaction.
c
c!convert to watts:
c	fusene=fusene*1.6022e-19
 
c--title for plots to tell us what is going on:
	futitl='T: D+T-->n    3: D+He3-->p    D: D+D-->n $'
 
	return
	end
