      function gami (a, x)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c evaluate the incomplete gamma function defined by
c
c gami = integral from t = 0 to x of exp(-t) * t**(a-1.0) .
c
c gami is evaluated for positive values of a and non-negative values
c of x.  a slight deterioration of 2 or 3 digits accuracy will occur
c when gami is very large or very small, because logarithmic variables
c are used.
c
cdmc      external alngam, alog, exp, gamit
c
      if (a.le.0.0) call seteru (25hgami    a must be gt zero, 25, 1, 2)
      if (x.lt.0.0) call seteru (25hgami    x must be ge zero, 25, 2, 2)
c
      gami = 0.0
      if (x.eq.0.0) return
c
c the only error possible in the expression below is a fatal overflow.
      factor = exp (alngam(a) + a*alog(x) )
c
      gami = factor * gamit(a, x)
c
      return
      end
      function gamit (a, x)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c evaluate tricomi-s incomplete gamma function defined by
c
c gamit = x**(-a)/gamma(a) * integral t = 0 to x of exp(-t) * t**(a-1.)
c
c and analytic continuation for a .le. 0.0.  gamma(x) is the complete
c gamma function of x.  gamit is evaluated for arbitrary real values of
c a and for non-negative values of x (even though gamit is defined for
c x .lt. 0.0).
c
c      a slight deterioration of 2 or 3 digits accuracy will occur when
c gamit is very large or very small in absolute value, because log-
c arithmic variables are used.  also, if the parameter a is very close
c to a negative integer (but not a negative integer), there is a loss
c of accuracy, which is reported if the result is less than half
c machine precision.
c
c ref. -- w. gautschi, an evaluation procedure for incomplete gamma
c functions, acm trans. math. software.
c
cdmc      external aint, alngam, alog, exp, gamr, r1mach, r9gmit, r9lgic,
cdmc     1  r9lgit, sqrt
      data alneps, sqeps, bot / 3*0.0 /
c
      if (alneps.ne.0.0) go to 10
      alneps = -alog(r1mach(3))
      sqeps = sqrt(r1mach(4))
      bot = alog(r1mach(1))
c
 10   if (x.lt.0.0) call seteru (21hgamit   x is negative, 21, 2, 2)
c
      if (x.ne.0.0) alx = alog(x)
      sga = 1.0
      if (a.ne.0.0) sga = sign (1.0, a)
      ainta = aint (a+0.5*sga)
      aeps = a - ainta
c
      if (x.gt.0.0) go to 20
      gamit = 0.0
      if (ainta.gt.0.0 .or. aeps.ne.0.0) gamit = gamr(a+1.0)
      return
c
 20   if (x.gt.1.0) go to 40
      if (a.ge.(-0.5) .or. aeps.ne.0.0) call algams (a+1.0, algap1,
     1  sgngam)
      gamit = r9gmit (a, x, algap1, sgngam, alx)
      return
c
 40   if (a.lt.x) go to 50
      t = r9lgit (a, x, alngam(a+1.0))
      if (t.lt.bot) call erroff
      gamit = exp(t)
      return
c
 50   alng = r9lgic (a, x, alx)
c
c evaluate gamit in terms of alog(gamic(a,x))
c
      h = 1.0
      if (aeps.eq.0.0 .and. ainta.le.0.0) go to 60
      call algams (a+1.0, algap1, sgngam)
      t = alog(abs(a)) + alng - algap1
      if (t.gt.alneps) go to 70
      if (t.gt.(-alneps)) h = 1.0 - sga*sgngam*exp(t)
      if (abs(h).gt.sqeps) go to 60
      call erroff
      call seteru (32hgamit   result lt half precision, 32, 1, 1)
c
 60   t = -a*alx + alog(abs(h))
      if (t.lt.bot) call erroff
      gamit = sign (exp(t), h)
      return
c
 70   t = t - a*alx
      if (t.lt.bot) call erroff
      gamit = -sga*sgngam*exp(t)
      return
c
      end
      function r9lgic (a, x, alx)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c compute the log complementary incomplete gamma function for large x
c and for a .le. x.
c
cdmc      external alog, r1mach
      data eps / 0.0 /
c
      if (eps.eq.0.0) eps = 0.5*r1mach(3)
c
      xpa = x + 1.0 - a
      xma = x - 1.0 - a
c
      r = 0.0
      p = 1.0
      s = p
      do 10 k=1,200
        fk = k
        t = fk*(a-fk)*(1.0+r)
        r = -t/((xma+2.0*fk)*(xpa+2.0*fk)+t)
        p = r*p
        s = s + p
        if (abs(p).lt.eps*s) go to 20
 10   continue
      call seteru (57hr9lgic  no convergence in 200 terms of continued f
     1raction, 57, 1, 2)
c
 20   r9lgic = a*alx - x + alog(s/xpa)
c
      return
      end
      function r9lgit (a, x, algap1)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c compute the log of tricomi-s incomplete gamma function with perron-s
c continued fraction for large x and for a .ge. x.
c
cdmc      external alog, r1mach, sqrt
      data eps, sqeps / 2*0.0 /
c
      if (eps.eq.0.0) eps = 0.5*r1mach(3)
      if (sqeps.eq.0.0) sqeps = sqrt(r1mach(4))
c
      if (x.le.0.0 .or. a.lt.x) call seteru (
     1  35hr9lgit  x should be gt 0.0 and le a, 35, 2, 2)
c
      ax = a + x
      a1x = ax + 1.0
      r = 0.0
      p = 1.0
      s = p
      do 20 k=1,200
        fk = k
        t = (a+fk)*x*(1.0+r)
        r = t/((ax+fk)*(a1x+fk)-t)
        p = r*p
        s = s + p
        if (abs(p).lt.eps*s) go to 30
 20   continue
      call seteru (57hr9lgit  no convergence in 200 terms of continued f
     1raction, 57, 3, 2)
c
 30   hstar = 1.0 - x*s/a1x
      if (hstar.lt.sqeps) call seteru (
     1  39hr9lgit  result less than half precision, 39, 1, 1)
c
      r9lgit = -x - algap1 - alog(hstar)
c
      return
      end
      function r9gmit (a, x, algap1, sgngam, alx)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c compute tricomi-s incomplete gamma function for small x.
c
cdmc      external alngam, alog, exp, r1mach
      data eps, bot / 2*0.0 /
c
      if (eps.eq.0.0) eps = 0.5*r1mach(3)
      if (bot.eq.0.0) bot = alog(r1mach(1))
c
      if (x.le.0.0) call seteru (24hr9gmit  x should be gt 0, 24, 1, 2)
c
      ma = a + 0.5
      if (a.lt.0.0) ma = a - 0.5
      aeps = a - float(ma)
c
      ae = a
      if (a.lt.(-0.5)) ae = aeps
c
      t = 1.0
      te = ae
      s = t
      do 20 k=1,200
        fk = k
        te = -x*te/fk
        t = te/(ae+fk)
        s = s + t
        if (abs(t).lt.eps*abs(s)) go to 30
 20   continue
      call seteru (54hr9gmit  no convergence in 200 terms of taylor-s se
     1ries, 54, 2, 2)
c
 30   if (a.ge.(-0.5)) algs = -algap1 + alog(s)
      if (a.ge.(-0.5)) go to 60
c
      algs = -alngam(1.0+aeps) + alog(s)
      s = 1.0
      m = -ma - 1
      if (m.eq.0) go to 50
      t = 1.0
      do 40 k=1,m
        t = x*t/(aeps-float(m+1-k))
        s = s + t
        if (abs(t).lt.eps*abs(s)) go to 50
 40   continue
c
 50   r9gmit = 0.0
      algs = -float(ma)*alog(x) + algs
      if (s.eq.0.0 .or. aeps.eq.0.0) go to 60
c
      sgng2 = sgngam*sign(1.0,s)
      alg2 = -x - algap1 + alog(abs(s))
c
      if (alg2.gt.bot) r9gmit = sgng2*exp(alg2)
      if (algs.gt.bot) r9gmit = r9gmit + exp(algs)
      return
c
 60   r9gmit = exp(algs)
      return
c
      end
      function gamr (x)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c this routine, not gamma(x), should be the fundamental one.
c
cdmc      external aint, exp, gamma
c
      gamr = 0.0
      if (x.le.0.0 .and. aint(x).eq.x) return
c
      call entsrc (irold, 1)
      if (abs(x).gt.10.0) go to 10
      gamr = 1.0/gamma(x)
      call erroff
      call entsrc (ir, irold)
      return
c
 10   call algams (x, alngx, sgngx)
      call erroff
      call entsrc (ir, irold)
      gamr = sgngx * exp(-alngx)
      return
c
      end
      subroutine algams (x, algam, sgngam)
c july 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c evaluate log abs (gamma(x)) and return the sign of gamma(x) in sgngam.
c sgngam is either +1.0 or -1.0.
c
cdmc      external aint, alngam
c
      algam = alngam(x)
      sgngam = 1.0
      if (x.gt.0.0) return
c
      int = amod (-aint(x), 2.0) + 0.1
      if (int.eq.0) sgngam = -1.0
c
      return
      end
      function alngam (x)
c august 1980 edition.   w. fullerton, c3, los alamos scientific lab.
c
cdmc      external aint, alog, gamma, r1mach, r9lgmc, sin, sqrt
      data sq2pil / 0.9189385332 0467274e0/
c sq2pil = alog(sqrt(2.*pi)),  sqpi2l = alog (sqrt(pi/2.))
      data sqpi2l / 0.2257913526 4472743e0/
      data pi     / 3.1415926535 8979324e0/
c
      data xmax, dxrel / 0., 0. /
c
      if (xmax.ne.0.) go to 10
      xmax = r1mach(2)/alog(r1mach(2))
      dxrel = sqrt (r1mach(4))
c
 10   y = abs(x)
      if (y.gt.10.0) go to 20
c
c alog (abs (gamma(x))) for  abs(x) .le. 10.0
c
      alngam = alog (abs (gamma(x)))
      return
c
c alog (abs (gamma(x))) for abs(x) .gt. 10.0
c
 20   if (y.gt.xmax) call seteru (
     1  38halngam  abs(x) so big alngam overflows, 38, 2, 2)
c
      if (x.gt.0.) alngam = sq2pil + (x-0.5)*alog(x) - x + r9lgmc(y)
      if (x.gt.0.) return
c
      sinpiy = abs (sin(pi*y))
      if (sinpiy.eq.0.) call seteru (31halngam  x is a negative integer,
     1  31, 3, 2)
c
      alngam = sqpi2l + (x-0.5)*alog(y) - x - alog(sinpiy) - r9lgmc(y)
c
      if (abs((x-aint(x-0.5))*alngam/x).lt.dxrel) call seteru (
     1'alngam  answer lt half precision because x too near negative ',
     2   68, 1, 1)
      return
c
      end
      function gamma (x)
c jan 1984 edition.   w. fullerton, c3, los alamos scientific lab.
      dimension gcs(23)
cdmc      external aint, alog, csevl, exp, inits, r1mach, r9lgmc, sin, sqrt
c
      data gcs   ( 1) / .0085711955 90989331e0/
      data gcs   ( 2) / .0044153813 24841007e0/
      data gcs   ( 3) / .0568504368 1599363e0/
      data gcs   ( 4) /-.0042198353 96418561e0/
      data gcs   ( 5) / .0013268081 81212460e0/
      data gcs   ( 6) /-.0001893024 529798880e0/
      data gcs   ( 7) / .0000360692 532744124e0/
      data gcs   ( 8) /-.0000060567 619044608e0/
      data gcs   ( 9) / .0000010558 295463022e0/
      data gcs   (10) /-.0000001811 967365542e0/
      data gcs   (11) / .0000000311 772496471e0/
      data gcs   (12) /-.0000000053 542196390e0/
      data gcs   (13) / .0000000009 193275519e0/
      data gcs   (14) /-.0000000001 577941280e0/
      data gcs   (15) / .0000000000 270798062e0/
      data gcs   (16) /-.0000000000 046468186e0/
      data gcs   (17) / .0000000000 007973350e0/
      data gcs   (18) /-.0000000000 001368078e0/
      data gcs   (19) / .0000000000 000234731e0/
      data gcs   (20) /-.0000000000 000040274e0/
      data gcs   (21) / .0000000000 000006910e0/
      data gcs   (22) /-.0000000000 000001185e0/
      data gcs   (23) / .0000000000 000000203e0/
c
      data pi /3.14159 26535 89793 24e0/
c sq2pil is alog (sqrt (2.*pi) )
      data sq2pil /0.91893 85332 04672 74e0/
      data ngcs, xmin, xmax, xsml, dxrel /0, 4*0.0 /
c
      if (ngcs.ne.0) go to 10
c
c ---------------------------------------------------------------------
c initialize.  find legal bounds for x, and determine the number of
c terms in the series required to attain an accuracy ten times better
c than machine precision.
c
      ngcs = inits (gcs, 23, 0.1*r1mach(3))
c
      call r9gaml (xmin, xmax)
      xsml = exp (amax1 (alog (r1mach(1)), -alog(r1mach(2))) + 0.01)
      dxrel = sqrt (r1mach(4))
c
c ---------------------------------------------------------------------
c finish initialization.  start evaluating gamma(x).
c
 10   y = abs(x)
      if (y.gt.10.0) go to 50
c
c compute gamma(x) for abs(x) .le. 10.0.  reduce interval and
c find gamma(1+y) for 0. .le. y .lt. 1. first of all.
c
      n = x
      if (x.lt.0.) n = n - 1
      y = x - float(n)
      n = n - 1
      gamma = 0.9375 + csevl(2.*y-1., gcs, ngcs)
      if (n.eq.0) return
c
      if (n.gt.0) go to 30
c
c compute gamma(x) for x .lt. 1.
c
      n = -n
      if (x.eq.0.) call seteru (14hgamma   x is 0, 14, 4, 2)
      if (x.lt.0. .and. x+float(n-2).eq.0.) call seteru (
     1  31hgamma   x is a negative integer, 31, 4, 2)
      if (x.lt.(-0.5) .and. abs((x-aint(x-0.5))/x).lt.dxrel) call
     1  seteru (68hgamma   answer lt half precision because x too near n
     2egative integer, 68, 1, 1)
      if (y.lt.xsml) call seteru (
     1  54hgamma   x is so close to 0.0 that the result overflows,
     2  54, 5, 2)
c
      do 20 i=1,n
        gamma = gamma / (x+float(i-1))
 20   continue
      return
c
c gamma(x) for x .ge. 2.
c
 30   do 40 i=1,n
        gamma = (y+float(i))*gamma
 40   continue
      return
c
c compute gamma(x) for abs(x) .gt. 10.0.  recall y = abs(x).
c
 50   if (x.gt.xmax) call seteru (32hgamma   x so big gamma overflows,
     1  32, 3, 2)
c
      gamma = 0.
      if (x.lt.xmin) call seteru (35hgamma   x so small gamma underflows
     1  , 35, 2, 0)
      if (x.lt.xmin) return
c
      gamma = exp((y-0.5)*alog(y) - y + sq2pil + r9lgmc(y) )
      if (x.gt.0.) return
c
      if (abs((x-aint(x-0.5))/x).lt.dxrel) call seteru (
     1  61hgamma   answer lt half precision, x too near negative integer
     2  , 61, 1, 1)
c
      sinpiy = sin (pi*y)
      if (sinpiy.eq.0.) call seteru (
     1  31hgamma   x is a negative integer, 31, 4, 2)
c
      gamma = -pi / (y*sinpiy*gamma)
c
      return
      end
      function r9lgmc (x)
c august 1977 edition.  w. fullerton, c3, los alamos scientific lab.
c
c compute the log gamma correction factor for x .ge. 10.0 so that
c  alog (gamma(x)) = alog(sqrt(2*pi)) + (x-.5)*alog(x) - x + r9lgmc(x)
c
      dimension algmcs(6)
cdmc      external alog, csevl, exp, inits, r1mach, sqrt
c
c series for algm       on the interval  0.          to  1.00000d-02
c                                        with weighted error   3.40e-16
c                                         log weighted error  15.47
c                               significant figures required  14.39
c                                    decimal places required  15.86
c
      data algmcs( 1) /    .1666389480 45186e0 /
      data algmcs( 2) /   -.0000138494 817606e0 /
      data algmcs( 3) /    .0000000098 108256e0 /
      data algmcs( 4) /   -.0000000000 180912e0 /
      data algmcs( 5) /    .0000000000 000622e0 /
      data algmcs( 6) /   -.0000000000 000003e0 /
c
      data nalgm, xbig, xmax / 0, 2*0.0 /
c
      if (nalgm.ne.0) go to 10
      nalgm = inits (algmcs, 6, r1mach(3))
      xbig = 1.0/sqrt(r1mach(3))
      xmax = exp (amin1(alog(r1mach(2)/12.0), -alog(12.0*r1mach(1))) )
c
 10   if (x.lt.10.0) call seteru (23hr9lgmc  x must be ge 10, 23, 1, 2)
      if (x.ge.xmax) go to 20
c
      r9lgmc = 1.0/(12.0*x)
      if (x.lt.xbig) r9lgmc = csevl (2.0*(10./x)**2-1., algmcs, nalgm)/x
      return
c
 20   r9lgmc = 0.0
      call seteru (34hr9lgmc  x so big r9lgmc underflows, 34, 2, 0)
      return
c
      end
      function csevl (x, cs, n)
c april 1977 version.  w. fullerton, c3, los alamos scientific lab.
c
c evaluate the n-term chebyshev series cs at x.  adapted from
c r. broucke, algorithm 446, c.a.c.m., 16, 254 (1973).  also see fox
c and parker, chebyshev polys in numerical analysis, oxford press, p.56.
c
c             input arguments --
c x      value at which the series is to be evaluated.
c cs     array of n terms of a chebyshev series.  in eval-
c        uating cs, only half the first coef is summed.
c n      number of terms in array cs.
c
      dimension cs(1)
c
      if (n.lt.1) call seteru (28hcsevl   number of terms le 0, 28, 2,2)
      if (n.gt.1000) call seteru (31hcsevl   number of terms gt 1000,
     1  31, 3, 2)
      if (x.lt.(-1.1) .or. x.gt.1.1) call seteru (
     1  25hcsevl   x outside (-1,+1), 25, 1, 1)
c
      b1 = 0.
      b0 = 0.
      twox = 2.*x
      do 10 i=1,n
        b2 = b1
        b1 = b0
        ni = n + 1 - i
        b0 = twox*b1 - b2 + cs(ni)
 10   continue
c
      csevl = 0.5 * (b0-b2)
c
      return
      end
      function inits (os, nos, eta)
c april 1977 version.  w. fullerton, c3, los alamos scientific lab.
c
c initialize the orthogonal series so that inits is the number of terms
c needed to insure the error is no larger than eta.  ordinarily, eta
c will be chosen to be one-tenth machine precision.
c
c             input arguments --
c os     array of nos coefficients in an orthogonal series.
c nos    number of coefficients in os.
c eta    requested accuracy of series.
c
      dimension os(nos)
c
      if (nos.lt.1) call seteru (
     1  35hinits   number of coefficients lt 1, 35, 2, 2)
c
      err = 0.
      do 10 ii=1,nos
        i = nos + 1 - ii
        err = err + abs(os(i))
        if (err.gt.eta) go to 20
 10   continue
c
 20   if (i.eq.nos) call seteru (28hinits   eta may be too small, 28,
     1  1, 2)
      inits = i
c
      return
      end
      subroutine r9gaml (xmin, xmax)
c april 1977 version.  w. fullerton, c3, los alamos scientific lab.
c
c calculate the minimum and maximum legal bounds for x in gamma(x).
c xmin and xmax are not the only bounds, but they are the only non-
c trivial ones to calculate.
c
c             output arguments --
c xmin   minimum legal value of x in gamma(x).  any smaller value of
c        x might result in underflow.
c xmax   maximum legal value of x in gamma(x).  any larger value will
c        cause overflow.
c
cdmc      external alog, r1mach
c
      alnsml = alog(r1mach(1))
      xmin = -alnsml
      do 10 i=1,10
        xold = xmin
        xln = alog(xmin)
        xmin = xmin - xmin*((xmin+0.5)*xln - xmin - 0.2258 + alnsml)
     1    / (xmin*xln + 0.5)
        if (abs(xmin-xold).lt.0.005) go to 20
 10   continue
      call seteru (27hr9gaml  unable to find xmin, 27, 1, 2)
c
 20   xmin = -xmin + 0.01
c
      alnbig = alog(r1mach(2))
      xmax = alnbig
      do 30 i=1,10
        xold = xmax
        xln = alog(xmax)
        xmax = xmax - xmax*((xmax-0.5)*xln - xmax + 0.9189 - alnbig)
     1    / (xmax*xln - 0.5)
        if (abs(xmax-xold).lt.0.005) go to 40
 30   continue
      call seteru (27hr9gaml  unable to find xmax, 27, 2, 2)
c
 40   xmax = xmax - 0.01
      xmin = amax1 (xmin, -xmax+1.)
c
      return
      end
      subroutine seteru (messg, nmessg, nerr, iopt)
      common /cseter/ iunflo
      integer messg(1)
      data iunflo / 0 /
c
      if (iopt.ne.0) call seterr (messg, nmessg, nerr, iopt)
      if (iopt.ne.0) return
c
      if (iunflo.le.0) return
      call seterr (messg, nmessg, nerr, 1)
c
      return
      end
      subroutine erroff
c
c  turns off the error state off by setting lerror=0.
c
cdmc      external i8save
c
      i=i8save(1,0,.true.)
      return
c
      end
      subroutine entsrc(irold,irnew)
c
c  this routine returns irold = lrecov and sets lrecov = irnew.
c
c  if there is an active error state, the message is printed
c  and execution stops.
c
c  irnew = 0 leaves lrecov unchanged, while
c  irnew = 1 gives recovery and
c  irnew = 2 turns recovery off.
c
c  error states -
c
c    1 - illegal value of irnew.
c    2 - called while in an error state.
c
cdmc      external i8save
c
      if (irnew.lt.0 .or. irnew.gt.2)
     1   call seterr(31hentsrc - illegal value of irnew,31,1,2)
c
      irold=i8save(2,irnew,irnew.ne.0)
c
c  if have an error state, stop execution.
c
      if (i8save(1,0,.false.) .ne. 0) call seterr
     1   (39hentsrc - called while in an error state,39,2,2)
c
      return
c
      end
      subroutine seterr (messg, nmessg, nerr, iopt)
c
c  this version modified by w. fullerton to dump if iopt = 1 and
c  not recovering.
c  seterr sets lerror = nerr, optionally prints the message and dumps
c  according to the following rules...
c
c    if iopt = 1 and recovering      - just remember the error.
c    if iopt = 1 and not recovering  - print, dump and stop.
c    if iopt = 2                     - print, dump and stop.
c
c  input
c
c    messg  - the error message.
c    nmessg - the length of the message, in characters.
c    nerr   - the error number. must have nerr non-zero.
c    iopt   - the option. must have iopt=1 or 2.
c
c  error states -
c
c    1 - message length not positive.
c    2 - cannot have nerr=0.
c    3 - an unrecovered error followed by another error.
c    4 - bad value for iopt.
c
c  only the first 72 characters of the message are printed.
c
c  the error handler calls a subroutine named fdump to produce a
c  symbolic dump. to complete the package, a dummy version of fdump
c  is supplied, but it should be replaced by a locally written version
c  which at least gives a trace-back.
c
      integer messg(1)
cdmc      external i1mach, i8save
c
c  the unit for error messages.
c
      iwunit=i1mach(4)
c
      if (nmessg.ge.1) go to 10
c
c  a message of non-positive length is fatal.
c
        write(iwunit,9000)
 9000   format(52h1error    1 in seterr - message length not positive.)
        go to 60
c
c  nw is the number of words the message occupies.
c
 10   nw=(min0(nmessg,72)-1)/i1mach(6)+1
c
      if (nerr.ne.0) go to 20
c
c  cannot turn the error state off using seterr.
c
        write(iwunit,9001)
 9001   format(42h1error    2 in seterr - cannot have nerr=0//
     1         34h the current error message follows///)
        call e9rint(messg,nw,nerr,.true.)
        itemp=i8save(1,1,.true.)
        go to 50
c
c  set lerror and test for a previous unrecovered error.
c
 20   if (i8save(1,nerr,.true.).eq.0) go to 30
c
        write(iwunit,9002)
 9002   format(23h1error    3 in seterr -,
     1         48h an unrecovered error followed by another error.//
     2         48h the previous and current error messages follow.///)
        call eprint
        call e9rint(messg,nw,nerr,.true.)
        go to 50
c
c  save this message in case it is not recovered from properly.
c
 30   call e9rint(messg,nw,nerr,.true.)
c
      if (iopt.eq.1 .or. iopt.eq.2) go to 40
c
c  must have iopt = 1 or 2.
c
        write(iwunit,9003)
 9003   format(42h1error    4 in seterr - bad value for iopt//
     1         34h the current error message follows///)
        go to 50
c
c  test for recovery.
c
 40   if (iopt.eq.2) go to 50
c
      if (i8save(2,0,.false.).eq.1) return
c
c     call eprint
c     stop
c
 50   call eprint
 60   call trdeath
      stop
c
      end
      subroutine eprint
c
c  this subroutine prints the last error message, if any.
c
      integer messg(1)
c
      call e9rint(messg,1,1,.false.)
      return
c
      end
      subroutine e9rint(messg,nw,nerr,save)
c
c  this routine stores the current error message or prints the old one,
c  if any, depending on whether or not save = .true. .
c
      integer messg(nw)
      logical save
cdmc      external i1mach, i8save
c
c  messgp stores at least the first 72 characters of the previous
c  message. its length is machine dependent and must be at least
c
c       1 + 71/(the number of characters stored per integer word).
c
      integer messgp(36),fmt(14),ccplus
c
c  start with no previous message.
c
      data messgp(1)/1h1/, nwp/0/, nerrp/0/
c
c  set up the format for printing the error message.
c  the format is simply (a1,14x,72axx) where xx=i1mach(6) is the
c  number of characters stored per integer word.
c
      data ccplus  / 1h+ /
c
      data fmt( 1) / 1h( /
      data fmt( 2) / 1ha /
      data fmt( 3) / 1h1 /
      data fmt( 4) / 1h, /
      data fmt( 5) / 1h1 /
      data fmt( 6) / 1h4 /
      data fmt( 7) / 1hx /
      data fmt( 8) / 1h, /
      data fmt( 9) / 1h7 /
      data fmt(10) / 1h2 /
      data fmt(11) / 1ha /
      data fmt(12) / 1hx /
      data fmt(13) / 1hx /
      data fmt(14) / 1h) /
c
      if (.not.save) go to 20
c
c  save the message.
c
        nwp=nw
        nerrp=nerr
        do 10 i=1,nw
 10     messgp(i)=messg(i)
c
        go to 30
c
 20   if (i8save(1,0,.false.).eq.0) go to 30
c
c  print the message.
c
        iwunit=i1mach(4)
        write(iwunit,9000) nerrp
 9000   format(7h error ,i4,4h in )
c
        call s88fmt(2,i1mach(6),fmt(12))
        write(iwunit,fmt) ccplus,(messgp(i),i=1,nwp)
c
 30   return
c
      end
      subroutine s88fmt( n, w, ifmt )
c
c  s88fmt  replaces ifmt(1), ... , ifmt(n) with
c  the characters corresponding to the n least significant
c  digits of w.
c
      integer n,w,ifmt(n)
c
      integer nt,wt,digits(10)
c
      data digits( 1) / 1h0 /
      data digits( 2) / 1h1 /
      data digits( 3) / 1h2 /
      data digits( 4) / 1h3 /
      data digits( 5) / 1h4 /
      data digits( 6) / 1h5 /
      data digits( 7) / 1h6 /
      data digits( 8) / 1h7 /
      data digits( 9) / 1h8 /
      data digits(10) / 1h9 /
c
      nt = n
      wt = w
c
 10   if (nt .le. 0) return
        idigit = mod( wt, 10 )
        ifmt(nt) = digits(idigit+1)
        wt = wt/10
        nt = nt - 1
        go to 10
c
      end
      integer function i8save(isw,ivalue,set)
c
c  if (isw = 1) i8save returns the current error number and
c               sets it to ivalue if set = .true. .
c
c  if (isw = 2) i8save returns the current recovery switch and
c               sets it to ivalue if set = .true. .
c
      logical set
c
      integer iparam(2)
c  iparam(1) is the error number and iparam(2) is the recovery switch.
c
c  start execution error free and with recovery turned off.
c
      data iparam(1) /0/,  iparam(2) /2/
c
      i8save=iparam(isw)
      if (set) iparam(isw)=ivalue
c
      return
c
      end
