	SUBROUTINE LOSSCO
!
! DMC 24 Mar 1993 -- patched (see below)
!
! Calculates the loss regions for the bounce-average fokker-planck
! code.  For trapped particles it assumes that the banana tip is
! located at the shell radius.  it calculates the range in pitch
! angle and energy which corresponds to bananas which extend beyound
! the limiter using conservation of canonical angular momentum in
! an axisymmetric device and is accurate for general banana width.
! For passing paticles, it uses a small banana width expansion of
! the canonical angular momentum equation to derive an expression
! for the shift of co- and counter- orbits from their average
! radial surface.  LOSSCO, DEPO, and DETECH should all use the
! same model for finite banana width effects.
!
!	Loss(i,j) = 0 <==> Loss Region
!	Loss(i,j) = 1 <==> Contained Region
!
! Original version by Rob Goldston, Oct. 3, 1981
! New version with loss region for co and counter particles as well
! by Greg Hammett, Oct. 24, 1984.
C
C	COMMON BLOCKS
C
	include 'FPPCOM'
 
C
C	ERASE OLD LOSS-CONES
C
	DO 100 I=0,maxen
	DO 100 J=1,maxxsi
	LOSS(I,J)=1
100	CONTINUE
 
c
c	Calculate loss regions for trapped particles
c
	zpi=abs(acos(-1.0))
	call momry(1.0,0.0,0,Rmax,zy,zjac)
	call momry(rminpro(nshe)/rmin,0.0,0,Rorb,zy,zjac)
	call momry(rminpro(nshe)/rmin,zpi,0,Rin,zy,zjac)
 
	IRAD=1+INT((numpsi-1)*rovera(nshe)+.5)
	DELPSI=PSI(numpsi)-PSI(IRAD)
	DO 800 J=maxxsi/2+1-njTrap,maxxsi/2+njTrap
	VLOSS=DELPSI/
	1	(AB*PMASS*SQRT(1.0-(1.0-xsi(j)**2)*RORB/RMAX)*RMAX)
	ELOSS=(VLOSS/1.384E6)**2*AB
	do 750 i=maxen,1,-1
	if(enerev(i) .lt. eloss)goto 800
750	LOSS(I,J)=0
 
800	CONTINUE
 
c
c	Calculate loss regions for passing particles
c
c The finite banan width model for DETECH, LOSSCO, and DEPO
c needs to be identical.  Presently, lossco and detech are similiar,
c but since detech expands psi around the detection point, while
c lossco expands psi around the average flux surface, the boundaries
c between confined and unconfined orbits and observable and
c unobservable orbits don't quite agree for passing particles (for
c trapped particles we use exact Pphi conservation in both cases
c so they do agree).  This new version of lossco didn't change anything
c perceptibly during ICRF heating.  It is questionable whether we need
c to make this even more exact for passing orbits, since most high
c energy particles are trapped anyway during ICRF heating.
c
c The biggest practical difference between this and the old lossco is
c that there is always a loss region, independent of NBI angle.  The
c large losses to be expected from counter-perp injection should be
c implemented by DEPO, where counter-perp injection should lead to
c trapped particles, many of which will be unconfined.
c
c The basic formulas here come from combining a definition of the
c average r**2 surface:
c	<r**2> = (vparin r2out + vparout r2in)/(vparin+vparout)
c with an expansion of the toroidal angular momentum equation:
c	Pphi = Psi(average) + delr**2out dPhi/dr**2 -m(R vpar)out
c	     = Psi(average) + delr**2in dPhi/dr**2 -m(R vpar)in
c
 
!gyro frequency:
	omci=9578.*Btor*rZbeam/Ab
 
	rad2=(rminpro(nshe)/rmin)**2
 
! loop over all passing pitch angles:
	do 900 jj=maxxsi/2+njTrap+1,maxxsi
! 1 for co particles, 2 for counter particles:
	do 900 jsign=1,2
	j=jj
	if(jsign .eq. 2)j=maxxsi+1-jj
 
	Rrefl=(Rorb)*(1-xsi(j)**2)
!counter particles can not shift in past Rrefl, so if Rrefl is
! inside the plasma they must be confined:
	if(xsi(j) .lt. 0.0 .and. Rrefl .gt. Rin)go to 900
 
	xout=abs(xsi(j))
!
! DMC PATCH 24 Mar 1993
! old code, blew up with SQRT(negative):
!	xin=sqrt(1.-Rrefl/Rin)
!
! there must be an ever so slight inconsistency btw this routine and
! the routine which defines njTrap, because Rrefl < Rin is expected for
! this passing particle but in 49990Z28 on DS01 today I got
! Rrefl = 188.65881, Rin = 188.65880
!
! here I add protection against a **small** violation of Rrefl < Rin
!  the code will still fail if the violation is not small
!
! Apr 19 -- also make sure xin strictly .gt. 0.0 at end
!   to prevent divide check further down...
!
	zrat = Rrefl/Rin
	if(zrat.gt.1.0) then
	  if(zrat.lt.1.0001) then
	    zrat=1.0
	  endif
	endif
	xin=sqrt(1.0-zrat)
	if(xin.eq.0.0) then
	  xin=1.0e-10       ! Apr 19 1993 -- dmc
	endif
!
! end DMC PATCH
!
	Const=2*qpro(nshe)/omci*xout/(xout+xin)*(Rorb*xout-Rin*xin)
 
	if(const .eq. 0.0)go to 900
	if(xsi(j) .gt. 0.0)then
	  vloss=rmin**2*(1-rad2)/const
	else
	  vloss=rmin**2*(1-rad2)*xout/xin/const
	endif
	eloss=Ab*(vloss/1.384e6)**2
	do 850 i=maxen,1,-1
	if(enerev(i) .lt. eloss)go to 900
850	loss(i,j)=0
 
900	continue
 
	return
	end
