	subroutine moms
C
c       Calculates the volume averaged beta and beta parallel of the
c       beam distribution function f1.  Also calculates other moments
c       of f1 such as the average density and temperature on a flux
c       surface, and the RF power absorbed on a flux surface.
C
C       ROB GOLDSTON  -- MAY 11, 1982
c
c--now performs a pretty complete power balance on each shell!!
c  In one test, only .2% of the power was missing.  To do a complete
c  power balance, we need to add: 1. Ohmic electric field power input
c  2. power lost when f is forced to be positive in subroutine imp.
c  3. NBI power input (now added), and 4. prompt nbi power loss.
c  Sept. 28, 1984, gwh.
c
c--gwh: modified to allow a source which keeps the density constant
c       even when CX and loss orbits are on.
c Speed up code by calculating common expressions in advance in
c       subroutine fstart and storing them in qjace, qjace, and enint.
c       Sept. 25, 1984, gwh.
C
C
C       ...COMMON BLOCKS
C
	include 'FPPCOM'
 
	real zdelfus(mxfureac)
	real zfmin(-1:maxen+1)
	real zlfmin(-1:maxen+1)
 
! Constant which appears in the calculation of beta and beta parallel:
 
! magnetic pressure and a 3/2 factor to convert kinetic energy
! to pressure:
	DENOM=1.5*Volfp(nshell)*BTOR**2/(8.0*3.141593)
 
c*********************************************************************
c								     *
c       Calculate the density and energy of particles averaged over  *
c       a flux surface.  These formulas are good to arbitrary        *
c       a/R and are generalized for arbitrary grids.	             *
c								     *
c*********************************************************************
 
c Zero all initial sums:
	sd=0.0  !sum for density
	sj=0.0  !sum for the beam current density
	spdiff=0.0 !sum for power lost to radial diffusion
	sdcx=0.0!sum for density loss rate from CX
	se=0.0  !sum for energy
	zepar=0.0 !sum for parallel energy
	srv=0.0	!sum for toroidal momentum
	smi=0.0	!sum for toroidal moment of inertia
	zrvcx=0.0 !CX loss of toroidal momentum
	zmicx=0.0 !CX loss of toroidal moment of inertia
	spe=0.0 !sum for power to electrons
	spi=0.0 !sum for power to ions
	spcx=0.0!sum for power to cx
	sddiff=0.0 !sum for density loss rate from radial diffusion
	zbomega=0.0 !sum for average beam rotation frequency
 
	do 13 i=1,mxfureac
13	zdelfus(i)=0.0	! sum for fusion reactions	
 
c--loop over all energies and pitch angles:
	do 200 j=1,maxxsi
	do 170 i=0,maxen
 
	sd=sd+qjace(i)*qjaca(j)*f1(i,j)
	sj=sj+bjinte(i)*bjinta(j)*f1(i,j)
	sdcx=sdcx+qjace(i)*qjaca(j)*rate(i)*f1(i,j)
	sddiff=sddiff+qjace(i)*qjaca(j)*ratedl(i,j)*f1(i,j)
	se=se+enint(i)*qjaca(j)*f1(i,j)
	zepar=zepar+enint(i)*qjaca(j)*vp2av(j)*f1(i,j)
	srv=srv+qpjace(i)*qpjaca(j)*f1(i,j)
	smi=smi+qjace(i)*qijaca(j)*f1(i,j)
	zrvcx=zrvcx+qpjace(i)*qpjaca(j)*rate(i)*f1(i,j)
	zmicx=zmicx+qjace(i)*qijaca(j)*rate(i)*f1(i,j)
	spe=spe+pebint(i)*qjaca(j)*f1(i,j)
	spi=spi+pibint(i)*qjaca(j)*f1(i,j)
	spcx=spcx+enint(i)*qjaca(j)*rate(i)*f1(i,j)
	spdiff=spdiff+enint(i)*qjaca(j)*ratedl(i,j)*f1(i,j)
	zbomega=zbomega+qwjace(i)*qwjaca(j)*f1(i,j)
 
c--various fusion reactions:
c  We don't use a do-loop here because we want the above "do 170"
c loop to be the innermost loop which will be vectorized.
c
c This calculation of 3 fusion reactions in the main MOMS loop
c is actually a few nanoseconds faster than the old version
c where 2 reactions were calculated in a separate loop in FUSOUT.
c It is faster on the VAX to put 3 calculations in a single loop
c rather than do the loop 3 times, because the VAX can then take
c advantage of its fast cache memory.
c
	zdelf=f1(i,j)*qjace(i)*qjaca(j)
 
	zdelfus(1)=zdelfus(1)+zdelf*sigvfu(1,i)
	zdelfus(2)=zdelfus(2)+zdelf*sigvfu(2,i)
c	zdelfus(3)=zdelfus(3)+zdelf*sigvfu(3,i)  ! D+D->p ignored for now.
	zdelfus(4)=zdelfus(4)+zdelf*sigvfu(4,i)
c	zdelfus(5)=zdelfus(5)+zdelf*sigvfu(5,i)	 ! T+T ignored for now.
c	zdelfus(6)=zdelfus(6)+zdelf*sigvfu(6,i)	 ! T+He3 ignored for now.
 
170	continue
200     continue
 
c--store the results:
	avbden(niTime)=sd       !average density (particles per cc)
	xjbeam(niTime)=sj       !average current density (amps/cm^2)
	peb(niTime)=spe*1.6022e-19      !Power to e from beam (Watts per cc)
	pib(niTime)=spi*1.6022e-19      !power to ions from beam (Watts per cc)
	Pdiff(niTime)=spdiff*1.6022e-19 !power lost by radial diffusion (W/cc)
	Pcxb(niTime)=spcx*1.6022e-19 !CX power loss (Watts/cc)
 
	avbtmom(nitime)=srv	!average beam toroidal momentum
	avbtmi(nitime)=smi	!average beam moment of inertia
	avbtmocx(nitime)=zrvcx	!Torque from beam to CX losses (plasma frame)
	avbtmicx(nitime)=zmicx	!Moment of inertia CX losses
 
	delbet=se*dVolfp(nshe)*1.6021e-12/denom
	betbm(niTime)=betbm(niTime)+delbet	    !beta
	delbet=zepar*dVolfp(nshe)*1.6021e-12/denom
	BetaPar(niTime)=BetaPar(niTime)+3.0*delbet  !parallel beta
 
	fuso(nitime,1)=zdelfus(1)   !fusion reaction rates
	fuso(nitime,2)=zdelfus(2)
	fuso(nitime,3)=zdelfus(3)
	fuso(nitime,4)=zdelfus(4)
	fuso(nitime,5)=zdelfus(5)
	fuso(nitime,6)=zdelfus(6)
 
!subtract off CX source if it is on:
	if(lcxsrc) Pcxb(niTime)=Pcxb(niTime)-
	1   3./2.*sdcx*Tnudis*1.6022e-19
 
c calculate equivalent temperature:
c (2/3) average energy (kev) = equivalent temperature:
	if(sd .gt. 0.0)then
		avbtemp(niTime)=2./3.*se/(sd*1000.)
	else
		avbtemp(niTime)=0.0
	endif
 
c and equivalent parallel temperature:
c average parallel energy = (1/2) Tparallel
	if(sd .gt. 0.0)then
	    avbtpar(niTime)=2.*zepar/(sd*1000.)
	else
	    avbtpar(niTime)=0.0
	endif
 
c average beam rotation frequency (in plasma frame):
	if(sd .gt. 0.0)then
	    avbtomeg(nitime)=zbomega/sd
	else
	    avbtomeg(nitime)=0.0
	endif
 
c find the rate at which beam ions are thermalizing (particles/cc/sec):
	bmtht(niTime)=0.0
	Pbtht(niTime)=0.0
 
	if(LLwbnd .eq. 0)then
 
	zpbec=0.0 !Pbe correction to conservative B.C. calculations
	zpbc=0.0 !Pb(i+e) correction to conservative B.C. calculations
	zpthrm=0.0 !Pbi from thermalization
 
	do 220 j=1,maxxsi
 
	bmtht(niTime)=bmtht(niTime)-qjaca(j)*(-drag(2)*f1(2,j)
	1   +(delten(1)/delten(2))*sqrt(enerev(1)/enerev(2))
	1   *(drag(3)*f1(3,j)
	1     +diffe(2)*(f1(3,j)-f1(2,j))/(enerev(3)-enerev(2))
	1     -drag(2)*f1(2,j)))
 
	zpbec=zpbec-qjaca(j)*((enerev(2)-enerev(1))*drage(2)*f1(2,j)
	1   +(enerev(1)-aven0)*(drage(1)*f1(1,j)
	1   +diffee(0)*(f1(1,j)-f1(0,j)) /(enerev(1)-enerev(0)) ))
 
	zpbc=zpbc-qjaca(j)*((enerev(2)-enerev(1))*drag(2)*f1(2,j)
	1   +(enerev(1)-aven0)*(drag(1)*f1(1,j)
	1   +diffe(0)*(f1(1,j)-f1(0,j)) /(enerev(1)-enerev(0)) ))
 
c thermalizing power (energy the beam ions still have left when they
c drop off the lowest grid point):
	zpthrm=zpthrm-qjaca(j)*(-enerev(2)*drag(2)*f1(2,j)
	1   +qjace(1)/qjace(2)*enerev(1)*(drag(3)*f1(3,j)
	1   +diffe(2)*(f1(3,j)-f1(2,j)) /(enerev(3)-enerev(2))
	1   -drag(2)*f1(2,j) ))
 
220     continue
 
	Pbtht(niTime)=zpthrm*1.6022e-19
	Peb(niTime)=Peb(niTime)+zpbec*1.6022e-19
	Pib(niTime)=Pib(niTime)+(zpbc-zpbec)*1.6022e-19
 
	endif
 
c*********************************************************************
c								     *
c       Calculate the RF power absorbed by the minority species.     *
c       This assumes that DEB and DEC contain only the RF	     *
c       contributions to the fokker-planck diffusion coefficients.   *
c	 See my notes "RF Heating Rate" from Nov. 28, 1983.	     *
c								     *
c*********************************************************************
	sum=0.0
	if(prftot .le. 0.0)goto 260
 
c	do 250 j=1,maxxsi
c	do 230 i=2,maxen-1
c230     sum=sum+f1(i,j)*( delxsi(j)*(deb(i,j)-deb(i-1,j))
c	1    +.25*(enerev(i+1)-enerev(i))*(dec(i,j+1)-dec(i,j-1))
c	1    +.25*(enerev(i)-enerev(i-1))*(dec(i-1,j+1)-dec(i-1,j-1)))
c	i=maxen
c	sum=sum+f1(i,j)*( delxsi(j)*(deb(i,j)-deb(i-1,j))
c	1    +.25*(enerev(i)-enerev(i-1))*(dec(i-1,j+1)-dec(i-1,j-1)))
c
c	sum=sum+delxsi(j)*( deb(1,j)*f1(1,j)
c	1       -deb(0,j)*(f1(1,j)-f1(0,j))*(enerev(1)-aven0)
c	1			       /(enerev(1)-enerev(0)) )
c	sum=sum+f1(1,j)*(
c	1    .25*(enerev(2)-enerev(1))*(dec(1,j+1)-dec(1,j-1))
c	1    +.25*(enerev(1)-aven0)*(dec(0,j+1)-dec(0,j-1)))
c	sum=sum+f1(0,j)*(
c	1    .25*(enerev(1)-aven0)*(dec(0,j+1)-dec(0,j-1)))
c250     continue
 
	call fpcross
 
	do 250 j=1,maxxsi
	do 230 i=2,maxen
230     sum=sum+f1(i,j)*( delxsi(j)*(deb(i,j)-deb(i-1,j)) )
	1 +enint(i)/qjace(i)/2*(fpfluxe(i-1,j)-fpfluxe(i,j))
 
	sum=sum+delxsi(j)*( deb(1,j)*f1(1,j)
	1       -deb(0,j)*(f1(1,j)-f1(0,j))*(enerev(1)-aven0)
	1			       /(enerev(1)-enerev(0)) )
	1   +enint(1)/qjace(1)/2*(fpfluxe(0,j)-fpfluxe(1,j))
	1   +enint(0)/qjace(0)/2*(fpfluxe(-1,j)-fpfluxe(0,j))
 
250     continue
 
c--actual rf power absorbed (Watts per cm**3)
260     realrf(niTime)=dVolnorm*25*sum*1.6022e-19
 
c*********************************************************************
c         							     *
c       Calculate power lost due to bad orbit losses		     *
c								     *
c*********************************************************************
c--loop over all pitch angles:
	se=0.0  !sum for energy
	do 600 j=1,maxxsi
c--sum over all energies:
	see=0.0
	do 570 i=0,maxen
c--skipping the region of velocity space which is confined:
	if(loss(i,j) .ne. 0.0)go to 570
c--note that this formula for dfdt is missing f1(i,j) terms because
c f1(i,j)=0.0 if loss(i,j)=1.0:
	dfdt=-alpha(i,j)*f1(i,j+1)-gama(i,j)*f1(i,j-1)
	1       -alphe(i,j)*f1(i+1,j)-game(i,j)*f1(i-1,j)
	1           -fpdtjac(i,j)*( fpfluxe(i,j)-fpfluxe(i-1,j)
	1           +fpfluxa(i,j)-fpfluxa(i,j-1) )
 
c	1       +decp(i,j)*(f1(i+1,j+1)+f1(i,j+1)
c	1	       -f1(i+1,j-1)-f1(i,j-1))
c	1       -decm(i,j)*(f1(i,j+1)+f1(i-1,j+1)
c	1	       -f1(i,j-1)-f1(i-1,j-1))
c	1       +dacp(i,j)*(f1(i+1,j+1)+f1(i+1,j)
c	1	       -f1(i-1,j+1)-f1(i-1,j))
c	1       -dacm(i,j)*(f1(i+1,j)+f1(i+1,j-1)
c	1	       -f1(i-1,j)-f1(i-1,j-1))
 
	dfdt=dfdt/dtreal
	see=see+enint(i)*dfdt
570     continue
 
	se=se+see*qjaca(j)
600     continue
	POrbLoss(niTime)=se*1.6022e-19  !Power lost (watts/cc)
 
c*********************************************************************
c								     *
c       Add in a source of particles to replace CX and bad orbit     *
c       losses.  This  is neccessary during RF heating when there is *
c       no beam source.  The source fmxwel is calculated in fstart.  *
c								     *
c*********************************************************************
! we use a particle conserving Krook operator to represent CX.
! We estimate the source rate needed to keep constant density from
! the calculation of sdcx in the do 200 loop.  This approximate source
! is added every time step by IMP.  Here we add a source to give
! exact particle conservation.  This source is due to any
! errors in the CX source estimate and to replace particle losses
! due to bad orbits.
	Pparts(nitime)=0.0
	if(.not. lnsrc)goto 315
 
	if(nfasti .ne. nthermi) then
	    ztarget=drions(nshe,nfasti)+drions(nshe,nthermi)
	else
	    ztarget=drions(nshe,nfasti)
	endif
 
	zsden=-(avbden(niTime)-ztarget)
 
!either add on a source of cold ions:
	if(zsden .gt. 0.0)then
	do 300 i=0,maxen
	do 300 j=1,maxxsi
300     f1(i,j)=f1(i,j)+zsden*fmxwel(i)*loss(i,j)
! Power from the particle source (Watts/cc):
	Pparts(niTime)=(3./2.)*(zsden/2/dtFPP)*Tidis*1.6022e-19
 
!or subtract off a fraction of the existing ions:
	else
	zratkeep=(avbden(nitime)+zsden)/avbden(nitime)
	do 309 i=0,maxen
	do 309 j=1,maxxsi
309	f1(i,j)=f1(i,j)*zratkeep
! Power from the particle source (Watts/cc):
	Pparts(niTime)=(3./2.)*(zsden/2/dtFPP)*avbTemp(nitime)
	1	*1.6022e-19
	endif
 
315	continue
 
!CX source:
	do 318 i=0,maxen
318	fsourc(i)=0.0
	if(lcxsrc)then
	do 320 i=0,maxen
	if(zsden .ge. 0.0)fsourc(i)=fcx(i)*dtreal*(sdcx+rmaxcx*zsden)
320	if(zsden .lt. 0.0)fsourc(i)=fcx(i)*dtreal*sdcx*zratkeep
	endif
 
!add in source of cold particles from the radial diffusion model:
350     if(lsdiff) fsourc(0)=fsourc(0) + dtreal*sddiff*fcold
 
c*********************************************************************
c
c Improved model of beam thermalization.
c
c Here we subtract out a "sink" of Maxwellian ions,
c
c 	d f / d t =   . . .  - nu_th  f_Maxwellian
c
c We subtract off as much f_Maxwellian as we possibly can, while still
c maintaining f positive everywhere.  Because of non-Maxwellian
c distortions even for the "thermal" distribution, we do the subtraction
c only between E=0 and E=4*Ti (which contains about 95% of the particles
c and 85% of the energy of a Maxwellian).  The upper bound E=4*Ti
c (actually that is an approximate value, modified when the loss cone
c extends to low energy near the edge) is determined in subroutine
c fstart and is stored in parameter mxthe, (enerev(mxthe) \approx 4*Ti).
c
c This models f=f_mw + f_tail.
c
c*********************************************************************
 
	if(lfptherm .ne. 1) goto 9999
 
c find fmin(E) = min_{over xsi} f(E,xsi)
c ?? This loop could be faster if i is the inner loop?:
	do i=0,maxen
	   zfmin(i)=f1(i,1)
	   do j=2,maxxsi
	      zfmin(i)=min(zfmin(i),f1(i,j))
	   enddo
	enddo
 
	data iexpmod /0/
	if(iexpmod .eq. 0) goto 9980
 
c use an analytic exponential f = exp(-E/T) model here,
c or the numerical equilibrium f below 9980:
 
	do i=0,mxthe
	   if(zfmin(i) .gt. 0.0) then
	      zlfmin(i)=log(zfmin(i))
	   else
	      zlfmin(i)=-100.0
	   endif
	enddo
 
c find the average energy of fmin in the thermal region < 4*Ti:
	zd=0.0
	ze=0.0
	do i=0,mxthe
	   zd=zd+qjace(i)*zfmin(i)
	   ze=ze+enint(i)*zfmin(i)
	enddo
	if(zd .ne. 0.0) then
	   zTtherm=ze/zd/1.5
	else
	   zTtherm=trions(nshe,nthermi)
	endif
c use zTtherm=Ti instead:
	zTtherm=trions(nshe,nthermi)
 
c subtract of maximum density of Maxwellian which still keeps f>0:
	zalpha=zlfmin(0)
	do i=1,mxthe
	   if(zalpha .gt. zlfmin(i)+enerev(i)/zTtherm) then
	      ifit=i
	      zalpha=zlfmin(i)+enerev(i)/zTtherm
	   endif
	enddo
 
	write(6,*) 'time, dn/dt, Ttherm =',time,
     >    exp(zalpha)/(2*dtfpp)*zTtherm**1.5, zTtherm
 
c subtract off the requested thermalizing Maxwellian:
	do j=1,maxxsi
	   do i=0,mxthe
	      f1(i,j)=f1(i,j)-exp(zalpha-enerev(i)/zTtherm)*loss(i,j)
	   enddo
	enddo
 
c calculate the thermal particle and power source rates
c
	bmtht(nitime)=exp(zalpha)/(2*dtfpp)*zTtherm**1.5
	Pbtht(nitime)=bmtht(nitime)*3./2.*zTtherm*1.6022e-19
	return
 
 9980	continue
c
c subtract off just enough of a Maxwellian so that f>0 everywhere.
c znuthdt = nu_th * dt, where nu_th is the "thermalization rate"
c
	znuthdt=zfmin(0)/fmxwel(0)
	do i=1,mxthe
	   znuthdt=min(znuthdt,zfmin(i)/fmxwel(i))
	enddo
 
	do j=1,maxxsi
	   do i=0,mxthe
	      f1(i,j)=f1(i,j)-znuthdt*fmxwel(i)*loss(i,j)
c mxthe should have been set small enough in subroutine fstart
c that loss(i,j) is always 1.  But just in case, leave it on here...
	   enddo
	enddo
 
c
c calculate the thermal particle and power source rates
c
	bmtht(nitime)=dthdis4*znuthdt/(2*dtfpp)
	Pbtht(niTime)=bmtht(nitime)*3./2.*Tidis4*1.6022e-19
 
 
c or just zero out f for all energies below (3/2)*Ti
c (This is what TRANSP's Monte Carlo model effectively does):
c	do j=1,maxxsi
c	   do i=0,maxen
c	      if(enerev(i) .lt. 1.5*trions(nshe,nthermi))
c	1	   f1(i,j)=0.0
c	   enddo
c	enddo
c haven't bothered defining bmtht and Pbtht for this option (which
c is commented out anyway).
 
 
9999	return
	end
