	subroutine orbshift(zE,zvparv,zRin,zZin,zrav,Zeta0,iunconf)
c
c Uses a finite banana width model to transform from the instantaneous
c position of a particle:
c
c  zE	    particle energy (keV)
c  zvparv   vpar/v
c  zRin	    R of particle
c  zZin	    Z of particle (height off the midplane)
c
c to the constant-of-the-motion variables:
c
c  zrav	    <r> averaged over particle orbit
c  Zeta0    vpar/v at Rmaj+<r> (with the same sign as zvparv)
c  iunconf  =0 confined orbits, =1 for unconfined orbits
c
 
C     SET UP COMMON BLOCKS
 
	include 'FPPCOM'
	external Pphiorb
	common /zPphiCmn/ zRtip, zmv, zPphi0 ! communication with Pphiorb
 
C DMC 22 FEB 1993  --  replacing IMSL rootfinder ZBREN with local
C  root finder ZEROIN.  ZEROIN needs machine epslon.
 
	data zzeps/0.0/
	if(zzeps.eq.0.0) zzeps=x02aje()
 
c Map (zRin,zZin) to (zxi,zxth) and back to (zR,zZ)
c to avoid errors which occur because (zxi,zxth) do not
c correspond exactly to (zRin,zZin) due to small mapping errors:
 
	call nbimap(zRin,zZin,zxi,zth,ier)
	call momry(zxi,zth,0,zR,zZ,zJac)
 
! ion velocity:
	VEL=1.384E6*SQRT(zE*1.0E3/AB)
	zdir=sign(1.0,zVparV) !sign of Vpar/V
 
! Conservation of Toroidal Angular Momentum:
! Pphi = constant = Psi(r) - m R vpar
!                 = Psimag - Psimec
! Sightline geometry gives
! Vpar/V = Rtan/R
 
! from conservation of E and mu,
! Vpar/V = sqrt ( 1. - Rrefl/R)
! Rrefl is where Vpar --> 0 and the particle would reflect
! assuming B = B0 R0 / R
	Rrefl=zR*(1.0-zVparV**2)
 
	zrl=zxi*rmin
	psimag=finteq1d(zrl,Psi,0.0,delrPsi,numPsi)
!debug:
! psimag is an approximate value based on two interpolations
! (one in NBIMAP and another in FINTEQ1D).  PPHIORB uses a
! different type of approximation which may lead to a slightly
! different result.  To avoid sqrt(negative number) errors later,
! we install a limiting check here, which is based on the fact
! that Psi(R,Z) > Psi(R,0):
!	psimag2 = finteq1d(zR,PsiRj,RjPsi1,delRjPsi,numPsij)
!	psimag=max(psimag,psimag2)
 
	PSIMEC=AB*PMASS*zR*VEL*zVparV
	Psitot=Psimag-Psimec
 
c as a quick debugging option, try a simple zero-banana width model:
	data iquick /0/
	if(iquick .eq. 1) then
	    zrav=zrl
	    call momry(zxi,0.0,0,zRmax,zy0,zjac)
	    arg=1.-Rrefl/zRmax
	    if(arg .lt. 0.0)arg=0.0
	    zeta0=zdir*sqrt(arg)
	    iunconf=0
	    if(zxi .gt. 1.0) iunconf=1
	    return
	endif
 
c
c initialize zPphiCmn common block to communicate with Pphiorb
c
        zRtip=Rrefl
        zmv=ab*Pmass*vel*zdir
        zPphi0=Psitot
 
! Determine if this is a trapped or a passing orbit.  Assuming that
! psi(r) is monotonically increasing, then a pretty good condition is
! Pphi > Psi( abs(Rrefl-Rmaj)) means this is a trapped orbit.
! Unfortunately this condition breaks down for pinch orbits, which
! may occur at high energy.  We will ignore this special case, and
! treat such orbits as if they were trapped.
 
	if(Rrefl .lt. Rplamin) goto 2001
	if(Pphiorb(Rrefl) .gt. 0.0) goto 2001
 
c************************************************
C	WE HAVE A BANANA ORBIT
c************************************************
	Psitip=Psitot
!the banana tip is not even in the plasma, skip it:
	IF(PSITIP.GT.PSI(numpsi)) GOTO 124
	IPSTP=1+INT((numpsi-1)*PSITIP/PSI(numpsi)+0.5)
!zrav is the minor radius of the banana tip:
	zrav=RPSI(IPSTP)
	call momry(zrav/rmin,0.0,0,zRmax,zY0,zjac)
	arg=1.-Rrefl/zRmax
	if(arg .lt. 0.0)arg=0.0
	zeta0=zdir*sqrt(arg)
 
!flag this orbit if it is unconfined:
	iunconf=0
	if( (Psitip + Ab*Pmass*Vel*sqrt(RplaMax*(RplaMax-Rrefl)))
	1	.gt. Psi(numpsi))iunconf=1
	return
 
c************************************************
C       WE HAVE A PASSING ORBIT
c************************************************
 
2001    CONTINUE
 
        if (zdir .gt. 0.0) goto 2100
 
c
c we have a counter passing orbit
c first check to see if it is confined
c
        if(zRtip .lt. Rplamin)then
	if(Pphiorb(Rplamin) .le. 0.0) goto 124
	endif
	goto 2200
 
c
c we have a co-going passing orbit
c first check to see if it is confined:
c
2100    continue
	if(Pphiorb(RplaMax) .le. 0.0) goto 124
c
c search for the minimum value of Pphiorb, where the top of the
c orbit is in (R,Psi) space.
c
2200	zRminim=Max(Rplamin,zRtip)
	zRmax=RplaMax
c
c uvmif calls are fairly expensive.  skip if possible:
c
	zRtop=Max(zRtip,Rmaj)
	zfmin=Pphiorb(zRtop)
	if(zfmin .lt. 0) goto 2300
 
	zxguess=(zRminim+zRmax)/2
	zfmin1=Pphiorb(zxguess)
	zfmin2=Pphiorb(zR)
	if(zfmin2 .lt. zfmin1) zxguess=zR
	zbound=min(zRmax-zxguess,zxguess-zRminim)
	zstep=zbound/8.
	zxacc=rmin/nshell/10.
	imaxfn=32
 
c a replacement for uvmif which I wrote myself:
	call fpminf(Pphiorb,zxguess,zstep,zRminim,zRmax,zxacc,
	1   imaxfn,zRtop)
 
c	call uvmif(Pphiorb,zxguess,zstep,zbound,zxacc,imaxfn,zRtop)
c	if(iercd() .ne. 0) then
c	    type *, 'error in uvmif?'
c	endif
 
c
c check to see if the orbit has a numerically zero width (this can occur
c for very high energy counter ions on the high field side, or
c very high energy co ions on the low field side, where the shift is so
c large that the ions don't go anywhere).
c
	zfmin=Pphiorb(zRtop)
	if(zfmin .ge. 0.0) then
	    if(abs(zz) .gt. zxacc)then
	write(6,*) 'ORBSHIFT warning: Orbit has numerically zero width,'
	write(6,*) '    yet zZ > zxacc.'
	write(6,*) '    zE, zvparv,zR,zz =',ze,zvparv,zR,zz
	    endif
	    Rin=zRtop
	    Rout=zRtop
	    goto 2400
	endif
 
c
c now search for solutions to 0=Pphiorb(R), the two places where
c the orbit crosses the midplane.
c
2300	zrelacc=rmin/Rmaj/nshell/10.
	zR1=zRminim
	zR2=zRtop
	imaxit=32
	zR2=zeroin(zR1,zR2,Pphiorb,0.01*zrelacc,zzeps)
cIMSL	call zbren(Pphiorb,0.0,zrelacc,zR1,zR2,imaxit)
	Rin=zR2
 
	zR1=zRtop
	zR2=zRmax
	imaxit=32
	zR2=zeroin(zR1,zR2,Pphiorb,0.01*zrelacc,zzeps)
cIMSL	call zbren(Pphiorb,0.0,zrelacc,zR1,zR2,imaxit)
	Rout=zR2
 
c
c calculate the average r**2 surface, weighted by length of time
c spent there (proportional to 1/vparallel):
c
2400	call nbimap(Rout,0.0,zxi,zth,ier)
	r2out=(zxi*rmin)**2
	call nbimap(Rin,0.0,zxi,zth,ier)
	r2in=(zxi*rmin)**2
 
	zpsitop=zmv*zRtop*sqrt(1.-zRtip/zRtop)+zPphi0
	zr2top=finteq1d(zpsitop,rpsi,0.0,delPsir,numpsi)
	zr2top=zr2top**2
 
	vpin=sqrt(1.-Rrefl/Rin)
	vpout=sqrt(1.-Rrefl/Rout)
	zvptop=sqrt(1.-Rrefl/zRtop)
	zdenom=vpin*zvptop+vpout*zvptop+2*vpin*vpout
	if(zdenom .eq. 0.0) then
	   r2av=(r2out+r2in+2*zr2top)/4
	else
	r2av=(vpin*zvptop*r2out+vpout*zvptop*r2in+vpin*vpout*2*zr2top)
	1   /zdenom
	endif
	zrav=sqrt(r2av)
 
! determine pitch angle at Rmaj+<r>:
	call momry(zrav/rmin,0.0,0,zRmax,zY0,zjac)
	arg=1.-Rrefl/zRmax
	IF(ARG.LT.0.0) ARG=0.0
	ZETA0=zDir*SQRT(ARG)
 
	iunconf=0
	return
c
c unconfined orbit:
c
124	iunconf=1
	return
	end
