	subroutine plotpb
! plot radial power balance
 
	include 'FPPCOM'
	character*80 string
	real rpbod(maxshe)  !bad Orbit + radial Diffusion losses
	real zbpith(maxshe) !slowing down + thermalization ion heating
 
!type out initial and final temperature profile to the log file:
	call finitt(0,780)
	write(lsysout,*) 'initial beam temperature profile:'
	write(lsysout,*) (rbtemp0(i),i=1,nshell)
	write(lsysout,*) 'final beam temperature profile:'
	write(lsysout,*) (rbtemp1(i),i=1,nshell)
	write(lsysout,*) 'type a character to continue...'
	call upause(0)
 
!add bad Orbit and radial Diffusion losses together to fit on one page:
!also add slowing down and thermalation ion heating powers together:
	do 50 i=1,maxshe
	rpbod(i)=rpbl(i)+rpbdif(i)
50      zbpith(i)=rpbi(i)+rbpth(i)
 
!plot initial and final density and temperature profiles:
	call initt(240)
	call binitt
	call fpgrlab
 
	call npts(nshell)
	call slimx(125,475)
	call slimy(125,700)
	call dlimx(0.,1.0)
	zymin=fppinf
	zymax=-fppinf
	call mnmx(rbtemp0,zymin,zymax)
	call mnmx(rbtemp1,zymin,zymax)
	call mnmx(rbtpar1,zymin,zymax)
	call dlimy(0.0,zymax)
	call symbl(ichar('i'))
	call check(rovera,rbtemp0)
	call dsplay(rovera,rbtemp0)
	call symbl(ichar('f'))
	call line(7474)
	call cplot(rovera,rbtemp1)
	call symbl(ichar('p'))
	call line(7474)
	call cplot(rovera,rbtpar1)
	call AGTcgt(735,'initial and final "Temperature"$')
	call AGTcgt(0, 'and final parallel "T"$')
	call AGTchs('r/a$')
	call AGTcvs('keV$')
 
!initial and final density on same page:
	zntot0=fpvint(rbden0)
	zntot1=fpvint(rbden1)
 
	call binitt
	call npts(nshell)
	call slimx(625,975)
	call slimy(125,700)
	call dlimx(0.,1.0)
	zymin=1.e20
	zymax=-1.e20
	call mnmx(rbden0,zymin,zymax)
	call mnmx(rbden1,zymin,zymax)
	call dlimy(0.0,zymax)
	call symbl(ichar('i'))
	call check(rovera,rbden0)
	call dsplay(rovera,rbden0)
	call symbl(ichar('f'))
	call line(7474)
	call cplot(rovera,rbden1)
	call AGTcgt(735,'initial & final density n(r)$')
	write(string,85) zntot0,zntot1
85	format('Ntot0=',1pe8.2,' Ntot1=',1pe8.2,'$')
	call AGTcgt(0,string)
	call AGTcvs('cm**-3$')
	call AGTchs('r/a$')
	call upause(0)
 
!plot beam current density profile:
	call binitt
	call erase
	call fpgrlab
 
	call npts(nshell)
	call slimx(150,500)
	call slimy(125,700)
	call dlimx(0.,1.0)
	zymin=1.e20
	zymax=-1.e20
	call mnmx(rbj0,zymin,zymax)
	call mnmx(rbj1,zymin,zymax)
	if(zymax .gt. 0.0 .and. zymin .gt. 0.0)zymin=0.0
	if(zymin .lt. 0.0 .and. zymax .lt. 0.0)zymax=0.0
	call dlimy(zymin,zymax)
	call symbl(ichar('i'))
	call check(rovera,rbj0)
	call dsplay(rovera,rbj0)
	call symbl(ichar('f'))
	call line(7474)
	call cplot(rovera,rbj1)
	call AGTcgt(0,'beam current density$')
	call AGTchs('r/a$')
	call AGTcvs('amps/cm^2$')
 
!integrate over the plasma cross section to find the total beam current:
	zsum0=0.0
	zsum1=0.0
	do 100 i=1,nshell
 
!find minor radius step size dr/a.  These formulas are good
! for a non-uniform radial grid, assuming that the zone boundaries lie
! half way between the zone centers (except at the origin and the edge).
 
! zm = inner boundary of the zone
! zp = outer boundary of the zone
 
	if(i .eq. 1)then
	zm=0.0
	zp= 0.5*(rovera(1)+rovera(2))
	else if(i .eq. nshell)then
	zm= 0.5*(rovera(nshell)+rovera(nshell-1))
	zp=1.0
	else
	zm=0.5*(rovera(i)+rovera(i-1))
	zp=0.5*(rovera(i)+rovera(i+1))
	endif
! area of this shell:
	zdarea=3.14159*rmin**2*(zp**2-zm**2)
	zsum0=zsum0+rbj0(i)*zdarea
100     zsum1=zsum1+rbj1(i)*zdarea
 
	call AGTmhs(550,600,'Total toroidal beam current:$')
	write(string,101)' Initial = ',zsum0,' Amps$'
101     format(a,1pe10.3,a)
	call AGTmhs(550,575,string)
	write(string,101)' Final   = ',zsum1,' Amps$'
	call AGTmhs(550,550,string)
	call upause(0)
 
!find total power flows (divide by 1000 to get Kilowatts):
	tr=FPVINT(rpbrf)/1000.
	tr0=FPVINT(rpbrf0)/1000.
	ti=FPVINT(zbpith)/1000.
	te=FPVINT(rpbe)/1000.
	tl=FPVINT(rpbl)/1000.
	tc=FPVINT(rpbc)/1000.
	ts=FPVINT(rpbs)/1000.
	tdw=FPVINT(rpbdw)/1000.
	tradif=FPVINT(rpbdif)/1000.
 
!Plot radial power balance:
	call binitt
	call erase
	call fpgrlab
 
	call npts(nshell)
	call slimx(150,500)
	call slimy(125,700)
	call dlimx(0.,1.0)
	pmin=1.e20
	pmax=-1.e20
	call mnmx(rpbrf,pmin,pmax)
	call mnmx(rpbrf0,pmin,pmax)
	call mnmx(zbpith,pmin,pmax)
	call mnmx(rpbe,pmin,pmax)
	call mnmx(rpbod,pmin,pmax)
	call mnmx(rpbc,pmin,pmax)
	call dlimy(pmin,pmax)
	call steps(nshell/2)
	call symbl(ichar('0'))
	call check(rovera,rpbrf0)
	call dsplay(rovera,rpbrf0)
	call symbl(ichar('R'))
	call cplot(rovera,rpbrf)
	call line(7474)
	call symbl(ichar('i'))
	call cplot(rovera,zbpith)
	call line(5454)
	call symbl(ichar('e'))
	call cplot(rovera,rpbe)
	call line(3434)
	call symbl(ichar('L'))
	call cplot(rovera,rpbod)
	call line(1414)
	call symbl(ichar('C'))
	call cplot(rovera,rpbc)
 
	call AGTcgt(0,'Prfnb0, Prfnb, Pi, Pe, Ploss, Pcx$')
	call AGTchs('r/a$')
	call AGTcvs('(Watts/cc)$')
 
!write total power balance results:
	call AGTmhs(550,625,'Global power balance:$')
	write(string,701)tr0,' KW initial RF+NB power$'
701     format(f7.0,a)
	call AGTmhs(550,600,string)
	write(string,701)tr,' KW final RF+NB power$'
	call AGTmhs(550,575,string)
	write(string,701)ti,' KW to ions$'
	call AGTmhs(550,550,string)
	write(string,701)te,' KW to electrons$'
	call AGTmhs(550,525,string)
	write(string,701)tl,' KW bad orbit losses$'
	call AGTmhs(550,500,string)
	write(string,701)tc,' KW CX losses$'
	call AGTmhs(550,475,string)
	write(string,701)ts,' KW particle source input$'
	call AGTmhs(550,450,string)
	write(string,701)tdw,' KW d/dt stored energy$'
	call AGTmhs(550,425,string)
	write(string,701)tradif,' KW radial diffusion$'
	call AGTmhs(550,400,string)
	residu=tr+ts-tdw-ti-te-tl-tc-tradif
	write(string,701)residu,' KW residual (OH,$'
	call AGTmhs(550,375,string)
	call AGTmhs(550,350,'	   f<0 and roundoff)$')
	call upause(0)
 
c plot out Inverse k_perp rho correction needed to make ICRF
c powers calculated by FPP and SPRUCE agree:
 
	if(prftot .gt. 0.0)then
	call binitt
	call erase
	call fpgrlab
 
	call npts(nshell)
	call slimx(150,500)
	call slimy(125,700)
	call dlimx(0.,1.0)
	pmin=1.e20
	pmax=-1.e20
	call mnmx(qldamp,pmin,pmax)
	call mnmx(rkpcpro,pmin,pmax)
	call dlimy(pmin,pmax)
	call ytype(2)	!log scale for the y-axis
	call symbl(ichar('q'))
	call check(rovera,qldamp)
	call dsplay(rovera,qldamp)
	call symbl(ichar('r'))
	call cplot(rovera,rkpcpro)
 
	call AGTcgt(735,'FLR correction P_FPP/P_Spruce$')
	call AGTcgt(0,'qldamp and residual rkpcpro$')
	call AGTchs('r/a$')
	call AGTcvs('$')
 
	call AGTmhs(550,625,'Average k_perp rho correction$')
	write(string,702) '     =', rkpcorr
702	format(a,1pe10.3,'$')
	call AGTmhs(550,600,string)
	call AGTmhs(550,550,'qldamp = estimated <J0**2> for ')
	call AGTmhs(550,525,'   the dielectric tensor found ')
	call AGTmhs(550,500,'   from the previous Fokker-')
	call AGTmhs(550,475,'   Planck iteration.')
	call AGTmhs(550,425,'rkpcpro = residual <J0**2>')
	call AGTmhs(550,400,'   found from the present')
	call AGTmhs(550,375,'   Fokker-Planck iteration.')
 
	call upause(0)
	endif
 
!type out total power balance results:
	call finitt(0,780)
	write(lsysout,*) '   '
	write(lsysout,*)
     1  '*************************************************'
	write(lsysout,*)
     1  'Power balance on each shell (all values in watts/cc):'
	write(lsysout,*)
     1  'dW/dt =Prfnb+Psource-Pic -Pith -Pe -Ploss -Pcxr - Pradif -Perr'
	write(lsysout,*) '     '
	write(lsysout,*) 'i(r) dW/dt     Prfnb0     Prfnb'
     >        //'      Psource    Pic        Pith'
	write(lsysout,*)
	2    '       Pe         Ploss      Pcx        Pradif     Perr'
 
	do i=nshsta,nshsto
	error=rpbrf(i)+rpbs(i)-zbpith(i)-rpbe(i)-rpbl(i)-rpbc(i)-rpbdw(i)
	1	-rpbdif(i)
	write(lsysout,755) i,rpbdw(i),rpbrf0(i),rpbrf(i),rpbs(i),
	1    rpbi(i),rbpth(i),rpbe(i),rpbl(i),rpbc(i),
	1    rpbdif(i), error
755     format(1x,i2,6(1x,1pe10.3),/,4x,5(1x,1pe10.3))
	enddo
 
	write(lsysout,*) '    '
	write(lsysout,*) 'Global power balance:'
	write(lsysout,701) tr0,' KW initial RF+NB power$'
	write(lsysout,701) tr,' KW final RF+NB power$'
	write(lsysout,701) ti,' KW to ions$'
	write(lsysout,701) te,' KW to electrons$'
	write(lsysout,701) tl,' KW bad orbit losses$'
	write(lsysout,701) tc,' KW CX losses$'
	write(lsysout,701) ts,' KW particle source input$'
	write(lsysout,701) tradif,' KW radial diffusion$'
	write(lsysout,701) tdw,' KW d/dt stored energy$'
	residu=tr+ts-tdw-ti-te-tl-tc-tradif
	write(lsysout,701) residu,' KW residual (OH,$'
	write(lsysout,*) '	   f<0 and roundoff)$'
	call upause(0)
 
	return
	end
