	SUBROUTINE PROF
C
C
c This program calculates ne, Te, Ti, T0 and n0 profiles from
c the information provided in the namelist input file.
C
C     SET UP ARRAYS AND COMMON BLOCKS
C
	include 'FPPCOM'
	include 'HE4COM'
	include 'FPONCOM'  ! standalone FPP-only common block
 
	character string*80
	real cxtime(maxshe)
	real zJac(2,2)
	real zRmajb(maxshe)  ! Volume/area Rmaj at zone boundaries
 
 
c compress a few main variables now if needed:
	if(cratio .ne. 1.00)then
 
	Rmaj=Rmaj/cratio
	rmin=rmin/cratio**.5
	Btor=Btor*cratio
	Placur=placur*cratio
c approximate compression for general geometry for compatibility:
	prmoma(0)=prmoma(0)/cratio
	fpshift0=fpshift0/cratio**.5
	do 5 im=1,mxgmom
	prmoma(im)=prmoma(im)/cratio**.5
5	pzmoma(im)=pzmoma(im)/cratio**.5
	do 7 i=1,maxshe
	prmom(i,0)=prmom(i,0)/cratio
	do 7 im=1,mxgmom
	prmom(i,im)=prmom(i,im)/cratio**.5
7	pzmom(i,im)=pzmom(i,im)/cratio**.5
 
	endif
 
c set up radial zones, xradzb(i) is the outer boundary of zone i.
c our radial coordinate x is sqrt(toroidal flux) normalized to
c the edge value, so that x=0 at the center, and x=1 at the edge.
c For now use a uniformly spaced xradzb grid:
	do 10 i=1,nshell
10	xradzb(i)=float(i)/float(nshell)
 
c The rest of the code does not make any assumptions about
c whether or not xradzb is uniformly spaced, allowing us to
c pack more radial zones where we need them.
c
 
c initialize general geometry specification:
c if the full geometry specification has not been made,
	if(prmom(nshell,0) .eq. 0.0) then
c use the simplest geometry model if necessary:
	if(prmoma(0) .eq. 0.0) then
	do 12 im=0,mxgmom  ! make sure they are all zeroed.
	prmoma(im)=0.0
12	pzmoma(im)=0.0
	prmoma(0)=rmaj
	prmoma(1)=rmin/fpellip**0.5
	pzmoma(0)=0.0
	pzmoma(1)=rmin*fpellip**0.5
	expmom=1.0
	fpellip0=fpellip
	endif
 
c or use the general edge moments geometry model:
	zea=pzmoma(1)/prmoma(1)	!edge ellipticity
	do 15 i=1,nshell
	prmom(i,0)=prmoma(0)+fpshift0*(1-xradzb(i)**2)**expmom
c local ellipticity:
	ze=zea+(fpellip0-zea)*(1-xradzb(i)**2)**expmom
	prmom(i,1)=prmoma(1)/(ze/zea)**0.5 *xradzb(i)
	pzmom(i,1)=pzmoma(1)*(ze/zea)**0.5 *xradzb(i)
	do 14 im=2,mxgmom
	prmom(i,im)=prmoma(im) - prmoma(im)*(1-xradzb(i)**2)**expmom
14	pzmom(i,im)=pzmoma(im) - pzmoma(im)*(1-xradzb(i)**2)**expmom
15	continue
	do 16 im=0,mxgmom
	prmom(0,im)=0.0
16	pzmom(0,im)=0.0
	prmom(0,0)=prmom(1,0)
 
	endif
 
c determine how many geometry moments are actually being used:
	do 18 im=mxgmom,1,-1
	ngmom=im
	do 18 i=1,nshell
	if(prmom(i,im) .ne. 0.0 .or. pzmom(i,im) .ne. 0.0) goto 19
18	continue
19	continue
 
c find radial zone centers:
	DO 20 I=1,NSHELL
	z1=0.0
	if(i .ne. 1) z1=xradzb(i-1)
	xradzc(i)=(z1+xradzb(i))/2.	! radial zone centers
20	continue
 
c find the toroidal flux enclosed by the plasma assuming that
c R*Btoroidal = constant, and find the volumes of each radial zone.
 
	tflxtot=0.0
	zpi=abs(acos(-1.0))
	inumth=max(nshell*2*4,80)
	zdth=zpi/inumth
 
	do 30 i=1,nshell
	dVolfp(i)=0.0
	dAreafp(i)=0.0
	zdx=xradzb(i)
	if(i .gt. 1)zdx=xradzb(i)-xradzb(i-1)
 
	do 30 j=1,inumth
	zth=(j-0.5)*zpi/inumth
	call momry(xradzc(i),zth,1,zR,zY,zJac)
	zJ=zjac(1,1)*zjac(2,2)-zjac(2,1)*zjac(1,2)
	dVolfp(i)=dVolfp(i)+2*zdx*zdth*2*zpi*zR*zJ
	dAreafp(i)=dAreafp(i)+2*zdx*zdth*zJ
	tflxtot=tflxtot +2*zdx*zdth*zJ*(Btor*Rmaj/zR)
30	continue
 
	Volfp(1)=dVolfp(1)
	Areafp(1)=dAreafp(1)
	do 33 i=2,nshell
	Volfp(i)=Volfp(i-1)+dVolfp(i)
33	Areafp(i)=Areafp(i-1)+dAreafp(i)
 
	do 35 i=1,nshell
35	zRmajb(i)=Volfp(i)/(2*zpi*Areafp(i))
 
	Rmajpro(1)=zRmajb(1)-0.5*xradzb(1)*
	1   (zRmajb(2)-zRmajb(1))/(xradzb(2)-xradzb(1))
 
	do 37 i=2,nshell
37	Rmajpro(i)=(zRmajb(i)+zRmajb(i-1))/2.	
 
c define rmaj and rmin from their volume and area definitions:
	zbr=Btor*rmaj
	Rmaj=zRmajb(nshell)
	Btor=zbr/rmaj
	rmin=sqrt(Areafp(nshell)/zpi)
	do 38 i=1,nshell
	rminpro(i)=xradzc(i)*rmin
c rovera is an old variable and is calculated for the parts of
c the code which have not yet been upgraded to general geometry
c and are still assuming circular plasmas:
38	rovera(i)=rminpro(i)/rmin
 
c also find q (the inverse rotational transform) at the edge:
c the q calculation is correct even for R*BToroidal not constant.
	if(qpro(nshell) .ne. 0.) then
	qlim=qpro(nshell)
	qax=qpro(1)
        QRAT=QLIM/QAX
	else
 
	qlim=0.0
	inumth=max(nshell*2*4,80)
	zdth=zpi/inumth
	do 40 i=1,inumth
	zth=(i-0.5)*zpi/inumth
c ensure this is evaluated slightly inside the boundary:
	call momry(0.9999,zth,1,zR,zY,zJac)
	zJ=zjac(1,1)*zjac(2,2)-zjac(2,1)*zjac(1,2)
	qlim=qlim+zdth*(zjac(1,2)**2+zjac(2,2)**2)/zJ/zR
40	continue
	qlim=qlim*2*tflxtot/(zpi*zpi*4*100.*placur)
 
c calculate q profile:
c--this simple model for q was originally derived for concentric
C circular flux surfaces, and assumes a current profile which is
C parabolic to the x power, where x=(qlim/q0)-1.
c
c For a circular plasma:
c
c       q = r Btor / (R Bpol)
c       qrat=qlim/qax = q (at limiter)/ q(on axis)
c
c       J(r)=j0*(1-(r/a)**2)**(qrat-1)
c
c       rova=r/a
c       q(r)=qlim rova**2 / (1 - (1-rova**2)**qrat)
c
c first find q at the limiter and qrat=q@limiter/q@axis:
C
c this was the cylindral limit (qlim for general geometry has already
c been calculated):
c       QLIM=RMIN**2*BTOR/(200.*PLACUR*RMAJ)
c
c We have extended this q model to general geometry in the obvious way:
 
        QRAT=QLIM/QAX
	do 55 i=1,nshell
	zra=rminpro(i)/rmin
	DENOM=1.0-(1.0-zra**2)**QRAT
55	qpro(i)=QLIM*zra**2/DENOM
 
	endif
 
c default value for nfptherm (used in calculating fusion rates):
	if(nfptherm .eq. 0) nfptherm=nions
	do i=1,nions
	   nbbfus(i)=0  ! in standalone FPP calculate even beam-beam fusion
	enddo
 
c
c If (Te0 .ne. 0.0), then use parabolic models for various plasma
c parameters:
c
c Te(r) = (Te0-Tea)*(1-r**2/a**2)**Teexp + Tea
c
	if(Te0 .ne. 0.0) then
 
c adjust main ion and main impurity ion fractions to match Zeff
c if it is set:
	if(rZeff .gt. 0.0) then
 
c first find the main impurity ion (with the largest Z):
	iimp=0
	zimp=0.0
	do 60 i=2,nions
	    if(nZions(i) .gt. zimp)then
		zimp=nZions(i)
		iimp=i
	    endif
60	continue
 
c then use quasineutrality and the definition of Zeff:
	z1=0.0
	z2=0.0
	do 70 i=2,nions
	    if(i .eq. iimp) goto 70
	    z1=z1+rations(i)*nZions(i)
	    z2=z2+rations(i)*nZions(i)**2
70	continue
	rations(1)=(Zimp-rZeff+z2-z1*Zimp)/(Zimp-nZions(1))/nZions(1)
	rations(iimp)=(1-z1-rations(1)*nZions(1))/Zimp
	write(lsysout,*) 'ion mix ni/ne:',(rations(i),i=1,nions)
	endif
 
c end of section using Zeff to adjust ion fractions.
 
c now use the parabolic-to-a-power model for various profiles:
	do 200 i=1,nshell
	x2=rovera(i)**2
	Trelec(i)=(Te0-Tea)*(1-x2)**Teexp + Tea
	drelec(i)=(delec0-deleca)*(1-x2)**delecexp + deleca
 
	do 190 iion=1,nions
	Trions(i,iion)=(Ti0(iion)-Tia(iion))*(1-x2)**Tiexp(iion)
	1   +Tia(iion)
	Trionspa(i,iion)=Trions(i,iion)
190	drions(i,iion)=rations(iion)*drelec(i)
200	continue
 
	endif
 
c check quasineutrality:
 
	do 210 i=1,nshell
	zne=0.0
	do 205 iion=1,nions
205	zne=zne+drions(i,iion)*nZions(iion)
	if( abs(zne-drelec(i)) .gt. .001*drelec(i)) then
	write(string,*)
	1   'error in prof:  quasineutrality violated on shell ',i
	call aborter(lsysout,string)
	endif
210	continue
 
c set parallel Temperature = average Temperature if not set:
	if(Trionspa(1,1) .lt. 0.0) then
	   do iion=1,nions
	      do i=1,nshell
		 Trionspa(i,iion)=Trions(i,iion)
	      enddo
	   enddo
	endif
 
c do compression:
	if(cratio .ne. 1.0)then
	do 410 i=1,nshell
	trelec(i)=trelec(i)*cratio**(4./3.)
	drelec(i)=drelec(i)*cratio**2.
	do 410 iion=1,nions
	trions(i,iion)=trions(i,iion)*cratio**(4./3.)
	trionspa(i,iion)=trions(i,iion)
410	drions(i,iion)=drions(i,iion)*cratio**2.
	endif
 
c****************************************************************
c for compatibility with older parts of the code which still
c use the older profile data, define the older profile data
c (tepro,tipro,denpro,rzeff,rzimp, etc...) here.  After the whole
c code is upgraded to the new multi-ion model, the variables defined
c here (down to the next row of "**********") will no longer be
c needed.
 
	nshellfr=nshell
	rminfr=rmin
 
	do 4105 ish=1,nshell
	tepro(ish)=trelec(ish)
	denpro(ish)=drelec(ish)/1.e13	
4105	tipro(ish)=trions(ish,1)  ! use thermal Ti (no beam-beam)
 
	ab=nAions(nfasti)
	rZbeam=nZions(nfasti)
 
	zne=fpvint(drelec)
	znb=fpvint(drions(1,nfasti))
	cbeam=znb/zne
c--concmi defaults to cbeam except for 2cd harmonic heating
	if(concmi .le. 0.0 .or. nharm .eq. 1)concmi=cbeam
 
c find the main ion species
c for which FRANTIC will calculate a neutral density profile.
        imain=1
        do 4106 iion=1,nions
4106    if(drions(1,iion) .gt. drions(1,imain))imain=iion
        AP=nAions(imain)
        aZmain=nZions(imain)
	aZneut=nZions(nfasti)
 
c calculate a volume-averaged Zeff:
	zeff=0.0
	do 4107 iion=1,nions
4107	zeff=zeff+nZions(iion)**2 *fpvint(drions(1,iion))	
	rZeff=zeff/fpvint(drelec)
	write(lsysout,*) 'volume average Zeff =',rZeff
 
c calculate the average impurity ion Z to give the proper depletion
c of the "main" ion species:
	zfmain=nZions(imain)*fpvint(drions(1,imain))/fpvint(drelec)
	rZimp=(rZeff-zfmain*nZions(imain))/(1.0-zfmain)
	Aim=rZimp*2.0
 
C
C A list of plasma species parameters (including electrons):
c
	APlasAMU(1)=5.4858026e-04    ! electrons are first in this list
        APlasAMU(2)=AP
        APlasAMU(3)=AIM
	AplasZ(1)=-1
        aplasz(2)=aZmain
        aplasz(3)=rZimp
        NumPlSp=3
        IF(rZeff.LT.1.001) NumPlSp=2
 
c*****************************************************************
 
c--calculate neutral density profile:
	IF(LAntic .EQ. 1) CALL GNEUT(AP,rZimp,rZeff,AIM,AN00,TEDG)
	if(Lantic .eq. 2) then
	write(lsysout,*) 'Warning:  option LANTIC=2 no longer working'
	write(lsysout,*) ' will use GNEUT instead of GNUSUB to'
	write(lsysout,*) ' calculate the neutral density profiles.'
c	CALL GNUSUB
	CALL GNEUT(AP,rZimp,rZeff,AIM,AN00,TEDG)
	endif
 
c--dummy neutral density profile:
	if(Lantic .eq. 3) then
	do 307 i=1,nshell
	neupro(i)=an00
307	tnupro(i)=tedg
	endif
 
 
C Setup up FPP's velocity space (energy and xsi) grid.
c Depo, detech, and detecv need to know the energy and xsi grids.
c They only work if the energy and xsi grids are the same for all
c minor radii:
	call ingrid(.1)
	write(lsysout,'(1x,a,1pe10.3,a)')
     >     'Energy grid parameters:  minimum dE=',enerev(1),' eV'
	write(lsysout,'(1x,a,1pe10.3,a)')
     >     '                     Maximum Energy=',enerev(maxen),' eV'
 
c First write a summary page of text to the PLOT file:
	call initt(120)
	call binitt
	call fpgrlab
 
	iy=700 ! Y position to write a string
	call agtmhs(10,iy,
	1 'FPP Bounce-Averaged Fokker-Planck calculation (Hammett)$'
	1 ,0)
	iy=iy-25
	if(lshoot .eq. 1 .or. lshoot .eq. -1)then
	call agtmhs(10,iy,
	1 'SPRUCE ICRF Heating with full FLR (Evrard, Smithe, et.al.)$'
	1 ,0)
	else if(lshoot .eq. 2)then
	call agtmhs(10,iy,
	1 'SHOOT ICRF Fast-wave propagation (Smithe and Colestock)$'
	1 ,0)
	endif
	iy=iy-50
 
	write(string,315)Rmaj,rmin,tflxtot
315	format('Rmaj/rmin=',1pe9.2,'/',1pe9.2,' (cm)   Tor. flux=',
	1   1pe9.2,' (G cm**2)$')
	call agtmhs(10,iy,string,0)
	write(string,317)Btor,Placur,qlim
317	format('Btor=',1pe9.2,' (G)   Placur=',1pe9.2,' (kA)   q_mhd=',
	1   1pe9.2,' $')
	iy=iy-25
	call agtmhs(10,iy,string,0)
	iy=iy-50
 
	do 330 i=1,mlincomm
	ilen=nchar(comments(i))
	if(ilen .lt. 1) goto 335
	call agtmhs(10,iy,comments(i),0)
330	iy=iy-25
335	continue
 
	call upause(0)
 
! plot electron and ion temperatures:
! first convert to KeV for the plot:
	do 420 i=1,nshell
	trelec(i)=trelec(i)/1000.
	tnupro(i)=tnupro(i)/1000.
420	tipro(i)=tipro(i)/1000.
c--erase screen:
	call binitt
	call erase
	call fpgrlab
 
	call place('ul4  ')	!Put graph in upper left corner
	CALL NPTS(NSHELL)
	zymin=1.e20
	zymax=-1.e20
	call mnmx(trelec,zymin,zymax)
	call mnmx(tipro,zymin,zymax)
	call mnmx(tnupro,zymin,zymax)
	call dlimy(0.0,zymax)
	call xneat(0) !use exact x limits:
	call dlimx(0.0,rmin)
	call xfrm(4) !supress grid lines
	call yfrm(4) !supress grid lines
	CALL CHECK(rminpro,trelec)
	CALL DSPLAY(rminpro, trelec)
	call frame  !@
c@	call dframe
	call line(7474)
	call cplot(rminpro,tipro)
	call line(5454)
	call cplot(rminpro,tnupro)
	call AGTcgt(0,'Te, Ti, T0 (KeV)$')
	CALL AGTCHS('r (cm)$')
 
! convert te and ti back to eV for the rest of the code:
	do 430 i=1,nshell
	trelec(i)=trelec(i)*1000.
	tnupro(i)=tnupro(i)*1000.
430	tipro(i)=tipro(i)*1000.
 
! plot electron density:
	call binitt
	call place('ur4  ')	!graph in upper right corner	
	CALL NPTS(NSHELL)
	zymin=0.0
	zymax=0.0
	call mnmx(denpro,zymin,zymax)
	call dlimy(zymin,zymax)
	call xneat(0) !use exact x limits:
	call dlimx(0.,rmin)
	call xfrm(4) !supress grid lines
	call yfrm(4) !supress grid lines
	CALL CHECK(rminpro, DENPRO)
	CALL DSPLAY(rminpro,DENPRO)
c draw a frame around the graph:
	call frame !@
c@	call dframe
	call AGTcgt(0,'ne (10**13/cc)$')
	call AGTchs('r (cm)$')
 
 
	CALL BINITT
	call place('ll4  ')
	CALL NPTS(NSHELL)
	CALL YTYPE(2)
	zymin=fppinf
	zymax=-fppinf
	call mnmx(NEUPRO,zymin,zymax)
	IF(zymin.EQ.zymax) zymin=zymax*.5
	CALL DLIMY(zymin,zymax)
	call xneat(0) !use exact x limits:
	call dlimx(0.,rmin)
	call xfrm(4) !supress grid lines
	call yfrm(4) !supress grid lines
	CALL CHECK(rminpro,NEUPRO)
	CALL DSPLAY(rminpro,NEUPRO)
c draw a frame around the graph:
	call frame !@
c@	call dframe
	call AGTcgt(0,'Calculated n0 (1/cc)$')
	call AGTchs('r (cm)$')
 
	write(string,600)Taup
600	format('Taup = ',e9.2,' secs$')
	call AGTcgt(375,string)
 
! calculate and plot some usefull parameters, like the slowing
! down time and the CX time:
 
! main plasma ion density / electron density =
        DHY=(rZimp-rZeff)/(rZimp-aplasz(2))/aplasz(2)
! impurity ion density / electron density =
        DIM=(rZeff-aplasz(2))/((rZimp-aplasz(2))*rZimp)
	zovera=dhy*aplasz(2)**2/Ap + dim*rZimp**2/Aim
 
	do 700 i=1,nshell
 
	if(llstix) then
c exact Stix formula for benchmarking with Stix:
	    alle=24. -alog( drelec(i)**.5 / trelec(i))
	else
c Slightly better formula for log(lambda), (best formulas are in
c the subroutine COULOG.FOR):
	    alle=24.8-alog( drelec(i)**.5 / trelec(i))
	endif
 
	tause(i)=6.32e8*ab*trelec(i)**1.5
     >            /(drelec(i)*rZbeam**2 *alle)
700	Ecrit(i) = Ab * 14.8 * trelec(i) * Zovera**(2./3.)
 
!charge exchange time
	do 720 i=1,nshell
	Erel=1.5*trions(i,nfasti)/Ab
	tnus=tnupro(i)
	zrate=neupro(i)*sigvbt(erel,Tnus/Ap,'CX')
c CXTIME is only used here for plotting.  If the CX rate is
c infinite, set CXTIME=TAUSE, just so the cxtime disappears from
c the plot.  The CXRATE used in the Fokker-Planck equation gets
c recalculated in subroutine FPCF anyway:
	if(zrate .ne. 0.0) then
	    cxtime(i)=1./zrate
	else
	    cxtime(i)=tause(i)
	endif
720	continue
 
	call binitt
	call place('lr4  ')
	call npts(nshell)
	call ytype(2)
	zymin=fppinf
	zymax=-fppinf
	call mnmx(tause,zymin,zymax)
	call mnmx(cxtime,zymin,zymax)
	call dlimy(zymin,zymax)
	call xneat(0) !use exact x limits:
	call dlimx(0.,rmin)
	call xfrm(4) !supress grid lines
	call yfrm(4) !supress grid lines
	call check(rminpro,tause)
	call dsplay(rminpro,tause)
c draw a frame around the graph:
	call frame !@
c@	call dframe
	call AGTcgt(370,'Slowing down time (secs)$')
	call AGTchs('r (cm)$')
	call line(7474)
	call cplot(rminpro,cxtime)
	call AGTcgt(0,'CX time (secs)$')
 
	call upause(0)
 
c	call finitt(0,0) !return to text screen for any messages...
c finitt now called from fpgeopl:
	call fpgeopl  !plot FPP VV and plasma geometry
 
	RETURN
	END
