	SUBROUTINE RadNeo
!
c dmc -- fpdifcon model enhanced by allowing radially varying anomolous
c  diffusivity -- diffusivity profile in fpdifca(...); fpdifcon still used
c   for control purposes (fpdifcon.gt.0.0 turns on use of fpdifca)
c
c remainder of comments by Greg Hammett:
c
! I have tested this subroutine a fair amount.  The fpdifcon and
! fpdiftra models work o.k.  The neoclassical model may need checking.
! In particular, there may be some associated neoclassical convective
! term because I am using a non-standard representation of the
! convective-diffusive equation.
!
! Adds the neoclassical terms to the coefficients of the radial
! transport operator.  I can't do this in the original RDOPER where
! the ripple transport terms are calculated because btime = btlam didn't
! exist until after one loop over all shells so that RDJACO gets called.
!
! evaluates the coefficients in the radial transport operator:
!
!	d f             1           1  d   [
!       ---  =    --------------- * -  --- [ - vrnorm(lambda,energy,r) f
!       d t       btime(lambda,r)   r  d r [
!
!
!					     d	 (          )    ]
!					   + --- ( Drnorm f )    ]
!					     d r (          )    ]
!
!
! Where vrnorm = v_r * r * btime
!   and Drnorm = D_r * r * btime
!
! Greg Hammett 31-Oct-1987.
!
	include 'FPPCOM'
 
	if(XneoMult .eq. 0.0 .and. fpdiftra .eq. 0.0) goto 500
 
c***************************************************************
c
c Add in a constant D for trapped particles, and add in a
c neoclassical model for D for trapped particles.
c
c***************************************************************
 
	do 100 ishell=1,nshell
	do 200 ilam=1,MaxLam
 
!verify that this orbit is trapped and is accessible to this flux surface:
	ZRtip = abs(rlambda(ilam))*Btor*Rmaj
	if (ZRtip .lt. Rmaj-rovera(ishell)*rmin
	1 .or. ZRtip .gt. Rmaj+rovera(ishell)*rmin)go to 200
 
!skip transport for 0 energy particles:
	do 300 ien=1,maxen
	if(ishell .gt. nshemax(ilam,ien))go to 300
 
	zr=rmin*rovera(ishell)
 
c Add in simplest neoclassical transport term to Drnorm.  (I am still
c a bit confused about whether there should be a corresponding Vrnorm
c term!):
        denom=1.0-(1.0-(rovera(ishell))**2)**qrat
        qloc=qlim*(rovera(ishell))**2/denom
	zBpol=Btor*zr/Rmaj/qloc		
	ZOmtheta=9.58e3*zBpol*rZbeam/Ab
	zv=sqrt(2*9.579e11*enerev(ien)/Ab)
	zlnlam=24.8-alog((denpro(ishell)*1.e13)**.5/Tepro(ishell))
	znuii=9.016e-8*rZbeam**2*rZeff*denpro(ishell)*1.e13*zlnlam
	1   /Ab**.5/enerev(ien)**1.5
 
c default for XneoMult=0.0 makes FPPRF upwards compatible:
	Drnorm(ilam,ien,ishell)=Drnorm(ilam,ien,ishell)+
	1   XneoMult*zr*btlam(ilam,ishell)* znuii*zv**2/2/zOmtheta**2
 
C add in a constant diffusion (because I have written in the form of a
C random-walk equation in "r*btlam"-space, there are convection and diffusion
C terms):
 
c fpdiftra is a constant radial diffusion coefficient applied to
c trapped particles only:
 
	Drnorm(ilam,ien,ishell)=Drnorm(ilam,ien,ishell)+
	1   zr*btlam(ilam,ishell)* abs(fpdiftra)
 
	if(ishell .lt. nshell)then
	    zdrp=rminpro(ishell+1)-rminpro(ishell)
	    zbtp=btlam(ilam,ishell+1)
	else
	    zdrp=rmin-rminpro(ishell)
	    zbtp=btlam(ilam,ishell)
	endif
 
	Vrnorm(ilam,ien,ishell)=Vrnorm(ilam,ien,ishell)+
	1   abs(fpdiftra)*( (zr+zdrp)*zbtp - zr*btlam(ilam,ishell))/
	1   zdrp
 
c add convection to barely-passing particles to balance the diffusion
c into them from barely-trapped particles:
 
	if(ishell .gt. 1)then
	if (ZRtip .lt. Rmaj-rminpro(ishell-1))then
	Vrnorm(ilam,ien,ishell-1)=Vrnorm(ilam,ien,ishell-1)+
	1   abs(fpdiftra)*( zr*btlam(ilam,ishell) )/
	1   (rminpro(ishell)-rminpro(ishell-1))
	endif
	endif
 
300	continue !end of ien loop
 
200 	continue !end of ilam loop
100	continue !end of ishell loop
 
500	continue
 
c***************************************************************
c
c Add in a constant D for all particles, trapped and passing.
c
c***************************************************************
 
	if(fpdifcon .eq. 0.0) return
 
	do 600 ishell=1,nshell
	do 800 ilam=1,MaxLam
 
c verify that this orbit is accessible to this flux surface:
	ZRtip = abs(rlambda(ilam))*Btor*Rmaj
	if (ZRtip .gt. Rmaj+rovera(ishell)*rmin)go to 800
 
!skip transport for 0 energy particles:
	do 900 ien=1,maxen
	if(ishell .gt. nshemax(ilam,ien))go to 900
 
	zr=rmin*rovera(ishell)
 
C add in a constant diffusion (because I have written in the form of a
C random-walk equation in "r*btlam"-space, there are convection and diffusion
C terms):
c
c fpdifcon is a constant radial diffusion coefficient
c ...replaced by fpdifca(...) a radially varying diffusion coefficient
c ...dmc Jan 1995
c
	zdf=abs(fpdifca(ishell))
	Drnorm(ilam,ien,ishell)=Drnorm(ilam,ien,ishell)+
	1   zr*btlam(ilam,ishell)* zdf
 
	if(ishell .lt. nshell)then
	    zdrp=rminpro(ishell+1)-rminpro(ishell)
	    zbtp=btlam(ilam,ishell+1)
	    zdfp=abs(fpdifca(ishell+1))
	else
	    zdrp=rmin-rminpro(ishell)
	    zbtp=btlam(ilam,ishell)
	    zdfp=abs(fpdifca(ishell))
	endif
 
	Vrnorm(ilam,ien,ishell)=Vrnorm(ilam,ien,ishell)+
	1   ( zdfp*(zr+zdrp)*zbtp - zdf*zr*btlam(ilam,ishell)) / zdrp
 
900	continue !end of ien loop
 
800 	continue !end of ilam loop
600	continue !end of ishell loop
 
	RETURN
	END
