	SUBROUTINE rdoper
!
! evaluates the coefficients in the radial transport operator:
!
!	d f             1           1  d   [
!       ---  =    --------------- * -  --- [ - Vrnorm(lambda,energy,r) f
!       d t       btime(lambda,r)   r  d r [
!
!
!					     d	 (          )    ]
!					   + --- ( Drnorm f )    ]
!					     d r (          )    ]
!
!
! Where vrnorm = V_r * r * btime
!   and Drnorm = D_r * r * btime
!
! We are calculating V_r due to the asymmetric edge drag mechanism
! discussed in my thesis and my 1986 APS invited talk, and D_r due to the
! Goldston and Towner (J. Plasma Physics 26 (1981) 283) banana drift
! ripple diffusion process.
!
! Greg Hammett 26-Oct-1986.
!
! GWH 10-Nov-1989:  Actually, there are ripple contributions to
! V_r as well.  This is in part because of the form I have written
! the radial Fokker-Planck equation in, but also because at each
! ripple step there is a random walk in the vertical Y position only,
! and not in R.  To calculate V_r correctly, one must go to second
! order:
!
! r = sqrt(Y**2+R**2)
!
! delta r = (dr/dY) delta Y + (1/2) (d**2 r/dY**2) (delta Y)**2 + ...
!
! so the average < delta r > is not zero!
!
 
	include 'FPPCOM'
 
!used to communicate with Pphiorb:
	external Pphiorb
        common /zPphiCmn/ ZRtip, zmv,zPphi0  ! Rtip = R of banana tip
                                       ! zmv = m * v of particle
				       ! zPphi0 = Pphi at banana tip.
 
C DMC 22 FEB 1993  --  replacing IMSL rootfinder ZBREN with local
C  root finder ZEROIN.  ZEROIN needs machine epslon.
 
	data zzeps/0.0/
	if(zzeps.eq.0.0) zzeps=x02aje()
 
!define lambda grid.  lambda = mu/E is a constant of the motion during
! edge drag and ripple transport, while xsi is not conserved.  So it is
! useful to transform to this new coordinate.
 
	zBmin=Btor*Rmaj/(Rmaj+rmin)
 
	imaxlam2=maxlam/2
	if(imaxlam2*2 .ne. maxlam) call aborter(lsysout,
	1   'rdoper:  MaxLam must be even.')
 
c xsi: 0 --> -1, lambda: -(1/Bmin) --> 0
 
	do 420 ilam=1,imaxlam2
	zx=0.0-(ilam-1.0)/(imaxlam2-1.0)
420	rlambda(ilam)=-(1.0-zx**2)/zBmin
 
c xsi: 1 --> 0, lambda: 0 --> (1/Bmin)
 
	do 430 ilam=1,imaxlam2
	zx=1.0-(ilam-1.0)/(imaxlam2-1.0)
430	rlambda(ilam+imaxlam2)=(1.0-zx**2)/zBmin
 
 
! find nshemax(lambda,energy) = index of radius of last confined orbit
 
!initialize to harmless values:
	do 114 ishell=1,nshell
	do 113 ilam=1,MaxLam
	do 112 ien=1,maxen
	vrnorm(ilam,ien,ishell)=0.0
	Drnorm(ilam,ien,ishell)=0.0
	nshemax(ilam,ien)=nshell
112	CONTINUE
113	CONTINUE
114	CONTINUE
 
!find the maximum confined radius for each energy and pitch angle:
	if(.not. Lbador)goto 35
	do 30 ilam=1,MaxLam
	ZRtip = abs(rlambda(ilam))*Btor*Rmaj
	if (ZRtip .lt. Rmaj-rmin
	1 .or. ZRtip .gt. Rmaj+rmin)go to 30
 
	do 20 ien=1,maxen
	zmv = Ab * pmass * sqrt(2*9.579e11*enerev(ien)/Ab)
! 9.579e11 = k ( ergs/eV) / m_proton (grams)
 
! Psi(numpsi) is at the limiter.  Consider a particle lost if it comes
! to Psi(numpsi-1), to avoid "small difference of two almost identical
! numbers" error later on:
 
	psimax=Psi(numpsi-1)-zmv*(Rmaj+rmin)*sqrt(1.0-ZRtip/(Rmaj+rmin))
	if(psimax .le. 0.0)psimax=0.0
	rmax=Rpsi(1+int((numpsi-1)*Psimax/Psi(numpsi)+0.5))
20	nshemax(ilam,ien)=lkupr(rmax/rmin,rovera,nshell)
30	continue
 
35	continue
	
	do 100 ishell=1,nshell
 
c find local q(r) and dq/dr:
        denom=1.0-(1.0-(rovera(ishell))**2)**qrat
        qloc=qlim*(rovera(ishell))**2/denom
	zr=rmin*rovera(ishell)
	zrovera2=rovera(ishell)**2
	zdqdr=2*qloc/zr*(1-zrovera2*qrat*(1-zrovera2)**(qrat-1)
	1   /denom)
 
	zmv = 0.0
	ZRtip = 0.0
	zPphi0=0.0
	zPphi0 = Pphiorb(Rmaj+rovera(ishell)*rmin)
	do 200 ilam=1,MaxLam
	ZRtip = abs(rlambda(ilam))*Btor*Rmaj
!verify that this orbit is trapped and is accessible to this flux surface:
	if (ZRtip .lt. Rmaj-rovera(ishell)*rmin
	1 .or. ZRtip .gt. Rmaj+rovera(ishell)*rmin)go to 200
 
!skip transport for 0 energy particles:
	do 300 ien=1,maxen
	if(ishell .gt. nshemax(ilam,ien))go to 300
 
c skip calculation of convective multiplier (which is time consuming)
c if convmult=0.0:
	if(convmult .eq. 0.0) goto 50
 
	zmv = Ab * pmass * sqrt(2*9.579e11*enerev(ien)/Ab)
c 9.579e11 = k ( ergs/eV) / m_proton (grams)
	
c find shift of co- leg of orbit:
	ZRminim=Rmaj + rovera(ishell)*rmin
	ZRmax = Rmaj + rmin
	maxit=30
c solves 0 = Pphiorb(R)
	ZRmax=zeroin(ZRminim,ZRmax,Pphiorb,0.00001,zzeps)
cIMSL	call zbren(Pphiorb,0.0,0.0001,ZRminim,ZRmax,maxit)
	rovap=(Zrmax-Rmaj)/rmin
c find shift of counter leg of orbit:
	zmv = - zmv
	Zrminim=ZRtip
	ZRmax = Rmaj + rovera(ishell)*rmin
	maxit=30
	ZRmax=zeroin(ZRminim,ZRmax,Pphiorb,0.00001,zzeps)
cIMSL	call zbren(Pphiorb,0.0,0.0001,ZRminim,ZRmax,maxit)
	rovam=abs(Zrmax-Rmaj)/rmin
 
c crude evaluation of integral dl sign(vpar)/taus:
	delthe=abs(2*acos((ZRtip-Rmaj)/rmin/rovera(ishell)))
	tauint=Rmaj*qloc*delthe*(1./FPINTRP(rovap,Tause,rovera,nshell)
	1	- 1./FPINTRP(rovam,Tause,rovera,nshell))
 
	vrnorm(ilam,ien,ishell)=convmult*1.02e2*(2*enerev(ien)*Ab)**.5
	1   /rZbeam/Btor/4./3.14159*tauint
 
! Calculate ripple diffusion:
 
!parameters for exponential ripple model:
!TFTR parameters (my improvement of Steve Scott's fit):
! This fit provides an upper bound on the ripple, and fits the ripple
! well for ripples above .1%.
!	data RipRmin /228.0/	    !major radius of minimum ripple (cm)
!	data Ripdmin /1.895e-5/   !ripple (peak to average) at that point
!	data RipexpL /19.17/	    !exponential scale length increase (cm)
!	data NumTF /20/	     	! # of TF coils
 
!evaluate ripple plateau diffusion D(E,xsi)
!(Goldston and Towner, J. Plasma Physics, v. 26, p. 283 (1981).)
50	if (xripmult .eq. 0.0) go to 300
	zcosth=(zRtip-Rmaj)/zr
	if(zcosth .lt. -1.0) goto 100 !passing particles don't see ripple
	zsinth=sqrt(1-zcosth**2)
	zy=zr*zsinth
 
!exponential model for ripple:
        zRipDlta=Ripdmin*exp(sqrt(zy**2+(zRtip-RipRmin)**2)/RipexpL)
 
        zRipAlfa= (zr/Rmaj)*zsinth/(NumTF * qloc * zRipDlta)
!Steve Scott's approximation for the correction to the 1/alpha**2 dependence
!of the (radial step size)**2 taken from fig. 13 of Goldston and Towner:
        if(zRipAlfa .gt. 2.5) zRipCorr= 1.0
        if(zRipAlfa .gt. .92 .and. zRipAlfa .le. 2.5)
	1                      zRipCorr=1.4961/zRipAlfa**(.44)
        if(zRipAlfa .le. .92) zRipCorr= 1.902*zRipAlfa**2.68
 
        zdrnorm= XRipMult*3.613e9*enerev(ien)**1.5
	1   *Ab**.5 *zsinth
	1   /(rZbeam**2 *Btor**2 *NumTF)*zRipCorr/zRipAlfa**2
 
c check to see if this is large enough to satisfy collisionless
c stochasticity criterion (Goldston, White, Boozer, PRL 47 (1981) 647),
c or is sufficiently collisional that the ripple phase is decorrelated
c from previous bounces (Goldston and Towner).  First find the
c GWB criterion:
 
c zv=particle velocity (cm/sec):
	zv = sqrt(2*9.579e11*enerev(ien)/Ab)
 
c zdelta is the radial step at each banana tip due to the ripple is
c given by dr = zdelta * cos(N phi):
 
	zdelta = sqrt( zdrnorm * 4 * 2 * 3.14159 * qloc * Rmaj
	1   /zv /zr)
 
	zdphidr=2*acos(zcosth)*zdqdr+2*qloc/zr*zcosth/zsinth
 
	istoch=0 ! default is not stochastic
	data zgwbth /0.5/ !GWB stochasticity threshold (may be 0.5-2.0)
c Roscoe White told me today (4/14/88) that 0.5 was consistently more
c accurate.
	if(numtf*zdelta*abs(zdphidr) .gt. zgwbth) istoch=1
 
c Now find the collisional decorrelation condition, successive
c ripple steps are uncorrelated if 2*nuii*taub > (r/R)/Numtf**2/q**2
c where nuii= nuperp/2 (nuperp^(i/i) given in the NRL) and taub
c is the 'half' bounce time.  Since FPP doesn't calculate taub
c until after the call to RDOPER, we use the approximate formula for
c deeply trapped ions.  (This should be improved later).
 
        zloglam=24.8-alog( drelec(ishell)**.5 / trelec(ishell))
	znuii=9.016e-8*rzbeam**2*rzeff*drelec(ishell)*zloglam
	1   /ab**.5/enerev(ien)**1.5
	ztaub=3.14159*Rmaj*qloc*sqrt(2.)/(zv*sqrt(zr/Rmaj))
	zcoldec=2*znuii*ztaub*(numtf*qloc)**2/(zr/Rmaj)
	data zcolth /0.5/ ! collisional decorrelation threshold
	if(zcoldec .gt. zcolth) istoch=1
 
	Drnorm(ilam,ien,ishell)=0.0
	if(istoch .eq. 1)then
	    Drnorm(ilam,ien,ishell)=zdrnorm
	    Vrnorm(ilam,ien,ishell)=Vrnorm(ilam,ien,ishell)
	1	+zdrnorm/zr*zcosth**2/zsinth**2
	endif
 
300	continue !end of ien loop
 
! If the next shell inward is inaccessible and Vrnorm is negative, force it to
! be zero to prevent flow into non-physical phase space:
	if(ishell .gt. 1)then
	    if(Zrtip .lt. Rmaj+rovera(ishell-1)*rmin)go to 200
	endif
	do 115 ien=1,maxen
	    if(Vrnorm(ilam,ien,ishell) .lt. 0.0)
	1	Vrnorm(ilam,ien,ishell)=0.0
115	CONTINUE
 
200 	continue !end of ilam loop
100	continue !end of ishell loop
 
	RETURN
	END
