	SUBROUTINE rdstep(f1rad,fnbsours)
c
c take a radial transport time step
c Greg Hammett 26-oct-1986
c
c Solves the equation:
c
c d/dt f = (1/Jacobian(r)) (1/r) d/dr ( - Vrnorm f + d/dr Drnorm f)
c
c Take care to difference this equation conservatively, and use
c upwind differences for the convective term.
c Also take care to prevent transport into or out of regions of
c phase space which are inacessible.
c
c GWH 21-Sep-1990: Upgraded to deal with passing+trapped particles.
c
	include 'FPPCOM'
	include 'F1COM'
 
!coefficients of tri-diagonal matrix resulting from the diffusion
! equation:
	real zalpha(maxshe)
	real zbeta(maxshe)
	real gamma(maxshe)
	real source(maxshe)
! and the coeficients of the recursive solution:
	real epsilon(maxshe)
	real delta(maxshe)
 
	real fmu1(MaxLam,maxshe)   ! f(r,const E, const mu) before
				      !     radial time step
	real fmu2(MaxLam,maxshe)   ! f after radial time step
 
	real zlammax(maxshe)	! maximum physical values for rlambda
	real zBlocal(maxshe)	! local B on the outer midplane
 
! define the radial time step:
	dtradial=timmy(loomax)-timmy(loomin)
 
c define zlammax:
	do 15 ishell=1,nshell
	zBlocal(ishell)= Btor*Rmaj/(Rmaj+rminpro(ishell))
15	zlammax(ishell)=1./ zBlocal(ishell)
 
	do  900 ien = 1,maxen  !ignore transport for 0 energy particles
	    do 800  ilam = 1, MaxLam
 
! zero out arrays:
	    do 50 ishell=1,nshell
		fmu1(ilam,ishell)=0.0
		fmu2(ilam,ishell)=0.0
50	  continue
 
!maximum confined shell at this energy and pitch angle:
		msconf=nshemax(ilam,ien)
		if(msconf .lt. 1) go to 800
 
! crude interpolation to find fmu1(ishell, fixed energy, fixed lambda):
		do 100 ishell=1,msconf
		x2=1.-abs(rlambda(ilam))*zBlocal(ishell)
		if(x2 .lt. 0.0) go to 100
		x=x2**0.5
		if (ilam .le. maxlam/2) x=-x
		jm=lkupr(x,xsi,maxxsi)
		jm=min(jm,maxxsi-1)
		fmu1(ilam,ishell) = (f1rad(ien,jm,ishell)*(xsi(jm+1)-x)
	1	   + f1rad(ien,jm+1,ishell)
	1	   *(x-xsi(jm)))/(xsi(jm+1)-xsi(jm))
 
100	     continue
	
c find the coefficients of the transport equation versus radius:
 
	do 199 i=1,msconf
 
c find grid step sizes:
 
	if(i .gt. 1 .and. i .lt. nshell)then
	    dr=(rminpro(i+1)-rminpro(i-1))/2.0
	    zdrp=(rminpro(i+1)-rminpro(i))
	    zdrm=(rminpro(i)-rminpro(i-1))
	endif
	if(i .eq. 1) then
	    dr=rmin*(rovera(2)+rovera(1))/2.
	    zdrp=(rminpro(i+1)-rminpro(i))
	    zdrm=rminpro(i)
	endif
	if(i .eq. nshell) then
	    dr=rmin*(1.-(rovera(nshell)+rovera(nshell-1))/2.)
	    zdrp=rmin-rminpro(i)
	    zdrm=(rminpro(i)-rminpro(i-1))
	endif
 
c evaluate coefficients of the flux at i+(1/2):
 
	Vrp=0.0
	zDpp=0.0
	ZDp=Drnorm(ilam,ien,i)
	if(i .lt. msconf)then
	    if(Vrnorm(ilam,ien,i+1) .lt. 0.0)Vrp=Vrnorm(ilam,ien,i+1)
	    Zdpp=Drnorm(ilam,ien,i+1)
	endif
 
c evaluate coefficients of the flux at i-(1/2):
	Vrm=0.0
	zDmm=0.0
	zDm=0.0
	if(i .gt. 1)then
	    if(Vrnorm(ilam,ien,i-1) .gt. 0.0)Vrm=Vrnorm(ilam,ien,i-1)
	    zDmm=Drnorm(ilam,ien,i-1)
	    ZDm=Drnorm(ilam,ien,i)
c force coefficients to zero if fmu1(i-1) is inacessible:
	    if (abs(rlambda(ilam)) .gt. zlammax(i-1)) then
	    Zdm=0.0
 
c debug:  zDmm and Vrm should already be zero if fmu1(i-1) is inacessible,
c but we can verify that this is true if we wish:
c	    if(zDmm .ne. 0.0 .or. Vrm .ne. 0.0) then
c	    type *, 'Warning: inaccessible'
c	    endif
 
	    endif
	endif
 
	zVr=Vrnorm(ilam,ien,i)
	if(i .eq. 1) zVr=Max(zVr,0.0)
c make sure there is no convection into an inaccessible region:
	if(i .gt. 1) then
	if(abs(rlambda(ilam)) .gt. zlammax(i-1)) zVr=Max(zVr,0.0)
	endif
 
c Debug:  For the time being, disallow flux across the trapped-passing
c boundary:
c	if(i .gt. 1)then
c	if(btlam(ilam,i-1) .le. 0.0) then
c	    zDmm=0.0
c	    zDm=0.0
c	    zVr=Max(zVr,0.0)
c	endif
c	endif
 
c Debug:  For the time being, disallow flux across the plasma edge,
c using conservative boundary conditions:
c	if(i .eq. msconf) then
c	    zDpp=0.0
c	    zDp=0.0
c	    Vrp=0.0
c	    zVr=Min(zVr,0.0)
c	endif
 
c find coefficients of the tridiagonal equation to be inverted:
c
c	alpha(i) f(i+1)  +  beta(i) f(i)  +  gamma(i) f(i-1)  =  S(i)
c
	if(btlam(ilam,i) .ne. 0.0)then
	    zalpha(i)=dtradial*(zDpp/zdrp - Vrp)
	1       / (btlam(ilam,i)*rminpro(i)*dr)
	    gamma(i)=dtradial*(zDmm/zdrm + Vrm) /
	1	(btlam(ilam,i)*rminpro(i)*dr)
	    zbeta(i)=-dtradial*(zDp/zdrp + zDm/zdrm
	1       + Abs(zVr) )
	1	/ (btlam(ilam,i)*rminpro(i)*dr) -1.0
	else
	    zalpha(i)=0.0
	    gamma(i)=0.0
	    zbeta(i)=-1.0
	endif
 
	source(i)=-fmu1(ilam,i)
 
199     continue
 
 
!solve the tri-diagonal matrix:
	delta(1)=source(1)/zbeta(1)     !boundary conditions
	epsilon(1)=-zalpha(1)/zbeta(1)
 
	if(msconf .ge. 2)then
	do 250 i=2,msconf
	epsilon(i)=-zalpha(i)/(zbeta(i)+gamma(i)*epsilon(i-1))
250     delta(i)=(source(i)-gamma(i)*delta(i-1))
	1       /(zbeta(i)+gamma(i)*epsilon(i-1))
	endif
 
	fmu2(ilam,msconf)=delta(msconf)
	if(msconf .ge. 2)then
	do 270 i=msconf-1,1,-1
270     fmu2(ilam,i)=epsilon(i)*fmu2(ilam,i+1)+delta(i)
	endif
 
c debug:  check for particle conservation (relevant only
c if conservative BC's are being used so there is no flux
c across the plasma boundary:
c	zsum1=0.0
c	zsum2=0.0
c	do 300 i=1,nshell
c	zsum1=zsum1+fmu1(ilam,i)*btlam(ilam,i)*rminpro(i)*dr
c300	zsum2=zsum2+fmu2(ilam,i)*btlam(ilam,i)*rminpro(i)*dr
c	if(abs(zsum1-zsum2) .gt. .01*zsum1 .and. zsum1 .gt.
c	1   1.e-30)then
c	    type *,' Warning in rdstep:  particles not conserved!'
c	endif
 
800     continue !end ilam loop
 
!***************************************************************
!
! crude interpolation to map fmu2 back onto f1rad:
!
! this is only a crude, slow interpolation, it could probably be greatly
! improved in the future:
!
!***************************************************************
 
	do 850 ishell=1,nshell
	do 825 j=1,maxxsi
 
	    rlam=(1.0-xsi(j)**2)/(Btor*Rmaj/(Rmaj+rminpro(ishell)))
 
! lookup in table of positive lambda's:
	    ilamp=lkupr(rlam,rlambdap,MaxLam/2)
	    ilamp=max(min(ilamp,MaxLam/2-1),1)
 
! transform to negative lambda's if needed:
	    if(xsi(j) .gt. 0.0) then
		ilm=maxlam/2+ilamp
		ilm1=ilm+1
	    else
		ilm=maxlam/2+1-ilamp
		rlam=-rlam
		ilm1=ilm-1
	    endif
 
! take care if the adjacent rlambda is inaccessible:
	    if(rlambdap(ilamp+1) .gt. zlammax(ishell)) then
	    zf1new=fmu2(ilm,ishell)
 
! otherwise, interpolate:
	    else
	    zf1new=(fmu2(ilm,ishell)*(rlambda(ilm1)-rlam)
	1	   +fmu2(ilm1,ishell)*(rlam-rlambda(ilm)))
	1	   /(rlambda(ilm1)-rlambda(ilm))
	    endif
 
c debug: use this when D is tiny to verify accuracy of the mapping
c from xsi to lambda and backagain.  When f(xsi) has sharp variations,
c such as during ICRF heating, these interpolation errors can become
c non-negligble.  The errors were not large enough to affect the
c volume integrated density, which was conserved to about 0.1%.
c To reduce these mapping errors, we have to increase MAXLAM,
c and/or go to a more accurate interpolation method.
c
c	data ztol /.02/
c	zerror=abs(zf1new-f1rad(ien,j,ishell))
c	if(zerror .gt. ztol*abs(f1rad(ien,j,ishell))
c	1   .and. zerror .gt. 1.e-5) then
c	write(lsysout,*) 'zf1new bad: zf1new,f1rad=',
c	1   zf1new,f1rad(ien,j,ishell)
c	write(lsysout,*) ' at ien,j,ishell=',ien,j,ishell
c	endif
 
	f1rad(ien,j,ishell)=zf1new
 
825	continue    ! end j loop
 
c symmetrize the trapped region of f:
	do 830 j=maxxsi/2+1,maxxsi
	zRtip=(1.0-xsi(j)**2)*(Rmaj+rminpro(ishell))
	if(zRtip .lt. (Rmaj-rminpro(ishell))) goto 850
	zavg=(f1rad(ien,j,ishell)+f1rad(ien,maxxsi+1-j,ishell))/2
	f1rad(ien,j,ishell)=zavg
	f1rad(ien,maxxsi+1-j,ishell)=zavg
830	continue
 
850	continue    ! end ishell loop
 
900     continue  !end ien loop
 
	return
	end
