	subroutine rfoper
!  evaluates the RF contributions to the F.P. operator.
!
! This routine became good for essentially general geometry sometime
! after the below date.  However, it still does assume B = Btor = K/R.
! Also, some of its algorithms may fail for bean-shaped plasmas.
!
! 20-Sep-1988 GWH:  moving towards non-circular geometry, although still
! assuming B = Btor = K/R.
!
! 2 RFOPER calls take about 3.5-4.8 seconds with lookup of resonance
! position in non-circular geometry.
!
! I have worked fairly hard at optimizing this routine.
! At most one could get another 15% speedup if the fintqe1d calls were
! replaced by instantaneous inline code, or another 12% if the acos calls
! could be replaced by single square root calls.  I don't think it is
! worth trying to do anymore optimization.
!
! includes several new effects different from the standard
! version before Sept. 1985:
! 1. generalized to kpar .ne. 0.
! 2. improved handling of the singularities.
! 3. Inclusion of E_minus J_n+1 terms.
!
! 31-Oct-1987: GWH, I increased the speed of the this routine by almost
! a factor of 4 by taking calls to FPINTRP to interpolate on a table of
! bessel function values out of the inner loop! RFOPER now takes 2.5 secs
! per call.  Eliminating the calls to bsjns only saves about .1 secs.
!
 
	include 'FPPCOM'
 
c  dmc Feb 1993 -- replace IMSL routine BSJNS with netlib routine
c  rjbesl -- tested OK Feb 23 1993 on DECstation;
c  cf TFTR.92 68828Z90 et al
 
	real zBrequ ! lookup in ThofBr array to find Theta(zBrequ,r)
c  jphe these two lines in common /fpbes/ (part of fppcom)
c	parameter (imaxbes=500)  ! # of points in Bessel tables
c	real beskik(0:imaxbes,6),arga(0:imaxbes) !array of Bessel functions
c  jphe 11-11-1994  end
	real Besa(0:imaxbes,4),arga(0:imaxbes) !array of Bessel functions
	real zddum,Besn(6)   !for IMSL routine jphe 11-nov-1994: besn(6) now.
	complex zkpcorr
 
	zpi=abs(acos(-1.0))
	omegrf=2*zpi*freqrf     !rf frequency (radians/sec)
	zxi=rminpro(nshe)/rmin
	call momry(zxi,0.0,0,zRmax,zY,zJac)
	omeg0=9.579e3*rZbeam*Btor*Rmaj/zRmax/Ab  !gyro-freq @ Bmin
	ckovm=9.579e11  !k(ergs/ev) over m(grams)
	xsistq=1-nharm*Omeg0/omegrf     !kpar=0 resonant pitch angle
 
c find position of the kpar=0 resonance layer:
	zBres=omegrf/nharm/rZbeam/9.579e3*Ab
	zRres=Rmaj*Btor/zBres
c find kparallel at resonance:
	zkparres=rkpar*antRcen/zRres
 
c find r2 and r1 of outer and inner boundaries of this radial zone:
	r2=xradzb(nshe)*rmin
	if(nshe .gt. 1)then
	r1=xradzb(nshe-1)*rmin
	else
	r1=0.0
	endif
 
	zPsi2=finteq1d(r2,Psi,0.0,delrPsi,numPsi)
	zPsi1=finteq1d(r1,Psi,0.0,delrPsi,numPsi)
 
	data zeovc /1.6021e-20/
	zPsi2=zPsi2/zeovc/rZbeam
	zPsi1=zPsi1/zeovc/rZbeam
 
!set up an array of bessel function values:
!first find the maximum value of kperp, and check that our Bessel
!function table covers necessary size:
 
	zmaxk=0.0
	do 11 j=1,nticrf
11      zmaxk=max(zmaxk,skperp(nshe,j))
 
c maximum argument of Bess function table (*1.5 for safety):
	zargmax=2.0*sqrt(2.*ckovm/Ab*enerev(maxen))*zmaxk/omeg0
	if(zargmax .lt. 1.0) zargmax=1.0
	zdel=zargmax/imaxbes
	if(zdel .gt. .1) then
	write(lsysout,*)
     1  'RFOPER warning: insufficient Bessel function resolution?'
	write(lsysout,*) '   Max argument = ',zargmax, '   zdel = ',zdel
	endif
	delbes=zdel
	do 12 i=0,imaxbes
	arga(i)=i*zdel
	zddum=arga(i)
cIMSL	call bsjns(zddum,4,Besn)
c jphe 11-nov-1994  increase to 6 to include 3 harmonic effects in rfkick
c
	call rjbesl(zddum,0.0,6,Besn,ingot)
	if(ingot .ne. 6) call aborter(6,'rjbesl error in rfoper.for')
	besa(i,1)=besn(1)
	besa(i,2)=besn(2)
	data Emcoef /1./ !1 = turn on, 0= turn off E_minus
	besa(i,3)=besn(3)*Emcoef
	besa(i,4)=besn(4)*Emcoef
c jphe transfer to common :
        beskik(i,1)=besn(1)
        beskik(i,2)=besn(2)
        beskik(i,3)=besn(3)
        beskik(i,4)=besn(4)
        beskik(i,5)=besn(5)
	beskik(i,6)=besn(6)
12      continue
 
c find resonant pitch angle in grid for plots:
	if(xsistq .lt. 0.0)then
		nJxsist=maxxsi/2+1
		go to 30
	endif
	xstar=sqrt(xsistq)
	do 15 j=maxxsi/2+1,maxxsi
	if(xsi(j) .lt. xstar)go to 15
	nJxsist=j
	go to 30
15      continue
	nJxsist=maxxsi
30      continue
 
c calculate a few constants outside of the major loop:
	zcon1=sqrt(2*ckovm/ab)
	a=omegrf/nharm/omeg0
 
c*********************************************
c  Loop over energy and pitch angle grid     *
c*********************************************
	do 1500 i=0,maxen
 
c calculate a few constants outside of the pitch angle loop:
c (the vax compiler does not do a good job of automatically
c optimizing these loops, so we need to help it manually.)
 
	zW=enerev(i)
	if(i .ne. maxen)then
	    zWup=(enerev(i)+enerev(i+1))/2.
	else
	    zWup=enerev(maxen)
	endif
 
	zvup2=2*ckovm/ab*zWup
	zvmid2=2*ckovm/ab*zW
	zvup=sqrt(zvup2)
	zvmid=sqrt(zvmid2)
	zbup=.25*zvup2*(zkparres/nharm/omeg0)**2
	zrtbup=sqrt(zbup)
	zbmid=.25*zvmid2*(zkparres/nharm/omeg0)**2
	zrtbmid=sqrt(zbmid)
 
	do 1500 j=1,maxxsi
 
 
c***************************************
c   evaluate deb and dec	       *
c***************************************
!  energy diffusion terms are evaluated between grid points.
!       deb(i,j) is deb halfway between i and i+1.
!       dec(i,j) is dec halfway between i and i+1.
!	deb(i,1)=deb(i,maxxsi)=dec(i,1)=dec(i,maxxsi)=0.0
!	    because vperp**2=0 there.  This is enforced in this
!	    subroutine to ensure the proper boundary conditions.
 
	x=xsi(j)	!pitch angle
	if(i .eq. maxen)go to 1200
	if(j .eq. 1 .or. j .eq. maxxsi) go to 1200  ! see above comment
	W=zWup
	zv2=zvup2
	zv=zvup
	b=zbup
	zsqrtb=zrtbup
 
!handling the xsi singularity:
! the xsi singularity is integrable, the integral can be found
! in Gradshteyn and Ryzhik.
 
	if(j .eq. 1)then
		x1=xsi(1)
	else
		x1=(xsi(j)+xsi(j-1))/2
	endif
	if(j .eq. maxxsi)then
		x2=xsi(maxxsi)
	else
		x2=(xsi(j)+xsi(j+1))/2
	endif
 
	arg1=1-a*(1-x1**2)+b*(1-x1**2)**2
	arg2=1-a*(1-x2**2)+b*(1-x2**2)**2
 
c if we are not near the singularity then we can use a simple
c average value:
	if(arg1 .le. 0.0 .and. arg2 .le. 0.0) goto 970 !no resonance
	if(arg1 .gt. 0.1 .and. arg2 .gt. 0.1 .and. abs(arg1-arg2)
	1    .lt. 0.1*max(arg1,arg2))then
	singx=sqrt(0.5*(1./arg1+1./arg2))/zv/nharm
	goto 942
	endif
 
!the following formula for the singularity breaks down for kpar .eq. 0
! use an alternate formula in that case:
	if(b/a .lt. 1.e-6)go to 930
 
	if(arg1 .ge. 0.0)then
	sinint1=alog(abs(2*zsqrtb*sqrt(arg1)-2*b*(1-x1**2)+a))
	1       /2/zsqrtb
	else
	sinint1=alog(sqrt(abs(a**2-4*b)))/2/zsqrtb
	endif
 
	if(arg2 .ge. 0.0)then
	sinint2=alog(abs(2*zsqrtb*sqrt(arg2)-2*b*(1-x2**2)+a))
	1       /2/zsqrtb
	else
	sinint2=alog(sqrt(abs(a**2-4*b)))/2/zsqrtb
	endif
 
	go to 940
!this is the kpar .eq. 0 formula:
930     arg1=1-a*(1-x1**2)
	sinint1=0
	if(arg1 .ge. 0.0)sinint1=sqrt(arg1)/a
	arg2=1-a*(1-x2**2)
	sinint2=0.0
	if(arg2 .ge. 0.0)sinint2=sqrt(arg2)/a
 
940     singx=abs(sinint1-sinint2)/abs(x)/abs(x2-x1)
	singx=singx/nharm/zv
 
! singx is the value of 1/|n vpar-kpar mu c/e|_res averaged
! over a xsi zone.
942	if(singx .le. 0.0)go to 970 !no resonance in this xsi zone.
 
!loop over two possible roots:
	do 960 isign=-1,1,2
	arg=1-a*(1-x**2)+b*(1-x**2)**2
 
!allow for the possibility that a tangent resonance occurs somewhere
! in this zone, although the center of the zone may miss the resonance:
	if(arg .lt. 0.0)arg=0.0
	vparres=zv*(isign*sqrt(arg)
	1       +sign(1.0,zkparres)*zsqrtb*(1-x**2))
! skip if vpar and x aren't of the same sign.  remember to symmetrize
! <Q> for trapped particles later.
	if( (x .ge. 0.0 .and. vparres .lt. 0.0) .or.
	1       (x .lt. 0.0 .and. vparres .ge. 0.0)) go to 960
	Bres=(omegrf-zkparres*vparres)/nharm/rZbeam/9.579e3*Ab
c position of the true Doppler-shifted resonance:
	Rresds=Rmaj*Btor/Bres
	zBrequ=Bres
! Omega-ci at resonance:
	omres=9.579e3*rZbeam*Bres/Ab
 
!Calculate d Omega / dl singularity:
! (a factor of Rq has been removed to cancel an Rq in BTIME)
 
	zxi=r2/rmin
	ir=nshe*2
 
	if(zBrequ .lt. Bminofr(ir)) then
	    zY2=0.0
	else if(zBrequ .gt. Bmaxofr(ir))then
	    zy2=0.0
	else
c test:
	zarg=(Bavgofr(ir)-zBrequ)/Bdelofr(ir)
	if(zarg .gt. 1.01 .or. zarg .lt. -1.01)
	1   call aborter(6,'zarg much too large')
 
	zthstar=acos(min(1.0,max(-1.0,zarg)))
	zY2=finteq1d(zthstar,YofBr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
	endif
 
	zxi=r1/rmin
	ir=nshe*2-2
	if(ir .ne. 0)then
 
	if(zBrequ .lt. Bminofr(ir)) then
	    zy1=0.0
	else if(zBrequ .gt. Bmaxofr(ir))then
	    zy1=0.0
	else
c test:
	zarg=(Bavgofr(ir)-zBrequ)/Bdelofr(ir)
	if(zarg .gt. 1.01 .or. zarg .lt. -1.01)
	1   call aborter(6,'zarg much too large')
 
	zthstar=acos(min(1.0,max(-1.0,zarg)))
	zY1=finteq1d(zthstar,YofBr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
	endif
 
	else
	    zy1=0.0
	endif
 
	singer=abs(zY2-zY1)*Bres*Rresds**2/Omres/abs(zPsi2-zPsi1)/
	1   Rmaj/qpro(nshe)
 
	if(singer .le. 0.0)go to 960 !resonance didn't intersect
				!this radial zone
 
! singer is now 1/|d Omega / dl| averaged over a radial zone,
! with Rmaj q divided out.
 
!vperp at resonance:
	vperp2=zv2-vparres**2
	if(vperp2 .lt. 0.0)vperp2=0.0
	vperp=sqrt(vperp2)
 
c theta where Doppler-shifted resonance intersects this radial shell
c (set theta=0 or pi if the resonance intersects this radial zone, but
c not the zone center!):
c NOTE: THIS ALGORITHM WILL NOT WORK PROPERLY IF THE PLASMA IS INDENTED
C OR SUFFICIENTLY HIGH BETA SUCH THAT BMAX IS NOT AT THETA+PI!
 
	ir=nshe*2-1
 
	if(zBrequ .lt. Bminofr(ir)) then
	  zth1=0.0
	  zthstar=0.0
	else if(zBrequ .gt. Bmaxofr(ir))then
	  zth1=zpi
	  zthstar=zpi
	else
c test:
	zarg=(Bavgofr(ir)-zBrequ)/Bdelofr(ir)
	if(zarg .gt. 1.01 .or. zarg .lt. -1.01)
	1   call aborter(6,'zarg much too large')
 
	zthstar=acos(min(1.0,max(-1.0,zarg)))
	zth1=finteq1d(zthstar,ThofBr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
	endif
	zthdsr=zth1
 
c Defining the correlation time for a single resonance as Tau_c, where
c Tau_c**2 is given in Eq. 2.30 of my thesis:
c
c	2 Tau_c**2 = | \integral d \tau \exp( - i \integral d \tau'
c		    (omega - k_\| v_\| - n \Omega) )  |
c
c The factor of 2 accounts for the fact that there are two resonances
c per bounce.  When the two resonances are well separated so the
c asymptotic form of the Airy function can be used, this reduces to
c Eq. 2.31 of my thesis.
c
c In terms of the variables used in this code, this becomes
c
c 	Tau_c**2 = 2 * Pi *singx * singer * Rmaj * qpro
c
c we now make sure Tau_c is less than the cutoff value given
c by the Airy function.  In my thesis (p. 49), I suggest using the
c maximum value of the Airy function.  Here, I will instead use the value of
c the Airy function at x=0 (where x is the argument of the Airy function.)
c This reduces the correlation time a bit more.
 
c zompp = d**2 Omega / d l**2:
	zompp=finteq1d(zthstar,omppofr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
c zomdd = - d**2 (n Omega + kparallel vparallel) / d t**2 at resonance:
cdmc	zomdd=-isign*vparres/singx*zompp+
cdmc	1   (1./singer/rmaj/qpro(nshe))**2 *nharm*vperp2/2./omres
c dmc -- recoded to avoid overflow
	znvp2ovo=nharm*vperp2/2./omres
	zomdd1=-isign*vparres/singx*zompp
	zomdd2=(1./singer/rmaj/qpro(nshe))**2
	zomdd=zomdd1 + zomdd2*znvp2ovo
c end dmc
	zt2inv=(abs(zomdd))**(2./3.)/7.9*2
	ztest=zt2inv*2*zpi*singx*singer*Rmaj*qpro(nshe)
	if(ztest .gt. 1.0) then
	singx=singx/ztest
	endif
 
c
c we now make sure that (Tau_c)**2 < tau_bounce**2/4.
c We enforce this by reducing singx if necessary.
c
	ztaub2=(btime(j)*2*zpi*qpro(nshe)*Rmaj)**2 /zv2
	ztest=4.*2*zpi*singx*singer*Rmaj*qpro(nshe)/ztaub2
	if(ztest .gt. 1.0) then
	singx=singx/ztest
	endif
 
c corresponding (nearest!) theta index, above midplane:
	jthdsra=1+0.5+zthdsr/2./zpi*nticrf
c and below midplane:
	jthdsrb=nticrf+2-jthdsra
 
c--Kennel-Engelmann quasilinear diffusion coeficient is proportional
c       to | theta | **2.  Here we ignore the Epar contibution
c       and just keep Eplus and Eminus.
 
c resonance above the midplane:
c--Bessel function argument:
	argum=skperp(nshe,jthdsra)*vperp/omres
	ibes=argum/zdel+0.5
	if(ibes .gt. imaxbes) then
	    write(lsysout,*)
     1      'RFOPER warning, argument outside Bessel table!'
	    ibes=imaxbes
	endif
 
	zkpcorr=Besa(ibes,nharm)+
	1   crfpol(nshe,jthdsra)*Besa(ibes,nharm+2)
	theta2=Eplus(nshe,jthdsra)*Eplus(nshe,jthdsra)*
	1   (real(zkpcorr)*real(zkpcorr)
	1   +aimag(zkpcorr)*aimag(zkpcorr))
 
c resonance below the midplane:
c--Bessel function argument:
	argum=skperp(nshe,jthdsrb)*vperp/omres
	ibes=argum/zdel+0.5
	if(ibes .gt. imaxbes) then
	    write(lsysout,*)
     1      'RFOPER warning, argument outside Bessel table!'
	    ibes=imaxbes
	endif
 
	zkpcorr=Besa(ibes,nharm)+
	1   crfpol(nshe,jthdsrb)*Besa(ibes,nharm+2)
	theta2=theta2+Eplus(nshe,jthdsrb)*Eplus(nshe,jthdsrb)*
	1   (real(zkpcorr)*real(zkpcorr)
	1   +aimag(zkpcorr)*aimag(zkpcorr))
 
	zfix=vperp2*singx*singer     ! dmc -- avoid overflow, 20 Oct 93
	rfdiff=zcon1*W*abs(x)/4.*8.9873e4
	1       *rZbeam**2
	1       *zfix*theta2         ! dmc -- avoid overflow, 20 Oct 93
! 8.9873e4 = (e/k)**2, e is electron charge (cgs), k is erg/ev
 
	deb(i,j)=deb(i,j)+rfdiff
	dec(i,j)=dec(i,j)-rfdiff*(x**2-xsistq)/2/x/W
960     continue
970     continue
 
c***************************************
c   evaluate dab and dac	       *
c***************************************
!  angle diffusion term is evaluated halfway between grid points.
!       dab(i,j) is dab halfway between j and j+1:
!       dac(i,j) is dab halfway between j and j+1:
!  We have found that setting dac(i,maxen)=0.0
!  gives better numerical stability, so we enforce this condition
!  below.
 
1200    if(j .eq. maxxsi .or. j .eq. maxxsi/2 .or. i .eq. 0)go to 1500
	W=zW
	zv2=zvmid2
	zv=zvmid
	b=zbmid
	zsqrtb=zrtbmid
	x=.5*(xsi(j)+xsi(j+1))
 
!handling the xsi singularity:
 
	x1=xsi(j)
	x2=xsi(j+1)
 
	arg1=1-a*(1-x1**2)+b*(1-x1**2)**2
	arg2=1-a*(1-x2**2)+b*(1-x2**2)**2
 
c if we are not near the singularity then we can use a simple
c average value:
	if(arg1 .le. 0.0 .and. arg2 .le. 0.0) goto 1270 !no resonance
	if(arg1 .gt. 0.1 .and. arg2 .gt. 0.1 .and. abs(arg1-arg2)
	1    .lt. 0.1*max(arg1,arg2))then
	singx=sqrt(0.5*(1./arg1+1./arg2))/zv/nharm
	goto 1242
	endif
 
!the following formula for the singularity breaks down for kpar .eq. 0
! use an alternate formula in that case:
	if(b/a .lt. 1.e-6)go to 1230
 
 
	if(arg1 .ge. 0.0)then
	sinint1=alog(abs(2*zsqrtb*sqrt(arg1)-2*b*(1-x1**2)+a))
	1       /2/zsqrtb
	else
	sinint1=alog(sqrt(abs(a**2-4*b)))/2/zsqrtb
	endif
 
	if(arg2 .ge. 0.0)then
	sinint2=alog(abs(2*zsqrtb*sqrt(arg2)-2*b*(1-x2**2)+a))
	1       /2/zsqrtb
	else
	sinint2=alog(sqrt(abs(a**2-4*b)))/2/zsqrtb
	endif
 
	go to 1240
!this is the kpar .eq. 0 formula:
1230    arg1=1-a*(1-x1**2)
	sinint1=0
	if(arg1 .ge. 0.0)sinint1=sqrt(arg1)/a
	arg2=1-a*(1-x2**2)
	sinint2=0.0
	if(arg2 .ge. 0.0)sinint2=sqrt(arg2)/a
 
1240    singx=abs(sinint1-sinint2)/abs(x)/abs(x2-x1)
	singx=singx/nharm/zv
 
! singx is the value of 1/|n vpar-kpar mu c/e|_res averaged
! over a xsi zone.
1242	if(singx .le. 0.0)go to 1270 !no resonance in this xsi zone.
 
!loop over two possible roots:
	do 1260 isign=-1,1,2
	arg=1-a*(1-x**2)+b*(1-x**2)**2
 
!allow for the possibility that a tangent resonance occurs somewhere
! in this zone, although the center of the zone may miss the resonance:
	if(arg .lt. 0.0)arg=0.0
	vparres=zv*(isign*sqrt(arg)
	1       +sign(1.0,zkparres)*zsqrtb*(1-x**2))
! skip if vpar and x aren't of the same sign.  remember to symmetrize
! <Q> for trapped particles later.
	if( (x .ge. 0.0 .and. vparres .lt. 0.0) .or.
	1       (x .lt. 0.0 .and. vparres .ge. 0.0)) go to 1260
	Bres=(omegrf-zkparres*vparres)/nharm/rZbeam/9.579e3*Ab
c position of the true Doppler-shifted resonance:
	Rresds=Rmaj*Btor/Bres
	zBrequ=Bres
! Omega-ci at resonance:
	omres=9.579e3*rZbeam*Bres/Ab
 
!Calculate d Omega / dl singularity:
! (a factor of Rq has been removed to cancel an Rq in BTIME)
 
 
	zxi=r2/rmin
	ir=nshe*2
 
	if(zBrequ .lt. Bminofr(ir)) then
	    zY2=0.0
	else if(zBrequ .gt. Bmaxofr(ir))then
	    zY2=0.0
	else
c test:
	zarg=(Bavgofr(ir)-zBrequ)/Bdelofr(ir)
	if(zarg .gt. 1.01 .or. zarg .lt. -1.01)
	1   call aborter(6,'zarg much too large')
 
	zthstar=acos(min(1.0,max(-1.0,zarg)))
	zy2=finteq1d(zthstar,YofBr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
	endif
 
	zxi=r1/rmin
	ir=nshe*2-2
	if(ir .ne. 0)then
 
	if(zBrequ .lt. Bminofr(ir)) then
	    zY1=0.0
	else if(zBrequ .gt. Bmaxofr(ir))then
	    zY1=0.0
	else
c test:
	zarg=(Bavgofr(ir)-zBrequ)/Bdelofr(ir)
	if(zarg .gt. 1.01 .or. zarg .lt. -1.01)
	1   call aborter(6,'zarg much too large')
 
	zthstar=acos(min(1.0,max(-1.0,zarg)))
	zY1=finteq1d(zthstar,YofBr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
	endif
 
	else
	    zY1=0.0
	endif
 
	singer=abs(zY2-zY1)*Bres*Rresds**2/Omres/abs(zPsi2-zPsi1)/
	1   Rmaj/qpro(nshe)
 
	if(singer .le. 0.0)go to 1260 !resonance didn't intersect
				!this radial zone
 
! singer is now 1/|d Omega / dl| averaged over a radial zone,
! with Rmaj q divided out.
 
!vperp at resonance:
	vperp2=zv2-vparres**2
	if(vperp2 .lt. 0.0)vperp2=0.0
	vperp=sqrt(vperp2)
 
c theta where Doppler-shifted resonance intersects this radial shell
c (set theta=0 or pi if the resonance intersects this radial zone, but
c not the zone center!):
c NOTE: THIS ALGORITHM WILL NOT WORK PROPERLY IF THE PLASMA IS INDENTED
C OR SUFFICIENTLY HIGH BETA SUCH THAT BMAX IS NOT AT THETA+PI!
 
	ir=nshe*2-1
 
	if(zBrequ .lt. Bminofr(ir)) then
	  zth1=0.0
	  zthstar=0.0
	else if(zBrequ .gt. Bmaxofr(ir))then
	  zth1=zpi
	  zthstar=zpi
	else
c test:
	zarg=(Bavgofr(ir)-zBrequ)/Bdelofr(ir)
	if(zarg .gt. 1.01 .or. zarg .lt. -1.01)
	1   call aborter(6,'zarg much too large')
 
	zthstar=acos(min(1.0,max(-1.0,zarg)))
	zth1=finteq1d(zthstar,ThofBr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
	endif
	zthdsr=zth1
 
c Defining the correlation time for a single resonance as Tau_c, where
c Tau_c**2 is given in Eq. 2.30 of my thesis:
c
c	2 Tau_c**2 = | \integral d \tau \exp( - i \integral d \tau'
c		    (omega - k_\| v_\| - n \Omega) )  |
c
c The factor of 2 accounts for the fact that there are two resonances
c per bounce.  When the two resonances are well separated so the
c asymptotic form of the Airy function can be used, this reduces to
c Eq. 2.31 of my thesis.
c
c In terms of the variables used in this code, this becomes
c
c 	Tau_c**2 = 2 * Pi *singx * singer * Rmaj * qpro
c
c we now make sure Tau_c is less than the cutoff value given
c by the Airy function.  In my thesis (p. 49), I suggest using the
c maximum value of the Airy function.  Here, I will instead use the value of
c the Airy function at x=0 (where x is the argument of the Airy function.)
c This reduces the correlation time a bit more.
 
c zompp = d**2 Omega / d l**2:
	zompp=finteq1d(zthstar,omppofr(1,ir),0.0,delthofB(ir),
	1   numthofB(ir))
c zomdd = - d**2 (n Omega + kparallel vparallel) / d t**2 at resonance:
cdmc	zomdd=-isign*vparres/singx*zompp+
cdmc	1   (1./singer/rmaj/qpro(nshe))**2 *nharm*vperp2/2./omres
c dmc -- recoded to avoid overflow
	znvp2ovo=nharm*vperp2/2./omres
	zomdd1=-isign*vparres/singx*zompp
	zomdd2=(1./singer/rmaj/qpro(nshe))**2
	zomdd=zomdd1 + zomdd2*znvp2ovo
c end dmc
	zt2inv=(abs(zomdd))**(2./3.)/7.9*2.
	ztest=zt2inv*2*zpi*singx*singer*Rmaj*qpro(nshe)
	if(ztest .gt. 1.0) then
	singx=singx/ztest
	endif
 
c
c we now make sure that (Tau_c)**2 < tau_bounce**2/4.
c We enforce this by reducing singx if necessary.
c
	ztaub2=(btime(j)*2*zpi*qpro(nshe)*Rmaj)**2 /zv2
	ztest=4.*2*zpi*singx*singer*Rmaj*qpro(nshe)/ztaub2
	if(ztest .gt. 1.0) then
	singx=singx/ztest
	endif
 
c corresponding (nearest!) theta index, above midplane:
	jthdsra=1+0.5+zthdsr/2./zpi*nticrf
c and below midplane:
	jthdsrb=nticrf+2-jthdsra
 
c--Kennel-Engelmann quasilinear diffusion coeficient is proportional
c       to | theta | **2.  Here we ignore the Epar contibution
c       and just keep Eplus and Eminus.
 
c resonance above the midplane:
c--Bessel function argument:
	argum=skperp(nshe,jthdsra)*vperp/omres
	ibes=argum/zdel+0.5
	if(ibes .gt. imaxbes) then
	    write(lsysout,*)
     1      'RFOPER warning, argument outside Bessel table!'
	    ibes=imaxbes
	endif
 
	zkpcorr=Besa(ibes,nharm)+
	1   crfpol(nshe,jthdsra)*Besa(ibes,nharm+2)
	theta2=Eplus(nshe,jthdsra)*Eplus(nshe,jthdsra)*
	1   (real(zkpcorr)*real(zkpcorr)
	1   +aimag(zkpcorr)*aimag(zkpcorr))
 
c resonance below the midplane:
c--Bessel function argument:
	argum=skperp(nshe,jthdsrb)*vperp/omres
	ibes=argum/zdel+0.5
	if(ibes .gt. imaxbes) then
	    write(lsysout,*)
     1      'RFOPER warning, argument outside Bessel table!'
	    ibes=imaxbes
	endif
 
	zkpcorr=Besa(ibes,nharm)+
	1   crfpol(nshe,jthdsrb)*Besa(ibes,nharm+2)
	theta2=theta2+Eplus(nshe,jthdsrb)*Eplus(nshe,jthdsrb)*
	1   (real(zkpcorr)*real(zkpcorr)
	1   +aimag(zkpcorr)*aimag(zkpcorr))
 
	zfix=vperp2*singx*singer     ! dmc -- avoid overflow, 20 Oct 93
	rfdiff=zcon1*W*abs(x)/4.*8.9873e4
	1       *rZbeam**2
	1       *zfix*theta2         ! dmc -- avoid overflow, 20 Oct 93
! 8.9873e4 = (e/k)**2, e is electron charge (cgs), k is erg/ev
 
	dab(i,j)=dab(i,j)+(x**2-xsistq)**2*rfdiff/4/x**2/w**2
	if(i .lt. maxen) dac(i,j)=dac(i,j)-rfdiff*(x**2-xsistq)/2/x/w
1260    continue
1270    continue
 
1500    continue
 
	return
	end
