	subroutine rfprof
c
c--this subroutine sets up RF power deposition profiles.
 
c--common blocks:
	include 'FPPCOM'
 
	real Ttail(maxshe)	!Stix model Ttail
	real sing(maxshe)	! 1/|r sin(theta)| averaged over a
				! radial shell
	real polariz(maxshe)    ! E_+/E_- polarization
	character string*80
 
c choice of RF Model:
c LSHOOT=+-1 to use Smithe's SPRUCE code
c LSHOOT=2 will use the Colestock-Smithe SHOOT Icrf code.
c LSHOOT=0 will use the old hand-entered ICRF guestimates as
c	massaged by this routine RFPROF.
c (LSHOOT=-1 is allowed for upwards compatibility with old versions
c where LSHOOT was a logical true or false.)
 
	if(lshoot .eq. 1 .or. lshoot .eq. -1)then
c general geometry SMITHE code:
		call fpspruce
		return
	else if (lshoot .eq. 2) then
c Smithe-Colestock SHOOT ICRF code:
c      	      call fpshoot2
	      call aborter(lsysout,'SHOOT no longer available!.')
	      return
	endif
 
c--Eplus is kept constant in the program, and the power changes
c	as f evolves.  In real experiments, the power is constant
c	and Eplus varies.  This is an area for future improvement.
c	We use Stix's simple formula Pstix to normalize Eplus to
c	give approximately the correct total power.  To be
c	rigorous one must look at the final power plot calculated
c	by the code including all the bessel function, doppler
c	broadening, and trapping effects, and adjust Eplus to keep
c	total power constant.
 
c--Two different models for calculating RF power profile:
c
c LRFmod=1  the user input the RF E-plus profile in array erfpro.
c
c LRFmod=2  the user input the RF power profile directly in
c		the erfpro array.
c
c LRFmod=3  the user typed in the exact values of E-plus to be
c 		used in the code.  Do not normalize.
c
!convert E-plus from Volts/cm to Statvolts/cm:
	do 20 i=1,nshell
20	Erfpro(i)=Erfpro(i)/299.79
 
c--Major radius of resonance layer:
c--1.524e3 = e/(m c 2 pi) in cgs
	Rres=nharm*1.524e3*rZbeam*Btor*Rmaj /(Ab*freqrf)
 
c--The Prf profile is related to the E-plus profile by:
c
c	        nmin zmin e c R (E-plus)**2     Rres
c	Prf = ( --------------------------- )   ----
c	            B r abs(sin(theta))         Raxis
c
c  taking care to average the singularity r sin(theta) over a radial
c  grid point.  Note that all quantities in parentheses are
c  evaluated in the resonance layer, including nmin!
 
	do 50 i=1,nshell
	pstovn(i)=0.0
 
	if(i .gt. 1) then
	r1=(rovera(i-1)+rovera(i))*rmin/2
	else
	r1=0.0
	endif
	if(i .lt. nshell)then
	r2=(rovera(i)+rovera(i+1))*rmin/2
	else
	r2=rmin
	endif
 
	deltar=r2-r1
	dummy1=r1**2-(Rres-Rmaj)**2
	if(dummy1 .lt. 0.0)dummy1=0.0
	dummy2=r2**2-(Rres-Rmaj)**2
	if(dummy2 .lt. 0.0)dummy2=0.0
 
	sing(i)=(sqrt(dummy2)-sqrt(dummy1))/deltar/rovera(i)/rmin
	if(sing(i) .le. 0.0)go to 50
	if(LRFmod .ne. 2)then
	pstovn(i)=1.e-7*concmi*denpro(i)*1.e13*rZbeam
	1	*4.8032*2.9979*Rres**2
	1	*sing(i)*erfpro(i)**2/(Btor*Rmaj/Rres*Rmaj)
! e*c = 4.8032e-10 * 2.9979e10
	else
	pstovn(i)=erfpro(i)
	endif
50	continue
 
c--normalize RF power profile:
	sum=0.0
	do 250 i=1,nshell
	sum=sum+rmin/nshell*rovera(i)*rmin*2*3.14159
	1	*2*3.14159*Rmaj*Pstovn(i)
250	continue
	if(LRFmod .eq. 3)Prftot=sum
	const=prftot/sum
 
c--calculate the power profile (use pstovn temporarily):
	do 275 i=nshell,1,-1
	if(LRFmod .ne. 2)then
	pstovn(i)=pstovn(i)*const
	erfpro(i)=erfpro(i)*sqrt(const)
 
	else
	pstovn(i)=pstovn(i)*const
 
	if(sing(i) .gt. 0.0)then
	eplus2=1.e7*pstovn(i)*Btor*Rmaj/Rres*Rmaj
	1	/(concmi*denpro(i)*1.e13*rZbeam
	1	*4.8032*2.9979*Rres**2*sing(i))
	erfpro(i)=sqrt(eplus2)
 
! use constant e-plus inside a region where Pstix=0.0
	else
	erfpro(i)=erfpro(i+1)
	endif
	endif
 
275	continue
 
c save minority power profile:
	do i=1,nshell
	pichspec(i,nfasti)=pstovn(i)
	enddo
 
c--calculate the k-perp profile if desired. Models:
c
c   Lkmodel = 0	cold plasma dispersion relation (Stix NF 1975 eq. 3)
c
c   Lkmodel = 1  kpar=0 limit of above, kperp=omega/Valfven
c
c   Lkmodel = +-2  kperp(r) already input in data file.
c   Lkmodel = -2   also set Eminus (right hand component) to zero
 
c for bulk ions:
	omegci=9.58e3*aplasz(2)*Btor*Rmaj/Rres/Ap
	zalpha=(freqrf*2*3.14159)/omegci
 
c transform kparallel from antenna to resonance layer:
	zkparres=rkpar*antRcen/Rres
 
	if(abs(Lkmodel) .ne. 2)then
c find kperp on all shells:
	do 285 i=1,nshell
	valfen=2.18e11*Btor*Rmaj/Rres
	1	/Sqrt(denpro(i)*1.e13 *Ap/aplasz(2))
	rkperp(i)=freqrf*2*3.14159/valfen
	if(Lkmodel .ne. 1)then
	rnpar=(zkparres/rkperp(i))**2
	factor=(1-2*rnpar-rnpar**2*(zalpha**2-1))
	1	/(1+rnpar*(zalpha**2-1))
c if the mode is evanescent, use the imaginary part of k:
	rkperp(i)=rkperp(i)*sqrt(abs(factor))
	endif
 
285	continue
	endif
 
c find the wave polarization from the cold plasma dispersion relation
c ignoring the minority (a la Stix):
	do 287 i=1,nshell
	valfen=2.18e11*Btor*Rmaj/Rres
	1	/Sqrt(denpro(i)*1.e13 *Ap/aplasz(2))
	rnpar=zkparres/(2.*3.14159*freqrf*Valfen)
 
	Polariz(i)=(zalpha-1-rnpar**2 *(zalpha**2-1))
	1       /(zalpha+1+rnpar**2 *(zalpha**2-1))
 
287	continue	!	enddo
 
c make plots only for the first and the last two iterations:
	if(nrloop .ne. 0 .and. nrloop .ne. mrloop-1 .and.
	1   nrloop .ne. mrloop) goto 390
 
!plot power profile:
	call initt(240)
	call binitt
	call fpgrlab
	call place('ul4  ')
	call npts(nshell)
	zymin=0.0
	zymax=0.0
	call mnmx(pstovn,zymin,zymax)
	call dlimy(zymin,zymax)
	call dlimx(0.,1.)
	call check(rovera,pstovn)
	call dsplay(rovera,pstovn)
	call AGTcgt(0,'Prf (W/cc)$')
	call AGTchs('r/a$')
	write(string,311)prftot/1.e6,Rres
311	format(1pe8.2,' MW RF Rres=',1pe8.2,' cm')
	call AGTmhs(100,745,string)
 
	call binitt
	call place('ur4  ')
	call npts(nshell)
	zymin=0.0
	zymax=0.0
	call mnmx(rkperp,zymin,zymax)
	call dlimy(zymin,zymax)
	call dlimx(0.,1.)
	call check(rovera,rkperp)
	call dsplay(rovera,rkperp)
	call AGTcgt(0,'k-perp (1/cm)$')
	call AGTchs('r/a$')
	write(string,317)rkpar
317	format('kpar = +- ',f5.3,'$')
	call AGTcgt(745,string)
 
!calculate and plot the Stix temperature Ttail=P*taus/n/3
	do 450 i=1,nshell
c--convert to power per particle in kev/sec
	zPovn= pstovn(i)/(1.6022e-6 * concmi * denpro(i)) /1000.
450	Ttail(i)=zPovn*tause(i)/3.
 
	call binitt
	call place('ll4  ')
	call npts(nshell)
	zymin=0.0
	zymax=0.0
	call mnmx(Ttail,zymin,zymax)
	call dlimy(zymin,zymax)
	call dlimx(0.,1.)
	call check(rovera,Ttail)
	call dsplay(rovera,Ttail)
	call AGTcgt(0,'Ttail (KeV) =P/n*taus/3$')
	call AGTchs('r/a$')
	write(string,320)concmi
320	format('nmin / ne =',f6.4,'$')
	call AGTcgt(370,string)
 
!convert E-plus to Volts/cm, plot it, and convert back
	do 335 i=1,nshell
335	Erfpro(i)=Erfpro(i)*299.79
	call binitt
	call place('lr4  ')
	call npts(nshell)
	zymin=0.0
	zymax=0.0
	call mnmx(Erfpro,zymin,zymax)
	call dlimy(zymin,zymax)
	call dlimx(0.,1.)
	call check(rovera,erfpro)
	call dsplay(rovera,erfpro)
	call AGTcgt(0,'E-plus (V/cm)$')
	call AGTchs('r/a$')
	call upause(0)
	call finitt(0,0)
	do 337 i=1,nshell
337	Erfpro(i)=Erfpro(i)/299.79
 
 
390	continue
 
c--convert to power per particle in ev/sec
c--convert to units used in the code: PSTOVN in ev/sec:
	do 400 i=1,nshell
400	pstovn(i)=pstovn(i)/(1.6022e-6 * concmi * denpro(i))
 
!	call plots(rovera,pstovn,nshell,'r/a',
!	1	'RF (ev/sec)','Stix RF power per particle profile')
 
c--divide E-plus by sqrt(2), half the power is in +kpar, the other
c	half in -kpar:
	do 513 i=1,nshell
513	Erfpro(i)=Erfpro(i)/sqrt(2.)
 
c--convert the outputs of this code, which were originally functions of
c r only, to be functions of r and theta for compatibility with Pat's
c SHOOT code:
 
	do i=1,nshell
	do j=1,nticrf+1
	Eplus(i,j)=Erfpro(i)
	crfpol(i,j)=1./polariz(i)
	skperp(i,j)=rkperp(i)
	enddo
	enddo
 
c Turn off the E-minus component to benchmark with Lodestar's code
c if desired (by setting Lkmodel=-2):
 
	if(Lkmodel .eq. -2)then
	do i=1,nshell
	do j=1,nticrf+1
	crfpol(i,j)=0.
	enddo
	enddo
	endif
 
	return
	end
