c	August,1994.
c	Richard Goldfinger (rcg)
 
c       This is the TRANSP sawtooth model for icrf heated minority species.
c	The distribution function for the minority  species
c	is replaced by its volume average for r<rmix, is left unchanged outside
c	rmix. This modified distribution function is then Fokker-Planck'ed
c	the next time fpprf for icrf is called.
c	This is done at each sawtooth event *while the minority is
c	non-thermal* (determined by the value of nthrf).
 
c *******************************************************
c	To invoke this model for ICRF tail sawtoothing, the
c	user needs to have these namelist switches set:
c	nlicrf=.t.,nlsaw=.t., nicrf=1 (use fpp+SPRUCE for icrf),
c	nlsawic=.t..
c *******************************************************
 
 
        subroutine sawfppic(f1rad)
 
	include 'FPPCOM'
	include 'F1COM'
	include 'TRCOM'
 
 
        integer ien,i,j,ishell,iupper,itop
	real zdr,zsum1,zsum2,zse1(maxshe),zse2(maxshe)
c	For each pitch angle and energy...
	real zfmu1(maxshe)	!f(r) before sawtooth flattening
	real zfmu2(maxshe)	!f(r) after sawtooth event
 
 
 
	if (nicrf .ne. 1)then
	write (nout,*) 'nicrf/fpp switch .ne. 1 using SAWFPPIC model!'
	write (nout,*) 'No minority ion sawtoothing (SAWFPPIC)'
	return
	endif
 
c	If nthrf=2, the icrf minority
c       was flagged as 'non-thermal' and was fp'ed in the last call to
c       fpichmn; if nthrf=1, fpichmn determined that thermalization was
c       complete and turned off the icrf.
	if ( nthrf .ne. 2) return
 
	write(nonlin,110) ta
	write(nout,110) ta
110	format(1x,' %SAWFPPIC routine called @ta= ', 1pe18.4)
 
c	debug file
crcgc	guess that 25 is an unused unit number!
c	open (unit=25,file='sawfpp_debug.', status='unknown')
c	write(25,110) ta
c	write (25,100)ta,tb,tlicrf,tcrfon
c100	format(1x,' ta tb tlicrf tcrfon  ',4e14.5)
c	write (25,*)'energy index runs from 0,maxen. Maxen:   ', maxen
c	write(25,*) 'pitchangle index runs from 1,maxxsi. Maxxsi:  ',maxxsi
c	write(25,*) 'rminpro', rminpro
c	write(25,*) 'outer zone bdy, XRADZB:',xradzb
c	write(25,*) 'zone center, XRADZC:',xradzc
c	write(25,112)
c	Actually, the following print statemnet for xi(i,2) is misleading
c	because the TRANSP array xi is defined from lcentr, not
c	from 1. 4-25-94
c112	format(/,x,' i rminpro   xi(i,2) xradzb xradzc')
c	write(25,111) (i,rminpro(i),xi(i,2),xradzb(i),xradzc(i),
c	1                i=1,maxshe)
111	format(x,(i4,4(1pe16.4)) )
 
c	*Calculate IMIX.* The analytic requirements on helflx will not be
c	re-checked here, since they were in the previous call to sawhck.
	imix=0
	do 10 j=lcentr,ledge
	jp1=j+1
	zhf1=helflx(j)
	zhf2=helflx(jp1)
	if ((zhf1 .gt. 0.0) .and. (zhf2 .le. 0.0)) then
c	end of mixing region found
	imix=j
	endif
10	continue
c	write(25,*) 'Mixing radius index found to be:', imix
c	write(25,*) '==> mixing r, rminpro(imix-lcentr+1), is'
c	1, rminpro(imix-lcentr+1)
 
c	The following is derived from Greg Hammett's rdstepdw routine.
 
c	Compute flux surface averaged energy density on shells
c	for diagnostic below (from moms routine).
c	zse1(ishell)=0.0  !sum for energy, on this shell, before mixing
c	zse2(ishell)=0.0  !sum for energy, on this shell, after mixing
 
c	Note-this diagnostic requires quantiies from FPP which are
c	NOT saved in TRCOM. Hence the diagnostic breaks in the
c	pathological event that a restart occurs and no call occurs
c	to the icrf/fpp package before this point is reached. In
c	this event mixing code should still work OK... because
c	rminsav and btxsiic ARE saved in TRCOM.
 
	do ishell=1,maxshe
	zse1(ishell)=0.0
	enddo
 
	do ishell=1,maxshe
	do j=1,maxxsi
	do i=0,maxen
	zse1(ishell)=zse1(ishell)+ enint(ishell)*25.*delxsi(j)
	1    *abs(xsi(j))*btxsiic(j,ishell)*f1rad(i,j,ishell)
	enddo
	enddo
	enddo
 
c	To ensure restart non-volatility, need to recompute nshell
	nshell=ledge-lcentr+1
 
c	Do the sawtooth flattening...
 
c	Loop over energy
	do  900 ien = 0,maxen
 
c	loop over midplane pitchangle
        do 800  j=1,maxxsi
 
	zsum1=0.0
	zsum2=0.0
 
c	Loop over radius, from center to zone which contains r=rmix,
c	the mixing radius, as determined above.
 
	do 50 ishell=1,nshell
	zfmu1(ishell)=0.0
	zfmu2(ishell)=0.0
50	continue
 
c	Set initial zfmu1(ishell)
 
	do 60 ishell=1,nshell
	zfmu1(ishell)=f1rad(ien,j,ishell)
c	For r<rmix, zfmu2 takes on vol. averaged value, calculated below.
	zfmu2(ishell)=zfmu1(ishell)	
60	continue
 
c	Indexing:
c	Since rminpro(i)==xradzc(i)*rmin==xi(lcentr+i-1,2)*rmin and since
c	the TRANSP indexing runs from J=lcentr,imix for sawtooth sections,
c	the rminpro corresponding to the zone which contains
c	the mixing radius is xi(imix,2)*rmin.
c	And therefore the index over rminpro must run from 1 to (imix-lcentr+1)
c	(Rminpro and btxsi were copied into rminsav and btxsiic in fptotric
c	to ensure restart non-volatility.)
 
	do 300 i=1,imix-lcentr+1
 
c find grid step sizes:
 
        if(i .gt. 1 .and. i .lt. nshell)then
            zdr=(rminsav(i+1)-rminsav(i-1))/2.0
        endif
        if(i .eq. 1) then
            zdr=(rminsav(2)+rminsav(1))/2.
        endif
        if(i .eq. nshell) then
            zdr=rmin-(rminsav(nshell)+rminsav(nshell-1))/2.
        endif
 
	zsum1=zsum1+zfmu1(i)*btxsiic(j,i)*rminsav(i)*zdr
	zsum2=zsum2+         btxsiic(j,i)*rminsav(i)*zdr
300     continue	!end r loop
 
crcg	diagnostics
c	iupper=imix-lcentr+1
c	iupper=min0(iupper,20)
c	write(25,*) 'after 300 loop. zsum1,2:  ',zsum1,zsum2
c	write(25,*) 'energy index,ien,  pitchAngle, j: ' ,ien,j
c	write (25,102) (fpfrad(ien+1,j,ishell,mibsp1),
c	write (25,102) (f1rad(ien,j,ishell),
c	1  ishell=1,iupper)
c	itop=min0(2*iupper,20)
c	write (25,102) (fpfrad(ien+1,j,ishell,mibsp1),
c	write (25,102) (f1rad(ien,j,ishell),
c	1  ishell=iupper,itop)
c102	format(1x,'fpfrad',/,2x,5(1pe14.5) )
 
 
c	 copy new sawtooth modified minority species  back into
c	 f1rad, df is unchanged for r>rmix
 
	do 325 ishell=1,imix-lcentr+1
	zfmu2(ishell)=zsum1/zsum2
        f1rad(ien,j,ishell)=zfmu2(ishell)
325     continue
 
c	Debug: Check for particle conservation , a la rdstepdw
c	zdr has always been constant in debug runs I've seen,
c	so following should work (GWH did same in rdstepdw).
c	This test is at each E and xsi.
 
	zsum1=0.0
	zsum2=0.0
	do 301 i=1,nshell
	zsum1=zsum1+zfmu1(i)*btxsiic(j,i)*rminsav(i)*zdr
	zsum2=zsum2+zfmu2(i)*btxsiic(j,i)*rminsav(i)*zdr
 
301	continue
 
c	Diagnostic
	if (abs(zsum1-zsum2) .gt. .01*zsum1 .and.
	1 zsum1 .gt. 1.e-30) then
	write(nout,*) ' Warning in sawfppic: particles not conserved!'
	write(nout,*) 'zsum1,zum2 :', zsum1,zsum2
	write(nonlin,*) ' Warning in sawfppic: particles not conserved!'
	write(nonlin,*) 'zsum1,zum2 :', zsum1,zsum2
	endif
 
 
800     continue	!end xsi(j) loop
900     continue	!end ien loop
 
c	Now compute total vol. integrated energy and compare with pre-mixing result
 
c	Compute energy on shells for diagnostic (from moms).
c	zse2 for energy, on this shell, after mixing
c	Initialize
	do ishell=1,maxshe
	zse2(ishell)=0.0
	enddo
	do ishell=1,maxshe
	do j=1,maxxsi
	do i=0,maxen
	zse2(ishell)=zse2(ishell)+ enint(ishell)*25.*delxsi(j)
	1   *abs(xsi(j))*btxsiic(j,ishell)*f1rad(i,j,ishell)
	enddo
	enddo
	enddo
 
crcgc	Do volume integral, within factor of 4*pi**2*R, of energy
	zsum1=0.0
	zsum2=0.0
	do i=1,maxshe
 
c find grid step sizes:
 
        if(i .gt. 1 .and. i .lt. nshell)then
            zdr=(rminsav(i+1)-rminsav(i-1))/2.0
        endif
        if(i .eq. 1) then
            zdr=(rminsav(2)+rminsav(1))/2.
        endif
        if(i .eq. nshell) then
            zdr=rmin-(rminsav(nshell)+rminsav(nshell-1))/2.
        endif
	zsum1=zsum2+ zse1(i)*rminsav(i)*zdr
	zsum2=zsum2+ zse2(i)*rminsav(i)*zdr
	enddo
 
c	write (25,*) 'zse1',zse1
c	write(25,*) 'zse2',zse2
c	write (25,*) 'zsum1,zsum2 total energy before and after'
c	write (6,*) 'zsum1,zsum2 total energy before and after'
c	write(25,9999)zsum1,zsum2
c	write(6,9999)zsum1,zsum2
c 9999	format(1x,'Sawfppic mixing',2(1pe20.12))
c	close(unit=25)
 
	return
	end
 
