      subroutine sptensor(tplus0,tplus1,tmins0,tmins1,alpha,alfven,
     %      w,zdRovR,bmag,dense,tempe,nions,awi,qi,densi,tempi,akz)
      include 'SPRINC'
      real awi(ionmax),qi(ionmax),densi(ionmax),tempi(ionmax)
      complex tplus0(ionmax+1),tplus1(ionmax+1),tmins0(ionmax+1),
     %        tmins1(ionmax+1),alpha(ionmax+1),uplus(ionmax+1),
     %        umins(ionmax+1),uzero(ionmax+1),akzz,uplust,uminst,
     %        uzerot,zfuncp,zfuncm,zfunavg
c GWH 15-Jan-1989: renamed from tensor to sptensor
c     TENSOR calculates the transverse dielectric tensor constants
c     to 1st order in plasma beta.  Electron Landau and TTMP terms
c     are included.  All 1st order ion contributions are included.
c     The assumptions are two:  1)  |kperp*vti/wci|**2<<1, the Larmor
c     condition, and 2)  |kperp*c/wpe|**2<<1, the small Eparallel
c     condition.  If the minoritry species has a large nT, then high
c     wavenumber normal modes are possible in the resonance layer,
c     thus violating the Larmor expansion criterion.
c       T(plus/minus)=t(plus/mins)0+(nperp**2)*t(plus/mins)1+
c                     nperp(dot)(-iDel(+/-)bHat(cross)Del)t(plus/mins)1/ako.
c       Kalpha=-alpha(nperp)(nperp)+iDel(alpha)(nperp)/ako; K=1.+K'+Kalpha.
c       K'(1,1)=K'(2,2)=Tplus+Tmins; K'(1,2)=-K'(2,1)=i*(Tplus-Tmins).
c
c GWH: modified on 8/14/89 to exclude the 1st order (in kperp**2 rho**2)
c contributions to the fundamental ion response.  This corresponds to
c expanding J0^2 = 1 and ignoring the next order contribution.
c For extremely energetic minority ions, the next order contribution
c can easily cause very large negative powers!
c
      data tmpmin,akpmin /.03,.02/
      nspec=nions+1
      abkp=max(abs(akz),akpmin)
      ako2=1.11265e-17*w**2
c
c
c     ELECTRON CONTRIBUTIONS
      wpe2=3.18261e3*dense
      wce=1.75882e11*bmag
      vte=1.87554e7*sqrt(max(tempe,tmpmin))
      alfven=1.+wpe2/wce**2
c
c     Zero order (fundamental) terms
      tplus0(nspec)=cmplx(-.5*wpe2/(w*(w+wce)),0.)
      tmins0(nspec)=cmplx(-.5*wpe2/(w*(w-wce)),0.)
c
c     Landau damping and TTMP terms
      argp=w/(abkp*vte)
      fact=.25*wpe2*ako2*vte/(w*abkp*wce**2)
      zfuncp=zfunavg(argp,0.0)
      tplus1(nspec)=cmplx(fact,0.)*((2.,0.)*zfuncp
     %                  +cmplx(.5*abkp*vte/(w-wce)+2.5*abkp*vte/(w+wce)
     %                            -abkp*vte/(w+2.*wce),0.))
      tmins1(nspec)=cmplx(fact,0.)*((2.,0.)*zfuncp
     %                  +cmplx(.5*abkp*vte/(w+wce)+2.5*abkp*vte/(w-wce)
     %                            -abkp*vte/(w-2.*wce),0.))
      alpha(nspec)=cmplx(4.*fact,0.)*
     %                     (zfuncp+cmplx(abkp*vte*w/(w**2-wce**2),0.))
      fact=2.*(wce/(abkp*vte))*fact
      zfuncp=-(2.,0.)*((1.,0.)+cmplx(argp,0.)*zfuncp)
      uplus(nspec)=cmplx(fact,0.)*(zfuncp+cmplx(abkp*vte/(w+wce),0.))
     %                                                       +(1.,0.)
      umins(nspec)=cmplx(fact,0.)*(zfuncp+cmplx(abkp*vte/(w-wce),0.))
     %                                                       -(1.,0.)
      fact=2.*(w*wce/(ako2*vte**2))*fact
      akzz=(1.,0.)-cmplx(fact,0.)*zfuncp
c
c
c     ION CONTRIBUTIONS
      do 10 i=1,nions
      wci=9.64853e7*bmag*qi(i)/awi(i)
      vti=4.39284e5*sqrt(max(tempi(i),tmpmin)/awi(i))
      wpi2=1.74951*qi(i)**2*densi(i)/awi(i)
      alfven=alfven+wpi2/wci**2
c
c     Zero order (fundamental) terms
      argp=(w-wci)/(abkp*vti)
      argm=(w+wci)/(abkp*vti)
      dzeta=zdRovR*wci/(abkp*vti)
      fact=.5*wpi2/(w*abkp*vti)
      zfuncp=zfunavg(argp,dzeta)
      zfuncm=zfunavg(argm,dzeta)
      tplus0(i)=cmplx(fact,0.)*zfuncp
      tmins0(i)=cmplx(fact,0.)*zfuncm
 
c
c     1st order fundamental terms
c
c initialize 1st order fundamental terms:
      tplus1(i)=0.0
      tmins1(i)=0.0
      alpha(i)=0.0
      uplus(i)=0.0
      umins(i)=0.0
 
c if you want to turn on 1st order fundamental lines, then uncomment
c the next 7 commented lines:
      fact=.5*ako2*(vti/wci)**2*fact
c      tplus1(i)=-cmplx(.5*fact,0.)*((5.,0.)*zfuncp+zfuncm)
c      tmins1(i)=-cmplx(.5*fact,0.)*((5.,0.)*zfuncm+zfuncp)
c      alpha(i)=-cmplx(2.*fact,0.)*(zfuncp+zfuncm)
      fact=2.*(wci/(abkp*vti))*fact
c      zfuncp=-(2.,0.)*((1.,0.)+cmplx(argp,0.)*zfuncp)
c      zfuncm=-(2.,0.)*((1.,0.)+cmplx(argm,0.)*zfuncm)
c      uplus(i)=-cmplx(fact,0.)*zfuncp
c      umins(i)=-cmplx(fact,0.)*zfuncm
 
c
c     2nd harmonic terms
      argp=(w-2.*wci)/(abkp*vti)
      argm=(w+2.*wci)/(abkp*vti)
      dzeta=2.*zdRovR*wci/(abkp*vti)
      fact=.5*(abkp*vti/wci)*fact
      zfuncp=zfunavg(argp,dzeta)
      zfuncm=zfunavg(argm,dzeta)
      tplus1(i)=tplus1(i)+cmplx(fact,0.)*zfuncp
      tmins1(i)=tmins1(i)+cmplx(fact,0.)*zfuncm
c
c     Landau terms
      argp=w/(abkp*vti)
      zfuncp=zfunavg(argp,0.0)
      tplus1(i)=tplus1(i)+cmplx(2.,0.)*fact*zfuncp
      tmins1(i)=tmins1(i)+cmplx(2.,0.)*fact*zfuncp
      alpha(i)=alpha(i)+cmplx(4.*fact,0.)*zfuncp
      fact=2.*(wci/(abkp*vti))*fact
      zfuncp=-(2.,0.)*((1.,0.)+cmplx(argp,0.)*zfuncp)
      uplus(i)=uplus(i)+cmplx(fact,0.)*zfuncp
      umins(i)=umins(i)+cmplx(fact,0.)*zfuncp
      fact=2.*(w*wci/(ako2*vti**2))*fact
      akzz=akzz-cmplx(fact,0.)*zfuncp
c
   10 continue
c
c     Include Eparallel corrections in symmetric form
      uplust=(0.,0.)
      uminst=(0.,0.)
      uzerot=(0.,0.)
      do 20 i=1,nspec
      uzero(i)=(.25,0.)*(uplus(i)+umins(i))
      uplust=uplust+uplus(i)
      uminst=uminst+umins(i)
   20 uzerot=uzerot+uzero(i)
      do 30 i=1,nspec
      tplus1(i)=tplus1(i)-cmplx(.5*abkp**2/ako2,0.)*
     %                           (uplus(i)*uzerot+uplust*uzero(i))/akzz
      tmins1(i)=tmins1(i)-cmplx(.5*abkp**2/ako2,0.)*
     %                           (umins(i)*uzerot+uminst*uzero(i))/akzz
   30 alpha(i)=alpha(i)-cmplx(.5*abkp**2/ako2,0.)*
     %                           (uplus(i)*uminst+uplust*umins(i))/akzz
c
      return
      end
