 
	SUBROUTINE TRTOFP(jicrf)
 
c
c transfers information from TRANSP to FPP format, so that FPP can do a
c Fokker-Planck time step.
c
c dmc jan 1995 -- added code to interface anomolous fast ion diffusivity
c profile, difb -> fpdifca
c
C Mod RCG August,1994 Set teeny fpp step, tcalc, if ta=tsaw (  ==>
C this is a call from SAWMOD  after minority ion sawtoothing by SAWFPPIC).
C
C	Updated:
C  09/15/93 TBT Set BkgMax=0 always.
 
 
c INPUT:
c jicrf=.t. to prepare for an ICRF calculation
c      =.f. to prepare for a NBI calculation
c
c ??????????????????? A question mark signifies a place where
c the coding needs improvement.
c
c This replaces calls to INPUTNML and PROF (and therfore gneut/frantic
c and gnusub) by FPPRF.
c
c There are a number of crude approximations in the conversion from
c TRANSP to FPP at the moment.  Read through the comments in this
c routine for details of how to improve it.
c
 
c TRANSP common block:
	include 'TRCOM'
 
c FPP common blocks:
	include 'FPPCOM'
c AZMAIN switch for Helium plasma cross-sections:
	include 'HE4COM'
 
	character*9 idate   ! today's date
	logical jicrf	! .t. to set up for ICRF heating instead of NBI
c
c  data stmts -- for portability, must precede executable code
c
	data imcrfdb /0/
	data irfmulti/2/
c
c	data cratio /1.00/
c
c--------------------------------------------------------------------
c define i/o units:
	lsysout=nonlin	!transp/olympus file for output to "terminal"
	lsysin=nin	!transp/olympus file for input from "terminal"
 
c********************************************************************
c set a few variables that used to be set with the debugger:
c these are control-type variables that do not change very often.
c We should consider putting them into a special control namelist to
c be read in only once:
	cxonly=.false.  !.f.=normal, .t.=CX calc only, no FP calculation
	lfget=.false.   !.t. = read f from disk at beginning of run
	lfsave=.false.  !.t. = save f onto disk at the end of the run
	batchm=.true.   !.t. =batch mode, do not prompt user for plots limits.
	LTransp=.True.  !FPP running under TRANSP, so FPP plots not needed
	nfpdepsw=nfpdepmd !2=TRANSP's accurate M.C. Beam deposition model
			!0=FPP's original gaussian model
			!1=FPP's 1988 divergence model
 
c  subroutine spect adds in a background maxwellian when
c	calculating the cx spectrum.  bkgmax gives the density
c	of the background maxwellian as a fraction of electron
c	density.  bkgmax=1 includes the regular background
c	bkgmax=0 turns off the background maxwellian.
 
C	bkgmax = 1  !+1=turn on background Maxwellian in CX spectrum.
C		    ! 0=turn OFF background Maxwellian in CX spectrum.	
C	if(jicrf) bkgmax=0
 
	bkgmax=0    ! TBT 9/93
 
C TBT	The FPP CX simulator is skipped by TRANSP which has its own
C       simulator. (If we ever want to use the FPP simulator, we need
C       to work on the logic for multiple ion species.
 
 
	cratio=1.00 ! compression ratio (1.00 = no compression)
	nplte=0	    !# of spectra vs. E plots to produce
	npltt=0	    !# of spectra vs. t plots to produce
	npltf=0	    !# of plots of f(v_precession) to produce
	Lplote=0    ! 0 = turn off plots of f(E)
	Lplota=0    ! 0 = turn off plots of f(angle)
	Lplotf=0    ! 0 = turn off contour plots of f(E,angle)
	LplotH=0    ! 0 = turn off plots of beam deposition H(r)'s.
	Lplod=0	    ! 0 = don't plot detector sensitivities (interactive
		    !	only).
c*********************************************************************
	LTr1st=.false.
	if(jicrf)then
	    if(tlicrf .eq. tcrfon) LTr1st=.true.
	else
	    if(nbstep .eq. 1) LTr1st=.true.
	endif
 
c graph label:
        call c9date(idate)    !routine which gives today's date
        runlabel='FPP/TRANSP '//idate//' '//runid//' '
 
c establish the plasma geometry so that FPVINT (volume integrator)
c will work:
c NSHELL = # of radial zones for FPP computations.
	nshell=ledge-lcentr+1
	if(nshell .gt. maxshe)then
	call aborter(lsysout,'FPP vs. TRANSP error: nshell>maxshe ')
	endif
 
C Cross-sectional equivalent minor radius:
 
	rmin=sqrt(parea/pi)
 
C Volume/Area equivalent major radius:
	Rmaj=RmajRA(lep1,1)
 
c define the radial zone grid:
 
	zdifbp=difb(lcentr)
 
	do 1120 i=1,nshell
	xradzb(i)=xi(lcentr+i,1)
	xradzc(i)=xi(lcentr+i-1,2)
	rminpro(i)=xradzc(i)*rmin
 
	dVolfp(i)=dVol(lcentr+i-1,2)
	dAreafp(i)=darea(lcentr+i-1)
 
c rovera(i)=r/a is calculated for old circular parts of FPP:
	rovera(i)=rminpro(i)/rmin
 
c dmc -- anomolous diffusivity profile
	  zdifb=difb(lcentr+i)
	  fpdifca(i)=0.5*(zdifbp+zdifb)
	  zdifbp=zdifb
c
1120	continue
 
	Areafp(1)=dAreafp(1)
	Volfp(1)=dVolfp(1)
	Rmajpro(1)=Volfp(1)/(2*pi*Areafp(1))
	do 1130 i=2,nshell
	Areafp(i)=Areafp(i-1)+dAreafp(i)
	Volfp(i)=Volfp(i-1)+dVolfp(i)
	Rmajpro(i)=Volfp(i)/(2*pi*Areafp(i))
1130	continue
 
	tflxtot=tflux*1.e8 ! Toroidal Magnetic Flux (Gauss*cm**2)
 
c This next section was originally in PROF.  If we ever do compression,
c we will have to fit this in somewhere (not here because Btor hasn't
c been defined yet!)
c compress a few main variables now if needed:
c	if(cratio .ne. 1.00)then
c	Rmaj=Rmaj/cratio
c	rmin=rmin/cratio**.5
c	Btor=Btor*cratio
c	Placur=placur*cratio
c	endif
 
c Set number of ions in FPP's multiple ion model.
c Number of hydrogenic ions + nsbeam beam ions + 1 impurity ion:
	nions=NG+nsbeam+1
	nfptherm=NG+1	! # of "thermal" ion species
c plus minority if present:
	zminsum=0.0
	do 1010 i=1,nzones
1010	zminsum=zminsum+rhmin(lcentr+i-1,2)
	if(zminsum .gt. 0.0) nions=nions+1
 
c read in ion masses and charges:
	do 1020 i=1,ng
	nAions(i)=aplasm(i)
1020	nZions(i)=backz(i)
	do 1022 i=1,nsbeam
	nAions(ng+i)=abeams(i)
1022	nZions(ng+i)=xzbeams(i)
	nAions(ng+nsbeam+1)=aimp
	nZions(ng+nsbeam+1)=xzimp
	if(zminsum .gt. 0.0)then
	    nAions(nions)=amini
	    nZions(nions)=xzmini
	endif
 
	do 1030 i=1,nions
        nbbfus(i)=0
1030	call namels(nZions(i),nAions(i),namions(i))
 
c identify the beam species:
	do 1033 i=ng+1,ng+nsbeam
        nbbfus(i)=1
	ilen=nchar(namions(i))
1033	namions(i)=namions(i)(1:ilen)//' beam'
	nfasti=ng+lsbeam
 
c identify the RF minority species:
	if(JICRF) nfasti=nions
	if(zminsum .gt. 0.0)then
            nbbfus(nions)=1
	    ilen=nchar(namions(nions))
	    namions(nions)=namions(nions)(1:ilen)//' RFmin'
	endif
 
C a few q parameters, never actually used by TRANSP version of FPP:
	qax=qgeo(lcentr)
	qlim=qgeo(lep1)
        QRAT=QLIM/QAX
 
c read in Te, Ti, ne, ni, q, and rotation frequency  profiles:
c we do interpolation here in case we ever decide to have a different
c number of FPP and TRANSP radial zones:
	do 1100 i=1,nshell
	zx=rminpro(i)/rmin
	trelec(i)=fpintrp(zx,te(lcentr,2),xi(lcentr,2),nzones)
	drelec(i)=fpintrp(zx,rhoel(lcentr,2),xi(lcentr,2),nzones)
	romega(i)=fpintrp(zx,omegag(lcentr),xi(lcentr,2),nzones)
	qpro(i)=fpintrp(zx,qgeo(lcentr),xi(lcentr,1),nzones+1)
 
	do 1040 iion=1,ng
	drions(i,iion)=fpintrp(zx,rhob(lcentr,iion,2),
	1	xi(lcentr,2),nzones)
	trions(i,iion)=fpintrp(zx,ti(lcentr,2),xi(lcentr,2),nzones)
	trionspa(i,iion)=trions(i,iion)
1040	continue
 
	do 1042 iion=ng+1,ng+nsbeam
	drions(i,iion)=fpintrp(zx,rhbs(lcentr,2,iion-ng),
	1		       xi(lcentr,2),nzones)
	trions(i,iion)=2./3.*fpintrp(zx,ebavps(lcentr,iion-ng),
	1                      xi(lcentr,2),nzones)
	trionspa(i,iion)=2.*fpintrp(zx,ebapl2s(lcentr,iion-ng),
	1                      xi(lcentr,2),nzones)
1042	continue
 
	drions(i,ng+nsbeam+1)=fpintrp(zx,rhi(lcentr,2),xi(lcentr,2),
	1                                nzones)
	trions(i,ng+nsbeam+1)=fpintrp(zx,ti(lcentr,2),xi(lcentr,2),
	1                                nzones)
	trionspa(i,ng+nsbeam+1)=trions(i,ng+nsbeam+1)
 
	if(zminsum .gt. 0.0)then
	drions(i,nions)=fpintrp(zx,rhmin(lcentr,2),xi(lcentr,2),nzones)
	trionspa(i,nions)=2.*fpintrp(zx,eminpar(lcentr),
	1   xi(lcentr,2),nzones)
	trions(i,nions)=2./3.*(0.5*trionspa(i,nions)+
	1   fpintrp(zx,eminper(lcentr),xi(lcentr,2),nzones))
	if(Ltr1st) then
	    trions(i,nions)= fpintrp(zx,ti(lcentr,2),xi(lcentr,2),nzones)
	    trionspa(i,nions)=trions(i,nions)
	endif
	endif
1100	continue
 
c set a reasonable lower bound on the temperatures of 0.02 eV
c (room temperature).
	do 1110 ish=1,nshell
	do 1110 iions=1,nions
	trions(ish,iions)=max(trions(ish,iions),0.02)
	trionspa(ish,iions)=max(trionspa(ish,iions),0.02)
1110	continue
 
c calculate an average ion temperature profile travion(ish)
	do 4103 ish=1,nshell
	ztiav=0.0
	zniav=0.0
	do 4102 iions=1,nions
	zniav=zniav+drions(ish,iions)
4102	ztiav=ztiav+drions(ish,iions)*trions(ish,iions)
4103	travion(ish)=ztiav/zniav
 
c****************************************************************
c for compatibility with older parts of the code which still
c use the older profile data, define the older profile data
c (tepro,tipro,denpro,rzeff,rzimp, etc...) here.  After the whole
c code is upgraded to the new multi-ion model, the variables defined
c here (down to the next row of "**********") will no longer be
c needed.
 
	nshellfr=nshell
	rminfr=rmin
 
	do 4105 ish=1,nshell
	tepro(ish)=trelec(ish)
	denpro(ish)=drelec(ish)/1.e13	
4105	tipro(ish)=trions(ish,1)  !use thermal ion temperature
! using travion is not a good approximation because self
! collisions are ignored for now.
 
	ab=nAions(nfasti)     !atomic mass A of the 'beam' species
	rZbeam=nZions(nfasti) !atomic charge Z of the 'beam' species
 
	zne=fpvint(drelec)
 
	if(zminsum .gt. 0.0)then
	znb=fpvint(drions(1,nions))
	cbeam=znb/zne
c concmi=nmin/ne for the fundamental minority species.
c concmi defaults to the same value as cbeam unless the "beam"
c is a 2cd harmonic majority species.  CONCMI is used
c to normalize the P/n profile in RFPROF.
 
	if(concmi .le. 0.0 .or. nharm .eq. 1)concmi=cbeam
	endif
 
c find the main ion species:
	imain=1
	do 4106 iion=1,ng
4106	if(drions(1,iion) .gt. drions(1,imain))imain=iion
	AP=nAions(imain)
	aZmain=nZions(imain)
	azneut=nZions(nfasti)
 
c calculate a volume-averaged Zeff:
	zeff=0.0
	do 4107 iion=1,nions
4107	zeff=zeff+nZions(iion)**2 *fpvint(drions(1,iion))	
	rZeff=zeff/fpvint(drelec)
 
c calculate the average impurity ion Z to give the proper depletion
c of the "main" ion species:
	zfmain=nZions(imain)*fpvint(drions(1,imain))/fpvint(drelec)
	rZimp=(rZeff-zfmain*nZions(imain))/(1.0-zfmain)
	Aim=rZimp*2.0    !AMU of the impurity
 
c
c A list of plasma species parameters (including electrons):
c
	APlasAMU(1)=5.4858026e-04    ! electrons are first in this list
        APlasAMU(2)=AP
        APlasAMU(3)=AIM
	AplasZ(1)=-1
        aplasz(2)=aZmain
        aplasz(3)=rZimp
        NumPlSp=3
        IF(rZeff.LT.1.001) NumPlSp=2
 
c*****************************************************************
 
C LOOP VOLTAGE.  Later use TRANSP EZ(j) electric field profile.  For
c now just use steady state edge value.
 
	VLoopfp=ez(ledge)*2*3.14159*Rmaj
 
C      TOROIDAL FIELD (a small B/Btexternal correction is made later
c	for ICRF):
	Btor=BzxR/Rmaj*1.e4
 
c********************************************************************
c
c Beam parameters.
c For now use FPP's simple beam deposition models.
c
c FPP's parallel beamlet model has been modified to give
c results fairly close to TRANSP's model including the effects of
c apertures and gaussian divergence (focussing has not yet been
c implemented, although it could be).  The solution
c for the beam intensity (as a function of width and height) can be
c written in terms of error functions, although special treatement is
c needed for the two limiting cases R_focal=R and R_focal>>R.
c (Currently using the R_focal>>R limit).
c
c Alternatively, we may later use TRANSP's DEP routines, which not
c only have all this focussing and divergence stuff in it, but track
c the beamlets through non-circular plasmas as well.   Doug is working
c on this!
c
c********************************************************************
 
	numnbi=nbeam
	if(numnbi .lt. 1)numnbi=1
c (fpp's logic is to turn off beams by setting the power to zero,
c not by setting numnbi to zero.)
 
c Ignore beams if we are setting up for an ICRF calculation:
	if(jicrf) then
	do 1142 i=1,numnbi
1142	pbeams(i)=0.
	goto 1147
	endif
 
c check the # of NBI's, and print out helpful error messages:
	if(numnbi .gt. maxnbi)then
	write(lsysout,*) ' '
	write(lsysout,*) 'We are about to bomb because numbni>MAXNBI.'
	write(lsysout,*) 'You can probably fix this easily by'
	write(lsysout,*) 'increasing the parameter MAXNBI in FPPCOM'
	write(lsysout,*) 'to be the same value as MIB in TRCOM,'
	write(lsysout,*) 'and then recompiling TRANSP.'
	call aborter(lsysout,'FPP vs. TRANSP error: numnbi>maxnbi')
	endif
 
c get basic information for all beams:
	do 1140 i=1,numnbi
	zthird=1.0-ffulla(i)-fhalfa(i)
c beam power (MegaWatts):
	pbeams(i)=xe*einja(i)*xninja(i)
	1   *(ffulla(i)+fhalfa(i)/2+zthird/3)/1.e6
	vbeams(i)=einja(i)  !beam voltage
	rbeams(i)=rtcena(i) !beam tangency radius (Rtan)
	if(.not. nlco(i)) rbeams(i)=-rbeams(i) !counter beams have Rtan<0
	BmixFull(i)=ffulla(i) !particle fraction at the full energy
	BmixHalf(i)=fhalfa(i) !particle fraction at the half energy
 
	fplbap(i)=xlbapa(i) !source to aperture distance
	fplbtn(i)=xlbtna(i) !source to tangency point distance
 
c if TRANSP's DEP has already calculated beam deposition, then get:
	bpshthru(i)=bpshna(i) !shine thru power lost (W)
	bpprompt(i)=bpprma(i) !prompt loss power (W)
 
1140	continue
 
c The present FPP beam deposition model cannot handle significant
c focussing nor ion sources which are off the midplane.  Check for this
c and print out an error message:
	if(nfpdepsw .ne. 2)then
	do 1145 i=1,numnbi
c ztopl= distance from the source to the plasma center or Rtan:
	z2=Rmaj**2-Rtcena(i)**2
	if(z2 .lt. 0.0) z2=0.0
	ztopl=xlbtna(i)-sqrt(z2)
 
	if(foclr .ne. 0.0)then
	if(ztopl/foclr .gt. divr) write(lsysout,*)
	1   'Warning! ignoring beam focussing in TRTOFP'
	if(ztopl/foclz .gt. divz) write(lsysout,*)
	1   'Warning! ignoring beam focussing in TRTOFP'
	endif
 
	if(xybsca(i) .ne. 0.0 .or. xybapa(i) .ne. 0.0) write(lsysout,*)
	1   'Warning! ignoring vertical displacement of beam in TRTOFP'
 
1145	continue
	endif
 
c beam shapes, Half width and half height of the edges:
c assumes and no focussing for now, so print out an error message
c if there is significant focussing or the sources are not on the midplane:
 
	if(nbshap .eq. 1)then
c rectangular source and aperture:
	fpbsHW=BmWidR	!beam source (grid) half width (cm)
	fpbsHH=BmWidZ	!beam source half height (cm)
	fpbaHW=Redge	!beam aperture half width (cm)
	fpbaHH=xZedge	!beam aperture half height (cm)
	fpbmdr=divR	!beam horizontal divergence sigma (radians)
	fpbmdz=divZ	!beam vertical divergence sigma (radians)
	else
c circular source and aperture, use equivalent area:
	if(nfpdepsw .ne. 2)then
	write(lsysout,*)'Warning!: using rectangular approximation to'
	write(lsysout,*)' circular beam cross-section in TRTOFP.'
	endif
	zarea=pi**.5/2.0
	fpbsHW=zarea*BmWidR   !beam grid half width (cm)
	fpbsHH=zarea*BmWidR   !beam grid half height (cm)
	fpbaHW=zarea*Redge    !beam aperture half width (cm)
	fpbaHH=zarea*Redge    !beam aperture half height (cm)
	fpbmdr=divR	      !beam horizontal divergence sigma (radians)
	fpbmdz=divZ	      !beam vertical divergence sigma (radians)
	endif
 
C      BEAM CURRENT RISE TIME
!Transp gives us the instantaneous P(t).
	trise=0.0
 
! tstart is the initial time of the FPP calculation:
! tstart+tcalc is the final time of the FPP calculation:
1147	tstart=tbm1
	tcalc=tbm2-tbm1
	toff=epsinv !set so that beam off time is ignored
 
!? Timing choices could be improved, add more control options
! to namelist:
	if(jicrf)then
	    tstart=tlicrf
	    dtfppmin=max(tb-ta,tb-tlicrf) !Set minimum time step
	    if(LTR1st)then
	    tcalc=dtfppmin !request minimum time step the first time.
 
c rcg. special teeny step if exactly on a sawtooth, and minority sawtooth model is called for
	    else if (ta .eq. tsaw) then	!This tsaw=ta flag is set in sawmod.
            tcalc=dtfppmin/100.
c Need to also make dtfppmin teeny...since fpprf enforces tcalc>=dtfppmin
	    dtfppmin=tcalc
 
	    else
	    tcalc=dticrf    !dticrf set by timbdy in heater-rcg.
	    endif
c rcg- end modification
	endif
 
! select time step:
! we can use a fancier model in the future, with the time step chose
! optimally, but for now, we will assume that FPP is called with the
! time step we need:
! dtFPP is the time between calls to MOMS.  Set dtFPP to tcalc/2.0
! to ensure that MOMS is called 3 times (at the start, half-way,
! and at the end), so that the fast ion stored energy is calculated
! more accurately when the background plasma temperature is changing
! rapidly:
	dtFPP=tcalc/2.0
! FPP actually advances by 2*dtfpp between MOMS calls:
	dtFPP=dtFPP/2.0
	nmulti(1)=1
	if(jicrf)nmulti(1)=irfmulti
	do 1155 i=2, nshell
1155	nmulti(i)=nmulti(1)
 
C  INNER WALL RADIUS
c Just use plasma inner edge for now, later use TRANSP's LIMLOS routine:
c?:
	call momry(1.0,pi,0,RinWal,zYdum,zjac)
 
c********************************************************************
c
c CX detector information (turned off for now)...
c
c********************************************************************
 
C      DIRECTION OF SIGHTLINE
	ldir=-1 ! 1=horizontal, 0=vertical, -1=none
 
C      NUMBER OF DETECTORS
	nline=0
	if(nline .gt. mxslin)then
	nline=mxslin
	write(lsysout,*) 'Warning: too many detector sightlines. '
	write(lsysout,*) 'Only the first ',nline,
	1   ' sightlines will be used.'
	endif
 
C      DETECTORS TANGENCIES
c      READ(iLUN41,*) (TANS(I), I=1,NLINE)
 
C	MAJOR RADIUS AT WHICH ALL SIGHT LINES CROSS.  (CM)
c	READ(iLUN41,*) RCrsfp
 
C	PHI ANGLE AT WHICH ALL SIGHT LINES CROSS.  (DEGREES)
c	READ(iLUN41,*) PhiCrsfp
 
C      ENERGY LIMITS FOR PLOTS
c elim1 is defunct, elim2 is the CX plot upper limit
c      READ(iLUN41,*) ELIM1,ELIM2
 
c*******************************************************************
c
c Neutral Profile from TRANSP
c
c*******************************************************************
c Transp has a fairly sophisticated neutral package.  FPP is currently
c only set up to handle a small fraction of TRANSP's neutral
c information.  We will ignore rotation of the neutrals.  FPP also
c only considers one neutral species, that is poloidally symmetric.
c The mass of this species (or the average mass for an average over
c multiple species) is the same as the plasma's mass Ap.
c
	Lantic=0    !1=FPP would use FRANTIC.
		    !2=FPP would use GNUSUB
		    !anything else: FPP won't change neutral arrays.
                    ! i.e., use TRANSP's neutral calculation.
 
	do 1160 i=1,nshell
	neupro(i)=0.0
	tnupro(i)=0.0
 
c find interpolation factors:
	zx=rminpro(i)/rmin
	jr=lkupr(zx,xi(lcentr,2),nzones)
	if(jr .lt. 1)then
	jr=1
	zw1=1.0
	zw2=0.0
	else if (jr .ge. nzones) then
	jr=nzones-1
	zw1=0.0
	zw2=1.0
	else
	zw1=(xi(lcentr+jr,2)-zx)/(xi(lcentr+jr,2)-xi(lcentr+jr-1,2))
	zw2=1.0-zw1
	endif
 
	do 1150 ispec=1,ng
	zwall=zw1*walln0(ispec,lcentr+jr-1)+zw2*walln0(ispec,lcentr+jr)
	zvol =zw1*voln0(ispec,lcentr+jr-1) +zw2*voln0(ispec,lcentr+jr)
	neupro(i)=neupro(i)+zwall+zvol
	zewall=zw1*ewall0(ispec,lcentr+jr-1)+zw2*ewall0(ispec,lcentr+jr)
	zevol= zw1*evol0(ispec,lcentr+jr-1) +zw2*evol0(ispec,lcentr+jr)
	tnupro(i)=tnupro(i)+zwall*zewall+zvol*zevol
1150	continue
 
	tnupro(i)=2./3.*tnupro(i)
1160	if(neupro(i) .gt. 0.0)tnupro(i)=tnupro(i)/neupro(i)
 
c crude correction for the fraction of fast CX neutrals which
c are recaptured before leaving the plasma:
	fpcxre=0.5
	do 1165 i=1,nshell
1165	neupro(i)=neupro(i)*(1.-fpcxre)
 
c the arrays walln0,voln0,ewall0,evol0 were define by TRANSP's PARM
c which was called from FPMAIN.  Note that rotation of the neutrals
c has been ignored.
 
c*******************************************************************
c
c Beam radial transport model
c
c*******************************************************************
 
	fpdifcon=0.0
	mrloop=0
 
cxx  for now, just set Dbeam=fpdifcon=cdifb, which is a fixed
cxx  number from the transp namelist:
c
c  dmc Jan 1995 -- now accept a profile for anomolous diffusivity,
c   fpp common array fpdifca (defined from DIFB above ...)
c
c   note also, nkdifb=2 means anomolous diffusion for fusion
c   product ions only, (not RF or beam fast ions)
c
 
	if(nmdifb .ne. 0) then
	   if(nkdifb.ne.2) then
	      fpdifcon=amax1(1.0,cdifb)  ! but use fpdifca(...)
	      mrloop=1
	   endif
	endif
 
c*******************************************************************
c
c Fishbone model
c
c*******************************************************************
 
C
C  FISHBONE MODEL # (SEE TEXT OF FISH.FOR)
c FISHBONE's ignored for now:
	LFishMdl=0
C  TIME BTWN FISHBONES (SEC'S)
c	READ(iLUN41,*) TFISH
C  FULL WIDTH OF LOSS REGIONS IN VPAR/V FOR MODEL 2
c	READ(iLUN41,*) WFISH2
C  FULL WIDTH OF LOSS REGION IN VPAR/V FOR MODEL 3
c	READ(iLUN41,*) WFISH3
C  FRACTION OF PARTICLES IN LOSS REGION THAT SURVIVE A FISH-BONE
c	READ(iLUN41,*) FSHFRC
C  MINIMUM ENERGY AFFECTED BY A FISH-BONE
c	READ(iLUN41,*) EFMIN
C  MAXIMUM ENERGY AFFECTED BY A FISH-BONE
c	READ(iLUN41,*) EFMAX
C  MINIMUM MINOR RADIUS AFFECTED BY A FISH-BONE
c	READ(iLUN41,*) RFMIN
C  MAXIMUM MINOR RADIUS AFFECTED BY A FISH-BONE
c	READ(iLUN41,*) RFMAX
 
c***************************************************
c                                                  *
c      read in rf data                             *
c                                                  *
c***************************************************
 
	prftot=0.0  !Total input RF power (watts)
	if(.not. jicrf) goto 2000
 
c For now, FPP/SHOOT can only deal with a single frequency.
c Assume that all antennas are identical and use the parameters
c given for the first antenna.
 
	do 1900 iant=1,nicha
	if(tb .ge. tonicha(iant) .and. tb .le. tofficha(iant))
	1	prftot=prftot+prficha(iant)
1900	continue
 
	freqrf=FRQicha(1)	!Frequency of ICRF in Hz
 
c For now, FPP/SHOOT only uses a single kparallel (actually +/-kpar),
c so pick a reasonable guess at the dominant kpar:
!? even my reasonable guess could be improved:
	zrelph=phicha(2,1)-phicha(1,1)
	if(abs(zrelph) .gt. 90.0)then
	rkpar=pi/sepicha(1)
	else
	zdist=sqrt(widicha(1)**2+sepicha(1)**2)
	rkpar=pi/2.0/zdist
	endif
 
! nharm = 1 for fundamental heating, = 2 for 2cd harmonic heating:
	nharm=1	!default
 
	LShoot=1 ! ICRF model 1= Smithe's SPRUCE code, 2=SHOOT, 0=guess
 
c LShoot .ne. 0 requires the following parameters (currently assumes
c two out-of-phase antennas are used):
 
	antRcen=RMJicha(1)+RMNicha(1)
	antwidth=widicha(1) !toroidal width of each antenna (cm)
	antsep=sepicha(1)   !toroidal separation between the two antennas (cm)
	antHH=abs(RMNicha(1))*sin(thicha(1)*pi/180./2.)
			    !antenna half height (cm)
 
C "Vacuum" plasma needed to avoid complications in the ICRF wave
c propagation:
	vacden=1.e11	! "vacuum" density (/cm**3)
	vactemp=100.	! "vacuum" temperature (eV)
 
! VV shape description in Fourier moments (in cm):
	if(mxVVmoms+1 .ne. mimom) call aborter(lsysout,
	1    'FPP and TRANSP have different number of VV moments')
 
	do 1905 imom=0,mxVVmoms
	VVRmoms(imom)=VVRmom(imom+1)
1905	VVZmoms(imom)=VVZmom(imom+1)
 
c ensure restartability by restoring some parameters from
c TRANSP common:
	if(.not. LTr1st)then
 
	rkpcorr=richkrho
 
	do 1910 ish=1,nshell
	zx=rminpro(ish)/rmin
ctmp: Temporarily use fpfrad to store rkpcpro until PORT.FOR upgraded:
	rkpcpro(ish)=fpfrad(mifpen,mifpxi,ish,mibsp1-1)
	qldamp(ish)=richdcor(lcentr+ish-1)
1910	continue
 
	endif
 
c Btor is the external magnetic field at Rmaj.  Make a small correction
c to model the effects of the poloidal field and diamagnetism on the
c position of the resonance layer.
	zpi=4.0*atan(1.0)
	zomegrf=2*zpi*freqrf  !RF angular frequency
	if(zomegrf .gt. 0.0) then
	  zRres = nharm*9.579e3*rzBeam*Btor*Rmaj/Ab/zomegrf
	else
	  zRres=Rmaj
	endif
	zcorr=fpintrp(zRres,fbx,rmajmp,nrmaj)
	Btor=Btor*zcorr
 
c LShoot=0 requires the following parameters:
 
c--LRFmod = 1 next line is RF E-plus profile
c  LRFmod = 2 next line is RF power profile
c	read(iLUN41,*)LRFmod
c--rf electric field profile (positive polarization):
c--(or rf power profile)
c	read(iLUN41,*) (erfpro(i),i=1,nshell)
c-- k-perp model (1= k-perp=w/vAlfven  2= enter profile on next line:)
c	read(iLUN41,*)Lkmodel
c--RF k-perp profile (cm-1):
c	read(iLUN41,*) (rkperp(i),i=1,nshell)
 
c*************************************************************
c
c End of ICRF parameters
c
c*************************************************************
 
2000	lradls=.false. !.f. = turns off a Krook model for radial diffusion
	lcxlos=.true.  !.t. = turn on the CX loss operator
	lbador=.true.  !.t. = turn on bad orbit losses
 
	if(jicrf)then
	lcxsrc=.true.  !.t. = turn on the source part of the CX operator
	lnsrc=.true.   !.t. = turn on a source to maintain density
	else
	lcxsrc=.false.
	lnsrc=.false.
	endif
 
c***************************************************************
c
c FOR DEBUGGING ONLY:
c
c Hack RF+BEAMS for benchmark with Ongena's Monte-Carlo RF model
c
c Use simplest RF model where all RF fields are Hardwired.
c comments are from fpbasic.dft:
c
	if(imcrfdb .eq. 1) then
	   write(lsysout,*)
	1	'warning from trtofp.for: debugging MC RF'
	   prftot=0.5e6		! total input rf power (watts)
	   FREQRF  =  103.02E+06 ! frequency of RF generator in Hz
	   RKPAR   =  0.06	! abs(kpar) (+/- used in code) (/cm)
	   NHARM   =  1		!1=fundamental 2=2cd harmonic ICRF heating
	   concmi=.1  ! fudge imaginary minority...
	   LSHOOT=0  ! ICRF model 1 or -1 =Smithe's SPRUCE 2=SHOOT 0=Guess
c  LSHOOT=1 or -1 Smithe's general geometry fast-wave propagation code
c  LSHOOT=2 Smithe and Colestock's circular fast-wave code
c  LSHOOT=0 Hand-entered guesses at the ICRF power profile, etc.
 
c  LSHOOT = 0 requires the following guesses at ICRF fields:
 
	   LRFMOD = 3 ! ERFpro is actually
              !        1=E-plus(r) 2=Prf(r) 3=Exact E-plus(r)
	      ! ERFPRO gets renormalized to give specified PRFTOT, unless
	      ! LRFMOD=3, then ERFPRO is E-plus exactly.
	   do i=1, nshell
	      ERFPRO(i) = 7.348469 ! Volt/cm
	      rkperp(i) = 0.	! ICRF k-perp(r) profile (cm^-1)
	   enddo
	   LKmodel=-2 !k_perp model 0= cold plasma k_perp +-2=input rkperp(r)
		! (-2 will turn off Eminus for benchmarking purposes.)
 
	   ! LLWbnd=0	! use 1 = conservative (RF) 0=sink (NBI) lower B.C.'s
           ! LLWbnd=-1 for TRANSP-like nbi BC's.
	   ! (LLWbnd set in fpvgrid...)
 
! VV shape description in Fourier moments (in cm):
	   if(mxVVmoms+1 .ne. mimom) call aborter(lsysout,
	1	'FPP and TRANSP have different number of VV moments')
 
	   do imom=0,mxVVmoms
	      VVRmoms(imom)=VVRmom(imom+1)
	      VVZmoms(imom)=VVZmom(imom+1)
	   enddo
	endif
 
c
c END OF DEBUGGING SECTION.
c
c***************************************************************
 
	Lbeam=0 !Lbeam=1 to read 3-D neutral information file
		!Lbeam=0 use only the neutrals calculated by TRANSP
 
c Lbeam=1 would call FP3DNU, but FP3DNU was broken by the upgrade
c from a single impurity model to a multiple ion species model.
 
c	if(lbeam .eq. 1) call fp3dnu
 
! calculate and some useful parameters, like the slowing
! down time and the critical energy:
 
! main plasma ion density / electron density =
	zDHY=(rZimp-rZeff)/(rZimp-aplasz(2))/aplasz(2)
! impurity ion density / electron density =
	zDIM=(rZeff-aplasz(2))/((rZimp-aplasz(2))*rZimp)
	zovera=zdhy*aplasz(2)**2/Ap + zdim*rZimp**2/Aim
 
 
	do 2700 i=1,nshell
 
	if(llstix) then
c exact Stix formula for benchmarking with Stix:
	    zalle=24. -alog( drelec(i)**.5 / trelec(i))
	else
c Slightly better formula for log(lambda), (best formulas are in
c the subroutine COULOG.FOR):
	    zalle=24.8-alog( drelec(i)**.5 / trelec(i))
	endif
 
	tause(i)=6.32e8*ab*trelec(i)**1.5
     >		  /(drelec(i)*rZbeam**2 * zalle)  ! dmc 9 jun 1992
                                            ! bugfix:  "alle" -> "zalle"
 
2700	Ecrit(i) = Ab * 14.8 * trelec(i) * Zovera**(2./3.)
 
	return
	END
